<?php
/**
 * Droptables
 *
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 *
 * @package   Droptables
 * @copyright Copyright (C) 2014 JoomUnited (http://www.joomunited.com). All rights reserved.
 * @copyright Copyright (C) 2014 Damien Barrère (http://www.crac-design.com). All rights reserved.
 * @license   GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 */
// no direct access
defined('_JEXEC') || die;

/**
 * Class droptablesBase
 */
class droptablesBase //phpcs:ignore Squiz.Classes.ValidClassName.NotCamelCaps -- Class name not change
{
    /**
     * Require once file style, ...
     *
     * @return void
     */
    public static function initComponent()
    {
        $oldRelease = self::getExtensionVersion('com_droptables');
        if (version_compare($oldRelease, '3.8.0', 'lt')) {//delete old file
            jimport('joomla.filesystem.folder');
            jimport('joomla.filesystem.file');
            $folder = JPATH_SITE . DIRECTORY_SEPARATOR . 'media' . DIRECTORY_SEPARATOR . 'droptables' . DIRECTORY_SEPARATOR;
            $files = glob($folder . '*_*.html');
            foreach ($files as $f) {
                JFile::delete($f);
            }

            require_once(JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR . 'com_droptables' . DIRECTORY_SEPARATOR . 'models' . DIRECTORY_SEPARATOR . 'dbtable.php');
            $modelDbTable = new DroptablesModelDbtable();

            $tables_query = $modelDbTable->getListQueryDbTable();
            if ($tables_query) {
                foreach ($tables_query as $table_query) {
                    if ($table_query[1] !== null) {
                        $modelDbTable->updateTableQuery($table_query[1], $table_query[0], false);
                    }
                }
            }
        }

        //Load language from non default position
        self::loadLanguage();

        // Register helper class
        JLoader::register('DroptablesHelper', JPATH_ADMINISTRATOR . '/components/com_droptables/helpers/droptables.php');
        JLoader::register('DroptablesTablesHelper', JPATH_ADMINISTRATOR . '/components/com_droptables/helpers/tables.php');
        // Register helper class
        JLoader::register('DroptablesComponentHelper', JPATH_ADMINISTRATOR . '/components/com_droptables/helpers/component.php');

        //Load scripts and stylesheets
        $document = JFactory::getDocument();
        $params   = JComponentHelper::getParams('com_droptables');
        $view_option = JFactory::getApplication()->input->get('option');
        $view_task = JFactory::getApplication()->input->get('task');
        $view = JFactory::getApplication()->input->get('view', null);
        $get_function = JFactory::getApplication()->input->get('function', null);

        if (self::isJoomla40() || $view_task !== 'configuration.display' || ($view === 'tables' && $get_function === 'jSelectTable_jform_request_id')) {
            JHtml::_('jquery.framework');
            $document->addScript(JURI::root() . 'components/com_droptables/assets/js/jquery-ui.js');
        }
        JHtml::_('bootstrap.framework');

        $app = JFactory::getApplication();
        if ($app->isClient('site')) {
            JHtml::_('bootstrap.loadCss');
            $document->addStyleSheet(JURI::root() . 'components/com_droptables/assets/css/frontstyle.css');
        }

        if ($view_option === 'com_droptables' && $view_task !== 'configuration.display' && $view_task !== 'dbtable.display' && $view !== 'foldertree') {
            $id_table = JFactory::getApplication()->input->get('id_table');
            if ($id_table === null) {//list cat
                $document->addStyleSheet(JURI::root() . 'components/com_droptables/assets/css/simple-scrollbar.css');
                $document->addStyleSheet(JURI::root() . 'components/com_droptables/assets/css/styleCat.css');
                $document->addScript(JURI::root() . 'components/com_droptables/assets/js/simple-scrollbar.js');
                $document->addScript(JUri::root() . 'components/com_droptables/assets/tablesorter/jquery.tablesorter.js');
                $document->addScript(JURI::root() . 'components/com_droptables/assets/js/bootbox.min.js');
                $document->addScript(JURI::root() . 'components/com_droptables/assets/js/jquery.droppable.min.js');
                if (!($view === 'tables' && $get_function === 'jSelectTable_jform_request_id')) {
                    $document->addScript(JURI::root() . 'components/com_droptables/assets/js/categories.js');
                }
            } else {
                $document->addStyleSheet(JURI::root() . 'components/com_droptables/assets/css/table-sprites.css');
                $document->addStyleSheet(JURI::root() . 'components/com_droptables/assets/css/simple-scrollbar.css');
                $document->addStyleSheet(JURI::root() . 'components/com_droptables/assets/css/handsontable.full.min.css?v=6.2');
                $document->addStyleSheet(JURI::root() . 'components/com_droptables/assets/codemirror/codemirror.css');
                $document->addStyleSheet(JURI::root() . 'components/com_droptables/assets/codemirror/3024-night.css');
                $document->addStyleSheet(JURI::root() . 'components/com_droptables/assets/css/tooltipster.bundle.min.css');
                $document->addStyleSheet(JURI::root() . 'components/com_droptables/assets/css/tooltipster-sideTip-borderless.min.css');
                $document->addStyleSheet(JURI::root() . 'components/com_droptables/assets/css/jquery.minicolors.css');
                $document->addStyleSheet(JURI::root() . 'components/com_droptables/assets/css/style.css');


                $document->addScript(JURI::root() . 'components/com_droptables/assets/js/jquery.minicolors.min.js');
                $document->addScript(JURI::root() . 'components/com_droptables/assets/js/simple-scrollbar.js');
                $document->addScript(JURI::root() . 'components/com_droptables/assets/js/less.js');
                $document->addScript(JURI::root() . 'components/com_droptables/assets/js/jquery.textselect.min.js');
                $document->addScript(JURI::root() . 'components/com_droptables/assets/js/jquery.leanModal.min.js');
                $document->addScript(JURI::root() . 'components/com_droptables/assets/js/bootbox.min.js');
                $document->addScript(JURI::root() . 'components/com_droptables/assets/codemirror/codemirror.js');
                $document->addScript(JURI::root() . 'components/com_droptables/assets/codemirror/mode/css/css.js');
                $document->addScript(JURI::root() . 'components/com_droptables/assets/js/chart.min.js');
                $document->addScript(JURI::root() . 'components/com_droptables/assets/js/handsontable.full.min.js?v=6.2');
                $document->addScript(JURI::root() . 'components/com_droptables/assets/js/formula.js');
                $document->addScript(JURI::root() . 'components/com_droptables/assets/js/accounting.js');
                $document->addScript(JURI::root() . 'components/com_droptables/assets/js/moment-jdateformatparser.js');
                $document->addScript(JURI::root() . 'components/com_droptables/assets/js/droptables_main.js');
                $document->addScript(JURI::root() . 'components/com_droptables/assets/js/tooltipster.bundle.min.js');
            }
        }

        if ((int)JFactory::getApplication()->input->getInt('caninsert', 'GET', 'bool') > 0) {
            $document->addScript(JURI::root() . 'components/com_droptables/assets/js/insert_table.js');
        }

        //Only load these file if import-export excel feature is enabled
        if ((string)$params->get('enable_import_excel', 1) === '1') {
            $document->addScript(JURI::root() . 'components/com_droptables/assets/js/dropzone.min.js');
            $document->addScript(JURI::root() . 'components/com_droptables/assets/js/jquery.fileDownload.js');
        }
    }

    /**
     * Require_once helpers/droptables.php, helpers/component.php, css/front.css
     *
     * @return void
     */
    public static function initFrontComponent()
    {
        //Load language from non default position
        self::loadLanguage();

        // Register helper class
        JLoader::register('DroptablesHelper', JPATH_ADMINISTRATOR . '/components/com_droptables/helpers/droptables.php');
        // Register helper class
        JLoader::register('DroptablesComponentHelper', JPATH_ADMINISTRATOR . '/components/com_droptables/helpers/component.php');
        $document = JFactory::getDocument();
        $document->addStyleSheet(JURI::root() . 'components/com_droptables/assets/css/front.css');
    }

    /**
     * Search a param into the component config
     *
     * @param string $path    Param name
     * @param null   $default Default value
     *
     * @return mixed
     */
    public static function getParam($path, $default = null)
    {
        $params = JComponentHelper::getParams('com_droptables');
        return $params->get($path, $default);
    }

    /**
     * Method to return the current joomla version
     *
     * @param string $format Format value to version
     *
     * @return string version
     */
    public static function getJoomlaVersion($format = 'short')
    {
        $method = 'get' . ucfirst($format) . 'Version';

        // Get the joomla version
        $instance = new JVersion();
        $version  = call_user_func(array($instance, $method));

        return $version;
    }

    /**
     * Method to check if current joomla version is 3.X
     *
     * @return boolean
     */
    public static function isJoomla40()
    {
        if (version_compare(self::getJoomlaVersion(), '4.0') >= 0) {
            return true;
        }
        return false;
    }

    /**
     * Method to check if current joomla version is 3.X
     *
     * @return boolean
     */
    public static function isJoomla30()
    {
        if (version_compare(self::getJoomlaVersion(), '3.0') >= 0) {
            return true;
        }

        return false;
    }

    /**
     * Method to check if current joomla version is 3.X
     *
     * @return boolean
     */
    public static function isJoomla39()
    {
        if (version_compare(self::getJoomlaVersion(), '3.10') < 0) {
            return true;
        }

        return false;
    }

    /**
     * Check if a component is installed and activated
     *
     * @param null   $extension Extension name
     * @param string $type      Extension type
     *
     * @return boolean
     */
    public static function isExtensionActivated($extension, $type = '')
    {
        $db    = JFactory::getDbo();
        $query = 'SELECT extension_id FROM #__extensions WHERE element=' . $db->quote($extension);

        if ($type !== '') {
            $query .= ' AND type=' . $db->quote($type);
        }
        $query .= ' AND enabled=1';
        $db->setQuery($query);
        if ($db->execute()) {
            if ($db->getNumRows() > 0) {
                return true;
            }
        }

        return false;
    }

    /**
     * Method to set config parameters
     *
     * @param array $datas Value to check
     *
     * @return boolean
     */
    public static function setParams($datas)
    {
        return DroptablesComponentHelper::setParams($datas);
    }

    /**
     * Load global file language
     *
     * @return void
     */
    public static function loadLanguage()
    {
        $lang = JFactory::getLanguage();
        $lang->load('com_droptables', JPATH_ADMINISTRATOR . '/components/com_droptables', null, true);
        $lang->load('com_droptables.override', JPATH_ADMINISTRATOR . '/components/com_droptables', null, true);
        $lang->load('com_droptables.sys', JPATH_ADMINISTRATOR . '/components/com_droptables', null, true);
    }

    /**
     * Get value of array/object
     *
     * @param array|object   $var     Array/object parent
     * @param string|integer $value   Key value
     * @param string         $default Default value
     *
     * @return mixed|string
     */
    public static function loadValue($var, $value, $default = '')
    {
        if (is_object($var) && isset($var->$value)) {
            return $var->$value;
        } elseif (is_array($var) && isset($var[$value])) {
            return $var[$value];
        }

        return $default;
    }

    /**
     * Check on Joomunited website the latest version number of the component
     *
     * @param null|string $extension Extension name
     *
     * @return false or version number (string)
     */
    public static function getLastExtensionVersion($extension = null)
    {
        if ($extension === null) {
            $extension = JFactory::getApplication()->input->getString('option', '');
        }
        if (ini_get('allow_url_fopen') === '1') {
            $content = file_get_contents('http://www.joomunited.com/UPDATE-INFO/updates.json');
        } else {
            return false;
        }
        $json = json_decode($content);

        return $json->extensions->$extension->version;
    }

    /**
     * Get version extansion
     *
     * @param null   $extension Extension name
     * @param string $type      Extension type
     *
     * @return boolean
     */
    public static function getExtensionVersion($extension = null, $type = '')
    {
        if ($extension === null) {
            $extension = JFactory::getApplication()->input->getString('option', '');
        }
        $db    = JFactory::getDbo();
        $query = 'SELECT manifest_cache FROM #__extensions WHERE element=' . $db->quote($extension);

        if ($type !== '') {
            $query .= ' AND type=' . $db->quote($type);
        }
        $db->setQuery($query);
        if ($db->execute()) {
            $manifest = $db->loadResult();
            $json     = json_decode($manifest);
            if (property_exists($json, 'version')) {
                return $json->version;
            }
        }

        return false;
    }
}
