<?php
/**
 * Droptables
 *
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 *
 * @package   Droptables
 * @copyright Copyright (C) 2014 JoomUnited (http://www.joomunited.com). All rights reserved.
 * @copyright Copyright (C) 2014 Damien Barrère (http://www.crac-design.com). All rights reserved.
 * @license   GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 */

defined('_JEXEC') || die;

//require_once(JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR . 'com_categories' . DIRECTORY_SEPARATOR . 'controllers' . DIRECTORY_SEPARATOR . 'categories.php');
jimport('joomla.filesystem.folder');

/**
 * Class DroptablesControllerCategories
 */
class DroptablesControllerCategories extends JControllerForm
{
    /**
     * Proxy for getModel
     *
     * @param string $name   The model name. Optional.
     * @param string $prefix The class prefix. Optional.
     * @param array  $config The config get model
     *
     * @return object The model.
     * @since  1.6
     */
    public function getModel($name = 'Category', $prefix = 'DroptablesModel', $config = array('ignore_request' => true))
    {
        $model = parent::getModel($name, $prefix, $config);
        return $model;
    }

    /**
     * Removes an item.
     *
     * @return void
     *
     * @since 11.1
     */
    public function delete()
    {
        // Check for request forgeries
        JSession::checkToken() || die(JText::_('JINVALID_TOKEN'));

        // Get items to remove from the request.
        $cid = JFactory::getApplication()->input->getInt('id_category', 0);

        if ($cid) {
            // Get the model.
            $model = $this->getModel();
            JFactory::getApplication()->setUserState('list.limit', 100000);

            $modelCats = $this->getModel('categories');
            $modelCats->setState('category.id', $cid);
            $modelCats->setState('filter.parentId', $cid);
            $modelCats->setState('category.recursive', true);
            $items = $modelCats->getItems();

            $canDo = DroptablesHelper::getActions();
            if ($canDo->get('core.delete')) {
                if (!$canDo->get('core.edit')) {
                    if ($canDo->get('core.edit.own')) {
                        $gallery = $model->getItem($cid);
                        if ((int)$gallery->created_user_id !== (int)JFactory::getUser()->id) {
                            $this->exitStatus('not permitted');
                        }
                    } else {
                        $this->exitStatus('not permitted');
                    }
                }
            } else {
                $this->exitStatus('not permitted');
            }

            //little hack because joomla always delete children, bug in version 3.1
            $joomla31 = false;
            if (version_compare(droptablesBase::getJoomlaVersion(), '3.1')) {
                $joomla31 = true;
            }


            $errors = array();
            // Remove the items.
            $modelCat = $this->getModel('category');
            $category = $modelCat->getCategory($cid);
            if ($model->delete($cid)) {
                foreach ($items as $item) {
                    if ((int)$item->id === (int)$cid[0]) {
                        continue;
                    }
                    $id = $item->id;
                    $category = $modelCat->getCategory($id);
                    if (!$model->delete($id)) {
                        $errors[] = 'error while deleting category'; //todo: translate
                    }
                }
            } else {
                $errors[] = $model->getError();
            }
            if (count($errors)) {
                $this->exitStatus(implode('<br/>', $errors));
            }
            $this->exitStatus(true);
        }
    }

    /**
     * Change order for category
     *
     * @param null|string  $position Position cat
     * @param null|string  $pk       Parent cat
     * @param null|integer $ref      Ref
     *
     * @return void
     */
    public function order($position = null, $pk = null, $ref = null)
    {
        $position = $position === null ? JFactory::getApplication()->input->getCmd('position', 'after') : $position;
        $pk = $pk === null ? JFactory::getApplication()->input->getInt('pk', null) : $pk;
        $ref = $ref === null ? JFactory::getApplication()->input->getInt('ref') : $ref;
        $model = $this->getModel();
        $canDo = DroptablesHelper::getActions();
        if (!$canDo->get('core.edit')) {
            if ($canDo->get('core.edit.own')) {
                $category = $model->getItem($pk);
                if ((int)$category->created_user_id !== (int)JFactory::getUser()->id) {
                    $this->exitStatus('not permitted');
                }
            } else {
                $this->exitStatus('not permitted');
            }
        }

        if ((int)$ref === 0) {
            $ref = 1;
        }
        if ($position !== 'after') {
            $position = 'first-child';
        }

        $table = $model->getTable();
        if ($table->moveByReference($ref, $position, $pk)) {
            $this->exitStatus(true, $pk . ' ' . $position . ' ' . $ref);
        }
        $this->exitStatus('problem');
    }

    /**
     * Return a json response
     *
     * @param boolean $status Status responsive
     * @param array   $datas  Data array to return with the json string
     *
     * @return void
     */
    private function exitStatus($status, $datas = array())
    {
        $response = array('response' => $status, 'datas' => $datas);
        echo json_encode($response);
        JFactory::getApplication()->close();
    }
}
