<?php
/**
 * Droptables
 *
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 *
 * @package   Droptables
 * @copyright Copyright (C) 2014 JoomUnited (http://www.joomunited.com). All rights reserved.
 * @copyright Copyright (C) 2014 Damien Barrère (http://www.crac-design.com). All rights reserved.
 * @license   GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 */

// no direct access
defined('_JEXEC') || die;

//require_once(JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR . 'com_categories' . DIRECTORY_SEPARATOR . 'controllers' . DIRECTORY_SEPARATOR . 'category.php');

/**
 * Class DroptablesControllerCategory
 */
class DroptablesControllerCategory extends JControllerForm
{
    /**
     * Id category
     *
     * @var integer
     */
    private $savedId;

    /**
     * Function set title for category
     *
     * @return void
     */
    public function setTitle()
    {
        $id_category = JFactory::getApplication()->input->getInt('id_category', 0);

        $model = $this->getModel();
        $canDo = DroptablesHelper::getActions();
        if (!$canDo->get('core.edit')) {
            if ($canDo->get('core.edit.own')) {
                $category = $model->getItem($id_category);
                if ((int)$category->created_user_id !== (int)JFactory::getUser()->id) {
                    $this->exitStatus('not permitted');
                }
            } else {
                $this->exitStatus('not permitted');
            }
        }
        $title = JFactory::getApplication()->input->getString('title');

        if ($model->setTitle($id_category, $title)) {
            $this->exitStatus(true);
        } else {
            $this->exitStatus(JText::_('COM_DROPTABLES_GLOBAL_ERROR'));
        }
    }

    /**
     * Method to add a category
     *
     * @return void
     */
    public function addCategory()
    {
        $canDo = DroptablesHelper::getActions();
        if (!$canDo->get('core.create')) {
            $this->exitStatus('not permitted');
        }
        $id_category = JFactory::getApplication()->input->getInt('id_category', 0);
        $name = JFactory::getApplication()->input->getString('name', JText::_('COM_DROPTABLES_MODEL_CATEGORY_DEFAULT_NAME'));
        if ($id_category > 0) {
            $datas = array();
            $datas['jform']['id'] = 0;
            $datas['jform']['extension'] = 'com_droptables';
            $datas['jform']['title'] = $name;
            $datas['jform']['alias'] = JText::_('COM_DROPTABLES_MODEL_CATEGORY_DEFAULT_NAME') . '-' . date('dmY-h-m-s', time());
            $datas['jform']['parent_id'] = $id_category;
            $datas['jform']['published'] = 1;
            $datas['jform']['language'] = '*';
            $datas['jform']['metadata']['tags'] = '';

            //Set state value to retreive the correct table
            $model = $this->getModel();
            $model->setState('category.extension', 'com_droptables');

            $app = JFactory::getApplication();
            $app->input->set('id', null, 'POST');
            foreach ($datas as $data => $val) {
                $app->input->set($data, $val, 'POST');
            }

            if ($this->save()) {
                // move to correct location
                if ($id_category !== 1) {
                    $table = $model->getTable();
                    $table->moveByReference($id_category, 'first-child', $this->savedId);
                }
                $this->exitStatus(true, array('id_category' => $this->savedId, 'name' => $name));
            }
        }

        $this->exitStatus('error while adding category');
    }

    /**
     * Method to save a record.
     *
     * @param string $key    The name of the primary key of the URL variable.
     * @param string $urlVar The name of the URL variable if different from the primary key (sometimes required
     * to avoid router collisions).
     *
     * @return boolean  True if successful, false otherwise.
     *
     * @throws \Exception Throw when application not start
     * @since  12.2
     */
    public function save($key = null, $urlVar = null)
    {
        $app = JFactory::getApplication();
        $lang = JFactory::getLanguage();
        $model = $this->getModel();

        $table = $model->getTable();

        $data = $app->input->get('jform', array(), 'array');

        property_exists($table, 'checked_out');
        $context = $this->option . '.edit.' . $this->context;

        // Determine the name of the primary key for the data.
        if (empty($key)) {
            $key = $table->getKeyName();
        }

        // To avoid data collisions the urlVar may be different from the primary key.
        if (empty($urlVar)) {
            $urlVar = $key;
        }

        $recordId = $app->input->getInt($urlVar);

        // Populate the row id from the session.
        $data[$key] = $recordId;

        // Access check.
        if (!$this->allowSave($data, $key)) {
            $this->setError(JText::_('JLIB_APPLICATION_ERROR_SAVE_NOT_PERMITTED'));
            $this->setMessage($this->getError(), 'error');

            $redirect_str = 'index.php?option=' . $this->option . '&view=' . $this->view_list;
            $redirect_str .= $this->getRedirectToListAppend();
            $this->setRedirect(JRoute::_($redirect_str, false));

            return false;
        }

        // Validate the posted data.
        // Sometimes the form needs some posted data, such as for plugins and modules.
        $form = $model->getForm($data, false);

        if (!$form) {
            $app->enqueueMessage($model->getError(), 'error');

            return false;
        }

        // to make it work when logged in as administrator on frontend then saving data
        unset($data['published']);
        // Test whether the data is valid.
        $validData = $model->validate($form, $data);

        // Check for validation errors.
        if ($validData === false) {
            // Get the validation messages.
            $errors = $model->getErrors();

            // Push up to three validation messages out to the user.
            for ($i = 0, $n = count($errors); $i < $n && $i < 3; $i++) {
                if ($errors[$i] instanceof Exception) {
                    $app->enqueueMessage($errors[$i]->getMessage(), 'warning');
                } else {
                    $app->enqueueMessage($errors[$i], 'warning');
                }
            }

            // Save the data in the session.
            $app->setUserState($context . '.data', $data);

            // Redirect back to the edit screen.
            $redirect_str = 'index.php?option=' . $this->option . '&view=' . $this->view_item;
            $redirect_str .= $this->getRedirectToItemAppend($recordId, $urlVar);
            $this->setRedirect(JRoute::_($redirect_str, false));

            return false;
        }

        // Attempt to save the data.
        $validData['published'] = 1;
        if (!isset($validData['parent_id'])) {
            $validData['parent_id'] = 1;
        }
        $model->setState('droptables.category.parent_id', $validData['parent_id']);
        if (!$model->save($validData)) {
            // Save the data in the session.
            $app->setUserState($context . '.data', $validData);

            // Redirect back to the edit screen.
            $this->setError(JText::sprintf('JLIB_APPLICATION_ERROR_SAVE_FAILED'));
            $this->setMessage($this->getError(), 'error');
            $redirect_str = 'index.php?option=' . $this->option . '&view=' . $this->view_item;
            $redirect_str .= $this->getRedirectToItemAppend($recordId, $urlVar);
            $this->setRedirect(JRoute::_($redirect_str, false));

            return false;
        }

        $text_prefix = $this->text_prefix . ($recordId === 0 && $app->isClient('site') ? '_SUBMIT' : '') . '_SAVE_SUCCESS';
        $this->setMessage(
            JText::_(
                ($lang->hasKey($text_prefix) ? $this->text_prefix
                    : 'JLIB_APPLICATION') . ($recordId === 0 && $app->isClient('site') ? '_SUBMIT' : '') . '_SAVE_SUCCESS'
            )
        );

        // Clear the record id and data from the session.
        $this->releaseEditId($context, $recordId);
        $app->setUserState($context . '.data', null);

        // Redirect to the list screen.
//        $redirect_str = 'index.php?option=' . $this->option . '&view=' . $this->view_list;
//        $redirect_str .= $this->getRedirectToListAppend();
//        $this->setRedirect(JRoute::_($redirect_str, false));

        // Invoke the postSave method to allow for the child class to access the model.
        $this->postSaveHookJ3($model, $validData);

        return true;
    }
    /**
     * Method run after save J3
     *
     * @param JModelLegacy $model     Model instance
     * @param array        $validData Data
     *
     * @return void
     * @throws \Exception Throw when Model name not found
     * @since  1.0
     */
    protected function postSaveHookJ3(JModelLegacy $model, $validData = array())
    {
        $this->savedId = $model->getState($model->getName() . '.id');
        parent::postSaveHook($model, $validData);
    }
    /**
     * Create new category for mysql tables
     *
     * @return void
     */
    public function addCategoryDBtable()
    {
        $dbtableModel = JModelLegacy::getInstance('Dbtable', 'DroptablesModel');
        $id_category = $dbtableModel->getCategoryId();

        $this->exitStatus(true, array('id_category' => $id_category));
    }

    /**
     * Return a json response
     *
     * @param boolean $status Status responsive
     * @param array   $datas  Data array to return with the json string
     *
     * @return void
     */
    private function exitStatus($status, $datas = array())
    {
        $response = array('response' => $status, 'datas' => $datas);
        echo json_encode($response);
        JFactory::getApplication()->close();
    }

    /**
     * Function save hook
     *
     * @param JModelLegacy $model     Model
     * @param array        $validData Data
     *
     * @return void
     */
    protected function postSaveHook(JModelLegacy $model, $validData = array())
    {
        $this->savedId = $model->getState($model->getName() . '.id');
        parent::postSaveHook($model, $validData);
    }
}
