<?php
/**
 * Droptables
 *
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 *
 * @package   Droptables
 * @copyright Copyright (C) 2014 JoomUnited (http://www.joomunited.com). All rights reserved.
 * @copyright Copyright (C) 2014 Damien Barrère (http://www.crac-design.com). All rights reserved.
 * @license   GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 */

defined('_JEXEC') || die;

jimport('joomla.application.component.controllerform');

/**
 * Class DroptablesControllerChart
 */
class DroptablesControllerChart extends JControllerForm
{
    /**
     * Function save the changing for chart
     *
     * @param null $key    Key
     * @param null $urlVar Url var
     *
     * @since 3.5.0
     *
     * @return void
     */
    public function save($key = null, $urlVar = null)
    {
        $id = JFactory::getApplication()->input->getInt('id', 0);
        $data = JFactory::getApplication()->input->getVar('jform', array());
        $model = $this->getModel();
        $data['id'] = $id;
        if ($model->save($data)) {
            $this->exitStatus(true);
        } else {
            $this->exitStatus(JText::_('COM_DROPTABLES_GLOBAL_ERROR'));
        }
    }

    /**
     * Function add chart table
     *
     * @since 3.5.0
     *
     * @return void
     */
    public function add()
    {
        $id_table = JFactory::getApplication()->input->getInt('id_table');
        $datas = JFactory::getApplication()->input->getString('datas');
        $model = $this->getModel();
        $newItem = $model->add($id_table, $datas);

        if ($newItem) {
            $table = $model->getItem($newItem);
            $this->exitStatus(true, array(
                'id' => $newItem,
                'datas' => $table->datas,
                'type' => 'Line',
                'title' => JText::_('COM_DROPTABLES_MODEL_CHART_DEFAULT_TITLE')
            ));
        } else {
            $this->exitStatus(JText::_('COM_DROPTABLES_GLOBAL_ERROR'));
        }
    }

    /**
     * Function copy table charts
     *
     * @since 3.5.0
     *
     * @return void
     */
    public function copy()
    {
        $id = JFactory::getApplication()->input->getInt('id');
        $model = $this->getModel();
        $newItem = $model->copy($id);
        if ($newItem) {
            $table = $model->getItem($newItem);
            $this->exitStatus(true, array('id' => $table->id, 'title' => $table->title, 'datas' => $table . datas));
        } else {
            $this->exitStatus(JText::_('COM_DROPTABLES_GLOBAL_ERROR'));
        }
    }

    /**
     * Function delete table chart
     *
     * @since 3.5.0
     *
     * @return void
     */
    public function delete()
    {
        $id = JFactory::getApplication()->input->getInt('id');
        $model = $this->getModel();
        if ($model->delete($id)) {
            $this->exitStatus(true);
        } else {
            $this->exitStatus(JText::_('COM_DROPTABLES_GLOBAL_ERROR'));
        }
    }

    /**
     * Change title chart
     *
     * @since 3.5.0
     *
     * @return void
     */
    public function setTitle()
    {
        $id = JFactory::getApplication()->input->getInt('id', 0);

        $model = $this->getModel();
        $canDo = DroptablesHelper::getActions();
        if (!$canDo->get('core.edit')) {
            if ($canDo->get('core.edit.own')) {
                $chart = $model->getItem($id);
                if ((int)$chart->author !== (int)JFactory::getUser()->id) {
                    $this->exitStatus('not permitted');
                }
            } else {
                $this->exitStatus('not permitted');
            }
        }

        $title = JFactory::getApplication()->input->getString('title');

        if ($model->setTitle($id, $title)) {
            $this->exitStatus(true);
        } else {
            $this->exitStatus(JText::_('COM_DROPTABLES_GLOBAL_ERROR'));
        }
    }

    /**
     * Return a json response
     *
     * @param boolean $status Status responsive
     * @param array   $datas  Data array to return with the json string
     *
     * @since 3.5.0
     *
     * @return void
     */
    private function exitStatus($status, $datas = array())
    {
        $response = array('response' => $status, 'datas' => $datas);
        echo json_encode($response);
        JFactory::getApplication()->close();
    }
}
