<?php
/**
 * Droptables
 *
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 *
 * @package   Droptables
 * @copyright Copyright (C) 2014 JoomUnited (http://www.joomunited.com). All rights reserved.
 * @copyright Copyright (C) 2014 Damien Barrère (http://www.crac-design.com). All rights reserved.
 * @license   GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 */

// no direct access
defined('_JEXEC') || die;

jimport('joomla.application.component.controllerform');

/**
 * Class DroptablesControllerConfig
 */
class DroptablesControllerConfiguration extends JControllerForm
{
    /**
     * Display the view
     *
     * @param null|string $cachable  Template
     * @param null|string $urlparams Link
     *
     * @return void
     */
    public function display($cachable = false, $urlparams = false)
    {
        // Access check.
        if (JFactory::getUser()->authorise('core.admin', 'com_droptables')) {
            // Force it to be the search view
            $this->input->set('view', 'configuration');

            parent::display();
        } else {
            $content  = '<div id="system-message-container" class="ju-message-permissions">';
            $content .= '<div class="alert alert-error"><button type="button" class="close" data-dismiss="alert">×</button>';
            $content .= '<h4 class="alert-heading">'. JText::_('COM_DROPTABLES_VIEW_DROPTABLES_CHECKERROR') .'</h4><div class="alert-message">'. JText::_('JERROR_ALERTNOAUTHOR');
            $content .= '</div></div></div>';

            echo $content;
        }
    }

    /**
     * Save params
     *
     * @return void
     */
    public function saveParams()
    {
        //Security check
        JSession::checkToken() || die('Invalid Token');

        $result = $this->saveToConfiguration();

        $type = 'success';
        $text = JText::_('COM_DROPTABLES_CONFIGURATION_MESSAGE_SAVE_SUCCESS');

        if (!$result) {
            $type = 'error';
            $text = JText::_('COM_DROPTABLES_CONFIGURATION_MESSAGE_SAVE_GLOBAL_CONFIG_ERROR');
        }

        JFactory::getSession()->set(
            'droptable_configuration_message',
            array(
                'type' => $type,
                'text' => $text
            )
        );
        $this->exitStatus(true, array('url' => JRoute::_('index.php?option=com_droptables&task=configuration.display', false)));
    }

    /**
     * Save params and close
     *
     * @return void
     */
    public function saveParamsAndClose()
    {
        //Security check
        JSession::checkToken() || die('Invalid Token');

        $result = $this->saveToConfiguration();

        $type = 'success';
        $text = JText::_('COM_DROPTABLES_CONFIGURATION_MESSAGE_SAVE_SUCCESS');

        if (!$result) {
            $type = 'error';
            $text = JText::_('COM_DROPTABLES_CONFIGURATION_MESSAGE_SAVE_GLOBAL_CONFIG_ERROR');
        }

        JFactory::getSession()->set(
            'droptable_configuration_message',
            array(
                'type' => $type,
                'text' => $text
            )
        );

        $this->exitStatus(true, array('url' => JRoute::_('index.php?option=com_droptables', false)));
    }

    /**
     * Close configuration
     *
     * @return void
     */
    public function closeConfiguration()
    {
        $this->exitStatus(true, array('url' => JRoute::_('index.php?option=com_droptables', false)));
    }

    /**
     * Save configuration
     *
     * @return boolean
     *
     * @throws Exception When getApplication don't run
     */
    public function saveToConfiguration()
    {
        $input = JFactory::getApplication()->input;
        // If we are on the save command we need the actual data
        $jformData = $input->get('jform', array(), 'array');

        if (empty($jformData)) {
            $this->setError(JText::_('COM_DROPTABLES_CONFIGURATION_NO_GET_FORM_DATA_ERROR'));
            return false;
        }

        $model = $this->getModel();

        if (isset($jformData['rules'])) {
            $permissions = $model->savePermissions($jformData);

            if (!$permissions) {
                return false;
            }
        } else {
            if (isset($jformData['include_editor_buttons'])) {
                $jformData['include_editor_buttons'] = str_replace(' ', '', $jformData['include_editor_buttons']);
            }
            if ($jformData['include_editor_buttons'] === '') {
                $jformData['include_editor_buttons'] = ' ';
            }
            $global = $model->saveGlobalConfig($jformData);

            if (!$global) {
                $this->setError(JText::_('COM_DROPTABLES_CONFIGURATION_MESSAGE_SAVE_GLOBAL_CONFIG_ERROR'));
                return false;
            }
        }

        return true;
    }

    /**
     * Exit a request serving a json result
     *
     * @param string $status Exit status
     * @param array  $datas  Echoed datas
     *
     * @since 1.0.3
     *
     * @return void
     */
    protected function exitStatus($status = '', $datas = array())
    {
        $response = array('response' => $status, 'datas' => $datas);
        echo json_encode($response);
        die();
    }
}
