<?php
/**
 * Droptables
 *
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 *
 * @package   Droptables
 * @copyright Copyright (C) 2014 JoomUnited (http://www.joomunited.com). All rights reserved.
 * @copyright Copyright (C) 2014 Damien Barrère (http://www.crac-design.com). All rights reserved.
 * @license   GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 */

// no direct access
defined('_JEXEC') || die;

jimport('joomla.application.component.controllerform');

/**
 * Class DroptablesControllerDbtable
 */
class DroptablesControllerDbtable extends JControllerForm
{
    /**
     * Display the view
     *
     * @param null|string $cachable  Template
     * @param null|string $urlparams Link
     *
     * @return void
     */
    public function display($cachable = false, $urlparams = false)
    {
        // Access check.
        if (JFactory::getUser()->authorise('com_droptables.dbtable', 'com_droptables')) {
            // Force it to be the search view
            $this->input->set('view', 'dbtable');

            parent::display();
        }
    }

    /**
     * Get model
     *
     * @param string $name   Model name
     * @param string $prefix Prefix
     * @param array  $config Config
     *
     * @return DroptablesModelDbtable|JModelLegacy
     */
    public function getModel($name = '', $prefix = '', $config = array('ignore_request' => true))
    {
        if (!DroptablesHelper::getActions()->get('com_droptables.dbtable')) {
            $this->exitStatus(false, JText::_('COM_DROPTABLES_VIEW_DROPTABLES_ERROR_PERMISSIONS'));
        }
        require_once(JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR . 'com_droptables' . DIRECTORY_SEPARATOR . 'models' . DIRECTORY_SEPARATOR . 'dbtable.php');
        return new DroptablesModelDbtable();
    }

    /**
     * Function change table
     *
     * @return void
     */
    public function changeTables()
    {
        $tables = JFactory::getApplication()->input->get('tables');
        $model = $this->getModel();
        $columns = $model->listMySQLColumns($tables);

        $this->exitStatus(true, array('columns' => $columns));
    }

    /**
     * Return a build query and preview table
     *
     * @return void
     */
    public function generateQueryAndPreviewdata()
    {
        $table_data = JFactory::getApplication()->input->get('table_data', array(), 'ARRAY');

        $model = $this->getModel();
        $result = $model->generateQueryAndPreviewdata($table_data);

        $this->exitStatus(true, $result);
    }

    /**
     * Create new table for selected mysql table
     *
     * @return void
     */
    public function createTable()
    {
        $table_data = JFactory::getApplication()->input->get('table_data', array(), 'ARRAY');
        $id_cat = JFactory::getApplication()->input->getInt('id_cat', 0);
        $model = $this->getModel();
        $result = $model->createTable($table_data, $id_cat);

        $this->exitStatus(true, $result);
    }

    //update table with new change

    /**
     * Change table
     *
     * @return void
     */
    public function updateTable()
    {
        $table_data = JFactory::getApplication()->input->get('table_data', array(), 'ARRAY');
        $id_table = (int)$table_data['id_table'];
        $model = $this->getModel();
        $result = $model->updateTable($id_table, $table_data);

        $this->exitStatus(true, $result);
    }

    /**
     * Function get query table
     *
     * @return void
     */
    public function getQuery()
    {
        $id_table = JFactory::getApplication()->input->getInt('id', 0);

        $model = $this->getModel();
        $parmas = $model->getTableQuery($id_table);

        $result = '';
        if ($parmas === null) {
            $parmas = $model->getTableParams($id_table, 'mysql_query');

            if ($parmas !== null) {
                $result = $parmas;
            }
        } else {
            $result = $parmas;
        }

        $this->exitStatus(true, $result);
    }

    /**
     * Function getTableColumnsOptions
     *
     * @param string|null $queryTable Query table
     *
     * @return void|array
     */
    public function getTableColumnsOptions($queryTable = null)
    {
        if ($queryTable !== null) {
            $table_data = false;
            $query = $queryTable;
        } else {
            $table_data = JFactory::getApplication()->input->getString('tableList', false);
            $query = JFactory::getApplication()->input->getString('query', null);
        }

        if ($query === null) {
            $this->exitStatus(false, $query);
        }

        $query = str_replace(array("\n\r", "\r\n", "\n", "\r", '&#10;'), ' ', $query);
        $query = str_replace('\\', '', $query);

        $model = $this->getModel();
        require_once(JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR . 'com_droptables' . DIRECTORY_SEPARATOR . 'classes' . DIRECTORY_SEPARATOR . 'php-sql-parser' . DIRECTORY_SEPARATOR . 'vendor' . DIRECTORY_SEPARATOR . 'autoload.php');
        $parser = new \PHPSQLParser\PHPSQLParser(false, true);
        if (strpos($query, '{user_id}') !== false) {
            $query = str_replace('{user_id}', ' {user_id}', $query);
        }
        if (strpos($query, '{user_email}') !== false) {
            $query = str_replace('{user_email}', ' {user_email}', $query);
        }
        $parserMysql = $parser->parse($query, true);

        $countTable = count($parserMysql['FROM']);
        $columns = $parserMysql['SELECT'];//array
        $count = count($columns);

        //list tables
        $table_in_query = array();
        $tables_list_alias = array();
        for ($i = 0; $i < $countTable; $i++) {
            $table_in_query[] = $parserMysql['FROM'][$i]['table'];
            if (!empty($parserMysql['FROM'][$i]['alias'])) {
                $tables_list_alias[$parserMysql['FROM'][$i]['alias']['no_quotes']['parts'][0]] = $parserMysql['FROM'][$i]['table'];
            }
        }

        //list columns
        $listColumns = array();
        $listColumnss = array();
        for ($i = 0; $i < $count; $i++) {
            $nameColumn = explode('.', $columns[$i]['base_expr']);
            if (!empty($nameColumn[1])) {
                $listColumns[] = $nameColumn[1];
                if (!empty($tables_list_alias[$nameColumn[0]])) {//table is alias
                    $listColumnss[] = $tables_list_alias[$nameColumn[0]] . '.' . $nameColumn[1];
                } else {
                    $listColumnss[] = $columns[$i]['base_expr'];
                }
            } else {
                $listColumns[] = $columns[$i]['base_expr'];
                $listColumnss[] = $columns[$i]['base_expr'];
            }
        }

        if (!empty($table_data) && !empty($table_data[0])) {//has option of db table
            $result = $model->getTableColumnsOptions($table_data);
        } elseif (!empty($query)) {//custom query
            $result = $model->getTableColumnsOptions($table_in_query, $listColumns);
        } else {
            $result = array();
        }

        $dataReturn = array();
        $dataReturn['column'] = $listColumnss;
        $dataReturn['table'] = $table_in_query;
        $dataReturn['result'] = $result;

        if ($queryTable !== null) {
            return $dataReturn;
        } else {
            $this->exitStatus(true, $dataReturn);
        }
    }

    /**
     * Removes all dangerous strings from query
     *
     * @param string $query Custom query
     *
     * @return boolean|array
     */
    public function checkCustomQuery($query)
    {
        preg_match_all('/CREATE|DELETE|DROP|INSERT|SET|UPDATE|REPLACE|create|delete|drop|insert|update|set|replace/i', $query, $check, PREG_OFFSET_CAPTURE);
        if (!empty($check[0])) {
            return $check[0];
        } else {
            return false;
        }
    }

    /**
     * Function applyCustomQuery
     *
     * @return void
     */
    public function applyCustomQuery()
    {
        $custom_mysql = JFactory::getApplication()->input->get('custom_mysql', 'none', 'raw');

        $model        = $this->getModel();
        $custom_mysql = str_replace(array("\n\r", "\r\n", "\n", "\r", '&#10;'), ' ', $custom_mysql);
        $custom_mysql = str_replace('\\', '', $custom_mysql);
        if (strpos($custom_mysql, '{user_id}') !== false) {
            $id_user = (int)JFactory::getUser()->id;
            $custom_mysql = str_replace('{user_id}', ' ' . $id_user, $custom_mysql);
        }
        if (strpos($custom_mysql, '{user_email}') !== false) {
            $email_user = JFactory::getUser()->email;
            $custom_mysql = str_replace('{user_email}', ' \'@' . $email_user . '@\'', $custom_mysql);
        }

        $check_query  = $this->checkCustomQuery($custom_mysql);

        if ($check_query !== false) {
            try {
                require_once(JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR . 'com_droptables' . DIRECTORY_SEPARATOR . 'classes' . DIRECTORY_SEPARATOR . 'php-sql-parser' . DIRECTORY_SEPARATOR . 'vendor' . DIRECTORY_SEPARATOR . 'autoload.php');
                $parser = new \PHPSQLParser\PHPSQLParser(false, true);
                $parser = $parser->parse($custom_mysql);

                $count = count($check_query[0]);
                for ($i = 0; $i < $count; $i++) {
                    if (!empty($check_query[$i][0]) && !empty($parser[$check_query[$i][0]])) {
                        $this->exitStatus(false, JText::_('COM_DROPTABLES_LAYOUT_DBTABLES_QUERY_ERROR'));
                    }
                }
            } catch (Exception $e) {
                $this->exitStatus(false, JText::_('COM_DROPTABLES_LAYOUT_DBTABLES_QUERY_ERROR'));
            }
        }

        $result = $model->applyCustomQuery($custom_mysql);
        if ($result['hasRow'] === false) {
            $this->exitStatus(false, $result);
        }
        $this->exitStatus(true, $result);
    }


    /**
     * Function return a json response
     *
     * @param boolean $status Status response
     * @param array   $datas  Array of datas to return with the json string
     *
     * @return void
     */
    private function exitStatus($status, $datas = array())
    {
        $response = array('response' => $status, 'datas' => $datas);
        echo json_encode($response);
        JFactory::getApplication()->close();
    }
}
