<?php

/**
 * Droptables
 *
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 *
 * @package   Droptables
 * @copyright Copyright (C) 2014 JoomUnited (http://www.joomunited.com). All rights reserved.
 * @copyright Copyright (C) 2014 Damien Barrère (http://www.crac-design.com). All rights reserved.
 * @license   GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 */
// no direct access
defined('_JEXEC') || die;

jimport('joomla.application.component.controllerform');
jimport('joomla.filesystem.file');

/**
 * Class DroptablesControllerExcel
 */
class DroptablesControllerExcel extends JControllerForm
{
    /**
     * Id table
     *
     * @var integer
     */
    private $id_table = 0;
    /**
     * Error message
     *
     * @var string
     */
    private $error_message = '';
    /**
     * List error message
     *
     * @var array
     */
    private $error_message_array = array();
    /**
     * File format
     *
     * @var array
     */
    private $allowed_ext = array('xls', 'xlsx');

    /**
     * Import file excel to table
     *
     * @return void|boolean
     */
    public function import()
    {
        $app = JFactory::getApplication();
        $json = array();
        $file = $app->input->getString('file');

        $targetPath = JPATH_SITE . '/tmp/uploads/';
        if ($file) {
            $file = $targetPath . $file;
        } else {
            $file = $this->uploadFile();
        }

        if ($file) {
            $id = $app->input->getInt('id_table');
            $onlydata = $app->input->getInt('onlydata');
            $ignoreCheck = $app->input->getInt('ignoreCheck');

            $modelTable = $this->getModel('table');

            if ($onlydata === 1) {//render new style
                $tableContent = (array)$modelTable->getItem($id, false, true);
                $this->fetchStyle = true;
            } else {
                $this->fetchStyle = false;
                $tableContent = (array)$modelTable->getItem($id, false, false);
            }

            $doUpdate = false;

            if (!isset($tableContent['params'])) {
                $tableContent['params'] = new stdClass();
            }

            $this->id_table = $id;
            $readFileExcel = $this->readFileExcel($file, true, false, $tableContent['params'], $this->fetchStyle);//return text, file, status

            if (isset($readFileExcel['status']) && $readFileExcel['status']) {//ok file
                $doUpdate = true;
            } elseif (isset($readFileExcel['text'])) {
                $this->exitStatus($readFileExcel['text']);
            } elseif (is_string($readFileExcel)) {
                $this->exitStatus($readFileExcel);
            } else {
                return false;
            }

            if ($onlydata === 1) {
                $modelTable->deleteOldStyle($id);
                $tableContent['style'] = new stdClass();
                $tableContent['style']->rows = new stdClass();
                $tableContent['style']->cols = new stdClass();
            }

            $tableContent['datas'] = $readFileExcel['data']['data'];
            $tableContent['params']->mergeSetting = $readFileExcel['data']['mergeSetting'];
            if ($this->fetchStyle) {
                $tableContent['params']->cell_types = $readFileExcel['data']['typeCells'];
                $tableContent['style']->rows = $readFileExcel['data']['style']['rows'];
                $tableContent['style']->cols = $readFileExcel['data']['style']['cols'];
                $tableContent['style'] = json_encode($tableContent['style']);

                $tableContent['styleCells'] = $readFileExcel['data']['style']['cell'];
            }
            $tableContent['numberRow'] = count($tableContent['datas']);
            $tableContent['numberCol'] = count($tableContent['datas'][0]);

            $count = count($readFileExcel['data']['hyperlink']);
            if ($readFileExcel['data']['hyperlink'] !== false) {
                if ($count > 0) {
                    $tableContent['params']->cell_types = $this->changeHyperlinksTable1($tableContent['params']->cell_types, $readFileExcel['data']['hyperlink']);
                }
            }
            $tableContent['params']->hyperlink = $readFileExcel['data']['hyperlink'];

            $tableContent['action'] = 'insert';

            if (isset($file) && file_exists($file)) {
                unlink($file);
            }

            if ($doUpdate) {
                //add table header = 1
                $header = 1;
                $tableContent['params']->headerOption = $header;
                $tableContent['header_data'] = array();
                for ($i = 0; $i < $header; $i++) {
                    $tableContent['header_data'][] = $tableContent['datas'][$i];
                }

                //update header option by merger cells
                $updateMergeCells = $modelTable->updateMergeCells($id, false, $tableContent);
                if (is_array($updateMergeCells)) {
                    $tableContent['params'] = $updateMergeCells;
                }
                if ($modelTable->saveTableSynfile($id, $tableContent)) {
                    $updated = true;
                } else {
                    $this->exitStatus(JText::_('COM_DROPTABLES_CTRL_EXCEL_SAVE_TABLE_ERROR'));
                }
            }
        } else {
            if (isset($file) && file_exists($file)) {
                unlink($file);
            }
            $this->exitStatus(JText::_('COM_DROPTABLES_CTRL_EXCEL_WRONG_FILE_EXTENSION'));
        }

        if (count($this->error_message_array) > 0) {
            $this->exitStatus($updated, $this->error_message_array);
        }
        $this->exitStatus($updated);
    }

    /**
     * Upload file to table
     *
     * @return boolean|string
     */
    private function uploadFile()
    {
        if (!empty($_FILES)) {
            $tempFile = $_FILES['file'];
            //check file extension
            //phpcs:ignore PHPCompatibility.Constants.NewConstants.ent_html401Found -- Not support php version 5.4 or earlier
            $tempFile['name'] = html_entity_decode($tempFile['name'], ENT_COMPAT | ENT_HTML401, 'UTF-8');
            if (!in_array(strtolower(JFile::getExt($tempFile['name'])), $this->allowed_ext)) {
                $this->error_message = JText::_('COM_DROPTABLES_CTRL_EXCEL_WRONG_FILE_EXTENSION');
                return false;
            }

            $targetPath = JPATH_SITE . '/tmp/uploads';
            $fileName = preg_replace('/([^a-zA-Z0-9-]+)/', '_', $_FILES['file']['name']);
            $targetFile = $targetPath . DIRECTORY_SEPARATOR . $fileName;

            if (JFile::upload($tempFile['tmp_name'], $targetFile)) {
                $file = $targetFile;
                return $file;
            } else {
                $this->error_message = JText::_('COM_DROPTABLES_CTRL_EXCEL_UPLOAD_FILE_ERROR');
                return false;
            }
        } else {
            $this->error_message = JText::_('COM_DROPTABLES_CTRL_EXCEL_UPLOAD_FILE_EMPTY');
            return false;
        }
    }

    /**
     * Export file xml
     *
     * @param string $adminOrFront Check front end or admin
     *
     * @return void
     */
    public function export($adminOrFront = '')
    {
        $file = $this->makeFile($adminOrFront);
        if ($file !== false && is_readable($file)) {
            header('Content-Description: File Transfer');
            header('Content-Type: application/octet-stream');
            header('Content-Disposition: attachment; filename=' . basename($file));
            header('Expires: 0');
            header('Cache-Control: must-revalidate');
            header('Pragma: public');
            header('Content-Length: ' . filesize($file));
            readfile($file);

            if (JFile::exists($file)) {
                unlink($file);
            }
        } else {
            echo $this->error_message;
        }

        $app = JFactory::getApplication();
        $app->close();
    }

    /**
     * Function create file xml
     *
     * @param string $adminOrFront Check front end or admin
     *
     * @since 3.5.0
     *
     * @return boolean|string
     */
    private function makeFile($adminOrFront = '')
    {
        $app = JFactory::getApplication();
        $stdFormat = array('xlsx' => 'Excel2007', 'xls' => 'Excel5');
        $format_excel = $app->input->getCmd('format_excel', 'xlsx');
        $id_table = $app->input->getInt('id');
        $onlydata = $app->input->getInt('onlydata');
        $front = $app->input->getInt('front');

        require_once(JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR . 'com_droptables' . DIRECTORY_SEPARATOR . 'models' . DIRECTORY_SEPARATOR . 'table.php');
        $modelTable = new DroptablesModelTable();
        $tableContent = (array)$modelTable->getItem($id_table, true, true, false, false, false);

        $params = $tableContent['params'];
        $style = $tableContent['style'];
        $datas = $tableContent['data'];

        if ($front === null && !(isset($params) && $params->download_button)) {
            $this->exitStatus(false, JText::_('COM_DROPTABLES_CTRL_EXCEL_WRONG_NOT_PERMISSION'));
        }

        if (!is_array($datas)) {
            $this->error_message = JText::_('COM_DROPTABLES_CTRL_EXCEL_EXPORT_TABLE_NOT_FOUND');
            return false;
        } else {
            require_once(JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR . 'com_droptables' . DIRECTORY_SEPARATOR . 'classes' . DIRECTORY_SEPARATOR . 'phpspreadsheet' . DIRECTORY_SEPARATOR . 'vendor' . DIRECTORY_SEPARATOR . 'autoload.php');

            $spreadsheet = new \PhpOffice\PhpSpreadsheet\Spreadsheet();
            $activeSheet = $spreadsheet->getActiveSheet();

            $activeSheet->fromArray($datas);
            $this->maxCell = $activeSheet->getHighestRowAndColumn();
            $columnIndex = \PhpOffice\PhpSpreadsheet\Cell\Coordinate::columnIndexFromString($this->maxCell['column']);

            $datas = $this->renderValueCalculateCell($datas, $this->maxCell);
            $activeSheet->fromArray($datas);

            if ($format_excel === 'xlsx') {
                $activeSheet->rangeToArray('A1:' . $this->maxCell['column'] . $this->maxCell['row'], '', false);
            } else {
                $data = $activeSheet->rangeToArray('A1:' . $this->maxCell['column'] . $this->maxCell['row'], '', true);
                $activeSheet->fromArray($data);
            }

            if ($onlydata && $onlydata === 1) {
                $style_row = array();
                $style_column = array();
                /*set style table*/
                if (isset($style->table)) {
                    $tableStyles = (array)$style->table;
                    if (!empty($tableStyles)) {
                        $activeSheet = $this->setStyleTable($activeSheet, $tableStyles);
                    }
                } else {
                    $tableStyles = array();
                }

                /*set width columns*/
                if (isset($style->cols) && !empty($style->cols)) {
                    $cI = 0;
                    foreach ($style->cols as $col) {
                        if (is_array($col) && isset($col[1]) && isset($col[1]->width)) {
                            $activeSheet->getColumnDimensionByColumn($cI + 1)->setWidth(floor($col[1]->width / 10)); //Excel unit: number of characters that can be displayed with the standard font
                            $style_column[$col[0]] = (array)$col[1];
                            $activeSheet = $this->setCellStyle1($activeSheet, $cI + 1, array(1, $this->maxCell['row']), (array)$col[1], $tableStyles);
                        } elseif (!is_array($col) && isset($col->{1}) && isset($col->{1}->width)) {
                            $activeSheet->getColumnDimensionByColumn($cI + 1)->setWidth(floor($col->{1}->width / 10)); //Excel unit: number of characters that can be displayed with the standard font
                            $style_column[$col->{0}] = (array)$col->{1};
                            $activeSheet = $this->setCellStyle1($activeSheet, $cI + 1, array(1, $this->maxCell['row']), (array)$col->{1}, $tableStyles);
                        }
                        $cI++;
                    }
                }

                /*set height rows*/
                if ($tableContent['type'] !== 'mysql' && isset($style->rows) && !empty($style->rows)) {
                    $rI = 0;
                    foreach ($style->rows as $row) {
                        $rI++;
                        if (is_array($row) && isset($row[1]) && isset($row[1]->height)) {
                            $activeSheet->getRowDimension($rI)->setRowHeight(floor($row[1]->height / 1.333333)); //px 2 pt
                            $activeSheet = $this->setCellStyle1($activeSheet, array(1, $columnIndex), $rI, (array)$row[1], $tableStyles);
                            $style_row[$row[0]] = (array)$row[1];
                        } elseif (!is_array($row) && isset($row->{1}) && isset($row->{1}->height)) {
                            $activeSheet->getRowDimension($rI)->setRowHeight(floor($row->{1}->height / 1.333333)); //px 2 pt
                            $activeSheet = $this->setCellStyle1($activeSheet, array(1, $columnIndex), $rI, (array)$row->{1}, $tableStyles);
                            $style_row[$row->{0}] = (array)$row->{1};
                        }
                    }
                } elseif ($tableContent['type'] === 'mysql') {
                    if (!empty($style->table->allRowHeight) && $style->table->allRowHeight > 0) {
                        $rowHeight = $style->table->allRowHeight / 1.333333;
                    }

                    for ($rI = 0; $rI < $this->maxCell['row']; $rI++) {
                        if (isset($style->rows) && isset($style->rows->{$rI})) {
                            $row = $style->rows->{$rI};
                            if (!empty($row[1]) && !empty($row[1]->height)) {
                                $activeSheet->getRowDimension($rI + 1)->setRowHeight(floor($row[1]->height / 1.333333)); //px 2 pt
                                $style_row[$row[0]] = (array)$row[1];
                                $activeSheet = $this->setCellStyle1($activeSheet, array(1, $columnIndex), $rI + 1, (array)$row[1], $tableStyles);
                            } elseif (!empty($row->{1}) && !empty($row->{1}->height)) {
                                $activeSheet->getRowDimension($rI + 1)->setRowHeight(floor($row->{1}->height / 1.333333)); //px 2 pt
                                $style_row[$row->{0}] = (array)$row->{1};
                                $activeSheet = $this->setCellStyle1($activeSheet, array(1, $columnIndex), $rI + 1, (array)$row->{1}, $tableStyles);
                            } else {
                                $activeSheet->getRowDimension($rI + 1)->setRowHeight(floor($rowHeight)); //px 2 pt
                                $activeSheet = $this->setCellStyle1($activeSheet, array(1, $columnIndex), $rI + 1, array(), $tableStyles);
                            }
                        } else {
                            $activeSheet = $this->setCellStyle1($activeSheet, array(1, $columnIndex), $rI + 1, array(), $tableStyles);
                        }
                    }
                }

                /*set style cells*/
                if (isset($style->cells)) {
                    $cellStyles = $style->cells;
                    if (!empty($cellStyles)) {
                        foreach ($cellStyles as $key => $cellCss) {
                            $check_valid_cells = explode('!', $key);
                            if (isset($check_valid_cells[0]) && (int)$check_valid_cells[0] >= 0 && isset($cellCss[2])) {
                                $cellCss[2] = array_merge(
                                    isset($style_row[$check_valid_cells[0]]) ? $style_row[$check_valid_cells[0]] : array(),
                                    isset($style_column[$check_valid_cells[1]]) ? $style_column[$check_valid_cells[1]] : array(),
                                    $cellCss[2]
                                );
                                $activeSheet = $this->setCellStyle1($activeSheet, $check_valid_cells[1] + 1, $check_valid_cells[0] + 1, $cellCss[2], $tableStyles);
                            }
                        }
                    }
                }

                if (is_array($params) && isset($params['mergeSetting'])) {
                    $mergeSettings = $params['mergeSetting'];
                } elseif (is_object($params) && isset($params->mergeSetting)) {
                    $mergeSettings = $params->mergeSetting;
                } else {
                    $mergeSettings = array();
                }

                if (is_string($mergeSettings)) {
                    $mergeSettings = json_decode($mergeSettings, true);
                }

                if (!empty($mergeSettings)) {
                    foreach ($mergeSettings as $mergeSetting) {
                        if (isset($mergeSetting['col'])) {
                            $activeSheet->mergeCellsByColumnAndRow($mergeSetting['col'] + 1, $mergeSetting['row'] + 1, $mergeSetting['col'] + $mergeSetting['colspan'], $mergeSetting['row'] + $mergeSetting['rowspan']);
                        } else {
                            $activeSheet->mergeCellsByColumnAndRow($mergeSetting[2] + 1, $mergeSetting[0] + 1, $mergeSetting[2] + $mergeSetting[3], $mergeSetting[0] + $mergeSetting[1]);
                        }
                    }
                }
            }

            if ($format_excel === 'xlsx') {
                $objWriter = new \PhpOffice\PhpSpreadsheet\Writer\Xlsx($spreadsheet);
            } elseif ($format_excel === 'xls') {
                $objWriter = new \PhpOffice\PhpSpreadsheet\Writer\Xls($spreadsheet);
            }

            jimport('joomla.filesystem.folder');
            $upload_dir = JPATH_SITE . '/tmp/uploads';
            if (!JFolder::exists($upload_dir)) {
                JFolder::create($upload_dir);
            }

            $tableContent['title'] = strtolower(preg_replace('/[^a-z0-9_]+/i', '_', $tableContent['title']));
            $file = $upload_dir . '/' . $tableContent['title'] . '.' . $format_excel;

            try {
                $objWriter->save($file);
            } catch (Exception $e) {
                $this->error_message = JText::_('COM_DROPTABLES_CTRL_EXCEL_EXPORT_CREATE_FILE_ERROR');
            }

            if (!JFile::exists($file) || !is_readable($file)) {
                $this->error_message = JText::_('COM_DROPTABLES_CTRL_EXCEL_EXPORT_CREATE_FILE_ERROR');
                return false;
            }
        }

        return $file;
    }

    /**
     * Function get Border Style cell
     *
     * @param integer $bWidth Width
     * @param string  $bStyle Style
     *
     * @return string
     */
    public function getBorderStyle($bWidth, $bStyle)
    {
        $borderStyle = PhpOffice\PhpSpreadsheet\Style\Border::BORDER_NONE;
        $bStyle = trim($bStyle);
        $bWidth = (int)$bWidth;
        if ($bWidth > 1) {
            switch ($bStyle) {
                case 'solid':
                    $borderStyle = PhpOffice\PhpSpreadsheet\Style\Border::BORDER_MEDIUM;
                    break;
                case 'dashed':
                    $borderStyle = PhpOffice\PhpSpreadsheet\Style\Border::BORDER_MEDIUMDASHED;
                    break;
                case 'dotted':
                    $borderStyle = PhpOffice\PhpSpreadsheet\Style\Border::BORDER_MEDIUMDASHDOT;
                    break;
            }
        } elseif ((int)$bWidth === 1) {
            switch ($bStyle) {
                case 'solid':
                    $borderStyle = PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN;
                    break;
                case 'dashed':
                    $borderStyle = PhpOffice\PhpSpreadsheet\Style\Border::BORDER_DASHED;
                    break;
                case 'dotted':
                    $borderStyle = PhpOffice\PhpSpreadsheet\Style\Border::BORDER_DOTTED;
                    break;
            }
        } else {
            $borderStyle = PhpOffice\PhpSpreadsheet\Style\Border::BORDER_NONE;
        }

        return $borderStyle;
    }

    /**
     * Check syn on front-end
     *
     * @var boolean
     */
    private $synFromFrontEnd = false;

    /**
     * Sync spreadsheet
     *
     * @since 3.4.0
     *
     * @return boolean|void
     */
    public function syncSpreadsheet()
    {
        $db = JFactory::getDbo();

        $query = 'SELECT t.id, t.params FROM #__droptables_tables as t WHERE t.type = \'html\' AND t.params LIKE \'%auto_sync":"1"%\'';
        $db->setQuery($query);
        $tables = $db->loadObjectList();

        if (empty($tables)) {
            return false;
        }

        $count = 0;

        $this->synFromFrontEnd = true;

        foreach ($tables as $table) {
            if (is_string($table->params)) {
                $params = json_decode($table->params, true);
            } else {
                $params = $table->params;
            }

            if (isset($params['spreadsheet_url']) && $params['spreadsheet_url'] !== '' && $params['auto_sync'] === '1') {//sync from gg
                if ($this->updateTableFromSpreadsheet($table->id, $params['spreadsheet_url'], $params['spreadsheet_style'], 'spreadsheet', 'require')) {
                    $count++;
                }
            }
            if (isset($params['excel_url']) && $params['excel_url'] !== '' && $params['excel_auto_sync'] === '1') {//sync from gg
                if ($this->updateTableFromSpreadsheet($table->id, $params['excel_url'], $params['excel_spreadsheet_style'], 'excel', 'require')) {
                    $count++;
                }
            }
        }

        $last_sync = (int)strtotime(date('Y-m-d H:i:s'));
        $DroptablesComponentHelper = new DroptablesComponentHelper();
        if (!$DroptablesComponentHelper->setParams(array('last_sync'=>$last_sync))) {
            $this->exitStatus(true, $this->error_message_array);
        }
        $this->exitStatus(true);
    }

    /**
     * Fetch file excel from Spreadsheet
     *
     * @since 3.4.0
     *
     * @return void
     */
    public function fetchSpreadsheet()
    {
        $app = JFactory::getApplication();

        $id_table = $app->input->getInt('id');
        $spreadsheet_url = $app->input->getString('spreadsheet_url');

        $autoSync = (int)$app->input->getInt('sync', 0);
        $syncType = (int)$app->input->getInt('syncType', 0);
        $fetchStyle = (int)$app->input->getInt('style', 0);

        if ($id_table && $spreadsheet_url) {
            $update = $this->updateTableFromSpreadsheet($id_table, $spreadsheet_url, $fetchStyle, $syncType);
            if (!$update) {
                $this->exitStatus(JText::_('COM_DROPTABLES_CTRL_EXCEL_SAVE_TABLE_ERROR'));
            } else {
                if (count($this->error_message_array) > 0) {
                    $this->exitStatus(true, array('sync' => $autoSync, 'style' => $fetchStyle, 'spreadsheet_url' => $spreadsheet_url, 'error' => $this->error_message_array));
                }
                $this->exitStatus(true, array('sync' => $autoSync, 'style' => $fetchStyle, 'spreadsheet_url' => $spreadsheet_url));
            }
        }
        if (count($this->error_message_array) > 0) {
            $this->exitStatus(true, array('autoSync' => $autoSync, 'style' => $fetchStyle, 'spreadsheet_url' => $spreadsheet_url, 'error' => $this->error_message_array));
        }
        $this->exitStatus(true, array('sync' => $autoSync, 'style' => $fetchStyle, 'spreadsheet_url' => $spreadsheet_url));
    }


    /**
     * Convert utf8
     *
     * @param array $array Array
     *
     * @return mixed
     */
    public function utf8Converter($array)
    {
        ini_set('mbstring.substitute_character', 'none');
        array_walk_recursive($array, function (&$item, $key) {
            if (!mb_detect_encoding($item, 'utf-8', true)) {
                $item = mb_convert_encoding($item, 'UTF-8', 'UTF-8');
            }
        });

        return $array;
    }

    /**
     * Function get style by columns
     *
     * @param object  $sheet       Spreadsheet->getActiveSheet();
     * @param integer $maxColIndex Number column
     * @param integer $ri          Row id
     * @param array   $tblStyles   Table style
     *
     * @return array
     */
    public function getStyleColumns($sheet, $maxColIndex, $ri, $tblStyles)
    {
        for ($ci = 1; $ci <= $maxColIndex; $ci++) {
            $column = \PhpOffice\PhpSpreadsheet\Cell\Coordinate::stringFromColumnIndex($ci);
            $cellStyle = $sheet->getStyle($column . ($ri));

//                        $formatCode[$ci] = array();
//                        $formatCode[$ci][$ri] = $cellStyle->getNumberFormat()->getFormatCode();

            /*get style for cell*/
            $tblStyles['cell'][$ri . '|' . $ci] = array();
            array_push($tblStyles['cell'][$ri . '|' . $ci], $ri, $ri, $ci, $ci);
            $styleFromGetStyle = $this->createCSSStyleExel($cellStyle);

            array_push($tblStyles['cell'][$ri . '|' . $ci], $styleFromGetStyle);

            if ($ci + $ci > 2) {
                $start_row1 = $ri;
                $start_col1 = isset($tblStyles['cell'][$ri . '|' . ($ci - 1)]) ? $ci - 1 : -1;

                if ($start_col1 !== -1 && isset($tblStyles['cell'][$start_row1 . '|' . $start_col1])) {
                    if (!isset($tblStyles['cell'][$start_row1 . '|' . $start_col1][4])) {
                        $start_row1 = $tblStyles['cell'][$start_row1 . '|' . $start_col1][0];
                        $start_col1 = $tblStyles['cell'][$start_row1 . '|' . $start_col1][2];
                    }
                    if ($tblStyles['cell'][$start_row1 . '|' . $start_col1][4] === $tblStyles['cell'][$ri . '|' . $ci][4]) {
                        $tblStyles['cell'][$start_row1 . '|' . $start_col1][1] = $tblStyles['cell'][$start_row1 . '|' . $start_col1][1] < $tblStyles['cell'][$ri . '|' . $ci][1] ? $tblStyles['cell'][$ri . '|' . $ci][1] : $tblStyles['cell'][$start_row1 . '|' . $start_col1][1];
                        $tblStyles['cell'][$start_row1 . '|' . $start_col1][3] = $tblStyles['cell'][$start_row1 . '|' . $start_col1][3] < $tblStyles['cell'][$ri . '|' . $ci][3] ? $tblStyles['cell'][$ri . '|' . $ci][3] : $tblStyles['cell'][$start_row1 . '|' . $start_col1][3];
                        $tblStyles['cell'][$ri . '|' . $ci][0] = $tblStyles['cell'][$start_row1 . '|' . $start_col1][0];
                        $tblStyles['cell'][$ri . '|' . $ci][1] = $tblStyles['cell'][$start_row1 . '|' . $start_col1][1];
                        $tblStyles['cell'][$ri . '|' . $ci][2] = $tblStyles['cell'][$start_row1 . '|' . $start_col1][2];
                        $tblStyles['cell'][$ri . '|' . $ci][3] = $tblStyles['cell'][$start_row1 . '|' . $start_col1][3];
                        unset($tblStyles['cell'][$ri . '|' . $ci][4]);
                    }
                }
            }

            if (!isset($tblStyles['cols']->{$ci - 1}[1]['width'])) {
                $tblStyles['cols']->{$ci - 1}[0] = $ci - 1;
                $width = $sheet->getColumnDimension($column)->getWidth();
                $tblStyles['cols']->{$ci - 1}[1]['width'] = $width * 10;

                if ($width < 0) {//error when reading the column width
                    $width = \PhpOffice\PhpSpreadsheet\Shared\Xls::sizeCol($sheet, $column);
                    $tblStyles['cols']->{$ci - 1}[1]['width'] = $width;
                }

                if ($width < 0) {
                    $tblStyles['cols']->{$ci - 1}[1]['width'] = 100;
                }
            }
        }
        return $tblStyles;
    }

    /**
     * Function creator array list data merge cell
     *
     * @param array $mergeRanges Value return $sheetActive->getMergeCells
     *
     * @return array
     */
    public function getMergeCell($mergeRanges)
    {
        $mergeSettings = array();

        foreach ($mergeRanges as $mergeRange) {
            list($tlCell, $rbCell) = explode(':', $mergeRange);

            list($tl_cNb, $tl_rNb) = \PhpOffice\PhpSpreadsheet\Cell\Coordinate::coordinateFromString($tlCell);
            list($br_cNb, $br_rNb) = \PhpOffice\PhpSpreadsheet\Cell\Coordinate::coordinateFromString($rbCell);
            $tl_cNb = \PhpOffice\PhpSpreadsheet\Cell\Coordinate::columnIndexFromString($tl_cNb);
            $br_cNb = \PhpOffice\PhpSpreadsheet\Cell\Coordinate::columnIndexFromString($br_cNb);

            $mergeSetting = array();
            $mergeSetting[0] = $tl_rNb - 1;
            $mergeSetting[2] = $tl_cNb - 1;
            $mergeSetting[1] = $br_rNb - $tl_rNb + 1;
            $mergeSetting[3] = $br_cNb - $tl_cNb + 1;
            $mergeSettings[] = $mergeSetting;
        }

        return $mergeSettings;
    }

    /**
     * Update table by spreadsheet
     *
     * @param integer     $id_table        Id table
     * @param string      $spreadsheet_url Spreadsheet url
     * @param integer     $fetchStyle      Check fetch style
     * @param integer     $syncType        Check type fetch
     * @param string|null $check           Droptables model table
     *
     * @since 3.4.0
     *
     * @return boolean
     */
    public function updateTableFromSpreadsheet($id_table, $spreadsheet_url, $fetchStyle, $syncType, $check = null)
    {
        if ($check === null) {
            $modelTable = $this->getModel('table');
        } else {
            require_once(JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR . 'com_droptables' . DIRECTORY_SEPARATOR . 'models' . DIRECTORY_SEPARATOR . 'table.php');
            $modelTable = new DroptablesModelTable();
        }
        $doUpdate = false;
        $hyperlinks = array();
        $this->id_table = $id_table;
        if ($fetchStyle === 1) {//render new style
            $this->fetchStyle = true;
            $tableContent = (array)$modelTable->getItem($id_table, false, true);
        } else {
            $this->fetchStyle = false;
            $tableContent = (array)$modelTable->getItem($id_table, false, false);
        }

        $doUpdate = false;

        if (strpos($spreadsheet_url, 'docs.google.com/spreadsheet') !== false) {
            $syncType = 'spreadsheet';
            $url_arr = explode('/', $spreadsheet_url);
            array_pop($url_arr);

            if (preg_match('/sharing/i', substr($spreadsheet_url, -8, 8)) === 1) {//link sharing
                $csv_url = implode('/', $url_arr) . '/export?format=xlsx';
            } else {
                //convert to url export csv
                $csv_url = implode('/', $url_arr) . '/pub?hl=en_US&single=true&output=csv';

                $url_query = parse_url($spreadsheet_url, PHP_URL_QUERY);
                if (!empty($url_query)) {
                    parse_str($url_query, $url_query_params);
                    if (isset($url_query_params['gid'])) {
                        $csv_url .= '&gid=' . $url_query_params['gid'];
                    }
                }
            }

            /*download file .csv*/
            try {
                $files = $this->getCsvDataFromUrl1($csv_url, $spreadsheet_url);
            } catch (Exception $e) {
                $files = false;
            }
            if ($files !== false) {
                //read file xlsx
                if (isset($files['xlsx'])) {
                    $readFileExcel = $this->readFileExcel($files['xlsx'], true, false, $tableContent['params'], $this->fetchStyle);//return text, file, status
                    $file = $files['xlsx'];
                }

                if (isset($files['csv'])) {
                    if (!(isset($readFileExcel['status']) && $readFileExcel['status'])) {//not get style from csv
                        $fetchStyle = 0;
                        $readFileExcel = $this->readFileExcel($files['csv'], true, false, $tableContent['params'], false);
                    }
                    unlink($files['csv']);
                }

                if (isset($readFileExcel['status']) && $readFileExcel['status']) {//ok file
                    $doUpdate = true;
                } elseif (isset($readFileExcel['text'])) {
                    if (!$this->synFromFrontEnd) {
                        $this->exitStatus($readFileExcel['text']);
                    }
                    return false;
                } elseif (is_string($readFileExcel)) {
                    if (!$this->synFromFrontEnd) {
                        $this->exitStatus($readFileExcel);
                    }
                    return false;
                } else {
                    return false;
                }
            } else {
                if (!$this->synFromFrontEnd) {
                    $this->exitStatus($this->error_message !== '' ? $this->error_message : JText::_('COM_DROPTABLES_ERROR_FETCH_FILE'));
                }
                return false;
            }
        } else {
            $syncType = 'excel';
            //download file
            $file = $this->downloadFileExcel($spreadsheet_url);

            if ($file) {
                $readFileExcel = $this->readFileExcel($file, true, false, $tableContent['params'], $fetchStyle === 1);
                if (isset($readFileExcel['status']) && $readFileExcel['status']) {
                    $doUpdate = true;
                } elseif (isset($readFileExcel['text'])) {
                    if (!$this->synFromFrontEnd) {
                        $this->exitStatus($readFileExcel['text']);
                    }
                    return false;
                } elseif (is_string($readFileExcel)) {
                    if (!$this->synFromFrontEnd) {
                        $this->exitStatus($readFileExcel);
                    }
                    return false;
                } else {
                    return false;
                }
            } else {
                if (!$this->synFromFrontEnd) {
                    $this->exitStatus($this->error_message);
                }
                return false;
            }
        }

        if ($fetchStyle === 1) {
            $data = $modelTable->deleteOldStyle($id_table);
            $tableContent['style'] = new stdClass();
            $tableContent['style']->rows = new stdClass();
            $tableContent['style']->cols = new stdClass();
        }

        $updated = false;
        if (isset($readFileExcel)) {
            $tableContent['datas'] = $readFileExcel['data']['data'];
            $tableContent['params']->mergeSetting = $readFileExcel['data']['mergeSetting'];

            if ($syncType === 'spreadsheet') {
                $tableContent['params']->spreadsheet_style = $fetchStyle;
                $spreadsheet_url = str_replace('%20', ' ', $spreadsheet_url);
                $tableContent['params']->spreadsheet_url = $spreadsheet_url;
            } elseif ($syncType === 'onedrive') {
                $tableContent['params']->onedrive_style = $fetchStyle;
                $spreadsheet_url = str_replace('%20', ' ', $spreadsheet_url);
                $tableContent['params']->onedrive_url = $spreadsheet_url;
            } else {
                $tableContent['params']->excel_spreadsheet_style = $fetchStyle;
                $spreadsheet_url = str_replace('%20', ' ', $spreadsheet_url);
                $tableContent['params']->excel_url = $spreadsheet_url;
            }

            if ($fetchStyle === 1) {
                $tableContent['params']->cell_types = $readFileExcel['data']['typeCells'];
                $tableContent['style']->rows = $readFileExcel['data']['style']['rows'];
                $tableContent['style']->cols = $readFileExcel['data']['style']['cols'];
                $tableContent['style'] = json_encode($tableContent['style']);

                $tableContent['styleCells'] = $readFileExcel['data']['style']['cell'];
            }

            $tableContent['numberRow'] = count($tableContent['datas']);
            $tableContent['numberCol'] = count($tableContent['datas'][0]);

            $count = count($readFileExcel['data']['hyperlink']);
            if ($readFileExcel['data']['hyperlink'] !== false) {
                if ($count > 0) {
                    $tableContent['params']->cell_types = $this->changeHyperlinksTable1($tableContent['params']->cell_types, $readFileExcel['data']['hyperlink']);
                }
            }
            $tableContent['params']->hyperlink = $readFileExcel['data']['hyperlink'];

            $tableContent['action'] = 'insert';

            if ($doUpdate) {
                //add table header = 1
                if (isset($tableContent['style']->table->headerOption) && $tableContent['style']->table->headerOption > 0) {
                    $tableContent['params']->headerOption = $tableContent['style']->table->headerOption;
                } else {
                    $tableContent['params']->headerOption = 1;
                }
                $tableContent['header_data'] = array();
                for ($i = 0; $i < $tableContent['params']->headerOption; $i++) {
                    $tableContent['header_data'][] = $tableContent['datas'][$i];
                }

                //update header option by merger cells
                $updateMergeCells = $modelTable->updateMergeCells($id_table, false, $tableContent);
                if (is_array($updateMergeCells)) {
                    $tableContent['params'] = $updateMergeCells;
                }
                if ($modelTable->saveTableSynfile($id_table, $tableContent)) {
                    $updated = true;
                } else {
                    if (!$this->synFromFrontEnd) {
                        $this->exitStatus(JText::_('COM_DROPTABLES_CTRL_EXCEL_SAVE_TABLE_ERROR'));
                    }
                    return false;
                }
            }
        } else {
            if (isset($file) && file_exists($file)) {
                unlink($file);
            }
            if (!$this->synFromFrontEnd) {
                $this->exitStatus(JText::_('COM_DROPTABLES_CTRL_EXCEL_SAVE_TABLE_ERROR'));
            }
            return false;
        }

        return $updated;
    }

    /**
     * Delete old style table
     *
     * @param integer $id Id table
     *
     * @return boolean
     */
    public function deleteOldStyle($id)
    {
        $dbo = JFactory::getDbo();
        $query = 'DELETE FROM #__droptables_range_styles WHERE id_table = ' . $id;
        $dbo->setQuery($query);
        $result = $dbo->execute();

        $query = 'DELETE FROM #__droptables_table_options WHERE id_table = ' . $id;
        $dbo->setQuery($query);
        $result = $dbo->execute();

        if ($result) {
            $query = 'UPDATE #__droptables_tables SET params = "", style = "", modified_time = ' . $dbo->quote(date('Y-m-d H:i:s')) . ' WHERE id = ' . $id;
            $dbo->setQuery($query);
            $result = $dbo->execute();
        }
        if (!$result) {
            return false;
        }

        return true;
    }

    /**
     * Function download file
     *
     * @param string $url       Url
     * @param string $extension File extension
     *
     * @return boolean|string
     */
    public function downloadFileExcel($url, $extension = '')
    {
        //check file extension
        $ext = strtolower(pathinfo($url, PATHINFO_EXTENSION));
        $newName = uniqid() . '.' . $ext;
        if (strpos($url, 'docs.google.com/spreadsheet') !== false) {
            if ($extension === '') {
                $newName = uniqid() . '.csv';
            } else {
                $newName = uniqid();
            }
        } elseif (!in_array($ext, $this->allowed_ext)) {
            $this->error_message = JText::_('COM_DROPTABLES_CTRL_EXCEL_WRONG_FILE_EXTENSION');
            return false;
        }

        $targetPath = JFactory::getApplication()->get('tmp_path');
        if (!file_exists($targetPath)) {
            mkdir($targetPath, 0777, true);
            $data = '<html><body bgcolor="#FFFFFF"></body></html>';
            $file = fopen($targetPath . 'index.html', 'w');
            fwrite($file, $data);
            fclose($file);
        }

        $targetFile = $targetPath . DIRECTORY_SEPARATOR . $newName . $extension;
        $url = str_replace(' ', '%20', $url);
        $data = $this->getDataFromUrl($url);
        if ($data === null) {
            $this->error_message = JText::_('COM_DROPTABLES_CTRL_EXCEL_NOT_READER_FILE');
            return false;
        }
        $file = fopen($targetFile, 'w');
        fwrite($file, $data);
        fclose($file);

        return $targetFile;
    }

    /**
     * Function get Data file .csv From Url
     *
     * @param string $spreadsheet_url Url
     * @param string $original_url    Original url
     *
     * @return string|boolean|array
     */
    public function getCsvDataFromUrl1($spreadsheet_url, $original_url)
    {
        $listFileDownload = array();

        //download csv is first
        $csvFile = $this->downloadFileExcel($spreadsheet_url);
        if ($csvFile) {
            $handle = fopen($csvFile, 'r');
            if ($handle !== false) {
                $listFileDownload['csv'] = $csvFile;
            }
            fclose($handle);
        }

        if (preg_match('/sharing/i', substr($original_url, -8, 8)) === 1) {//link sharing
            $sharing = true;
            $xlsxFile = $this->downloadFileExcel($spreadsheet_url, '.xlsx');
            if ($xlsxFile) {
                $handle = fopen($xlsxFile, 'r');
                if ($handle !== false) {
                    $listFileDownload['xlsx'] = $xlsxFile;
                }
                fclose($handle);

                return $listFileDownload;
            }
        }

        if (!$csvFile && (!isset($xlsxFile) || !$xlsxFile)) {
            $this->error_message = JText::_('COM_DROPTABLES_CTRL_EXCEL_WRONG_FILE_EXTENSION');
            return false;
        }

        if (isset($listFileDownload['csv'])) {
            $spreadsheet_url = str_replace('output=csv', 'output=xlsx', $spreadsheet_url);
            $file = $this->downloadFileExcel($spreadsheet_url, '.xlsx');

            /*if cann't get file from gg (first time)*/
            if ($file === false) {
                $url_query = parse_url($original_url, PHP_URL_QUERY);
                $url_arr = explode('/', $original_url);
                array_pop($url_arr);
                $csv_url = implode('/', $url_arr);

                if (!empty($url_query)) {
                    parse_str($url_query, $url_query_params);
                    if (isset($url_query_params['gid'])) {
                        $csv_url .= '/export?format=xlsx&id=' . $url_query_params['gid'];
                    }
                }

                $file = $this->downloadFileExcel($csv_url);
            }

            if ($file !== false) {
                $listFileDownload['xlsx'] = $file;
            }

            return $listFileDownload;
        } else {
            $this->error_message = JText::_('COM_DROPTABLES_CTRL_EXCEL_WRONG_FILE_EXTENSION');
            return false;
        }
    }

    /**
     * Function replace ';' to ',' of cell value in calculate cell
     *
     * @param array $datas   Data cell
     * @param array $maxCell Max number row and col
     *
     * @return mixed
     */
    public function renderValueCalculateCell($datas, $maxCell)
    {
        for ($i = 0; $i < $maxCell['row']; $i++) {
            $datas[$i] = array_map(
                function ($data) {
                    $data = (string)$data;
                    $key = substr($data, 0, 1);
                    $pattern = '@^=(DATE|DAY|DAYS|DAYS360|AND|OR|XOR|SUM|COUNT|MIN|MAX|AVG|CONCAT|date|day|days|days360|and|or|xor|sum|count|min|max|avg|concat)\\((.*?)\\)$@';
                    if ($key === '=' && preg_match($pattern, $data, $matches)) {
                        $data = str_replace(';', ',', $data);
                    }

                    return $data;
                },
                $datas[$i]
            );
        }

        return $datas;
    }

    /**
     * Function set style for cell
     *
     * @param object        $activeSheet ActiveSheet
     * @param integer|array $col         Col
     * @param integer|array $row         Row
     * @param array         $css         Css
     * @param array         $tableStyles Data option table
     *
     * @return mixed
     */
    public function setCellStyle1($activeSheet, $col, $row, $css, $tableStyles)
    {
        $selection = array();
        if (is_array($row)) {
            $selection[1] = $row[0];//row start
            $selection[3] = $row[1];//row end
        } else {
            $selection[1] = $row;
            $selection[3] = $row;
        }
        if (is_array($col)) {
            $selection[0] = $col[0];//column start
            $selection[2] = $col[1];//column end
        } else {
            $selection[0] = $col;
            $selection[2] = $col;
        }
        $getStyle = $activeSheet->getStyleByColumnAndRow($selection[0], $selection[1], $selection[2], $selection[3]);

        //font
        if (isset($css['cell_font_bold']) && $css['cell_font_bold']) {
            $getStyle->getFont()->setBold(true);
        }
        if (isset($css['cell_font_underline']) && $css['cell_font_underline']) {
            $getStyle->getFont()->setUnderline(true);
        }
        if (isset($css['cell_font_italic']) && $css['cell_font_italic']) {
            $getStyle->getFont()->setItalic(true);
        }
        if (isset($css['cell_font_color']) && $css['cell_font_color']) {
            $getStyle->getFont()->getColor()->setRGB(str_replace('#', '', $css['cell_font_color']));
        }
        if (isset($css['cell_font_family']) && $css['cell_font_family']) {
            $getStyle->getFont()->setName($css['cell_font_family']);
        }
        if (isset($css['cell_font_size']) && $css['cell_font_size']) {
            $getStyle->getFont()->setSize($css['cell_font_size'] * 72 / 96); //points = pixels * 72 / 96
        }

        //Alignment
        if (isset($css['cell_vertical_align']) && $css['cell_vertical_align']) {
            if ($css['cell_vertical_align'] === 'middle') {
                $vertical = 'center';
            } else {
                $vertical = $css['cell_vertical_align'];
            }
            $getStyle->getAlignment()->setVertical($vertical);
        }
        if (isset($css['cell_text_align']) && $css['cell_text_align']) {
            $horizontal = $css['cell_text_align'];
            $getStyle->getAlignment()->setHorizontal($horizontal);
        }

        //Fill
        if (!isset($css['cell_background_color']) || isset($css['cell_background_color']) && $css['cell_background_color'] === '') {
            if (isset($tableStyles['allAlternate']) && isset($tableStyles['allAlternate']->old)) {
                if (is_array($col)) {
                    if ($selection[1] % 2 === (($tableStyles['allAlternate']->header !== '') ? 0 : 1)) {
                        $css['cell_background_color'] = $tableStyles['allAlternate']->even;
                    } else {
                        $css['cell_background_color'] = $tableStyles['allAlternate']->old;
                    }
                    if ($selection[1] === 1 && $tableStyles['allAlternate']->header !== '') {
                        $css['cell_background_color'] = $tableStyles['allAlternate']->header;
                    } elseif ($this->maxCell['row'] === $selection[1] && $tableStyles['allAlternate']->footer !== '') {
                        $css['cell_background_color'] = $tableStyles['allAlternate']->footer;
                    }
                }
            } else {
                if (isset($css['AlternateColor'])) {
                    $css['cell_background_color'] = $this->getAlternateColor($selection[1] - 1, $css, $tableStyles);
                }
            }
        }

        if (isset($css['cell_background_color'])) {
            $fill_color = str_replace('#', '', $css['cell_background_color']);
            if ($fill_color === null || $fill_color === '') {
                $fill_color = 'ffffff';
            }
            $getStyle->getFill()->setFillType(PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID);
            $getStyle->getFill()->getStartColor()->setRGB($fill_color);
        }

        //Border
        if (isset($css['cell_border_bottom']) && $css['cell_border_bottom']) {
            list($bWidth, $bStyle, $bColor) = array_merge(explode(' ', $css['cell_border_bottom']), array('', ''));
            if ($bColor !== '') {
                $borderStyle = $this->getBorderStyle($bWidth, $bStyle);
                $getStyle->getBorders()->getBottom()->setBorderStyle($borderStyle);
                $bColor = str_replace('#', '', trim($bColor));
                $getStyle->getBorders()->getBottom()->getColor()->setRGB($bColor);
            }
        }

        if (isset($css['cell_border_top']) && $css['cell_border_top']) {
            list($bWidth, $bStyle, $bColor) = array_merge(explode(' ', $css['cell_border_top']), array('', ''));
            if ($bColor !== '') {
                $borderStyle = $this->getBorderStyle($bWidth, $bStyle);
                $getStyle->getBorders()->getTop()->setBorderStyle($borderStyle);
                $bColor = str_replace('#', '', trim($bColor));
                $getStyle->getBorders()->getTop()->getColor()->setRGB($bColor);
            }
        }

        if (isset($css['cell_border_left']) && $css['cell_border_left']) {
            list($bWidth, $bStyle, $bColor) = array_merge(explode(' ', $css['cell_border_left']), array('', ''));
            if ($bColor !== '') {
                $borderStyle = $this->getBorderStyle($bWidth, $bStyle);
                $getStyle->getBorders()->getLeft()->setBorderStyle($borderStyle);
                $bColor = str_replace('#', '', trim($bColor));
                $getStyle->getBorders()->getLeft()->getColor()->setRGB($bColor);
            }
        }

        if (isset($css['cell_border_right']) && $css['cell_border_right']) {
            list($bWidth, $bStyle, $bColor) = array_merge(explode(' ', $css['cell_border_right']), array('', ''));
            if ($bColor !== '') {
                $borderStyle = $this->getBorderStyle($bWidth, $bStyle);
                $getStyle->getBorders()->getRight()->setBorderStyle($borderStyle);
                $bColor = str_replace('#', '', trim($bColor));
                $getStyle->getBorders()->getRight()->getColor()->setRGB($bColor);
            }
        }

        return $activeSheet;
    }

    /**
     * Function set background color cell by alternate color
     *
     * @param integer $row         Row number
     * @param array   $css         Data css cell
     * @param array   $tableStyles Table styles
     *
     * @return string
     */
    public function getAlternateColor($row, $css, $tableStyles)
    {
        $fill_color = '';
        if (!isset($tableStyles['alternateColorValue'])) {
            return '';
        } else {
            if (is_array($tableStyles['alternateColorValue'])) {
                $alternateColorValue = (array)$tableStyles['alternateColorValue'][$css['AlternateColor']];
            } else {
                $alternateColorValue = (array)$tableStyles['alternateColorValue']->{$css['AlternateColor']};
            }
            if (empty($alternateColorValue)) {
                return '';
            }
        }

        $numberRow = 0;
        if ($alternateColorValue['header'] === '') {
            $numberRow = -1;
        }
        switch ($row) {
            case $alternateColorValue['selection'][0]:
                if ($numberRow === -1) {
                    $fill_color .= $alternateColorValue['even'];
                } else {
                    $fill_color .= $alternateColorValue['header'];
                }
                break;
            case $alternateColorValue['selection'][2]:
                if ($alternateColorValue['footer'] === '') {
                    if (($row - (int)($alternateColorValue['selection'][0] + $numberRow)) % 2) {
                        $fill_color .= $alternateColorValue['even'];
                    } else {
                        $fill_color .= $alternateColorValue['old'];
                    }
                } else {
                    $fill_color .= $alternateColorValue['footer'];
                }
                break;
            default:
                if (($row - (int)($alternateColorValue['selection'][0] + $numberRow)) % 2) {
                    $fill_color .= $alternateColorValue['even'];
                } else {
                    $fill_color .= $alternateColorValue['old'];
                }
                break;
        }

        return $fill_color;
    }

    /**
     * Function set style of table(freeze_row, col, symbol...)
     *
     * @param object $activeSheet ActiveSheet
     * @param array  $tableStyle  Data style table
     *
     * @return mixed
     */
    public function setStyleTable($activeSheet, $tableStyle)
    {
        /*freeze col and arow*/
        if (isset($tableStyle['freeze_col']) && $tableStyle['freeze_col']) {
            $freeze_col = (int)$tableStyle['freeze_col'] + 1;
        } else {
            $freeze_col = 1;
        }

        if (isset($tableStyle['freeze_row']) && $tableStyle['freeze_row']) {
            $freeze_row = (int)$tableStyle['freeze_row'] + 1;
        } else {
            $freeze_row = 1;
        }

        if ($freeze_row * $freeze_col !== 1) {
            $activeSheet->freezePaneByColumnAndRow($freeze_col, $freeze_row);
        }

        return $activeSheet;
    }

    /**
     * Convert number to alphabet
     *
     * @param integer|string $number Number
     *
     * @return string
     */
    public function numberToAlphabet($number)
    {
        $number = intval($number);
        if ($number <= 0) {
            return '';
        }
        $alphabet = '';
        while ($number !== 0) {
            $p = ($number - 1) % 26;
            $number = intval(($number - $p) / 26);
            $alphabet = chr(65 + $p) . $alphabet;
        }
        return $alphabet;
    }

    /**
     * List error code when use PhpSpreadsheet
     *
     * @var array
     */
    private $errorCodes = array(
        '#NULL!' => 0,
        '#DIV/0!' => 1,
        '#VALUE!' => 2,
        '#REF!' => 3,
        '#NAME?' => 4,
        '#NUM!' => 5,
        '#N/A' => 6,
    );

    /**
     * Function read data file excel(import, fetch)
     *
     * @param string  $file             Url file excel
     * @param boolean $data             Check get data cell
     * @param boolean $ignoreCheck      Ignore check
     * @param object  $tableOptionStyle Table style data option
     * @param boolean $renderStyle      Check get style table
     *
     * @return boolean|array
     */
    public function readFileExcel($file, $data, $ignoreCheck, $tableOptionStyle, $renderStyle = false)
    {
        $tableContent = array();
        $tblStyles = array();
        $hyperlinks = array();
        $cell_type = array();

        require_once(JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR . 'com_droptables' . DIRECTORY_SEPARATOR . 'classes' . DIRECTORY_SEPARATOR . 'phpspreadsheet' . DIRECTORY_SEPARATOR . 'vendor' . DIRECTORY_SEPARATOR . 'autoload.php');

        if (!is_readable($file)) {
            return array(
                'text' => JText::_('COM_DROPTABLES_IMPORT_NOT_READABLE'),
                'file' => basename($file),
                'status' => false
            );
        }

        try {
            $spreadsheet = \PhpOffice\PhpSpreadsheet\IOFactory::load($file);
            \PhpOffice\PhpSpreadsheet\Calculation\Calculation::getInstance($spreadsheet)->clearCalculationCache();
            //Load $inputFileName to a Spreadsheet Object
            if (!$spreadsheet->getSheetCount()) {
                return false;
            }
            $sheet = $spreadsheet->getActiveSheet();
            $maxCell = $sheet->getHighestRowAndColumn();
            // If there are more than 100 rows we need to re-check number of rows actually have data to avoid a sheet with many empty rows
            if ($maxCell['row'] > 100) {
                $valueDatas = $sheet->rangeToArray('A1:' . $maxCell['column'] . $maxCell['row'], null, false, true, false);
                //we need to first know how many rows actually have data
                $row_count = $maxCell['row'];
                // read through the data and see how many rows actually have data
                //the idea is that for every row, the first or second cell should be mandatory...
                //if we find one that is not, we stop there...
                do {
                    $row_count--;
                } while ((!$valueDatas[$row_count][0] || $valueDatas[$row_count][0] === 'NULL') &&
                (!$valueDatas[$row_count][1] || $valueDatas[$row_count][1] === 'NULL')
                );

                $maxCell['row'] = $row_count + 1;
            }

            //get style cells and FormatCode cells
            $formatCode = array();
            $maxColIndex = \PhpOffice\PhpSpreadsheet\Cell\Coordinate::columnIndexFromString($maxCell['column']);

            /*convert Separator for cells*/
            if (isset($tableOptionStyle->decimal_symbol)) {
                \PhpOffice\PhpSpreadsheet\Shared\StringHelper::setDecimalSeparator($tableOptionStyle->decimal_symbol);
            } else {
                \PhpOffice\PhpSpreadsheet\Shared\StringHelper::setDecimalSeparator('.');
            }

            if (isset($tableOptionStyle->thousand_symbol)) {
                \PhpOffice\PhpSpreadsheet\Shared\StringHelper::setThousandsSeparator($tableOptionStyle->thousand_symbol);
            } else {
                \PhpOffice\PhpSpreadsheet\Shared\StringHelper::setThousandsSeparator(',');
            }

            if ($data) {
                \PhpOffice\PhpSpreadsheet\Calculation\Calculation::getInstance($spreadsheet)->disableCalculationCache();
                \PhpOffice\PhpSpreadsheet\Calculation\Calculation::getInstance()->setCalculationCacheEnabled(false);
                $datas = $sheet->rangeToArray('A1:' . $maxCell['column'] . $maxCell['row'], '', false, true, false);
                $datas = $this->utf8Converter($datas);
                \PhpOffice\PhpSpreadsheet\Calculation\Calculation::getInstance($spreadsheet)->clearCalculationCache();
                try {
                    $valueDatas = $sheet->rangeToArray('A1:' . $maxCell['column'] . $maxCell['row'], '', true, true, false);
                    $valueDatas = $this->utf8Converter($valueDatas);
                } catch (\Exception $e) {
                    $this->error_message_array[] = $e->getMessage();
                }

                if (isset($datas)) {
                    /*change type cell value to string and replace ',' to ';' in calculate*/
                    $pattern = '@^=(DATE|DAY|DAYS|DAYS360|AND|OR|XOR|SUM|MULTIPLY|DIVIDE|COUNT|MIN|MAX|AVG|CONCAT|date|day|days|days360|and|or|xor|sum|multiply|divide|count|min|max|avg|concat)\\((.*?)\\)$@';
                    for ($i = 0; $i < $maxCell['row']; $i++) {
                        $count = count($datas[$i]);
                        for ($c = 0; $c < $count; $c++) {
                            $data_cell = (string)$datas[$i][$c];
                            $key = substr($data_cell, 0, 1);
                            $cell_position = $this->numberToAlphabet($c + 1) . '' . ($i + 1);

                            if (isset($valueDatas)) {
                                $datas[$i][$c] = $valueDatas[$i][$c];
                                if ($key === '=' && preg_match($pattern, $data_cell, $matches)) {//has calculater cell render to formula
                                    $datas[$i][$c] = str_replace(',', ';', $data_cell);
                                } elseif (isset($valueDatas[$i][$c]) && $valueDatas[$i][$c] !== '#NUM!') {//has formula cell not render to formula
                                    if ($key === '=' && $valueDatas[$i][$c] === $datas[$i][$c]) {//not render value
                                        $this->error_message_array[] = '' . $cell_position;
                                    }
                                }
                            } else {//has formula cell be not read, $valueDatas is understand
                                try {
                                    \PhpOffice\PhpSpreadsheet\Calculation\Calculation::getInstance($spreadsheet)->clearCalculationCache();
                                    $datas[$i][$c] = $sheet->getCell($cell_position)->getCalculatedValue();
                                    if (isset($this->errorCodes[$datas[$i][$c]])) {
                                        $this->error_message_array[] = '' . $cell_position;
                                    }
                                    $datas[$i][$c] = $datas[$i][$c] === null ? '' : $datas[$i][$c];
                                } catch (\Exception $e) {
                                    $this->error_message_array[] = '' . $cell_position;
                                }
//                                $datas[$i][$c] = str_replace(',', ';', $data_cell);
                            }

                            if ($renderStyle && preg_match_all('@<(\w+)\b.*?>.*?</\1>@si', $data_cell, $matches2) > 0) {
                                $cell_type[] = array($i, $c, 1, 1, 'html');
                            }
                        }
                    }
//                    $datas = $this->changeValueCalculateCell($datas, $valueDatas, $maxCell);
                }
            }

            if (count($this->error_message_array) > 0) {
                $erros1 = '' . implode(', ', $this->error_message_array);
                $erros = 'Import is finished! Note that the import tool wasn\'t able to read some cell data and therefore have removed some values. Cells concerned:' . implode(', ', $this->error_message_array);
                JLog::add($erros1, JLog::WARNING, 'droptables');

                if (count($this->error_message_array) > 10) {
                    $erros = '' . implode(', ', array(
                            $this->error_message_array[0],
                            $this->error_message_array[1],
                            $this->error_message_array[2],
                            $this->error_message_array[3],
                            $this->error_message_array[4],
                            $this->error_message_array[5],
                            '...',
                            $this->error_message_array[count($this->error_message_array) - 2],
                            $this->error_message_array[count($this->error_message_array) - 1]
                        ));
                } else {
                    $erros = $erros1;
                }

                JFactory::getSession()->set(
                    'droptable_read_excel_message',
                    array(
                        'type' => false,
                        'text' => $erros
                    )
                );
            }

            if (empty($datas)) {
                switch (json_last_error()) {
                    case JSON_ERROR_NONE:
                        return array('text' => ' - No errors', 'status' => false);
                    case JSON_ERROR_DEPTH:
                        return array('text' => ' - Maximum stack depth exceeded', 'status' => false);
                    case JSON_ERROR_STATE_MISMATCH:
                        return array('text' => ' - Underflow or the modes mismatch', 'status' => false);
                    case JSON_ERROR_CTRL_CHAR:
                        return array('text' => ' - Unexpected control character found', 'status' => false);
                    case JSON_ERROR_SYNTAX:
                        return array('text' => ' - Syntax error, malformed JSON', 'status' => false);
                    // phpcs:ignore PHPCompatibility.Constants.NewConstants.json_error_utf8Found -- the use of JSON_ERROR_UTF8 has check
                    case JSON_ERROR_UTF8:
                        return array(
                            'text' => ' - Malformed UTF-8 characters, possibly incorrectly encoded',
                            'status' => false
                        );
                    default:
                        return array('text' => ' - Unknown error', 'status' => false);
                }
            }

            if ($renderStyle) {
                $tblStyles['cols'] = new stdClass();
                $tblStyles['rows'] = new stdClass();
                $tblStyles['cell'] = array();
                $maxCells = (int)$maxCell['row'] * $maxColIndex;

                for ($ri = 1; $ri <= $maxCell['row']; $ri++) {
                    if ($file !== '' && $maxCells > 100000 && $ri >= 1000 && $ri % 1000 === 0) {//large table :(
                        $spreadsheet->disconnectWorksheets();
                        unset($spreadsheet);
                        $spreadsheet = \PhpOffice\PhpSpreadsheet\IOFactory::load($file);
                        $sheet = $spreadsheet->getActiveSheet();
                    }

                    $tblStyles = $this->getStyleColumns($sheet, $maxColIndex, $ri, $tblStyles);

                    $height = $sheet->getRowDimension($ri)->getRowHeight();
                    $tblStyles['rows']->{$ri - 1}[0] = $ri - 1;
                    if ($height === -1) {
                        $height = 18;
                    }

                    $tblStyles['rows']->{$ri - 1}[1]['height'] = floor($height * 1.333333); ////1 point = 1.333333 px
                }
            }

            if (isset($tblStyles['cell'])) {
                $count = count($tblStyles['cell']);
                $new_cell_style = array();
                for ($i = 0; $i < $count; $i++) {
                    $r = (int)($i / $maxColIndex);
                    $c = $i % $maxColIndex;
                    $key = ($r + 1) . '|' . ($c + 1);
                    if (isset($tblStyles['cell'][$key][4])) {
                        if (isset($tblStyles['cell'][($tblStyles['cell'][$key][0] - 1) . '|' . $tblStyles['cell'][$key][2]])) {
                            $row_before = $tblStyles['cell'][($tblStyles['cell'][$key][0] - 1) . '|' . $tblStyles['cell'][$key][2]];
                            if ($row_before[3] === $tblStyles['cell'][$key][3]) {
                                if (!isset($row_before[4])) {
                                    $row_before = $tblStyles['cell'][$row_before[0] . '|' . $row_before[2]];
                                }

                                if (!empty($row_before[4]) && $tblStyles['cell'][$key][4] === $row_before[4]) {
                                    $new_cell_style[$row_before[0] . '|' . $row_before[2]][1] = $tblStyles['cell'][$key][1];
                                    $tblStyles['cell'][$row_before[0] . '|' . $row_before[2]][1] = $tblStyles['cell'][$key][1];
                                    $tblStyles['cell'][$key][0] = $row_before[0];
                                    $tblStyles['cell'][$key][1] = $row_before[2];
                                    $tblStyles['cell'][$key][2] = $row_before[2];
                                    $tblStyles['cell'][$key][3] = $row_before[3];
                                    unset($tblStyles['cell'][$key][4]);
                                }
                            }
                        }
                        if (isset($tblStyles['cell'][$key][4])) {
                            $new_cell_style[$key] = $tblStyles['cell'][$key];
                        }
                    }
                }
                $tblStyles['cell'] = $new_cell_style;
            }
        } catch (\PhpOffice\PhpSpreadsheet\Reader\Exception $e) {
            $this->error_message = $e->getMessage();
            return $e->getMessage();
        }

        //Read Merged Cells info
        $mergeSettings = array();
        $mergeRanges = $spreadsheet->getActiveSheet()->getMergeCells();
        if (count($mergeRanges)) {
            $mergeSettings = $this->getMergeCell($mergeRanges);
        }

        //disconnect worksheets
        $spreadsheet->disconnectWorksheets();
        unset($spreadsheet);

        $hyperlinks = $this->getHyperlinkFromgg($file, 0);

        $tableContent = array(
            'data' => $datas,
            'style' => $tblStyles,
            'mergeSetting' => $mergeSettings,
            'typeCells' => $cell_type,
            'hyperlink' => $hyperlinks
        );

//        var_dump($this->id_table);

        return array('data' => $tableContent, 'status' => true);
    }

    /**
     * Get file and data table from url
     *
     * @param string  $spreadsheet_url Url file
     * @param array   $tableContent    Data table
     * @param string  $original_url    Original url file
     * @param integer $fetchStyle      Check fetch style
     *
     * @since 3.4.0
     *
     * @return array|boolean
     */
    public function getCsvDataFromUrl($spreadsheet_url, $tableContent, $original_url, $fetchStyle)
    {
        if (isset($tableContent['style']) && is_string($tableContent['style'])) {
            $tableContent['style'] = json_decode($tableContent['style'], false);
        } else {
            $tableContent['style'] = new stdClass();
        }
        $csvFile = $this->downloadFile($spreadsheet_url);
        $spreadsheet_data = array();
        $handle = fopen($csvFile, 'r');

        if ($handle !== false) {
            while (($data = fgetcsv($handle, 1000, ',')) !== false) {
                $data = str_replace("\n", '<br>', $data);
                $data = str_replace('"', '&quot;', $data);
                $spreadsheet_data[] = $data;
            }
            fclose($handle);
            unlink($csvFile);

            $spreadsheet_url = str_replace('output=csv', 'output=xlsx', $spreadsheet_url);
            $file = $this->getFilexlsFromgg($spreadsheet_url);
            if ($file === false) {
                $url_query = parse_url($original_url, PHP_URL_QUERY);
                $url_arr = explode('/', $original_url);
                array_pop($url_arr);
                $csv_url = implode('/', $url_arr);
                if (!empty($url_query)) {
                    parse_str($url_query, $url_query_params);
                    if (isset($url_query_params['gid'])) {
                        $csv_url .= '/export?format=xlsx&id=' . $url_query_params['gid'];
                    }
                }
                $file = $this->getFilexlsFromgg($csv_url);
            }
            if ($file !== false) {
                //save spreadsheet_style and spreadsheet_url
                if (!isset($tableContent['style']->table)) {
                    $tableContent['style'] = new stdClass();
                    $tableContent['style']->table = new stdClass();
                    $tableContent['style']->rows = new stdClass();
                    $tableContent['style']->cols = new stdClass();
                    $tableContent['style']->cells = new stdClass();
                }
                $tableContent['style']->table->spreadsheet_style = $fetchStyle;
                $tableContent['style']->table->spreadsheet_url = $original_url;
                if ($fetchStyle === 1) {
                    $tableContentStyle = $this->addStyleTableImport($file, $tableContent['style']->table);
                } else {
                    $tableContentStyle = json_encode($tableContent['style']);
                }

                $hyperlink = $this->getHyperlinkFromgg($file, 1);
            } else {
                $tableContentStyle = json_encode($tableContent['style']);
                $hyperlink = array();
            }
        } else {
            return false;
        }
        $data = array(
            'value' => $spreadsheet_data,
            'hyperlink' => $hyperlink,
            'style' => $tableContentStyle
        );
        return $data;
    }

    /**
     * Check string is json
     *
     * @param string  $string      String to check
     * @param boolean $return_data Default data return
     *
     * @since 3.4.0
     *
     * @return boolean
     */
    public function isJson($string, $return_data = false)
    {
        $data = json_decode($string);
        return (json_last_error() === JSON_ERROR_NONE) ? ($return_data ? $data : true) : false;
    }

    /**
     * Function download file
     *
     * @param string $url Link to file
     *
     * @since 3.4.0
     *
     * @return boolean|string
     */
    public function downloadFile($url)
    {
        //check file extension
        $ext = strtolower(pathinfo($url, PATHINFO_EXTENSION));
        $newName = uniqid() . '.' . $ext;
        if (strpos($url, 'docs.google.com/spreadsheet') !== false) {
            $newName = uniqid() . '.csv';
        } elseif (!in_array($ext, $this->allowed_ext)) {
            $this->error_message = JText::_('COM_DROPTABLES_CTRL_EXCEL_WRONG_FILE_EXTENSION');
            return false;
        }

        $targetPath = JFactory::getApplication()->get('tmp_path');
        if (!file_exists($targetPath)) {
            mkdir($targetPath, 0777, true);
            $data = '<html><body bgcolor="#FFFFFF"></body></html>';
            $file = fopen($targetPath . 'index.html', 'w');
            fwrite($file, $data);
            fclose($file);
        }

        $targetFile = $targetPath . DIRECTORY_SEPARATOR . $newName;
        $url = str_replace(' ', '%20', $url);
        $data = $this->getDataFromUrl($url);
        $file = fopen($targetFile, 'w');
        fwrite($file, $data);
        fclose($file);

        return $targetFile;
    }

    /**
     * Get file from google sheet
     *
     * @param string $url_arr Link file
     *
     * @since 3.4.0
     *
     * @return boolean|string
     */
    public function getFilexlsFromgg($url_arr)
    {
        $datafile = $this->getDataFromUrl($url_arr);
        if ($datafile === null) {
            return false;
        }

        $targetPath = JFactory::getApplication()->get('tmp_path');
        if (file_exists($targetPath)) {
            $targetFile = $targetPath . DIRECTORY_SEPARATOR . 'getfilexlsfromgg.xlsx';
        } else {
            return false;
        }
        $file = fopen($targetFile, 'cw');
        fwrite($file, $datafile);
        fclose($file);
        return $targetFile;
    }

    /**
     * Add style for table
     *
     * @param string $file                   Link file
     * @param array  $tableContentStyleTable Content data table
     *
     * @since 3.4.0
     *
     * @return false|string
     */
    public function addStyleTableImport($file, $tableContentStyleTable)
    {
        JLoader::register('PHPExcel_IOFactory', JPATH_ADMINISTRATOR . '/components/com_droptables/classes/phpexcel/PHPExcel/IOFactory.php');

        $xr = PHPExcel_IOFactory::createReaderForFile($file);
        $xr->setReadDataOnly(false);
        $objPHPExcel = $xr->load($file);
        $sheet = $objPHPExcel->getActiveSheet();
        $maxCell = $sheet->getHighestRowAndColumn();
        // If there are more than 100 rows we need to re-check number of rows actually have data to avoid a sheet with many empty rows
        if ($maxCell['row'] > 100) {
            $sheetData = $sheet->toArray(null, true, true, true);
            //we need to first know how many rows actually have data
            $row_count = (int)$maxCell['row'];
            // read through the data and see how many rows actually have data
            //the idea is that for every row, the first or second cell should be mandatory...
            //if we find one that is not, we stop there...
            do {
                $row_count--;
            } while ((!$sheetData[$row_count]['A'] || $sheetData[$row_count]['A'] === 'NULL') &&
            (!$sheetData[$row_count]['B'] || $sheetData[$row_count]['B'] === 'NULL')
            );

            //get the actual number of rows with data, removing the column labels
            $maxCell['row'] = $row_count + 1;
        }

        $maxColIndex = PHPExcel_Cell::columnIndexFromString($maxCell['column']);
        $tblStyles['cols'] = array();

        for ($ci = 0; $ci < $maxColIndex; $ci++) {
            $width = $sheet->getColumnDimension(PHPExcel_Cell::stringFromColumnIndex($ci))->getWidth();
            $tblStyles['cols'][$ci][0] = $ci;
            if ($width === -1) {
                $width = 10;
            }
            $tblStyles['cols'][$ci][1]['width'] = $width * 10; //Excel unit: number of characters that can be displayed with the standard font
        }

        $tblStyles['rows'] = array();
        for ($ri = 0; $ri < $maxCell['row']; $ri++) {
            $height = $sheet->getRowDimension($ri + 1)->getRowHeight();
            $tblStyles['rows'][$ri][0] = $ri;
            if ($height === -1) {
                $height = 18;
            }
            $tblStyles['rows'][$ri][1]['height'] = floor($height * 1.333333); ////1 point = 1.333333 px
        }

        $tblStyles['cells'] = array();
        for ($ri = 0; $ri < $maxCell['row']; $ri++) {
            for ($ci = 0; $ci < $maxColIndex; $ci++) {
                $tblStyles['cells'][$ri . '!' . $ci] = array($ri, $ci);

                $cellStyle = $sheet->getStyle(PHPExcel_Cell::stringFromColumnIndex($ci) . ($ri + 1));
                $cellCss = $this->createCSSStyle($cellStyle);
                $tblStyles['cells'][$ri . '!' . $ci][2] = $cellCss;
            }
        }
        $tblStyles['table'] = $tableContentStyleTable;
        $tableContentStyle = json_encode($tblStyles);


        //disconnect worksheets
        $objPHPExcel->disconnectWorksheets();
        unset($objPHPExcel);

        return $tableContentStyle;
    }

    /**
     * Funciton get Hyperlink from file xls getSheet(0) will get a sheet first
     *
     * @param string  $targetFile Url file
     * @param integer $check      Check delete file xls
     *
     * @return array|boolean
     */
    public function getHyperlinkFromgg($targetFile, $check)
    {
        if (!is_readable($targetFile)) {
            return array(
                'text' => JText::_('COM_DROPTABLES_IMPORT_NOT_READABLE'),
                'file' => basename($targetFile),
                'status' => false
            );
        }

        $xr = new stdClass();
        $xr = \PhpOffice\PhpSpreadsheet\IOFactory::createReaderForFile($targetFile);
        $xr->setReadDataOnly(false);
        $objPHPExcel = $xr->load($targetFile);

        $worksheet = $objPHPExcel->getSheet(0);
        $sheet = $objPHPExcel->getActiveSheet();
        $maxCell = $sheet->getHighestRowAndColumn();
        $maxColIndex = \PhpOffice\PhpSpreadsheet\Cell\Coordinate::columnIndexFromString($maxCell['column']);

        $styletable = array();
        $pattern = '@^=(HYPERLINK)\\((.*?)\\)$@';
        $pattern2 = '#\bhttps?://[^,\s()<>]+(?:\([\w\d]+\)|([^,[:punct:]\s]|/))#';
        for ($ri = 1; $ri <= $maxCell['row']; $ri++) {
            for ($ci = 1; $ci <= $maxColIndex; $ci++) {
                $indexCell = $worksheet->getCell(\PhpOffice\PhpSpreadsheet\Cell\Coordinate::stringFromColumnIndex($ci) . $ri);
                $cell = $indexCell->getValue();
                if (preg_match($pattern, $cell, $matches)) {
                    $cells = explode(',', $matches[2]);
                    preg_match_all($pattern2, $cells[0], $val0);
                    $styletable[($ri - 1) . '!' . ($ci - 1)] = array();
                    $styletable[($ri - 1) . '!' . ($ci - 1)]['hyperlink'] = $val0[0][0];
                    $styletable[($ri - 1) . '!' . ($ci - 1)]['text'] = preg_replace('/"/', '', $cells[1]);
                }

                $getHyperlink = $indexCell->getHyperlink()->getUrl();
                if ($getHyperlink !== '' && !isset($styletable[($ri - 1) . '!' . ($ci - 1)])) {
                    $styletable[($ri - 1) . '!' . ($ci - 1)] = array();
                    $styletable[($ri - 1) . '!' . ($ci - 1)]['hyperlink'] = $getHyperlink;
                    $styletable[($ri - 1) . '!' . ($ci - 1)]['text'] = $cell;
                }
            }
        }

        //disconnect worksheets
        $objPHPExcel->disconnectWorksheets();
        unset($objPHPExcel);

        if ($check === 1) {
            unlink($targetFile);
        }

        return $styletable;
    }

    /**
     * Add hyperlinks to table
     *
     * @param array $tableContent Data of table
     * @param array $hyperlinks   Hyperlinks data of table
     *
     * @return mixed
     */
    public function changeHyperlinksTable1($tableContent, $hyperlinks)
    {
        foreach ($hyperlinks as $key => $hyperlink) {
            $keyArray = explode('!', $key);
            if (!in_array(array($keyArray[0], $keyArray[1], 1, 1, 'html'), $tableContent)) {
                $tableContent[] = array($keyArray[0], $keyArray[1], 1, 1, 'html');
            }
        }

        return $tableContent;
    }

    /**
     * Add hyper link for table
     *
     * @param array  $tableContent Data table
     * @param string $hyperlinks   Link to file google sheet
     *
     * @since 3.4.0
     *
     * @return mixed
     */
    public function changeHyperlinksTable($tableContent, $hyperlinks)
    {
        if (isset($tableContent['style']) && is_string($tableContent['style'])) {
            $tableContent['style'] = json_decode($tableContent['style'], false);
        }

        $count = count((array)$tableContent['style']);
        if ($count < 1) {
            $tableContent['style'] = new stdClass();
            $tableContent['style']->table = new stdClass();
            $tableContent['style']->rows = new stdClass();
            $tableContent['style']->cols = new stdClass();
            $tableContent['style']->cells = new stdClass();
        } else {
            if ($tableContent['style']->cells === null) {
                $tableContent['style']->cells = new stdClass();
            }
        }

        $cells = new stdClass();

        foreach ($hyperlinks as $key => $hyperlink) {
            $keyArray = explode('!', $key);
            $cells->{$key} = array(
                0 => (int)$keyArray[0],
                1 => (int)$keyArray[1]
            );
            $cells->{$key}[2] = new stdClass();
            $cells->{$key}[2]->cell_type = 'html';
            if (isset($tableContent['style']->cells->{$key}[2])) {
                $cells->{$key}[2] = (object)array_merge((array)$tableContent['style']->cells->{$key}[2], (array)$cells->{$key}[2]);
            }
        }
        $tableContent['style']->cells = (object)array_merge((array)$tableContent['style']->cells, (array)$cells);
        $tableContent['style'] = json_encode($tableContent['style']);
        return $tableContent;
    }

    /**
     * Get data from link google sheet
     *
     * @param string $url Link google sheet
     *
     * @since 3.4.0
     *
     * @return boolean|string
     */
    public function getDataFromUrl($url)
    {
        $ch = curl_init();
        $timeout = 5;
        $agent = 'Mozilla/5.0 (Windows NT 6.2; WOW64; rv:17.0) Gecko/20100101 Firefox/17.0';

        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, $timeout);
        curl_setopt($ch, CURLOPT_USERAGENT, $agent);
        curl_setopt($ch, CURLOPT_REFERER, JUri::root());
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        $data = curl_exec($ch);

        if (curl_error($ch)) {
            $error = curl_error($ch);
            curl_close($ch);

            throw new Exception($error);
        }
        $info = curl_getinfo($ch);

        curl_close($ch);
        $statusCode = (string)$info['http_code'];

        if ($statusCode[0] === '2') {
            return $data;
        } else {
            return null;
        }
    }

    /**
     * Get csv data
     *
     * @param string $csv Data csv of table
     *
     * @since 3.4.0
     *
     * @return array
     */
    public function csvToArray($csv)
    {
        $arr = array();
        $lines = explode("\n", $csv);
        foreach ($lines as $row) {
            $arr[] = str_getcsv($row, ',');
        }

        return $arr;
    }


    /**
     * Create CSS style
     *
     * @param PhpOffice\PhpSpreadsheet\Style\Style $pStyle PhpOffice\PhpSpreadsheet\Style\Style
     *
     * @return string
     */
    private function createCSSStyleExel(PhpOffice\PhpSpreadsheet\Style\Style $pStyle)
    {
        // Construct CSS
        $css = '{';
        $css .= $this->createCSSStyleAlignmentExcel($pStyle->getAlignment());
        $css .= $this->createCSSStyleBordersExcel($pStyle->getBorders());
        $css .= $this->createCSSStyleFontExcel($pStyle->getFont());
        $css .= $this->createCSSStyleFillExcel($pStyle->getFill());
        $css .= '}';

        // Return
        return $css;
    }

    /**
     * Create CSS style (PhpSpreadsheet style alignment)
     *
     * @param PhpOffice\PhpSpreadsheet\Style\Alignment $getAlignment PhpOffice\PhpSpreadsheet\Style\Alignment
     *
     * @return string
     */
    private function createCSSStyleAlignmentExcel(PhpOffice\PhpSpreadsheet\Style\Alignment $getAlignment)
    {
        // Construct CSS
        $css = '';
        $css = '"cell_vertical_align":"' . $this->mapVAlignExcel($getAlignment->getVertical()) . '",';

        // Create CSS
        $textAlign = $this->mapHAlignExcel($getAlignment->getHorizontal());
        if ($textAlign) {
            $css .= '"cell_text_align":"' . $textAlign . '",';
            if (in_array($textAlign, array('left', 'right'))) {
                $css .= '"cell_padding_' . $textAlign . '":"' . (string)((int)$getAlignment->getIndent() * 9) . 'px",';
            }
        }

        // Return
        return $css;
    }

    /**
     * Create CSS style (PhpSpreadsheet style font)
     *
     * @param PhpOffice\PhpSpreadsheet\Style\Font $pStyle PhpOffice\PhpSpreadsheet\Style\Font
     *
     * @return string
     */
    private function createCSSStyleFontExcel(PhpOffice\PhpSpreadsheet\Style\Font $pStyle)
    {
        // Construct CSS
        $css = '';

        // Create CSS
        if ($pStyle->getBold()) {
            $css .= '"cell_font_bold":true,';
        }
        if ((string)$pStyle->getUnderline() !== PhpOffice\PhpSpreadsheet\Style\Font::UNDERLINE_NONE && $pStyle->getStrikethrough()) {
            $css .= '"cell_font_underline":true,';
        } elseif ((string)$pStyle->getUnderline() !== PhpOffice\PhpSpreadsheet\Style\Font::UNDERLINE_NONE) {
            $css .= '"cell_font_underline":true,';
        } elseif ($pStyle->getStrikethrough()) {
            $css .= '"cell_font_underline":false,';
        }
        if ($pStyle->getItalic()) {
            $css .= '"cell_font_italic":true,';
        }

        $css .= '"cell_font_color":"#' . $pStyle->getColor()->getRGB() . '",';
        $css .= '"cell_font_family":"' . $pStyle->getName() . '",';
        $css .= '"cell_font_size":"' . floor($pStyle->getSize() * 96 / 72) . '",';

        // Return
        return $css;
    }

    /**
     * Create CSS style (PhpSpreadsheet style borders)
     *
     * @param PhpOffice\PhpSpreadsheet\Style\Borders $pStyle PhpOffice\PhpSpreadsheet\Style\Borders
     *
     * @return string
     */
    private function createCSSStyleBordersExcel(PhpOffice\PhpSpreadsheet\Style\Borders $pStyle)
    {
        // Construct CSS
        $css = '';

        // Create CSS
        $css .= '"cell_border_bottom":"' . $this->createCSSStyleBorderExcel($pStyle->getBottom()) . '",';
        $css .= '"cell_border_top":"' . $this->createCSSStyleBorderExcel($pStyle->getTop()) . '",';
        $css .= '"cell_border_left":"' . $this->createCSSStyleBorderExcel($pStyle->getLeft()) . '",';
        $css .= '"cell_border_right":"' . $this->createCSSStyleBorderExcel($pStyle->getRight()) . '",';

        // Return
        return $css;
    }

    /**
     * Create CSS style (PhpSpreadsheet style border)
     *
     * @param PhpOffice\PhpSpreadsheet\Style\Border $pStyle PhpOffice\PhpSpreadsheet\Style\Border
     *
     * @return string
     */
    private function createCSSStyleBorderExcel(PhpOffice\PhpSpreadsheet\Style\Border $pStyle)
    {
        // Create CSS
        $borderStyle = $this->mapBorderStyleExcel($pStyle->getBorderStyle());
        $css = $borderStyle . ' #' . $pStyle->getColor()->getRGB() . (($borderStyle === 'none') ? '' : ' !important');

        // Return
        return $css;
    }

    /**
     * Create CSS style (PhpSpreadsheet style fill)
     *
     * @param PhpOffice\PhpSpreadsheet\Style\Fill $pStyle PhpOffice\PhpSpreadsheet\Style\Fill
     *
     * @return string
     */
    private function createCSSStyleFillExcel(PhpOffice\PhpSpreadsheet\Style\Fill $pStyle)
    {
        // Construct HTML
        $css = '';

        // Create CSS
        $value = $pStyle->getFillType() === PhpOffice\PhpSpreadsheet\Style\Fill::FILL_NONE ?
            '' : '#' . $pStyle->getStartColor()->getRGB();
        $css .= '"cell_background_color":"' . $value . '"';

        // Return
        return $css;
    }

    /**
     * Map VAlign
     *
     * @param string $vAlign Vertical alignment
     *
     * @return string
     */
    private function mapVAlignExcel($vAlign)
    {
        switch ($vAlign) {
            case \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_BOTTOM:
                return 'bottom';
            case \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_TOP:
                return 'top';
            case \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER:
            case \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_JUSTIFY:
                return 'middle';
            default:
                return 'baseline';
        }
    }

    /**
     * Map HAlign
     *
     * @param string $hAlign Horizontal alignment
     *
     * @return string|false
     */
    private function mapHAlignExcel($hAlign)
    {
        switch ($hAlign) {
            case \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_GENERAL:
                return false;
            case \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_LEFT:
                return 'left';
            case \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_RIGHT:
                return 'right';
            case \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER:
            case \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER_CONTINUOUS:
                return 'center';
            case \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_JUSTIFY:
                return 'justify';
            default:
                return false;
        }
    }

    /**
     * Map border style
     *
     * @param integer $borderStyle Sheet index
     *
     * @return string
     */
    private function mapBorderStyleExcel($borderStyle)
    {
        switch ($borderStyle) {
            case PhpOffice\PhpSpreadsheet\Style\Border::BORDER_NONE:
                return 'none';
            case PhpOffice\PhpSpreadsheet\Style\Border::BORDER_DASHDOT:
                return '1px dashed';
            case PhpOffice\PhpSpreadsheet\Style\Border::BORDER_DASHDOTDOT:
                return '1px dotted';
            case PhpOffice\PhpSpreadsheet\Style\Border::BORDER_DASHED:
                return '1px dashed';
            case PhpOffice\PhpSpreadsheet\Style\Border::BORDER_DOTTED:
                return '1px dotted';
            case PhpOffice\PhpSpreadsheet\Style\Border::BORDER_DOUBLE:
                return '3px double';
            case PhpOffice\PhpSpreadsheet\Style\Border::BORDER_HAIR:
                return '1px solid';
            case PhpOffice\PhpSpreadsheet\Style\Border::BORDER_MEDIUM:
                return '2px solid';
            case PhpOffice\PhpSpreadsheet\Style\Border::BORDER_MEDIUMDASHDOT:
                return '2px dashed';
            case PhpOffice\PhpSpreadsheet\Style\Border::BORDER_MEDIUMDASHDOTDOT:
                return '2px dotted';
            case PhpOffice\PhpSpreadsheet\Style\Border::BORDER_MEDIUMDASHED:
                return '2px dashed';
            case PhpOffice\PhpSpreadsheet\Style\Border::BORDER_SLANTDASHDOT:
                return '2px dashed';
            case PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THICK:
                return '3px solid';
            case PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN:
                return '1px solid';
            default:
                return '1px solid'; // map others to thin
        }
    }

    /**
     * Return a json response
     *
     * @param boolean $status Status response
     * @param array   $datas  Array of datas to return with the json string
     *
     * @since 3.4.0
     *
     * @return void
     */
    private function exitStatus($status, $datas = array())
    {
        $response = array('response' => $status, 'datas' => $datas);
        echo json_encode($response);
        JFactory::getApplication()->close();
    }
}
