<?php
/**
 * Droptables
 *
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 *
 * @package   Droptables
 * @copyright Copyright (C) 2014 JoomUnited (http://www.joomunited.com). All rights reserved.
 * @copyright Copyright (C) 2014 Damien Barrère (http://www.crac-design.com). All rights reserved.
 * @license   GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 */

// no direct access
defined('_JEXEC') || die;

jimport('joomla.application.component.controllerform');

/**
 * Class DroptablesControllerTable
 */
class DroptablesControllerTable extends JControllerForm
{
    /**
     * Save table
     *
     * @param null|integer $key    Key table
     * @param null|string  $urlVar Url value
     *
     * @return void
     */
    public function save($key = null, $urlVar = null)
    {
        $id = JFactory::getApplication()->input->getInt('id', 0);
        $jform = JFactory::getApplication()->input->get('jform', array(), 'ARRAY');
        $check_role = $this->checkRoleTable((int) $id, 'save');
        if ($check_role === 0) {
            $this->exitStatus(JText::_('COM_DROPTABLES_TABLES_DONT_PERMISSION'));
        }
        $model = $this->getModel();
        $data = array($id, $jform);
        if ($model->save($data)) {
            $params = $model->getTableParams($id, '');

            if (is_string($params)) {
                $params = json_decode($params);
            }
            if (isset($params->params) && is_string($params->params)) {
                $params = json_decode($params->params);
            }

            //update cells type
            if (isset($params->cell_types)) {
                $this->exitStatus(true, array('id' => $id, 'type' => array('update_cells_type' => json_encode($params->cell_types))));
            }

            $this->exitStatus(true);
        } else {
            $this->exitStatus(JText::_('COM_DROPTABLES_GLOBAL_ERROR'));
        }
    }

    /**
     * Function allow save
     *
     * @param array  $data Data
     * @param string $key  Key
     *
     * @return boolean
     */
    public function allowSave($data, $key = 'id')
    {
        $recordId = isset($data[$key]) ? $data[$key] : '0';

        if ($recordId) {
            return JFactory::getUser()->authorise('core.edit', $this->option) || JFactory::getUser()->authorise('core.edit.own', $this->option);
        } else {
            return $this->allowAdd($data);
        }
    }

    /**
     * Add new table
     *
     * @return void
     */
    public function add()
    {
        $id_category = JFactory::getApplication()->input->getInt('id_category');
        $name = JFactory::getApplication()->input->getString('name');
        $model = $this->getModel();
        $newItem = $model->add($id_category, $name);

        if ($newItem) {
            $this->exitStatus(true, array('id' => $newItem[0], 'title' => $name, 'modified_time' => $newItem[1], 'author' => $newItem[2]));
        } else {
            $this->exitStatus(JText::_('COM_DROPTABLES_GLOBAL_ERROR'));
        }
    }

    /**
     * Copy table
     *
     * @return void
     */
    public function copy()
    {
        $id = JFactory::getApplication()->input->getInt('id');
        $model = $this->getModel();
        $newItem = $model->copy($id);
        if ($newItem) {
            $table = $model->getItem($newItem);
            $this->exitStatus(true, array('id' => $newItem, 'title' => $table->title));
        } else {
            $this->exitStatus(JText::_('COM_DROPTABLES_GLOBAL_ERROR'));
        }
    }

    /**
     * Delete table
     *
     * @return void
     */
    public function delete()
    {
        $ids = JFactory::getApplication()->input->getString('ids', '');
        $model = $this->getModel();
        $check_role = $this->checkRoleTable($ids, 'delete');
        if ($check_role === 0) {
            $this->exitStatus(JText::_('COM_DROPTABLES_TABLES_DONT_PERMISSION'));
        }
        if ($model->delete($ids)) {
            $this->exitStatus(true);
        } else {
            $this->exitStatus(JText::_('COM_DROPTABLES_GLOBAL_ERROR'));
        }
    }

    /**
     * Check role user for table
     *
     * @param string $id    Id of table|category
     * @param string $check Var check function get checkRoleTable
     *
     * @return integer
     */
    private function checkRoleTable($id, $check)
    {
        $canDo = DroptablesHelper::getActions();
        $droptables_edit_category = $canDo->get('core.edit');
        $droptables_edit_own_category = $canDo->get('core.edit.own');
        $droptables_delete_tables = $canDo->get('com_droptables.delete');
        $droptables_create_tables = $canDo->get('com_droptables.create');
        $droptables_edit_tables = $canDo->get('com_droptables.edit');
        $droptables_edit_own_tables = $canDo->get('com_droptables.edit.own');
        $droptables_access_database_table = $canDo->get('com_droptables.dbtable');

        $idUser = JFactory::getUser()->id;

        if ($check === 'delete' && !empty($droptables_delete_tables)) {
            return 1;
        } elseif ($check === 'new' && !empty($droptables_create_tables)) {
            return 1;
        } elseif ($check === 'save') {
            $dbo = JFactory::getDbo();
            $range_style = '#__droptables_tables';
            $query = 'SELECT t.author, t.type FROM ' . $range_style . ' as t WHERE t.id = ' . (int)$id;
            $dbo->setQuery($query);
            $author = $dbo->loadObjectList();

            if (empty($author)) {
                return 0;
            }

            $data = (int)$author[0]->author === (int)$idUser ? 1 : 0;
            $type = $author[0]->type;
            if ($type === 'html' && !empty($droptables_edit_tables)) {
                return 1;
            } elseif ($type === 'html' && !empty($droptables_edit_own_tables)) {
                return $data;
            } elseif ($type === 'mysql'
                && !empty($droptables_access_database_table)) {//$droptables_edit_database_table
                return 1;
            }
            return 0;
        } elseif ($check === 'createRowDbTable' && !empty($droptables_edit_own_tables)) {
            $dbo = JFactory::getDbo();
            $range_style = '#__droptables_tables';
            $query = 'SELECT t.author, t.type, t.params FROM ' . $range_style . ' as t WHERE t.id = ' . (int)$id;
            $dbo->setQuery($query);
            $author = $dbo->loadObjectList();

            if (empty($author)) {
                return 0;
            }

            $type = $author[0]->type;
            if ($type !== 'mysql') {
                return 0;
            }

            if (isset($author[0]->params) && is_string($author[0]->params)) {
                $params = json_decode($author[0]->params, true);
            }
            if (!empty($params['table_editing'])
                && !empty($droptables_access_database_table)) {//$wptm_edit_database_table
                return 1;
            }
            return 0;
        } else {
            return 0;
        }
    }

    /**
     * Set table author
     *
     * @return void
     */
    public function setUser()
    {
        $id = JFactory::getApplication()->input->getInt('id');
        $user = JFactory::getApplication()->input->getInt('user');
        $model = $this->getModel();
        if ($model->setUser($id, $user)) {
            $this->exitStatus(true);
        } else {
            $this->exitStatus(JText::_('COM_DROPTABLES_GLOBAL_ERROR'));
        }
    }

    /**
     * Change title table
     *
     * @return void
     */
    public function setTitle()
    {
        $id = JFactory::getApplication()->input->getInt('id', 0);

        $model = $this->getModel();
        $canDo = DroptablesHelper::getActions();
        if (!$canDo->get('core.edit')) {
            if ($canDo->get('core.edit.own')) {
                $table = $model->getItem($id);
                if ((int)$table->author !== (int)JFactory::getUser()->id) {
                    $this->exitStatus('not permitted');
                }
            } else {
                $this->exitStatus('not permitted');
            }
        }

        $title = JFactory::getApplication()->input->getString('title');

        if ($model->setTitle($id, $title)) {
            $this->exitStatus(true);
        } else {
            $this->exitStatus(JText::_('COM_DROPTABLES_GLOBAL_ERROR'));
        }
    }

    /**
     * Change table order
     *
     * @return void
     */
    public function reorder()
    {
        $ids = JFactory::getApplication()->input->getString('order', '');
        $model = $this->getModel();
        if ($model->setPosition($ids)) {
            $modelTables = $this->getModel('tables');
            $tables = $modelTables->getItems();
            $tables = DroptablesTablesHelper::categoryObject($tables);
            $this->exitStatus(true, $tables);
        } else {
            $this->exitStatus(JText::_('COM_DROPTABLES_GLOBAL_ERROR'));
        }
    }

    /**
     * Change parent category
     *
     * @return void
     */
    public function changeCategory()
    {
        $id_table = JFactory::getApplication()->input->getInt('id', 0);
        $id_cat = JFactory::getApplication()->input->getInt('category', 0);
        $model = $this->getModel();
        if ($model->changeCategory($id_table, $id_cat)) {
            $modelTables = $this->getModel('tables');
            $tables = $modelTables->getItems();
            $tables = DroptablesTablesHelper::categoryObject($tables);
            $this->exitStatus(true, $tables);
        } else {
            $this->exitStatus(JText::_('COM_DROPTABLES_GLOBAL_ERROR'));
        }
    }

    /**
     * Delete new row in db_table
     *
     * @return void
     */
    public function deleteRowDbTable()
    {
        $id       = JFactory::getApplication()->input->getInt('value', 'none');
        $field    = JFactory::getApplication()->input->getString('field', 'none');
        $table    = JFactory::getApplication()->input->getString('table', 'none');
        $id_table = JFactory::getApplication()->input->getInt('id_table', 'none');
        $check_role = $this->checkRoleTable($id_table, 'createRowDbTable');

        if ($check_role === 0) {
            $this->exitStatus(JText::_('You don\'t have permission!'));
        }
        $model = $this->getModel();
        if ($model->deleteRowDbTable($table, $field, $id)) {
            $this->exitStatus(true, array('id' => $id_table, 'type' => array('reload_table' => true)));
        } else {
            $this->exitStatus(JText::_('error while delete row'));
        }
    }

    /**
     * Create new row in db_table
     *
     * @return void
     */
    public function createRowDbTable()
    {
        $id_table   = JFactory::getApplication()->input->getInt('id', 'none');
        $datas      = JFactory::getApplication()->input->getString('data', 'none');
        $dbtable    = JFactory::getApplication()->input->getString('dbtable', 'none');
        $check_role = $this->checkRoleTable($id_table, 'createRowDbTable');
        if ($check_role === 0) {
            $this->exitStatus(JText::_('You don\'t have permission!'));
        }
        $model = $this->getModel();
        if ($model->createRowDbTable((int) $id_table, $dbtable, $datas)) {
            $this->exitStatus(true, array('id' => $id_table, 'type' => array('reload_table' => true)));
        } else {
            $this->exitStatus(JText::_('error while changing table'));
        }
    }

    /**
     * Function convert table to new data
     *
     * @return void
     */
    public function updateOldTable()
    {
        $id_table = JFactory::getApplication()->input->getInt('id', 0);
        $model = $this->getModel();
        $data = $model->updateOldTable($id_table);
        if ($data !== false) {
            $this->exitStatus(true, $data);
        } else {
            $this->exitStatus('Error while convert table!');
        }
    }

    /**
     * Function convert table to new data
     *
     * @return void
     */
    public function updateTable362()
    {
        $id_table = JFactory::getApplication()->input->getInt('id', 0);
        $model = $this->getModel();
        $data = $model->updateTable362($id_table);
        if ($data !== false) {
            $this->exitStatus(true, $data);
        } else {
            $this->exitStatus('Error while convert table!');
        }
    }

    /**
     * Return a json response
     *
     * @param boolean $status Status response
     * @param array   $datas  Array of datas to return with the json string
     *
     * @since 3.4.0
     *
     * @return void
     */
    private function exitStatus($status, $datas = array())
    {
        $response = array('response' => $status, 'datas' => $datas);
        echo json_encode($response);
        JFactory::getApplication()->close();
    }
}
