<?php
/**
 * Droptables
 *
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 *
 * @package   Droptables
 * @copyright Copyright (C) 2014 JoomUnited (http://www.joomunited.com). All rights reserved.
 * @copyright Copyright (C) 2014 Damien Barrère (http://www.crac-design.com). All rights reserved.
 * @license   GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 */


//-- No direct access
defined('_JEXEC') || die('=;)');


jimport('joomla.plugin.plugin');
jimport('joomla.application.categories');
jimport('joomla.filesystem.file');

/**
 * Class plgContentdroptables
 */
class plgContentdroptables extends JPlugin  //phpcs:ignore Squiz.Classes.ValidClassName.NotCamelCaps -- Class name not change
{
    /**
     * Check the first get function
     *
     * @var integer
     */
    public static $Once = 1;

    /**
     * Example before display content method
     * Method is called by the view and the results are imploded and displayed in a placeholder
     *
     * @param string  $context    The context for the content passed to the plugin.
     * @param object  $article    The content object.  Note $article->text is also available
     * @param object  $params     The content params
     * @param integer $limitstart The 'page' number
     *
     * @return boolean
     */
    public function onContentPrepare($context, $article, $params = array(), $limitstart = 0)
    {
        $this->context = $context;

        $article->text = preg_replace_callback('@<img[^>]*?data\-droptablestable="([0-9]+)".*?>@', array($this, 'replace'), $article->text);

        /*
         * Sync page code use cUrl
         *
         * */
        $componentParams = JComponentHelper::getParams('com_droptables');
        $sync_periodicity = (int)$componentParams->get('sync_periodicity');
        if (!empty($sync_periodicity)) {
            $curSyncInterval = $this->curSyncInterval($componentParams);

            if ($curSyncInterval >= $sync_periodicity && self::$Once === 1) {
                $doc = JFactory::getDocument();
                JHtml::_('jquery.framework');
                $script = "jQuery(document).ready(function(){
                        jQuery.ajax({
                            url: '" . JUri::root() . "index.php?option=com_droptables&task=frontexcel.syncSpreadsheet',
                            type: 'POST',
                            success: function (datas) {
                            },
                            error: function (jqxhr, textStatus, error) {
                            },
                        });
                    });";
                $doc->addScriptDeclaration($script);

                self::$Once = 0;
            }
        }
        return true;
    }

    /**
     * Function curSyncInterval
     *
     * @param object $params Component params
     *
     * @return integer
     */
    private function curSyncInterval($params)
    {
        //get last_log param
        if ($params->get('last_sync') !== null) {
            $last_log = $params->get('last_sync');
            $time_old = (int)$last_log;
        } else {
            $time_old = 0;
        }

        $time_new = (int)strtotime(date('Y-m-d H:i:s'));
        $timeInterval = $time_new - $time_old;
        $curtime = $timeInterval / 60;

        return $curtime;
    }

    /**
     * Get chart data
     *
     * @param string $cellRange Cells range
     * @param array  $datas     Data table(data cell, format and style)
     *
     * @return array [1] is value has currency..., [2] is raw
     */
    public function readDataChartBySpreadsheet($cellRange, $datas)
    {
        $data0 = array();
        $data1 = array();
        $tblStyles = $datas->style;
        if (!isset($tblStyles->table)) {
            $tblStyles->table = new stdClass();
        }

        $data_value = $datas->data;

        require_once(JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR . 'com_droptables' . DIRECTORY_SEPARATOR . 'classes' . DIRECTORY_SEPARATOR . 'phpspreadsheet' . DIRECTORY_SEPARATOR . 'vendor' . DIRECTORY_SEPARATOR . 'autoload.php');
        $spreadsheet = new \PhpOffice\PhpSpreadsheet\Spreadsheet();
        $activeSheet = $spreadsheet->createSheet(1);
        $maxRows = count($data_value);

        JLoader::register('DroptablesHelper', JPATH_ADMINISTRATOR . '/components/com_droptables/helpers/droptables.php');
        $droptableHelper = new DroptablesHelper();
        $activeSheet->fromArray($droptableHelper::renderValueCalculateCell($data_value, $maxRows), null, 'A1');

        $arr_cellRanges = json_decode($cellRange);
        $countCellRanges = count($arr_cellRanges);
        for ($i = 0; $i < $countCellRanges; $i++) {
            $row = $arr_cellRanges[$i];
            $countRow = count($row);
            $data0[$i] = array();
            $data1[$i] = array();
            for ($j = 0; $j < $countRow; $j++) {
                list($r, $c) = explode(':', $row[$j]);
                if (isset($tblStyles->cells[$r . '!' . $c])) {
                    $tblStyle = $tblStyles->cells[$r . '!' . $c][2];
                } else {
                    $tblStyle = array();
                }

                if (isset($tblStyle['decimal_count']) && $tblStyle['decimal_count'] !== false
                    || isset($tblStyle['decimal_symbol']) && $tblStyle['decimal_symbol'] !== false
                    || isset($tblStyle['currency_symbol']) && $tblStyle['currency_symbol'] !== false) {
                    $droptableHelper::$thousand_symbol_cell = (isset($tblStyle['thousand_symbol']) && $tblStyle['thousand_symbol'] !== false)
                        ? $tblStyle['thousand_symbol'] : ((isset($tblStyle['thousand_symbol_second']) && $tblStyle['thousand_symbol_second'] !== false) ? $tblStyle['thousand_symbol_second'] : $droptableHelper::$thousand_symbol);
                    $droptableHelper::$decimal_count_cell = (isset($tblStyle['decimal_count']) && $tblStyle['decimal_count'] !== false)
                        ? $tblStyle['decimal_count'] : ((isset($tblStyle['decimal_count_second']) && $tblStyle['decimal_count_second'] !== false) ? $tblStyle['decimal_count_second'] : $droptableHelper::$decimal_count);
                    $droptableHelper::$decimal_symbol_cell = (isset($tblStyle['decimal_symbol']) && $tblStyle['decimal_symbol'] !== false)
                        ? $tblStyle['decimal_symbol'] : ((isset($tblStyle['decimal_symbol_second']) && $tblStyle['decimal_symbol_second'] !== false) ? $tblStyle['decimal_symbol_second'] : $droptableHelper::$decimal_symbol);
                    $droptableHelper::$currency_symbol_cell = (isset($tblStyle['currency_symbol']) && $tblStyle['currency_symbol'] !== false)
                        ? $tblStyle['currency_symbol'] : ((isset($tblStyle['currency_symbol_second']) && $tblStyle['currency_symbol_second'] !== false) ? $tblStyle['currency_symbol_second'] : $droptableHelper::$currency_symbol);
                    $droptableHelper::$symbol_position_cell = (isset($tblStyle['symbol_position']) && $tblStyle['symbol_position'] !== false)
                        ? $tblStyle['symbol_position'] : ((isset($tblStyle['symbol_position_second']) && $tblStyle['symbol_position_second'] !== false) ? $tblStyle['symbol_position_second'] : $droptableHelper::$symbol_position);
                    $has_format_cell = true;
                } else {
                    $droptableHelper::$thousand_symbol_cell = null;
                    $droptableHelper::$decimal_count_cell = null;
                    $droptableHelper::$decimal_symbol_cell = null;
                    $droptableHelper::$currency_symbol_cell = null;
                    $droptableHelper::$symbol_position_cell = null;
                    $has_format_cell = false;
                }
                if (!isset($data_value[$r]) || !isset($data_value[$r][$c])) {
                    $data0[$i][$j] = 0;
                    $data1[$i][$j] = null;
                } else {
                    $cell_value = $data_value[$r][$c];
                    $position = array();
                    $position[] = $droptableHelper::getNameFromNumber($c);
                    $position[] = $r + 1;
                    if (preg_match('@^=(DATE|DAY|DAYS|DAYS360|AND|OR|XOR|SUM|MULTIPLY|DIVIDE|COUNT|MIN|MAX|AVG|CONCAT|date|day|days|days360|and|or|xor|sum|multiply|divide|count|min|max|avg|concat)\\((.*?)\\)$@', $cell_value, $matches)) {
                        $formula = strtoupper($matches[1]);
                        //check formula function to replace input value
                        if (in_array($formula, $droptableHelper::$math_formula)) {
                            $droptableHelper::$decimal_count_cell = (isset($tblStyle['decimal_count_second']) && $tblStyle['decimal_count_second'] !== false) ? $tblStyle['decimal_count_second'] : $droptableHelper::$decimal_count_cell;
                            $droptableHelper::$decimal_symbol_cell = (isset($tblStyle['decimal_symbol_second']) && $tblStyle['decimal_symbol_second'] !== false) ? $tblStyle['decimal_symbol_second'] : $droptableHelper::$decimal_symbol_cell;
                            $droptableHelper::$thousand_symbol_cell = (isset($tblStyle['thousand_symbol_second']) && $tblStyle['thousand_symbol_second'] !== false) ? $tblStyle['thousand_symbol_second'] : $droptableHelper::$thousand_symbol_cell;
                            $droptableHelper::$currency_symbol_cell = (isset($tblStyle['currency_symbol_second']) && $tblStyle['currency_symbol_second'] !== false) ? $tblStyle['currency_symbol_second'] : $droptableHelper::$currency_symbol_cell;
                            $droptableHelper::$symbol_position_cell = (isset($tblStyle['symbol_position_second']) && $tblStyle['symbol_position_second'] !== false) ? $tblStyle['symbol_position_second'] : $droptableHelper::$symbol_position_cell;
                        }
                        $calculaterCell2 = $droptableHelper::calculaterCell2($data_value, $matches, $activeSheet, $position, true);
                        if (!is_array($calculaterCell2)) {
                            $data1[$i][$j] = null;
                            $data0[$i][$j] = $calculaterCell2;
                        } else {
                            $data1[$i][$j] = $calculaterCell2[1];
                            $data0[$i][$j] = $calculaterCell2[0];
                        }
                    } elseif ($has_format_cell) {
                        $col1 = preg_replace('/[-|0-9|,|\.|' . $droptableHelper::$currency_symbol_cell . '| ]/', '', $cell_value);

                        $cell_value = preg_replace('/[' . $droptableHelper::$thousand_symbol_cell . '| ]/', '', $cell_value);
                        $cell_value = preg_replace('/[' . $droptableHelper::$decimal_symbol_cell . '| ]/', '.', $cell_value);
                        $data1[$i][$j] = preg_replace('/[' . $droptableHelper::$currency_symbol_cell . '| ]/', '', $cell_value);
                        if ($col1 === '') {
                            $cell_value = preg_replace('/[' . $droptableHelper::$currency_symbol_cell . '| ]/', '', $cell_value);
                            $cell_value = number_format(floatval($cell_value), $droptableHelper::$decimal_count_cell, $droptableHelper::$decimal_symbol_cell, $droptableHelper::$thousand_symbol_cell);
                        }

                        if (isset($tblStyle['currency_symbol']) && $tblStyle['currency_symbol'] !== '' && $col1 === '') {
                            $cell_value = ((int) $droptableHelper::$symbol_position_cell === 0) ? $droptableHelper::$currency_symbol_cell . ' ' . $cell_value : $cell_value . ' ' . $droptableHelper::$currency_symbol_cell;
                        }
                        $data0[$i][$j] = $cell_value;
                    } else {
                        $data0[$i][$j] = $cell_value;
                        $data1[$i][$j] = null;
                    }
                }
            }
        }
        return array($data0, $data1);
    }

    /**
     * Convert the table content to front end
     *
     * @param array $match Match string
     *
     * @return string|array|boolean
     */
    private function replace($match)
    {
        $lang = JFactory::getLanguage();
        $lang->load('com_droptables', JPATH_ADMINISTRATOR . '/components/com_droptables', null, true);
        $lang->load('com_droptables.override', JPATH_ADMINISTRATOR . '/components/com_droptables', null, true);
        $lang->load('com_droptables.sys', JPATH_ADMINISTRATOR . '/components/com_droptables', null, true);

        JModelLegacy::addIncludePath(JPATH_SITE . '/components/com_droptables/models/', 'DroptablesModel');
        JTable::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_droptables/tables/', 'DroptablesTablesTable');
        require_once('chartStyleSet.php');

        $version = '4.0.0';

        $exp = '@<img.*data\-droptables\-chart="([0-9]+)".*?>@';
        preg_match($exp, $match[0], $matches);
        require_once(JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR . 'com_droptables' . DIRECTORY_SEPARATOR . 'models' . DIRECTORY_SEPARATOR . 'table.php');
        $droptablesModelTableBackend = new DroptablesModelTable();
        if (count($matches) > 0) { //is a chart
            $content = '';
            JModelLegacy::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_droptables/models/', 'DroptablesModel');
            $model = JModelLegacy::getInstance('Chart', 'droptablesModel');

            $chart = $model->getChart($matches[1]);
            // get table config
            $modelTable = JModelLegacy::getInstance('Table', 'droptablesModel');
            if ($chart) {
                $chartConfig = json_decode($chart->config);
                if (isset($chartConfig->useFirstRowAsGraph)) {
                    $this->useFirstRowAsGraph = $chartConfig->useFirstRowAsGraph;
                }

                $table = $droptablesModelTableBackend->getItem($chart->id_table, true, true, null, false);
                $params = JComponentHelper::getParams('com_droptables');
                if (empty($table) || empty($table->data)) {
                    return '';
                }

                $table->data = is_string($table->data) ? json_decode($table->data) : $table->data;
                $table->style = is_string($table->style) ? json_decode($table->style) : $table->style;

                $chartData = $this->readDataChartBySpreadsheet($chart->datas, $table);

                $symbol_position = (!empty($params['symbol_position'])) ? $params['symbol_position'] : 0;
                $symbol_position = (!empty($table->style->table->symbol_position)) ? $table->style->table->symbol_position : $symbol_position;
                $currency_symbol = (!empty($params['currency_sym'])) ? $params['currency_sym'] : '$';
                $currency_symbol = (!empty($table->style->table->currency_symbol)) ? $table->style->table->currency_symbol : $currency_symbol;
                $decimal_symbol = (!empty($params['decimal_sym'])) ? $params['decimal_sym'] : '.';
                $decimal_symbol = (!empty($table->style->table->decimal_symbol)) ? $table->style->table->decimal_symbol : $decimal_symbol;
                $decimal_count = (!empty($params['decimal_count'])) ? $params['decimal_count'] : 0;
                $decimal_count = (!empty($table->style->table->decimal_count)) ? $table->style->table->decimal_count : $decimal_count;
                $thousand_symbol = (!empty($params['thousand_sym'])) ? $params['thousand_sym'] : ',';
                $thousand_symbol = (!empty($table->style->table->thousand_symbol)) ? $table->style->table->thousand_symbol : $thousand_symbol;

                $jsChartData = $this->buildJsChartData(
                    $chartData,
                    $chart->type,
                    $chartConfig,
                    $currency_symbol,
                    $decimal_symbol,
                    $thousand_symbol
                );

                if (!$chartConfig) {
                    $chartConfig = new stdClass();
                }

                $chartConfig->width = isset($chartConfig->width) ? $chartConfig->width : 500;
                $chartConfig->height = isset($chartConfig->height) ? $chartConfig->height : 375;
                $chartConfig->chart_align = isset($chartConfig->chart_align) ? $chartConfig->chart_align : 'center';
                $symbol = '';

                $js = 'var DropChart = {};' . "\n";
                $reactVar = new stdClass();
                $js .= 'DropChart.id = "' . $matches[1] . '" ; ' . "\n";
                $reactVar->id = $matches[1];
                $js .= 'DropChart.type = "' . $chart->type . '" ; ' . "\n";
                $reactVar->type = $chart->type;
                $js .= 'DropChart.data = ' . $jsChartData . '; ' . "\n";
                $reactVar->data = $jsChartData;
                $js .= 'DropChart.currency_symbols = "' . $symbol . '"; ' . "\n";
                $reactVar->currency_symbols = $symbol;
                $js .= 'DropChart.places = ' . $decimal_count . '; ' . "\n";
                $reactVar->places = $decimal_count;
                $js .= 'DropChart.unit_symbols = ' . $symbol_position . '; ' . "\n";
                $reactVar->unit_symbols = $symbol_position;
                $js .= 'DropChart.decimal_symbols = "' . $decimal_symbol . '"; ' . "\n";
                $reactVar->decimal_symbols = $decimal_symbol;
                $js .= 'DropChart.thousand_symbols = "' . $thousand_symbol . '"; ' . "\n";
                $reactVar->thousand_symbols = $thousand_symbol;

//                $tblParams = $table->params;
//
//                $chartConfig = json_decode($chart->config);
//                if (isset($chartConfig->useFirstRowAsGraph)) {
//                    $this->useFirstRowAsGraph = $chartConfig->useFirstRowAsGraph;
//                }
//
//                $chartData = $this->getChartData($chart->datas, $datas);
//
//                $currency_symbol = (empty($table_config->table->currency_symbol))
//                    ? $params->get('default_currency_symbol', '$')
//                    : $table_config->table->currency_symbol;
//                $unit_symbols = (empty($table_config->table->unit_symbols))
//                    ? $params->get('default_symbol_position', '0')
//                    : $table_config->table->unit_symbols;
//                $places = (empty($table_config->table->places))
//                    ? $params->get('default_decimal_place', '0')
//                    : $table_config->table->places;
//                $decimal_symbols = (empty($table_config->table->decimal_symbols))
//                    ? $params->get('default_decimal_symbol', '.')
//                    : $table_config->table->decimal_symbols;
//                $thousand_symbols = (empty($table_config->table->thousand_symbols))
//                    ? $params->get('default_thousand_symbol', ',')
//                    : $table_config->table->thousand_symbols;
//                $jsChartData = $this->buildJsChartData(
//                    $chartData,
//                    $chart->type,
//                    $chartConfig,
//                    $currency_symbol,
//                    $thousand_symbols,
//                    $decimal_symbols
//                );

//                if (!$chartConfig) {
//                    $chartConfig = new stdClass();
//                }
//                $chartConfig->width = isset($chartConfig->width) ? $chartConfig->width : 500;
//                $chartConfig->height = isset($chartConfig->height) ? $chartConfig->height : 375;
//                $chartConfig->chart_align = isset($chartConfig->chart_align) ? $chartConfig->chart_align : 'center';
//                $symbol = '';
//                $countChartData = count($chartData);
//                for ($i = 0; $i < $countChartData; $i++) {
//                    $countChartDataI = count($chartData[$i]);
//                    for ($j = 0; $j < $countChartDataI; $j++) {
//                        if (strpos($chartData[$i][$j], $currency_symbol) !== false) {
//                            $chartData[$i][$j] = str_replace($thousand_symbols, '', $chartData[$i][$j]);
//                            $chartData[$i][$j] = str_replace($decimal_symbols, '.', $chartData[$i][$j]);
//                            if (is_numeric(str_replace($currency_symbol, '', $chartData[$i][$j]))) {
//                                $symbol = $currency_symbol;
//                                break;
//                            }
//                        }
//                    }
//                }
//                $js = 'var DropChart = {};' . "\n";
//                $reactVar = new stdClass();
//                $js .= 'DropChart.id = "' . $chart->id . '" ; ' . "\n";
//                $reactVar->id = $chart->id;
//                $js .= 'DropChart.type = "' . $chart->type . '" ; ' . "\n";
//                $reactVar->type = $chart->type;
//                $js .= 'DropChart.data = ' . $jsChartData . '; ' . "\n";
//                $reactVar->data = $jsChartData;
//                $js .= 'DropChart.currency_symbol = "' . $symbol . '"; ' . "\n";
//                $reactVar->currency_symbol = $symbol;
//                $js .= 'DropChart.places = ' . $places . '; ' . "\n";
//                $reactVar->places = $places;
//                $js .= 'DropChart.unit_symbols = ' . $unit_symbols . '; ' . "\n";
//                $reactVar->unit_symbols = $unit_symbols;
//                $js .= 'DropChart.decimal_symbols = "' . $decimal_symbols . '"; ' . "\n";
//                $reactVar->decimal_symbols = $decimal_symbols;
//                $js .= 'DropChart.thousand_symbols = "' . $thousand_symbols . '"; ' . "\n";
//                $reactVar->thousand_symbols = $thousand_symbols;

                if (isset($chartConfig->useFirstRowAsGraph)) {
                    $reactVar->useFirstRowAsGraph = $chartConfig->useFirstRowAsGraph;
                    $js .= 'DropChart.useFirstRowAsGraph = "' . $reactVar->useFirstRowAsGraph . '"; ' . "\n";
                }

                if ($chart->config) {
                    $js .= 'DropChart.config = ' . $chart->config . '; ' . "\n";
                    $reactVar->config = $chart->config;
                } else {
                    $js .= 'DropChart.config = {} ; ' . "\n";
                    $reactVar->config = new stdClass();
                }
                $js .= ' if(typeof DropCharts === "undefined") { var DropCharts = []; } ; ' . "\n";

                $js .= ' DropCharts.push(DropChart) ; ' . "\n";

                JHtml::_('jquery.framework');
                $document = JFactory::getDocument();
                $document->addScript(JUri::base() . 'components/com_droptables/assets/js/chart.min.js');
                $document->addScriptDeclaration($js);
                $document->addScript(JUri::base() . 'components/com_droptables/assets/js/dropchart.js?v=' . $version);

                $content = '<div class="chartContainer droptables" id="chartContainer' . $chart->id . '" data-id-chart="' . $chart->id . '">';
                $align = '';
                switch ($chartConfig->chart_align) {
                    case 'left':
                        $align = ' margin : 0 auto 0 0; ';
                        break;
                    case 'right':
                        $align = ' margin : 0 0 0 auto ';
                        break;
                    case 'none':
                    case 'center':
                    default:
                        $align = ' margin : 0 auto 0 auto ';
                        break;
                }

                $content .= '<div class="canvasWraper" style="max-height:' . $chartConfig->height
                    . 'px; max-width:' . $chartConfig->width . 'px;' . $align . '" >';
                $content .= '<canvas class="canvas"  height="' . $chartConfig->height . '" width="' . $chartConfig->width . '"></canvas>';
                $content .= '</div></div>';
//                $content .= '<script>' . $js . '</script>';

                return $content;
            }
        } else {
            JHtml::_('jquery.framework');
            $document = JFactory::getDocument();
//            $model = JModelLegacy::getInstance('Table', 'droptablesModel');
            $table = $droptablesModelTableBackend->getItem($match[1], true, true, true);

            $configParams = JComponentHelper::getParams('com_droptables');

            if (empty($table) || empty($table->data)) {
                return '';
            }

            $datas = is_string($table->data) ? json_decode($table->data) : $table->data;
            $style = is_string($table->style) ? json_decode($table->style) : $table->style;
            $params = $table->params;
            $table_type = !empty($table->type) ? $table->type : 'html';

            $hightLight = !isset($configParams['highlighting_enable']) ? 0 : (int)$configParams['highlighting_enable'];
            $table->hightlight_color = !isset($configParams['highlighting_color']) ? '#ffffaa' : $configParams['highlighting_color'];
            $table->hightlight_font_color = !isset($configParams['highlighting_color_text']) ? '#ffffff' : $configParams['highlighting_color_text'];
            $table->hightlight_opacity = !isset($configParams['highlighting_opacity']) ? '0.9' : $configParams['highlighting_opacity'];
            $default_order_sortable = isset($style->table->default_order_sortable) ? (int)$style->table->default_order_sortable : 0;
            $default_sort = isset($style->table->default_sortable) ? (int)$style->table->default_sortable : 0;
            $enable_pagination = isset($style->table->enable_pagination) ? (int)$style->table->enable_pagination : 0;

            //todo: usergroup use?
//            $table_usergroups = isset($style->table->usergroup) ? $style->table->usergroup : array();
//            if (!empty($table_usergroups) && !in_array('1', $table_usergroups)) {
//                $user = JFactory::getUser();
//                $groups = $user->get('groups');
//                $result = array_intersect($table_usergroups, $groups);
//                if (empty($result)) {
//                    return '';
//                }
//            }

            if (!isset($style->table) || count((array)$style->table) < 1) {
                $style->table = new stdClass();
            }
            if (!isset($style->table->freeze_col)) {
                $style->table->freeze_col = 0;
            }
            if (!isset($style->table->freeze_row)) {
                $style->table->freeze_row = 0;
            }
            if (!isset($style->table->enable_filters)) {
                $style->table->enable_filters = 0;
            }
            $sortable = false;
            if (isset($style->table->use_sortable) && (int)$style->table->use_sortable === 1) {
                $sortable = true;
            }
            $responsive_type = 'scroll';
            if (isset($style->table->responsive_type) && (string)$style->table->responsive_type === 'hideCols') {
                $responsive_type = 'hideCols';
            }
            if (!isset($style->table->enable_filters)) {
                $style->table->enable_filters = false;
            }
            if (!isset($style->table->table_height)) {
                $style->table->table_height = 500;
            }

            $content = '';
            $table->id = $match[1];
            $table->sortable = $sortable;
            $table->enable_filters = $style->table->enable_filters;
            $data_style = $this->styleRender($table);

            $document = JFactory::getDocument();
            $document->addStyleSheet(JUri::base() . 'media/droptables/' . JFile::makeSafe($table->id . '_' . $table->hash . '.css'));
            $document->addStyleSheet(JURI::root() . 'components/com_droptables/assets/css/front.css?t=1');
            $document->addStyleSheet(JURI::root() . 'components/com_droptables/assets/DataTables/datatables.min.css');
            $document->addStyleSheet(JURI::root() . 'components/com_droptables/assets/tipso/tipso.css');
            if (isset($table->data) && $table->data !== null && !empty($table->data) || $enable_pagination) {
                JHtml::_('jquery.framework');
                JText::script('COM_DROPTABLES_HIDDEN_COLUMNS_BUTTON');

                // hightlight
                if ((int)$hightLight < 1) {
                    $table->hightlight_color = 'not hightlight';
                }

                $document->addScript(JURI::base() . 'components/com_droptables/assets/js/moment.min.js');
                $document->addScript(JURI::base() . 'components/com_droptables/assets/js/moment-jdateformatparser.js');
                $document->addScript(JUri::base() . 'components/com_droptables/assets/DataTables/datatables.min.js');

                /* add tipso lib when tooltip cell exists*/
                $document->addScript(JUri::base() . 'components/com_droptables/assets/tipso/tipso.js');
                $document->addScript(JUri::base() . 'components/com_droptables/assets/js/jquery.fileDownload.js?v=' . $version);
                $document->addScript(JUri::base() . 'components/com_droptables/assets/js/front.js?v=' . $version);

                $check_sortable = '';
                $content .= '<div class="droptables_table tablesorter-bootstrap ' . $check_sortable . '" data-id="' . (int)$table->id . '" data-hightlight="' . $hightLight . '">';
                /*button download table*/
                if (isset($params->download_button) && $params->download_button) {
                    $content .= '<input type="button" data-href="#" href="javascript:void(0);" class="download_droptables" value="' . JText::_('COM_DROPTABLES_DOWNLOAD_TABLE') . '"/>';
                }
                $limit = isset($style->table->limit_rows) ? (int)$style->table->limit_rows : 0;
                $tableContent = $this->getTableContent($table, true, $data_style);

                if ($tableContent) {
                    $content .= $tableContent;

                    // Simple performance check to email cloak
                    if (strpos($tableContent, 'joomla-hidden-mail') !== false) {
                        JLoader::register('DroptablesBase', JPATH_ADMINISTRATOR . '/components/com_droptables/classes/droptablesBase.php');
                        if (droptablesBase::isJoomla40()) {
                            JFactory::getDocument()->getWebAssetManager()
                                ->useScript('webcomponent.hidden-mail');
                        }
                    }
                }
                //phpcs:ignore PHPCompatibility.Constants.NewConstants.ent_html401Found -- Not support php version 5.4 or earlier
                $content = html_entity_decode($content, ENT_COMPAT | ENT_HTML401, 'UTF-8');

                $content .= '<script>droptables_ajaxurl = "' . JUri::root() . 'index.php?option=com_droptables";</script>';

                $content .= '</div><style>' . $this->hightLightCss . '</style>';
            }

            return $content;
        }

        return false;
    }

    /**
     * Get table Html Content
     *
     * @param object  $table      Table object
     * @param boolean $getData    Get table datas
     * @param array   $data_style Style value
     *
     * @return string|boolean
     */
    public function getTableContent($table, $getData, $data_style)
    {
        $params = $table->params;

        JLoader::register('DroptablesHelper', JPATH_ADMINISTRATOR . '/components/com_droptables/helpers/droptables.php');

        $configParams = JComponentHelper::getParams('com_droptables');

        if (!empty($data_style['data'])) {
            $valueTable = DroptablesHelper::htmlRender($table, $configParams, $data_style['data'], $table->hash, $getData);
        } else {
            $valueTable = DroptablesHelper::htmlRender($table, $configParams, array(), $table->hash, $getData);
        }

        if ((isset($params->table_type) && $params->table_type === 'html') || $table->type === 'html') {
            $folder = JPATH_SITE . DIRECTORY_SEPARATOR . 'media' . DIRECTORY_SEPARATOR . 'droptables' . DIRECTORY_SEPARATOR;
            $file = $folder . JFile::makeSafe($table->id . '_' . $table->hash . '.html');

            $content = file_get_contents($file);
            return $content;
        } elseif (is_string($valueTable) && $valueTable !== '') {
            return $valueTable;
        } else {
            return 'table is empty';
        }

        return false;
    }

    /**
     * Function convert alpha string to integer
     *
     * @param string $col Alpha string
     *
     * @return integer
     */
    private function convertAlpha($col)
    {
        //function from http://www.bradino.com/php/excel-column-convert-letters-to-numbers/
        $col = str_pad($col, 2, '0', STR_PAD_LEFT);
        $i = ($col[0] === '0') ? 0 : (ord($col[0]) - 64) * 26;
        $i += ord($col[1]) - 64;

        return $i;
    }

    /**
     * Render style to front-end
     *
     * @param object $table Table style data
     *
     * @return array
     */
    private function styleRender($table)
    {
        $style = is_string($table->style) ? json_decode($table->style) : $table->style;
        if ($style === null) {
            return array();
        }
        $contents = array();
        $style_tooltip = array();
        $componentParams = JComponentHelper::getParams('com_droptables');

        $hightlight_color = $table->hightlight_color;
        $hightlight_font_color = $table->hightlight_font_color;
        $hightlight_opacity = $table->hightlight_opacity;

        if ($hightlight_color !== 'not hightlight') {
            require_once('chartStyleSet.php');
            $chartStyleObj = new ChartStyleSet($hightlight_color);
            $highlighting_rgbcolor = $chartStyleObj->hex2rgba($hightlight_color, $hightlight_opacity);
            $table->hightlight_css = '.droptables-highlight-horizontal, .droptables-highlight-vertical  {  color: ' . $hightlight_font_color . ' !important; background: ' . $highlighting_rgbcolor . ' !important; }';
        } else {
            $table->hightlight_css = '';
        }
        $this->hightLightCss = $table->hightlight_css;

        $tableContent = 'table {';

        //render global table params
        if (isset($style->table->alternate_row_odd_color) && $style->table->alternate_row_odd_color) {
            $tableContent .= 'tr:nth-child(even) td {background-color: ' . $style->table->alternate_row_odd_color . ';}';
        }
        if (isset($style->table->alternate_row_even_color) && $style->table->alternate_row_even_color) {
            $tableContent .= 'tr:nth-child(odd) td {background-color: ' . $style->table->alternate_row_even_color . ';}';
        }
        if (isset($style->table->row_border) && $style->table->row_border) {
            $tableContent .= 'tr td {border-bottom: ' . $style->table->row_border . ';}';
        }

        if (isset($style->table->headerOption) && (int)$style->table->headerOption > 1) {
            $header_content = array();
            for ($i = 0; $i < $style->table->headerOption; $i++) {
                $header_content[] = !empty($style->header_data) && !empty($style->header_data[$i]) ? $style->header_data[$i] : $table->data[$i];
            }
        } else {
            $header_content = $table->data[0];
        }

        //render global rows
        if (!(!isset($style->table->allRowHeight) || $style->table->allRowHeight === '')) {
            $tableContent .= ' td, th {height: ' . (int)$style->table->allRowHeight . 'px;}';
        }

        if (isset($style->rows)) {
            foreach ($style->rows as $numberRow => $row) {
                $tableContent .= '.dtr' . (int)$numberRow . ' {';

                if (is_object($row) && isset($row->{1}->height)) {
                    $styleRows = $row->{1};
                    $tableContent .= ' height: ' . (int)$row->{1}->height . 'px;';
                } elseif (is_array($row) && isset($row[1]->height)) {
                    $styleRows = $row[1];
                    $tableContent .= ' height: ' . (int)$row[1]->height . 'px;';
                } else {
                    $styleRows = null;
                }

                if ($styleRows !== null) {
                    foreach ($styleRows as $numberStyleRow => $styleRow) {
                        if ($numberStyleRow !== 'height') {
                            $tableContent .= $this->addStyleColsRows(
                                $styleRow,
                                null,
                                null,
                                $numberStyleRow
                            );
                        }
                    }
                }

                $tableContent .= '}';

                //responsive_type repeated header
                if (isset($style->table->responsive_type) && $style->table->responsive_type === 'repeatedHeader') {
                    $tableContent .= ' &.repeatedHeaderTrue tbody .dtr' . (int)$numberRow . ' {';

                    if (is_object($row) && isset($row->{1}->height)) {
                        $tableContent .= ' min-height: ' . (int)$row->{1}->height . 'px;';
                    } elseif (is_array($row) && isset($row[1]->height)) {
                        $tableContent .= ' min-height: ' . (int)$row[1]->height . 'px;';
                    } else {
                        $tableContent .= ' min-height: 30px;';
                    }

                    $tableContent .= ';}';
                }
            }
        }

        if (isset($style->cols)) {
            foreach ($style->cols as $numberCol => $col) {
                if (!empty($col)) {
                    $firstCol = is_object($col) ? $col->{1} : $col[1];
                    $tableContent .= '.dtc' . (int)($numberCol) . ' {';
                    if (isset($firstCol->width)) {
//                        $tableContent .= ' width: ' . (int)$firstCol->width . 'px !important; max-width: ' . (int)$firstCol->width . 'px !important; min-width: ' . (int)$firstCol->width . 'px;';//(commit 694549b40a39c283c2b214de6389fd9420139670 wptm)
                        $tableContent .= ' width: ' . (int)$firstCol->width . 'px; min-width: ' . (int)$firstCol->width . 'px;';
                    }

                    foreach ($firstCol as $numberStyleCol => $styleCol) {
                        if ($numberStyleCol !== 'width') {
                            $tableContent .= $this->addStyleColsRows(
                                $styleCol,
                                null,
                                null,
                                $numberStyleCol
                            );
                        }
                    }
                    $tableContent .= '}';

                    //responsive_type repeated header

                    if (isset($style->table->responsive_type) && $style->table->responsive_type === 'repeatedHeader') {
                        $tableContent .= '&.repeatedHeaderTrue { tbody td:nth-of-type(' . ((int)($numberCol) + 1) . '):before {content:';
                        if (!empty($header_content[(int)($numberCol)])) {
                            if (is_array($header_content[0])) {
                                $tableContent .= '"' . $header_content[0][(int)($numberCol)] . '"';
                            } else {
                                $tableContent .= '"' . $header_content[(int)($numberCol)] . '"';
                            }
                        } else {
                            $tableContent .= '""';
                        }
                        $tableContent .= ';}';
                        $tableContent .= '&:not(.style_repeated) tbody tr.odd td.dtc' . (int)($numberCol) . ' {width: auto !important;border: none !important; border-bottom: 1px solid #ffffff !important;';
                        $tableContent .= ';}';
                        $tableContent .= '&:not(.style_repeated) tbody tr.even td.dtc' . (int)($numberCol) . ' {width: auto !important;border: none !important; border-bottom: 1px solid #eaeaea !important;';
                        $tableContent .= ';}}';
                    }
                }
            }
        }

        if ($table->type === 'mysql') {
            if (isset($style->table->allAlternate)) {
                if (!empty($style->table->allAlternate->old)) {
                    $tableContent .= 'tbody tr.even td {background-color: ' . $style->table->allAlternate->old . '}';
                }
                if (!empty($style->table->allAlternate->even)) {
                    $tableContent .= 'tbody tr.odd td {background-color: ' . $style->table->allAlternate->even . '}';
                }
                if (!empty($style->table->allAlternate->header)) {
                    $tableContent .= 'thead tr:first-child th {background-color: ' . $style->table->allAlternate->header . '}';
                }
                if (!empty($style->table->allAlternate->footer)) {
                    $tableContent .= 'tbody tr:last-child td {background-color: ' . $style->table->allAlternate->footer . '}';
                }
            }
        }

        $content2 = '';
        if (isset($style->table->width) && $style->table->width > 0) {
            $content2 .= '& {width : ' . $style->table->width . 'px;}';
        }

        if (!isset($style->table->table_align)) {
            $style->table->table_align = 'center';
        }

        switch ($style->table->table_align) {
            case 'right':
                $content2 .= '& {margin : 0 0 0 auto}';
                break;
            case 'left':
                $content2 .= '& {margin : 0 auto 0 0}';
                break;
            case 'none':
            case 'center':
            default:
                $content2 .= '& {margin : 0 auto 0 auto}';
                break;
        }
        $content2 .= '}';

        $i = 0;
        $alternateColorValue = !empty($style->table->alternateColorValue) ? $style->table->alternateColorValue : null;
        $content = '';


        if (!empty($style->table->mergeSetting)) {
            $mergeSetting = is_string($style->table->mergeSetting) ? json_decode($style->table->mergeSetting, true) : $style->table->mergeSetting;
        }

        foreach ($style->cells as $cell) {
            $rowStyle = null;
            $colStyle = null;

            if ((is_object($style->rows) && isset($style->rows->{(int)($cell[0])}))) {
                $rowStyle = $style->rows->{(int)($cell[0])};
                $rowStyle = is_object($rowStyle) ? $rowStyle->{1} : $rowStyle[1];
            } elseif (!is_object($style->rows) && isset($style->rows[(int)($cell[0])])) {
                $rowStyle = $style->rows[(int)($cell[0])];
                $rowStyle = is_object($rowStyle) ? $rowStyle->{1} : $rowStyle[1];
            }
            if ((is_object($style->cols) && isset($style->cols->{(int)($cell[1])}))) {
                $colStyle = $style->cols->{(int)($cell[1])};
                $colStyle = is_object($colStyle) ? $colStyle->{1} : $colStyle[1];
            } elseif (!is_object($style->cols) && isset($style->cols[(int)($cell[1])])) {
                $colStyle = $style->cols[(int)($cell[1])];
                $colStyle = is_object($colStyle) ? $colStyle->{1} : $colStyle[1];
            }

            $cell_style = '';
            $cell_style_background = '';
            //render global table params
                $AlternateColor = null;
            if (isset($cell[2]['AlternateColor'])) {
                if (is_object($alternateColorValue) && isset($alternateColorValue->{$cell[2]['AlternateColor']})) {
                    $AlternateColor = $alternateColorValue->{$cell[2]['AlternateColor']};
                } elseif (is_array($alternateColorValue) && isset($alternateColorValue[$cell[2]['AlternateColor']])) {
                    $AlternateColor = $alternateColorValue[$cell[2]['AlternateColor']];
                }
            }

            if (isset($AlternateColor)) {
                $numberRow = 0;
                if (isset($AlternateColor->header) && $AlternateColor->header === '') {
                    $numberRow = -1;
                }
                switch ($cell[0]) {
                    case $AlternateColor->selection[0]:
                        if ($AlternateColor->header === '') {
                            $cell_style_background .= 'background-color: ' . $AlternateColor->even . '; ';
                        } else {
                            $cell_style_background .= 'background-color: ' . $AlternateColor->header . '; ';
                        }
                        break;
                    case $AlternateColor->selection[2]:
                        if ($AlternateColor->footer === '') {
                            if (($cell[0] - (int)($AlternateColor->selection[0] + $numberRow)) % 2) {
                                $cell_style_background .= 'background-color: ' . $AlternateColor->even . '; ';
                            } else {
                                $cell_style_background .= 'background-color: ' . $AlternateColor->old . '; ';
                            }
                        } else {
                            $cell_style_background .= 'background-color: ' . $AlternateColor->footer . '; ';
                        }
                        break;
                    default:
                        if (($cell[0] - (int)($AlternateColor->selection[0] + $numberRow)) % 2) {
                            $cell_style_background .= 'background-color: ' . $AlternateColor->even . '; ';
                        } else {
                            $cell_style_background .= 'background-color: ' . $AlternateColor->old . '; ';
                        }
                        break;
                }
            }

            $cell_background = $this->addStyleColsRows(
                isset($colStyle) && isset($colStyle->cell_background_color) ? $colStyle->cell_background_color : null,
                isset($rowStyle) && isset($rowStyle->cell_background_color) ? $rowStyle->cell_background_color : null,
                $cell,
                'cell_background_color'
            );

            if ($cell_style_background !== '') {
                if ($table->enable_filters || $table->sortable) {//if user filter|sort then AlternateColor not by row id
                    $content .= '.row_index' . (int)($cell[0]) . ' .dtc' . (int)($cell[1]) . ' {' . $cell_style_background . '}';
                } else {
                    $cell_style .= $cell_style_background;
                }
                if ($cell_background !== '') {
                    $content .= 'tr .dtr'.(int)($cell[0]).'.dtc'.(int)($cell[1]).' {'. $cell_background . '}';
                }
            } else {
                $cell_style .= $cell_background;
            }

            if (is_array($cell[2])) {
                if (!empty($mergeSetting) && count($mergeSetting) > 0) {
                    $count_mergeSetting = count($mergeSetting);
                    for ($j = 0; $j < $count_mergeSetting; $j++) {
                        if (is_object($mergeSetting[$j])) {
                            $merge_cells = (array)$mergeSetting[$j];
                        } else {
                            $merge_cells = $mergeSetting[$j];
                        }
                        if (isset($merge_cells[2])
                            && (int)($cell[1]) === $merge_cells[2]
                            && ($merge_cells[1] + $merge_cells[0] - 1 >= (int)($cell[0]))
                            && (int)($cell[0]) >= $merge_cells[0]) {
                            $cell_end = ($cell[0]) . '!' . ($merge_cells[2] + $merge_cells[3] - 1);

                            if (isset($style->cells[$cell_end]) && isset($style->cells[$cell_end][2]) && isset($style->cells[$cell_end][2]['cell_border_right'])) {
                                $cell[2]['cell_border_right'] = $style->cells[$cell_end][2]['cell_border_right'];
                            }
                        }
                    }
                }

                foreach ($cell[2] as $keyStyleCell => $styleCell) {
                    if (in_array($keyStyleCell, array(
                        'cell_border_top',
                        'cell_border_right',
                        'cell_border_bottom',
                        'cell_border_left',
                        'cell_font_family',
                        'cell_font_size',
                        'cell_font_color',
                        'cell_font_bold',
                        'cell_font_italic',
                        'cell_font_underline',
                        'cell_text_align',
                        'cell_padding_left',
                        'cell_vertical_align',
                        'cell_padding_top',
                        'cell_padding_right',
                        'cell_padding_bottom',
                        'cell_background_radius_left_top',
                        'cell_background_radius_right_top',
                        'cell_background_radius_right_bottom',
                        'cell_background_radius_left_bottom',
                        'cell_border_bottom'))) {
                        $cell_style .= $this->addStyleColsRows(
                            isset($colStyle) && isset($colStyle->{$keyStyleCell}) ? $colStyle->{$keyStyleCell} : null,
                            isset($rowStyle) && isset($rowStyle->{$keyStyleCell}) ? $rowStyle->{$keyStyleCell} : null,
                            $cell,
                            $keyStyleCell
                        );
                    }
                }

                $text_wrapping = (isset($colStyle) && isset($colStyle->text_wrapping)) ? $colStyle->text_wrapping : null;
                $text_wrapping = (isset($rowStyle) && isset($rowStyle->text_wrapping)) ? $rowStyle->text_wrapping : $text_wrapping;
                $text_wrapping = (isset($cell[2]['text_wrapping']) && false !== $cell[2]['text_wrapping']) ? $cell[2]['text_wrapping'] : $text_wrapping;
                if (isset($text_wrapping) && ($text_wrapping === 'normal' || $text_wrapping === 'hidden')) {
                    $cell_text_wrapping = $this->addStyleColsRows(
                        null,
                        null,
                        $cell,
                        'text_wrapping',
                        $text_wrapping
                    );
                    $firstCol = $colStyle ? $colStyle->width : null;
                    if (isset($cell[2]['cell_padding_left']) && (int)$cell[2]['cell_padding_left'] > 0) {
                        $firstCol -= (int)$cell[2]['cell_padding_left'];
                    }
                    if (isset($cell[2]['cell_padding_right']) && (int)$cell[2]['cell_padding_right'] > 0) {
                        $firstCol -= (int)$cell[2]['cell_padding_right'];
                    }

                    $htmlTag = 'p';
                    if (isset($cell[2]['cell_type']) && (string)$cell[2]['cell_type'] === 'html') {
                        $htmlTag = 'div';
                    }
                    if ($firstCol) {
                        $content .= '.dtr'.(int)($cell[0]).'.dtc'.(int)($cell[1]).' {overflow: visible;}';
                        $content .= '.dtr'.(int)($cell[0]).'.dtc'.(int)($cell[1]).' > ' . $htmlTag . ' {'. $cell_text_wrapping . ' max-width: ' . (int)$firstCol . 'px;}';
                    } else {
                        $content .= '.dtr'.(int)($cell[0]).'.dtc'.(int)($cell[1]).' {overflow: visible;}';
                        $content .= '.dtr'.(int)($cell[0]).'.dtc'.(int)($cell[1]).' > ' . $htmlTag . ' {'. $cell_text_wrapping . ' max-width: 100%;}';
                    }
                }
            }

            $content .= '.dtr' . (int)($cell[0]) . '.dtc' . (int)($cell[1]) . ' {' . $cell_style . '}';
            if (isset($cell[2]['tooltip_width']) && !empty($cell[2]['tooltip_width'])) {
                $style_tooltip[(int)($cell[0]) . '_' . (int)($cell[1])] = $cell[2]['tooltip_width'];
            }
            $i++;

            if ($i > 0 && $i % 1000 === 0) {
                $contents[$i / 1000 - 1] = $tableContent . $content . $content2;
                $content = '';
            }
        }

        $count_content = count($contents);
        $contents[$count_content] = $tableContent . $content . $content2;

        JLoader::register('DroptablesHelper', JPATH_ADMINISTRATOR . '/components/com_droptables/helpers/droptables.php');
        DroptablesHelper::compileStyleFrontEnd($table, $contents, $table->css);

        $data_return = array(
            'tooltip' => $style_tooltip
        );

        return array ('data' => $data_return);
    }

    /**
     * Merge style col, row, cell
     *
     * @param string $cols  Col  style
     * @param string $rows  Row  style
     * @param array  $cell  List style
     * @param string $key   Name style
     * @param string $value Data cell style
     *
     * @return string
     */
    public function addStyleColsRows($cols, $rows, $cell, $key, $value = '')
    {
        if ($value === '') {
            if (isset($cell) && isset($cell[2][$key]) && false !== $cell[2][$key]) {
                $value = $cell[2][$key];
            } elseif (isset($rows)) {
                $value = $rows;
            } elseif (isset($cols)) {
                $value = $cols;
            }
        }

        $content = '';
        if (isset($value)) {
            switch ($key) {
                case 'text_wrapping':
                    if ($value !== '') {
                        if ($value === 'normal') {
                            $content = ' word-wrap: normal; ';
                            $content .= ' overflow: visible; ';
                            $content .= ' white-space: nowrap; ';
                        } elseif ($value === 'break-word') {
                            $content = ' word-wrap: break-word; ';
                            $content .= ' overflow: visible; ';
                        } else {
                            $content = ' overflow: hidden; ';
                            $content .= ' white-space: nowrap; ';
                        }
                    }
                    break;
                case 'cell_background_color':
                    if ($value !== '') {
                        $content = ' background-color: ' . $value . '; ';
                    }
                    break;
                case 'cell_border_top':
                    $check = explode(' ', $value);
                    if ($check[0] !== 'none' && $value !== '') {
                        $content = ' border-top: ' . $value . '; ';
                    }
                    break;
                case 'cell_border_right':
                    $check = explode(' ', $value);
                    if ($check[0] !== 'none' && $value !== '') {
                        $content = ' border-right: ' . $value . '; ';
                    }
                    break;
                case 'cell_border_bottom':
                    $check = explode(' ', $value);
                    if ($check[0] !== 'none' && $value !== '') {
                        $content = ' border-bottom: ' . $value . '; ';
                    }
                    break;
                case 'cell_border_left':
                    $check = explode(' ', $value);
                    if ($check[0] !== 'none' && $value !== '') {
                        $content = ' border-left: ' . $value . '; ';
                    }
                    break;
                case 'cell_font_family':
                    if ($value !== '') {
                        $content = ' font-family: ' . $value . '; ';
                    }
                    break;
                case 'cell_font_size':
                    if ($value !== '') {
                        $content = ' font-size: ' . $value . 'px;';
                    }
                    break;
                case 'cell_font_color':
                    if ($value !== '') {
                        $content = ' color: ' . $value . '; ';
                    }
                    break;
                case 'cell_font_bold':
                    if (!empty($value)) {
                        $content = ' font-weight: bold;';
                    }
                    break;
                case 'cell_font_italic':
                    if (!empty($value)) {
                        $content = ' font-style: italic;';
                    }
                    break;
                case 'cell_font_underline':
                    if (!empty($value)) {
                        $content = ' text-decoration: underline;';
                    }
                    break;
                case 'cell_text_align':
                    if ($value !== '') {
                        $content = ' text-align: ' . $value . '; ';
                    }
                    break;
                case 'cell_vertical_align':
                    if ($value !== '') {
                        $content = ' vertical-align: ' . $value . '; ';
                    }
                    break;
                case 'cell_padding_left':
                    if ($value !== '') {
                        $content = ' padding-left: ' . $value . 'px;';
                    }
                    break;
                case 'cell_padding_top':
                    if ($value !== '') {
                        $content = ' padding-top: ' . $value . 'px;';
                    }
                    break;
                case 'cell_padding_right':
                    if ($value !== '') {
                        $content = ' padding-right: ' . $value . 'px;';
                    }
                    break;
                case 'cell_padding_bottom':
                    if ($value !== '') {
                        $content = ' padding-bottom: ' . $value . 'px;';
                    }
                    break;
                case 'cell_background_radius_left_top':
                    if ($value !== '') {
                        $content = ' border-top-left-radius: ' . $value . 'px;';
                    }
                    break;
                case 'cell_background_radius_right_top':
                    if ($value !== '') {
                        $content = ' border-top-right-radius: ' . $value . 'px;';
                    }
                    break;
                case 'cell_background_radius_right_bottom':
                    if ($value !== '') {
                        $content = ' border-bottom-right-radius: ' . $value . 'px;';
                    }
                    break;
                case 'cell_background_radius_left_bottom':
                    if ($value !== '') {
                        $content = ' border-bottom-left-radius: ' . $value . 'px;';
                    }
                    break;
            }
        }

        return $content;
    }

    /**
     * Validate color
     *
     * @param string $color Color value
     *
     * @return boolean
     */
    private function validateColor($color)
    {
        $result = false;
        if (isset($color[0]) && $color[0] === '#') {
            $color = substr($color, 1);
        }

        //Check if color has 6 or 3 characters and get values
        if (strlen($color) === 6) {
            if (preg_match('/^[a-f0-9]{6}$/i', $color)) {
                //hex color is valid
                $result = true;
            }
        } elseif (strlen($color) === 3) {
            if (preg_match('/^[a-f0-9]{3}$/i', $color)) {
                //hex color is valid
                $result = true;
            }
        }
        return $result;
    }

    /**
     * Build js for chart
     *
     * @param array  $data            Table content
     * @param string $type            Type chart
     * @param object $config          Chart option data
     * @param string $currency_symbol Currency symbols
     * @param string $decimal_symbol  Decimal Symbols
     * @param string $thousand_symbol Thousand symbols
     *
     * @return false|string
     */
    private function buildJsChartData($data, $type, $config, $currency_symbol, $decimal_symbol, $thousand_symbol)
    {
        $result = '';

        if (!$config || !is_object($config)) {
            $config = new stdClass();
            $config->pieColors = '';
            $config->colors = '';
        }

        $config->dataUsing = isset($config->dataUsing) ? $config->dataUsing : 'row';
        $config->useFirstRowAsLabels = isset($config->useFirstRowAsLabels) ? $config->useFirstRowAsLabels : false;

        $dataSets = $this->getDataSets(
            $data,
            $config->dataUsing,
            $currency_symbol,
            $decimal_symbol,
            $thousand_symbol
        );

        if (!isset($dataSets->data) || (count($dataSets->data) === 0)) {
            return $result;
        }

        switch ($type) {
            case 'PolarArea':
            case 'Pie':
            case 'Doughnut':
//                $chartData = $this->convertForPie($dataSets->data, $dataSets->axisLabels, $config->pieColors);
                $chartData = $this->convertForLine($dataSets, $config, $config->pieColors, 'pieColors');
                break;

            case 'Bar':
            case 'Radar':
            case 'Line':
            default:
                $chartData = $this->convertForLine($dataSets, $config, $config->colors, 'colors');
                break;
        }

        $result = json_encode($chartData);
        return $result;
    }

    // check column is int

    /**
     * Replec cell value
     *
     * @param object $cellsData       Cell content
     * @param string $currency_symbol Currency symbols
     *
     * @return array
     */
    private function replaceCell($cellsData, $currency_symbol)
    {
        $data1 = array();
        $i = 0;
        $data2 = -1;
        foreach ($cellsData as $k => $v) {
            $v1 = preg_replace('/[0-9\.\,\-]/', '', $v);
            $v1 = str_replace($currency_symbol, '', $v1);
            if (trim($v1) === '') {
                $data1[$i] = $k;
                $i++;
            } else {
                $data2 = $k;
            }
        }
        $data = array();
        $data[0] = $data1;
        if ($data2 !== -1) {
            $data[1] = $data2;
        }
        return $data;
    }

    /**
     * Get data cell to chart
     *
     * @param array  $cellsData       Data cells
     * @param string $dataUsing       Data Switch
     * @param string $currency_symbol Currency symbol
     * @param string $decimal_symbol  Decimal symbol
     * @param string $thousand_symbol Thousand symbol
     *
     * @return stdClass
     */
    private function getDataSets($cellsData, $dataUsing, $currency_symbol, $decimal_symbol, $thousand_symbol)
    {
        $result = new stdClass();
        $result->data = array();
        $result->data1 = array();
        $result->data_raw = array();
        $result->data_raw1 = array();
        $result->graphLabel = array();//text in line
        $result->axisLabels = array();//text in x-axis
        $result->currency_symbol = array();//text in x-axis
        $axisLabels = array();
        $deleteLine = array();

        if ($dataUsing !== 'row') {//convert to column type
            $cellsData[0] = $this->transposeArr($cellsData[0]);
            $cellsData[1] = $this->transposeArr($cellsData[1]);
        }

        $result->deleteData = array();

        foreach ($cellsData[0] as $k => $value) {
            $checkCellsHaveNaN = 0;
            $countCellInLine = count($value);
            $deleteData1 = array();
            $check_currency_symbol = 0;
            $data = array();
            $data1 = array();
            $result->data_raw[$k] = array();
            $result->data_raw1[$k] = array();

            for ($i = 0; $i < $countCellInLine; $i++) {
                $result->data_raw[$k][] = $value[$i];
                $result->data_raw1[$k][] = $cellsData[1][$k][$i];

                $cell_value = $this->getStrangeCharacters2($value[$i], $cellsData[1][$k][$i], $currency_symbol, $decimal_symbol, $thousand_symbol);
                $checkCellsHaveNaN += $cell_value[2];
                $data[] = $cell_value[0];
                $data1[] = $cell_value[1];

                if ($cell_value[2] === 1) {//have strange characters or is null
                    $deleteData1[$i] = 1;
                }
                if ($cell_value[3] === 1) {
                    $check_currency_symbol++;
                }
            }

            if ($checkCellsHaveNaN === $countCellInLine || ($checkCellsHaveNaN + 2 > $countCellInLine && $countCellInLine > 2)) {//line Have NaN
                $axisLabels[] = $data;
                $deleteLine[] = $k;
            } else {//get this line, that have cell value
                foreach ($deleteData1 as $ii => $deleteData) {
                    if (!isset($result->deleteData[$ii])) {
                        $result->deleteData[$ii] = 0;
                    }
                    $result->deleteData[$ii] += 1;
                }

                $result->graphLabel[] = $data[0];//array key 1, 2, 3,...||$value first value
                $result->data[] = $data;
                $result->data1[] = $data1;

                if ($check_currency_symbol > 1) {
                    $result->currency_symbol[] = 1;
                } else {
                    $result->currency_symbol[] = -1;
                }
            }
        }

        $numberLine = count($result->data);
        $useFirstRowAsGraph = isset($this->useFirstRowAsGraph) ? $this->useFirstRowAsGraph : true;
        //if line number > 1 then not get cell is graphLabel else < 1 then get it
        if ($numberLine > 1 && count($result->data_raw) > 1 && !(count($result->data_raw) === 2 && $useFirstRowAsGraph !== true)) {//have > 1 line in chart
            for ($i = 0; $i < $numberLine; $i++) {
                array_shift($result->data[$i]);
                array_shift($result->data1[$i]);
            }
            $result->arrayShiftData = true;

            if (isset($result->deleteData[0])) {
                unset($result->deleteData[0]);
            }
        }

        if (count($axisLabels) > 0) {//useFirstRowAsGraph become useless
            $result->axisLabels = $axisLabels[0];
        } elseif ($numberLine > 0) {//axisLabels from $cellsData[0] || all line be passed validated
            $result->axisLabels = $result->data_raw[0];
            if ($useFirstRowAsGraph !== true) {
                array_shift($result->data);
                array_shift($result->data1);
                array_shift($result->currency_symbol);
                array_shift($result->graphLabel);
            }
        }

        if (!empty($result->arrayShiftData)) {
            array_shift($result->axisLabels);
        }

        foreach ($result->deleteData as $ii => $deleteData) {//not deleted yet cells not pass
            if ($numberLine !== $deleteData) {
                unset($result->deleteData[$ii]);
            }
        }
        $result->data1 = $this->convertToNumber($result->data1);
        return $result;
    }
    /**
     * Get data cell to chart
     *
     * @param string $cell_value      Data cells
     * @param string $cell_value_raw  Data Switch
     * @param string $currency_symbol Currency symbol
     * @param string $decimal_symbol  Decimal symbol
     * @param string $thousand_symbol Thousand symbol
     *
     * @return array
     */
    public function getStrangeCharacters2($cell_value, $cell_value_raw, $currency_symbol, $decimal_symbol, $thousand_symbol)
    {
        $value = array();
        $value[0] = $cell_value;
        $value0 = $cell_value_raw === null ?  str_replace(' ', '', $cell_value) : str_replace(' ', '', $cell_value_raw);
        $value1 = str_replace($currency_symbol, '', $value0);
        if ($cell_value_raw === null) {
            $value1 = str_replace($thousand_symbol, '', $value1);
            $value1 = str_replace($decimal_symbol, '.', $value1);
        }
        $value[1] = preg_replace('/[^0-9|\.|-]/', '', $value1);

        $value[2] = 0;
        $value[3] = 0;//currency_symbol

        $value1 = preg_replace('/[0-9\.\,\-| ]/', '', $value1);
        if ($value1 !== '' || $cell_value === '' || $cell_value === null) {//have strange characters or is null
            $value[2] = 1;
        }

        if ($cell_value !== '' && (strrpos($cell_value, $currency_symbol) !== false && $cell_value_raw !== null)) {
            $value[3] = 1;
        }
        return $value;
    }

    /**
     * Get valid  chart data
     *
     * @param array  $cellsData       Cells content
     * @param string $currency_symbol Currency symbols
     * @param string $decimal_symbol  Decimal symbol
     * @param string $thousand_symbol Thousand symbol
     *
     * @return array
     */
    public function getValidChartData($cellsData, $currency_symbol, $decimal_symbol, $thousand_symbol)
    {
        $results = array();
        $resultIndexes = array();
        $tempIndexes = array();
        $rowIndexes = array();
        $countFirstCellsData = count($cellsData[0]);
        $countCellsData = count($cellsData);
        for ($i = 0; $i < $countFirstCellsData; $i++) {
            $resultIndexes[$i] = $i;
        }
        for ($i = 0; $i < $countCellsData; $i++) {
            $ValidRow = $this->isGetValidRow($cellsData[$i], $currency_symbol, $decimal_symbol, $thousand_symbol);
            if ($ValidRow['validRow'] > 1) {
                array_push($results, $ValidRow['cellDataVal']);
                array_push($rowIndexes, $i);
                $tempIndexes = $ValidRow['getValidRow'];
                $resultIndexes = $this->intersection($tempIndexes, $resultIndexes, $ValidRow['validRow']);
            }
        }
        $count = count($results);
        for ($i = 0; $i < $count; $i++) {
            $tempArr = array();
            $countIndex = count($resultIndexes);
            for ($j = 0; $j < $countIndex; $j++) {
                array_push($tempArr, $results[$i][$resultIndexes[$j]]);
            }
            $results[$i] = $tempArr;
        }
        return array($results, $resultIndexes, $rowIndexes);
    }

    //check and get index of valid number in the array

    /**
     * Get valid row
     *
     * @param array  $cellData        Cell content to row
     * @param string $currency_symbol Currency symbols
     * @param string $decimal_symbol  Decimal symbol
     * @param string $thousand_symbol Thousand symbol
     *
     * @return array
     */
    public function isGetValidRow($cellData, $currency_symbol, $decimal_symbol, $thousand_symbol)
    {
        $countCellData = count($cellData);
        $count = 0;
        $result = array();
        for ($i = 0; $i < $countCellData; $i++) {
            $cellData[$i] = (string)$cellData[$i];
            $cellData[$i] = str_replace($currency_symbol, '', $cellData[$i]);
            $cellData[$i] = str_replace($thousand_symbol, '', $cellData[$i]);
            $cellData[$i] = str_replace($decimal_symbol, '.', $cellData[$i]);

            $cellData[$i] = str_replace('/[\,\-]/g', '', $cellData[$i]);
            $cellData[$i] = str_replace('/[ ]/g', '', $cellData[$i]);
            $v = preg_replace('/[0-9\\.]/', '', $cellData[$i]);
            if ($v === '') {
                $count++;
                array_push($result, $i);
            }
        }
        return array('validRow' => $count, 'getValidRow' => $result, 'cellDataVal' => $cellData);
    }

    /**
     * Get intersection values of two array
     *
     * @param array   $tempIndexes       Temp index
     * @param array   $resultIndexes     Result index
     * @param integer $lengthTempIndexes Count temp index
     *
     * @return array
     */
    public function intersection($tempIndexes, $resultIndexes, $lengthTempIndexes)
    {
        $data = array();
        for ($i = 0; $i < $lengthTempIndexes; $i++) {
            if (in_array($tempIndexes[$i], $resultIndexes)) {
                array_push($data, $tempIndexes[$i]);
            }
        }
        return $data;
    }

    /**
     * Get empty array
     *
     * @param integer $len Count value array
     *
     * @return array
     */
    public function getEmptyArray($len)
    {
        $result = array();
        for ($i = 0; $i < $len; $i++) {
            $result[$i] = '    ';
        }
        return $result;
    }

    /**
     * Convert string to integer
     *
     * @param array|string $arr Value to convert
     *
     * @return array
     */
    public function convertToNumber($arr)
    {
        if (is_array($arr)) {
            $countArr = count($arr);
            for ($i = 0; $i < $countArr; $i++) {
                if (is_array($arr[$i])) {
                    $countArrI = count($arr[$i]);
                    for ($j = 0; $j < $countArrI; $j++) {
                        $arr[$i][$j] = str_replace(',', '', $arr[$i][$j]);
                    }
                } else {
                    $arr[$i] = str_replace(',', '', $arr[$i]);
                }
            }
        } else {
            $arr = str_replace(',', '', $arr);
        }
        return $arr;
    }

    /**
     * Convert values of in-line cells
     *
     * @param object $dataSets       Data chart after change
     * @param object $config         Use First Row As Labels
     * @param string $colors         Color lines in chart
     * @param string $checkTyleColor Color input in chart
     *
     * @return stdClass
     */
    private function convertForLine($dataSets, $config, $colors, $checkTyleColor)
    {
        $result = new stdClass();
        $result->datasets = array();
        if (!is_array($dataSets->data1) || (count($dataSets->data1) === 0)) {
            return $result;
        }

        $useFirstRowAsLabels = $config->useFirstRowAsLabels;

        $numberLine = count($dataSets->data1);
        $countDatasets = count($dataSets->data1[0]);
        $result->data_format = array();
        for ($i = 0; $i < $numberLine; $i++) {
            $dataSet = new stdClass();
            $dataSet->label = $dataSets->graphLabel[$i];
            $dataSet->currency_symbol = $dataSets->currency_symbol[$i];
            $result->labels = array();
            if (isset($dataSets->data) && $dataSets->data[$i] !== null) {
                $result->data_format[] = $dataSets->data[$i];
            } else {
                $result->data_format[] = $dataSets->data1[$i];
            }

            if ($checkTyleColor === 'pieColors') {
                $dataSet->highlight = array();
                $dataSet->backgroundColor = array();
                $dataSet->borderColor = array();
                $dataSet->pointBackgroundColor = array();
                $dataSet->pointColor = array();
                $dataSet->pointBorderColor = array();
                $dataSet->pointHighlightFill = array();
            }

            for ($j = 0; $j < $countDatasets; $j++) {
                if (!(isset($dataSets->deleteData)
                    && ((!empty($dataSets->arrayShiftData) && isset($dataSets->deleteData[$j + 1]))
                        || (empty($dataSets->arrayShiftData) && isset($dataSets->deleteData[$j]))
                    )
                )) {//thoa man
                    if (!isset($dataSet->data)) {
                        $dataSet->data = array();
                    }
                    $dataSet->data[] = $dataSets->data1[$i][$j];//data da duoc remove tu truoc

                    if ($useFirstRowAsLabels) {
                        $result->labels[] = $dataSets->axisLabels[$j];
                    } else {
                        $result->labels[] = '';
                    }

                    if ($checkTyleColor === 'pieColors') {
                        $pieColors = $this->getStyleSet($j, $colors);
                        $dataSet->highlight[] = $pieColors->highlight;
                        $dataSet->backgroundColor[] = $pieColors->backgroundColor;
                        $dataSet->borderColor[] = $pieColors->borderColor;
                        $dataSet->pointBackgroundColor[] = $pieColors->pointBackgroundColor;
                        $dataSet->pointColor[] = $pieColors->pointColor;
                        $dataSet->pointBorderColor[] = $pieColors->pointBorderColor;
                        $dataSet->pointHighlightFill[] = $pieColors->pointHighlightFill;
                    }
                }
            }
            if ($checkTyleColor !== 'pieColors') {
                $styleSet = $this->getStyleSet($i, $colors);
                $dataSet = (object)array_merge((array)$dataSet, (array)$styleSet);
            }
            $result->datasets[$i] = $dataSet;
        }

        return $result;
    }

    /**
     * Convert color
     *
     * @param array  $datacell   Cell data
     * @param string $axisLabels Axis labels
     * @param string $pieColors  Pie colors
     *
     * @return array
     */
    private function convertForPie($datacell, $axisLabels, $pieColors)
    {
        require_once('chartStyleSet.php');

        $datas = array();
        $data = array();

        $defaultColors = array('#F7464A', '#46BFBD', '#FDB45C', '#949FB1', '#4D5360');
        $highlights = array('#FF5A5E', '#5AD3D1', '#FFC870', '#A8B3C5', '#616774');
        if (!$pieColors) {
            $colors = $defaultColors;
        } else {
            $colors = explode(',', $pieColors);
        }
        $datas['labels'] = $axisLabels;
        $countCell = count($datacell);
        for ($i = 0; $i < $countCell; $i++) {
            $data[$i]['data'] = $datacell[$i];
            $data[$i]['backgroundColor'] = array();
            $data[$i]['borderColor'] = array();
            $data[$i]['pointBackgroundColor'] = array();
            $data[$i]['pointBorderColor'] = array();
            $countCellI = count($datacell[$i]);
            for ($j = 0; $j < $countCellI; $j++) {
                if (!isset($colors[$j])) {
                    $colors[$j] = $defaultColors[$j % 5];
                }
                $color = new ChartStyleSet($colors[$j], $i * 0.1);
                $data[$i]['backgroundColor'][$j] = $color->backgroundColor;
                $data[$i]['borderColor'][$j] = $color->borderColor;
                $data[$i]['pointBackgroundColor'][$j] = $color->pointBackgroundColor;
                $data[$i]['pointBorderColor'][$j] = $color->pointBorderColor;
            }
        }
        $datas['datasets'] = $data;
        return $datas;
    }

    /**
     * Alter brightness
     *
     * @param string  $colourstr Color str
     * @param integer $steps     Steps
     *
     * @return string
     */
    public function alter_brightness($colourstr, $steps)//phpcs:ignore PSR1.Methods.CamelCapsMethodName.NotCamelCaps -- Function unknown location of use
    {
        $colourstr = str_replace('#', '', $colourstr);
        $rhex = substr($colourstr, 0, 2);
        $ghex = substr($colourstr, 2, 2);
        $bhex = substr($colourstr, 4, 2);

        $r = hexdec($rhex);
        $g = hexdec($ghex);
        $b = hexdec($bhex);

        $r = max(0, min(255, $r + $r * $steps));
        $g = max(0, min(255, $g + $g * $steps));
        $b = max(0, min(255, $b + $b * $steps));

        return '#' . dechex($r) . dechex($g) . dechex($b);
    }

    /**
     * Get color style
     *
     * @param integer $i      Number array
     * @param string  $colors Color
     *
     * @return object
     */
    private function getStyleSet($i, $colors)
    {
        $result = null;
        $defaultColors = array('#DCDCDC', '#97BBCD', '#4C839E');

        if (!$colors) {
            $arrColors = $defaultColors;
        } else {
            $arrColors = explode(',', $colors);
        }

        if (count($arrColors) && isset($arrColors[$i])) {
            $color = $arrColors[$i];
        } else {
            $color = $defaultColors[$i % 3];
        }

        require_once('chartStyleSet.php');
        $result = new ChartStyleSet($color);
        return $result;
    }

    /**
     * Check number array
     *
     * @param array  $arr             Array content
     * @param string $currency_symbol Currency symbols
     *
     * @return boolean
     */
    private function isNumbericArray($arr, $currency_symbol)
    {
        $valid = true;
        $count = count($arr);
        for ($c = 0; $c < $count; $c++) {
            if ($arr[$c] !== '') {
                $arr[$c] = str_replace($currency_symbol, '', (string)$arr[$c]);
                $arr[$c] = preg_replace('/[\.\,\-]/', '', $arr[$c]);
                if (!is_numeric($arr[$c])) {
                    $valid = false;
                }
            }
        }
        return $valid;
    }

    /**
     * Check row has number
     *
     * @param array  $cells           Cells array
     * @param string $currency_symbol Currency symbols
     *
     * @return boolean
     */
    private function hasNumbericRow($cells, $currency_symbol)
    {
        $rValid = true;
        $rNaN = 0;
        $countCells = count($cells);
        for ($r = 0; $r < $countCells; $r++) {
            $valid = true;
            $countCellR = count($cells[$r]);
            for ($c = 0; $c < $countCellR; $c++) {
                $cells[$r][$c] = str_replace($currency_symbol, '', (string)$cells[$r][$c]);
                if (!is_numeric(preg_replace('/[\.\,\-]/', '', $cells[$r][$c]))) {
                    $valid = false;
                }
            }

            if (!$valid) {
                $rNaN++;
            }
        }
        if ($rNaN === count($cells)) {
            $rValid = false;
        }

        return $rValid;
    }

    /**
     * Transpose array
     *
     * @param array $array Array test
     *
     * @return array
     */
    private function transposeArr($array)
    {
        $transposed_array = array();
        if (is_array($array)) {
            foreach ($array as $row_key => $row) {
                if (is_array($row) && !empty($row)) { //check to see if there is a second dimension
                    foreach ($row as $column_key => $element) {
                        $transposed_array[$column_key][$row_key] = $element;
                    }
                } else {
                    $transposed_array[0][$row_key] = $row;
                }
            }
            return $transposed_array;
        }
    }

    /**
     * Get chart data
     *
     * @param string $cellRange Cell range
     * @param string $tData     Table data
     *
     * @return array
     */
    private function getChartData($cellRange, $tData)
    {
        $result = array();
        $arr_cellRanges = json_decode($cellRange);
        $countCellRanges = count($arr_cellRanges);
        for ($i = 0; $i < $countCellRanges; $i++) {
            $row = $arr_cellRanges[$i];
            $countRow = count($row);
            for ($j = 0; $j < $countRow; $j++) {
                $result[$i][$j] = $this->getCellData($row[$j], $tData);
            }
        }

        return $result;
    }

    /**
     * Get cell data
     *
     * @param string $cellPos   Cell pos
     * @param array  $tableData Table data
     *
     * @return string
     */
    private function getCellData($cellPos, $tableData)
    {
        $result = '';
        list($r, $c) = explode(':', $cellPos);
        $result = $tableData[$r][$c];
        return $result;
    }
}
