<?php
/**
 * Droptables
 *
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 *
 * @package   Droptables
 * @copyright Copyright (C) 2014 JoomUnited (http://www.joomunited.com). All rights reserved.
 * @copyright Copyright (C) 2014 Damien Barrère (http://www.crac-design.com). All rights reserved.
 * @license   GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 */


defined('_JEXEC') || die;

use Joomla\CMS\Helper\ContentHelper;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Factory;

/**
 * Class DroptablesHelper
 */
class DroptablesHelper
{
    /**
     * A cache for the available actions.
     *
     * @var object
     */
    public static $actions;
    /**
     * Calculator result
     *
     * @var string|integer|array
     */
    public static $resultCalc;
    /**
     * Date formats
     *
     * @var string|array
     */
    public static $date_formats;
    /**
     * Date
     *
     * @var array
     */
    public static $dateTime;
    /**
     * Turn count
     *
     * @var integer
     */
    public static $n;
    /**
     * Math formula
     *
     * @var array
     */
    public static $math_formula = array('SUM','MULTIPLY','DIVIDE','COUNT','MIN', 'MAX','AVG');

    /**
     * Boolean formula
     *
     * @var array
     */
    public static $and_formula = array('AND','OR', 'XOR');
    /**
     * Time formula
     *
     * @var array
     */
    public static $date_formula = array('DATE','DAY','DAYS','DAYS360');
    /**
     * Symbol position
     *
     * @var string
     */
    public static $symbol_position;
    /**
     * Symbol position cell
     *
     * @var string
     */
    public static $symbol_position_cell = '';
    /**
     * Currency symbol
     *
     * @var string
     */
    public static $currency_symbol;
    /**
     * Currency symbol cell
     *
     * @var string
     */
    public static $currency_symbol_cell = '';
    /**
     * Decimal symbol
     *
     * @var string
     */
    public static $decimal_symbol;
    /**
     * Decimal symbol cell
     *
     * @var string
     */
    public static $decimal_symbol_cell = '';
    /**
     * Decimal count
     *
     * @var string
     */
    public static $decimal_count;
    /**
     * Decimal count cell
     *
     * @var string
     */
    public static $decimal_count_cell = '';
    /**
     * Thousand symbol
     *
     * @var string
     */
    public static $thousand_symbol;
    /**
     * Thousand symbol cell
     *
     * @var string
     */
    public static $thousand_symbol_cell = '';
    /**
     * Date var
     *
     * @var array
     */
    public static $date;
    /**
     * Configure the Linkbar.
     *
     * @param string $vName The name of the active view.
     *
     * @return void
     * @since  1.6
     */
    public static function addSubmenu($vName)
    {
        /*JSubMenuHelper::addEntry(
            JText::_('COM_MESSAGES_ADD'),
            'index.php?option=com_messages&view=message&layout=edit',
            $vName == 'message'
        );

        JSubMenuHelper::addEntry(
            JText::_('COM_MESSAGES_READ'),
            'index.php?option=com_messages',
            $vName == 'messages'
        );*/
    }

    /**
     * List all tables for selection
     *
     * @return string[]
     */
    public static function allTableList()
    {

        $path_model = JPATH_ROOT . '/administrator/components/com_droptables/models/';
        JModelLegacy::addIncludePath($path_model, 'DroptablesModelCategories');
        $modelCategories = JModelLegacy::getInstance('categories', 'DroptablesModel');
        JModelLegacy::addIncludePath($path_model, 'DroptablesModelTables');
        $modelTables = JModelLegacy::getInstance('tables', 'DroptablesModel');

        JFactory::getApplication()->setUserState('list.limit', 100000);
        $categories = $modelCategories->getItems();
        $tables = $modelTables->getItems();

        JLoader::register('DroptablesTablesHelper', JPATH_ADMINISTRATOR . '/components/com_droptables/helpers/tables.php');
        $tables = DroptablesTablesHelper::categoryObject($tables);

        $totalCat = count($categories);
        $listArray = array('' => 'Select table');
        for ($i = 0; $i < $totalCat; $i++) {
            $listArray['cat-'.$categories[$i]->id] = str_repeat('-', ($categories[$i]->level - 1)) . ' Category: ' . $categories[$i]->title;
            if (isset($tables[$categories[$i]->id])) {
                $subTables = $tables[$categories[$i]->id];
                foreach ($subTables as $table) {
                    $listArray['table-'.$table->id] = str_repeat('-- ', ($categories[$i]->level )) . ' ' . $table->title;
                }
            }
        }

        return $listArray;
    }

    /**
     * Compile style table/chart
     *
     * @param object $table     Data table
     * @param array  $styles    Style table
     * @param string $customCss Custom css
     *
     * @return boolean
     */
    public static function compileStyleFrontEnd($table, $styles, $customCss)
    {
        jimport('joomla.filesystem.folder');
        jimport('joomla.filesystem.file');
        $folder = JPATH_SITE . DIRECTORY_SEPARATOR . 'media' . DIRECTORY_SEPARATOR . 'droptables' . DIRECTORY_SEPARATOR;
        $file = $folder . JFile::makeSafe($table->id . '_' . $table->hash . '.css');
        if (!JFile::exists($file)) {
            $files = glob($folder . $table->id . '_*.css');
            foreach ($files as $f) {
                JFile::delete($f);
            }
        } else {
            return true;
        }

        $countStyles = count($styles);
        $contents    = array();
        $contents[0] = '';

        JLoader::register('lessc', JPATH_ADMINISTRATOR . '/components/com_droptables/classes/lessphp.php');
        JLoader::register('csstidy', JPATH_ADMINISTRATOR . '/components/com_droptables/classes/csstidy/class.csstidy.php');
        $less = new lessc;

        for ($i = 0; $i < $countStyles; $i ++) {
            $style = $styles[$i];

            try {
                set_time_limit(0);
                ini_set('max_execution_time', -1);
                $contents[$i] = $less->compile('#droptablestable' . $table->id . '.droptablestable {' . $style . '}');
            } catch (Exception $exc) {
                return false;
            }

            try {
                $customContent = $less->compile('#droptablestable' . $table->id . '.droptablestable table {' . $customCss . '}');
            } catch (Exception $exc) {
                $customContent = '';
            }

            $contents[$i] .= $customContent;
            $csstidy      = new csstidy();
            $csstidy->parse($contents[$i]);

            $less->setFormatter('compressed');

            try {
                //phpcs:ignore WordPress.XSS.EscapeOutput.OutputNotEscaped -- The inner variables were esc
                $contents[$i] = $less->compile($csstidy->print->plain());
            } catch (Exception $exc) {
                return false;
            }
        }

        if (isset($contents[1])) {
            $content = implode($contents);
        } else {
            $content = $contents[0];
        }

        $style = $table->style;
        $alignCss = '';
        $table_align = !isset($style->table->table_align) ? 'left' : $style->table->table_align;
        switch ($table_align) {
            case 'left':
                $alignCss .= 'margin : 0 auto 0 0';
                break;
            case 'right':
                $alignCss .= 'margin : 0 0 0 auto';
                break;
            case 'none':
                break;
            case 'center':
            default:
                $alignCss .= 'margin : 0 auto';
                break;
        }
        $content .= '#droptablestable' . $table->id . '.droptablestable .dataTables_scrollHead .dataTables_scrollHeadInner { '. $alignCss .' }';

        if (JFile::write($file, $content)) {
            return true;
        }

        return true;
    }

    /**
     * Compile style
     *
     * @param object $table     Table data
     * @param string $style     Table style
     * @param string $customCss Custom style
     *
     * @return boolean
     */
    public static function compileStyle($table, $style, $customCss)
    {
        jimport('joomla.filesystem.folder');
        jimport('joomla.filesystem.file');

        $folder = JPATH_SITE . DIRECTORY_SEPARATOR . 'media' . DIRECTORY_SEPARATOR . 'droptables' . DIRECTORY_SEPARATOR;
        $file = $folder . JFile::makeSafe($table->id . '_' . $table->hash . '.css');
        if (!JFile::exists($file)) {
            $files = glob($folder . $table->id . '_*.css');
            foreach ($files as $f) {
                JFile::delete($f);
            }
        } else {
            return true;
        }

        JLoader::register('lessc', JPATH_ADMINISTRATOR . '/components/com_droptables/classes/lessphp.php');
        JLoader::register('csstidy', JPATH_ADMINISTRATOR . '/components/com_droptables/classes/csstidy/class.csstidy.php');

        $less = new lessc;
        try {
            set_time_limit(0);
            ini_set('max_execution_time', -1);
            $content = $less->compile('#droptablestable' . $table->id . '.droptablestable {' . $style . '}');
        } catch (Exception $exc) {
            return false;
        }

        try {
            $customContent =
                $less->compile('#droptablestable' . $table->id . '.droptablestable table tbody {' . $customCss . '}');
        } catch (Exception $exc) {
            $customContent = '';
        }

        if (!empty($customContent)) {
            $content .= $customContent;
        }
        $csstidy = new csstidy();
        $csstidy->parse($content);

        $less->setFormatter('compressed');
        try {
            $content = $less->compile($csstidy->print->plain());
        } catch (Exception $exc) {
            return false;
        }
        if (JFile::write($file, $content)) {
            return true;
        }
        return false;
    }

    /**
     * Convert date data
     * convert string to m/d/Y
     *
     * @param array   $number      Number Number
     * @param array   $date_format Date format
     * @param boolean $timezone    Check timezone
     *
     * @return boolean|string
     */
    public static function convertDay($number, $date_format, $timezone)
    {
        $F_name = array('january', 'february', 'march', 'april', 'may', 'june', 'july', 'august', 'september', 'october', 'november', 'december');
        $M_name = array('jan', 'feb', 'mar', 'apr', 'may', 'jun', 'jul', 'aug', 'sept', 'oct', 'nov', 'dec');
        $date_array = array();
        $date = array();
        $number = (!!$number) ? $number : array();
        if (count($date_format) !== count($number)) {
            return false;
        }
        $countDate = count($date_format);
        for ($n = 0; $n < $countDate; $n++) {
            $number[$n] = (!!$number[$n]) ? $number[$n] : '';
            if ($date_format[$n] === 'd' || $date_format[$n] === 'j') {
                $date_array[2] = ($number[$n] !== '') ? $number[$n] : 0;
            } elseif ($date_format[$n] === 'S' || $date_format[$n] === 'jS' || $date_format[$n] === 'dS') {
                preg_match_all('/[0-9]+/', $number[$n], $date[2]);
                $date_array[2] = $date[2][0][0];
            } elseif ($date_format[$n] === 'm' || $date_format[$n] === 'n') {
                $date_array[1] = $number[$n];
            } elseif ($date_format[$n] === 'F') {
                $date_array[1] = array_search(strtolower($number[$n]), $F_name) + 1;
            } elseif ($date_format[$n] === 'M') {
                $date_array[1] = array_search(strtolower($number[$n]), $M_name) + 1;
            } elseif ($date_format[$n] === 'Y' || $date_format[$n] === 'y') {
                $date_array[3] = $number[$n];
            } elseif (strtolower($date_format[$n]) === 'g' || strtolower($date_format[$n]) === 'h') {
                $date_array[4] = $number[$n];
            } elseif (strtolower($date_format[$n]) === 'ga' || strtolower($date_format[$n]) === 'ha') {
                preg_match_all('/[0-9]+/', $number[$n], $date[4]);
                $number[$n] = preg_replace('/[0-9]+/', '', $number[$n]);
                $date_array[4] = (strtolower($number[$n]) === 'am') ? (int)$date[4][0][0] : (int)$date[4][0][0] + 12;
            } elseif (strtolower($date_format[$n]) === 'a') {
                $date_array[7] = $number[$n];
            } elseif (strtolower($date_format[$n]) === 'i' || strtolower($date_format[$n]) === 'ia') {
                preg_match_all('/[0-9]+/', $number[$n], $date[5]);
                $date_array[5] = $date[5][0][0];
            } elseif (strtolower($date_format[$n]) === 's' || strtolower($date_format[$n]) === 'sa') {
                preg_match_all('/[0-9]+/', $number[$n], $date[6]);
                $date_array[6] = $date[6][0][0];
            } elseif ($date_format[$n] === 'T') {
                $date_array[8] = $number[$n];
            } elseif ($date_format[$n] === 'r') {
                if (array_search(strtolower($number[$n]), $F_name) + 1 > 0) {
                    $date_array[1] = array_search(strtolower($number[$n]), $F_name) + 1;
                } else {
                    $date_array[1] = array_search(strtolower($number[$n]), $M_name) + 1;
                }
                return $date_array[1] . '/' . $number[1] . '/' . $number[3] . ' ' . $number[4] . ':' . $number[5] . ':' . $number[6] . ' ' . $number[7];
            }
        }
        if ($date_array[3] === '' || $date_array[2] === '' || $date_array[2] > 31 || $date_array[1] > 12) {
            return false;
        }

        $date_array[4] = (!empty($date_array[4])) ? (int)$date_array[4] : '00';
        $date_array[5] = (!empty($date_array[5])) ? (int)$date_array[5] : '00';
        $date_array[6] = (!empty($date_array[6])) ? (int)$date_array[6] : '00';
        $date_array[7] = (!empty($date_array[7])) ? $date_array[7] : '';
        $date_array[8] = (!empty($date_array[8])) ? $date_array[8] : '';
        $date_array[8] = ($timezone === true) ? $date_array[8] : '';
        if (strtolower($date_array[7]) === 'pm') {
            $date_array[4] = $date_array[4] + 12;
        }
        return (int)$date_array[1] . '/' . (int)$date_array[2] . '/' . $date_array[3] . ' ' . $date_array[4] . ':' . $date_array[5] . ':' . $date_array[6] . $date_array[8];
    }

    /**
     * Set format symbols
     *
     * @param float          $resultCalc       Result calculator
     * @param string|integer $places           Decimal Count
     * @param string         $thousand_symbols Thousand symbols
     * @param string         $decimal_symbols  Decimal symbols
     * @param string|integer $unit_symbols     Unit symbols
     * @param string         $value_unit       Value unit symbols
     *
     * @return string
     */
    public static function formatSymbols($resultCalc, $places, $thousand_symbols, $decimal_symbols, $unit_symbols, $value_unit)
    {
        $places = (int)$places;
        $array_resultCalc = str_split((string)round($resultCalc, $places));
        $decimal = array_search('.', $array_resultCalc);
        $decimal = ($decimal !== false) ? $decimal : count($array_resultCalc);
        if ($decimal === count($array_resultCalc)) {
            $array_resultCalc[count($array_resultCalc)] = '.';
        }

        $data = '';
        $j = ($decimal > 3) ? $decimal % 3 : -1;
        if ($array_resultCalc[0] === '-') {
            $j = ($decimal - 1 > 3) ? ($decimal - 1) % 3 + 1 : -1;
            $array_resultCalc[0] = (int)$unit_symbols === 0 ? $array_resultCalc[0] . $value_unit : $array_resultCalc[0];
        } else {
            $array_resultCalc[0] = (int)$unit_symbols === 0 ? $value_unit . $array_resultCalc[0] : $array_resultCalc[0];
        }
        $decimal1 = $decimal;
        for ($i = 0; $i < $decimal1 + 1 + $places; $i++) {
            if ($i + 1 === $j && $array_resultCalc[$i] !== '-') {
                $data .= is_numeric($array_resultCalc[$i]) ? $array_resultCalc[$i] . $thousand_symbols : $array_resultCalc[$i];
            } elseif ($j !== -1 && $i + 1 - $j !== 0 && ($i + 1 - $j) % 3 === 0 && $i < $decimal - 1) {
                $data .= $array_resultCalc[$i] . $thousand_symbols;
            } elseif ($i === $decimal && $places !== 0) {
                $data .= $decimal_symbols;
            } elseif (empty($array_resultCalc[$i])) {
                $data .= '0';
            } elseif ($array_resultCalc[$i] !== $decimal_symbols) {
                $data .= $array_resultCalc[$i];
            }
        }
        return ((int)$unit_symbols === 0) ? $data : $data . ' ' . $value_unit;
    }

    /**
     * Convert droptables's dateformat to momentjs's dateformat
     *
     * @param string $value PHP date format
     *
     * @return string
     */
    public static function momentjsFormat($value)
    {
        $value = str_replace('M', 'MMM', $value);
        $value = str_replace('F', 'MMMM', $value);
        $value = str_replace('m', 'MM', $value);
        $value = str_replace('n', 'M', $value);

        $value = str_replace('D', 'ddd', $value);
        $regex = '/(?<!d)d(?!d)/i';
        $value = preg_replace($regex, 'DD', $value);

        $value = str_replace('j', 'D', $value);
        $value = str_replace('z', 'DDDD', $value);
        $value = str_replace('S', 'Do', $value);
        $value = str_replace('l', 'dddd', $value);
        $value = str_replace('N', 'e', $value);

        $value = str_replace('w', 'do', $value);
        $value = str_replace('W', 'w', $value);

        $value = str_replace('Y', 'YYYY', $value);
        $value = str_replace('y', 'YY', $value);
        $value = str_replace('o', 'Y', $value);

        $value = str_replace('h', 'hh', $value);
        $value = str_replace('g', 'h', $value);
        $value = str_replace('H', 'HH', $value);
        $value = str_replace('G', 'H', $value);

        $value = str_replace('i', 'mm', $value);
        $value = str_replace('s', 'ss', $value);
        $value = str_replace('T', 'z', $value);
        return $value;
    }

    /**
     * Convert php date format to tableSorter date format
     *
     * @param string $dateFormat PHP date format
     *
     * @return string
     */
    public static function convertDateFormat($dateFormat)
    {
        $dateFormat = strtolower($dateFormat);
        $d1 = strpos($dateFormat, 'd');
        $m1 = strpos($dateFormat, 'm');
        $y1 = strpos($dateFormat, 'y');

        $tempArr = array();
        if ($d1 !== false) {
            $tempArr[$d1] = 'dd';
        }
        if ($m1 !== false) {
            $tempArr[$m1] = 'mm';
        }
        if ($y1 !== false) {
            $tempArr[$y1] = 'yyyy';
        }

        ksort($tempArr);
        return implode('', array_values($tempArr));
    }

    /**
     * Function replace ';' to ',' of cell value in calculate cell
     *
     * @param array  $datas   Data cell
     * @param string $maxRows Max number row
     *
     * @return mixed
     */
    public static function renderValueCalculateCell($datas, $maxRows)
    {
        for ($i = 0; $i < $maxRows; $i++) {
            $datas[$i] = array_map(
                function ($data) {
                    $data = (string)$data;
                    $key = substr($data, 0, 1);
                    $pattern = '@^=(DATE|DAY|DAYS|DAYS360|AND|OR|XOR|SUM|COUNT|MIN|MAX|AVG|CONCAT|date|day|days|days360|and|or|xor|sum|count|min|max|avg|concat)\\((.*?)\\)$@';
                    if ($key === '=' && preg_match($pattern, $data, $matches)) {
                        $data = str_replace(';', ',', $data);
                    }

                    return $data;
                },
                $datas[$i]
            );
        }

        return $datas;
    }

    /**
     * Convert number to string
     *
     * @param number $num Number
     *
     * @return string
     */
    public static function getNameFromNumber($num)
    {
        $numeric = $num % 26;
        $letter = chr(65 + $numeric);
        $num2 = intval($num / 26);
        if ($num2 > 0) {
            return self::getNameFromNumber($num2 - 1) . $letter;
        } else {
            return $letter;
        }
    }

    /**
     * Convert number by format
     *
     * @param string  $inputData      Input value
     * @param string  $decimalSymbol  Input decimal symbol
     * @param integer $decimalCount   Input decimal count
     * @param string  $thousandSymbol Input thousand symbol
     *
     * @return string
     */
    public static function convertNumberByFormat($inputData, $decimalSymbol, $decimalCount = 9, $thousandSymbol = '')
    {
        $format = array(
            'currency_symbol' => self::$currency_symbol_cell !== null ? self::$currency_symbol_cell : self::$currency_symbol,
            'decimal_symbol' => self::$decimal_symbol_cell !== null ? self::$decimal_symbol_cell : self::$decimal_symbol,
            'thousand_symbol' => self::$thousand_symbol_cell !== null ? self::$thousand_symbol_cell : self::$thousand_symbol,
        );
//        $currency_symbol = self::$currency_symbol_cell !== null ? self::$currency_symbol_cell : self::$currency_symbol;
//        $thousand_symbol = self::$thousand_symbol_cell !== null ? self::$thousand_symbol_cell : self::$thousand_symbol;
//        $decimal_symbol = self::$decimal_symbol_cell !== null ? self::$decimal_symbol_cell : self::$decimal_symbol;
        $newData = preg_replace('/[-|0-9|,|\.|' . $format['currency_symbol'] . '| ]/', '', $inputData);
        if ($newData !== '') {//contains strange characters
            $inputData = '';
        } else {
            $charactersRemove = $format['thousand_symbol'] === ' ' ? ($format['decimal_symbol'] === '.' ? ',' : '\.') : $format['thousand_symbol'];
            if ($charactersRemove !== null && $format['currency_symbol'] !== null) {
                $inputData = preg_replace('/[' . $charactersRemove . '|' . $format['currency_symbol'] . '| ]/', '', $inputData);
            } elseif ($format['currency_symbol'] !== null) {
                $inputData = preg_replace('/[' . $format['currency_symbol'] . '| ]/', '', $inputData);
            }

            $decimal_symbol = $format['decimal_symbol'] === '.' ? '\.' : $format['decimal_symbol'];
            if ($decimal_symbol !== null) {
                $inputData = preg_replace('/' . $decimal_symbol . '/', '.', $inputData);
            }

            if ($inputData === '' || $inputData === null) {//if value null
                return $inputData;
            }

            $inputData = number_format($inputData, $decimalCount, $decimalSymbol, $thousandSymbol);
            if (is_nan(floatval($inputData))) {
                $inputData = '';
            }
        }
        return $inputData;
    }

    /**
     * Get and replace a cell value
     *
     * @param array   $datas                    Cells value
     * @param string  $row                      Position row cell
     * @param string  $col                      Position col cell
     * @param integer $formulaGroup             Formula group
     * @param boolean $checkCurrencySymbolExist Check currency symbol exist
     * @param object  $activeSheet              PhpSpreadsheet
     *
     * @return array
     */
    public static function getCell($datas, $row, $col, $formulaGroup, $checkCurrencySymbolExist, $activeSheet)
    {
        $data = '';
        $d    = $datas[$row][self::convertAlpha($col) - 1];
        if ($d !== null) {
            if ($formulaGroup === 1) {//convert value to number
//                $currency_symbol = self::$currency_symbol_cell !== null ? self::$currency_symbol_cell : self::$currency_symbol;
                $checkCurrencySymbolExist = strpos($d, self::$currency_symbol) > -1 ? true : $checkCurrencySymbolExist;
                $d = self::convertNumberByFormat($d, '.', 9, '');
            }
            $activeSheet->getCell($col . ($row + 1))->setValue($d);
            $data = $d;
        } else {
            $data = null;
        }
        return array($data, $checkCurrencySymbolExist);
    }

    /**
     * Multiply calculater
     *
     * @param array  $values     Cell value
     * @param string $value_unit Value unit
     *
     * @return string
     */
    public static function multiplyCalculation($values, $value_unit = '')
    {
        $resultCalc = new stdClass();
        $resultCalc->resultCalc = 1;
        array_map(function ($foo) use ($resultCalc) {
            $foo = preg_replace('/[ ]+/', '', $foo);
            if (is_numeric($foo)) {
                $resultCalc->resultCalc = $resultCalc->resultCalc * $foo;
            }
        }, $values);
        return $resultCalc->resultCalc;
    }

    /**
     * Divide calculater
     *
     * @param array  $values     Cell value
     * @param string $value_unit Value unit
     *
     * @return string
     */
    public static function divideCalculation($values, $value_unit = '')
    {
        $resultCalc = new stdClass();
        $resultCalc->resultCalc = 0;

        $count = count($values);
        if ($count !== 2) {
            return 'NaN';
        }
        $values[1] = preg_replace('/[ ]+/', '', $values[1]);

        if (!is_numeric($values[1]) || (int)$values[1] === 0 || $values[1] === '') {
            return 'NaN';
        }

        $values[0] = preg_replace('/[ |A-Za-z]+/', '', $values[0]);

        if (!is_numeric($values[0])) {
            return 'NaN';
        }

        $resultCalc->resultCalc = $values[0] / $values[1];
        return $resultCalc->resultCalc;
    }


    /**
     * Date calculater function
     *
     * @param string $values       Cell value
     * @param array  $date_formats Date format
     * @param string $number       Number value
     *
     * @return string
     */
    public static function dateCalculation($values, $date_formats, $number)
    {
        $M_name = array('', 'jan', 'feb', 'mar', 'apr', 'may', 'jun', 'jul', 'aug', 'sept', 'oct', 'nov', 'dec');
        $D_name = array('sun', 'mon', 'tue', 'wed', 'thu', 'fri', 'sat');

        $cellHtml = '';
        if (count($values) === 1) {
            preg_match_all('/[a-zA-Z0-9|+|-|\\\]+/', $number, $values);
        }
        $text_date = self::convertDay($values, $date_formats[0], false);
        if ($text_date !== false) {
            $date_string = date_create($text_date);
            if ($date_string !== false) {
                preg_match_all('/[^A-Z|^\\\|^a-z]+/', self::$date_formats, $format_resultCalc);
                $date_string = getdate($date_string->getTimestamp());
                $date        = array();
                $date['D']   = $D_name[$date_string['wday']];
                $date['l']   = $date_string['weekday'];
                $date['j']   = $date_string['mday'];
                $date['d']   = ((int) $date_string['mday'] < 10) ? '0' . $date_string['mday'] : $date_string['mday'];
                $date['F']   = $date_string['month'];
                $date['M']   = $M_name[$date_string['mon']];
                $date['n']   = $date_string['mon'];
                $date['m']   = ((int) $date_string['mon'] < 10) ? '0' . $date_string['mon'] : $date_string['mon'];
                $date['Y']   = $date_string['year'];
                $date['y']   = (int) $date_string['year'] % 100;
                foreach ($date_formats[0] as $date_format => $key) {
                    if (strpos($key, '\\') !== false
                        || in_array($key, array('a', 'A', 'g', 'G', 'h', 'H', 'i', 's', 'T')) !== false
                    ) {
                        $date[$key] = $values[$date_format];
                    }
                    $cellHtml .= '' . $date[$key] . (!empty($format_resultCalc[0][$date_format]) ? $format_resultCalc[0][$date_format] : '');
                }
            } else {
                return 'NaN';
            }
        } else {
            return 'NaN';
        }
        return $cellHtml;
    }

    /**
     * Check valid date with date format
     *
     * @param string $date   Date
     * @param string $format PHP date format
     *
     * @return boolean
     */
    public static function validateDate($date, $format = 'Y-m-d')
    {
        $d = DateTime::createFromFormat($format, $date);
        // The Y ( 4 digits year ) returns TRUE for any integer with any number of digits so changing the comparison from == to === fixes the issue.
        return $d && $d->format($format) === $date;
    }

    /**
     * Calculation for cell
     *
     * @param array   $datas       Cells data array
     * @param string  $valueCell   Cell value
     * @param object  $activeSheet PhpSpreadsheet
     * @param array   $position    Cell position
     * @param boolean $getRawData  Get cell data before convert format
     *
     * @return string|array
     */
    public static function calculaterCell2($datas, $valueCell, $activeSheet, $position, $getRawData = false)
    {
        $value_missing = array();
        $value_missing['col'] = null;
        $value_missing['row'] = null;

        $returnCellValue = '';
        $returnCellValueRaw = '';
        $rangers = explode(';', $valueCell[2]);
        $formula = strtoupper($valueCell[1]);
        $values = array();
        $positionCells = array();
        $checkCurrencySymbolExist = false;

        //check formula function to replace input value
        if (in_array($formula, self::$math_formula) || in_array($formula, self::$and_formula)) {//math calculation ans boolean formula
            $formulaGroup = 1;
        } elseif (in_array($formula, self::$date_formula)) {//date calculation
            $formulaGroup = 2;
        } else {
            $formulaGroup = 3;
        }

        foreach ($rangers as $cell) {
            $vals     = explode(':', $cell);
            if (count($vals) === 1) { //single cell
                preg_match_all('@([a-zA-Z]+)([0-9]+)@', $cell, $val0);
                $data = '';
                if ($val0[0] !== array()) {
                    $count = count($val0[0]);
                    for ($i = 0; $i < $count; $i ++) {
                        if (isset($datas[$val0[2][$i] - 1]) && $datas[$val0[2][$i] - 1][self::convertAlpha(strtoupper($val0[1][$i])) - 1] === null) {
                            $datas[$val0[2][$i] - 1][self::convertAlpha(strtoupper($val0[1][$i])) - 1] = '';
                        }
                        if (!isset($datas[$val0[2][$i] - 1]) || !isset($datas[$val0[2][$i] - 1][self::convertAlpha(strtoupper($val0[1][$i])) - 1])) {//check exist cell
                            $value_missing['col'][] = self::convertAlpha(strtoupper($val0[1][$i])) - 1;
                            $value_missing['row'][] = $val0[2][$i] - 1;
                            continue;
                        }
                        $dataCells = self::getCell($datas, $val0[2][$i] - 1, strtoupper($val0[1][$i]), $formulaGroup, $checkCurrencySymbolExist, $activeSheet);
                        $data = $dataCells[0];
                        $positionCells[] = array($val0[2][$i] - 1, strtoupper($val0[1][$i]));
                        $checkCurrencySymbolExist = $dataCells[1];
                    }
                } else {
                    $data = $cell;
                }
                $values[] = $data;
            } else { //range
                if ($formulaGroup === 2 && $formula !== 'DAY') {
                    return $returnCellValue;
                }
                preg_match('@([a-zA-Z]+)([0-9]+)@', $vals[0], $val1);
                preg_match('@([a-zA-Z]+)([0-9]+)@', $vals[1], $val2);
                if ($val1 !== array() && $val2 !== array()) {
                    $rowRange = array($val1[2] - 1, $val2[2] - 1);

                    $convertVal1 = self::convertAlpha($val1[1]) - 1;
                    $convertVal2 = self::convertAlpha($val2[1]) - 1;
                    $ColRange = array($convertVal1, $convertVal2);

                    for ($i = $rowRange[0]; $i <= $rowRange[1]; $i ++) {
                        for ($j = $ColRange[0]; $j <= $ColRange[1]; $j ++) {
                            if (isset($datas[$i]) && $datas[$i][$j] === null) {
                                $datas[$i][$j] = '';
                            }
                            if (!isset($datas[$i]) || !isset($datas[$i][$j])) {//check exist cell
                                $value_missing['col'][] = $j;
                                $value_missing['row'][] = $i;
                                continue;
                            }

                            $colCell = strtoupper(self::getNameFromNumber($j));
                            $dataCells = self::getCell($datas, $i, $colCell, $formulaGroup, $checkCurrencySymbolExist, $activeSheet);
                            $checkCurrencySymbolExist = $dataCells[1];
                            $values[] = $dataCells[0];
                            $positionCells[] = array($i, $colCell);
                            $data = $dataCells[0];
                        }
                    }
                } else {
                    $values[] = $cell;
                }
            }
        }

        if ($value_missing['row'] !== null) {
            return $value_missing;
        }

        if (in_array($formula, self::$math_formula)) {
            /*math formula*/
            if ($formula === 'MULTIPLY') {//phpspreadsheet not support
                $returnCellValue = self::multiplyCalculation($values);
            } elseif ($formula === 'DIVIDE') {//phpspreadsheet not support
                $returnCellValue = self::divideCalculation($values);
            } else {
                if ($formula === 'AVG') {
                    $activeSheet->setCellValue($position[0] . $position[1], '=AVERAGE(' . $valueCell[2] . ')');
                }

                $returnCellValue = $activeSheet->getCell($position[0] . $position[1])->getCalculatedValue();
            }

            if ($getRawData) {
                $returnCellValueRaw = floatval($returnCellValue);
            }

            if ($returnCellValue !== '') {
//                    var_dump(self::$decimal_count_cell, self::$decimal_symbol_cell, self::$thousand_symbol_cell);
                $returnCellValue = number_format(
                    floatval($returnCellValue),
                    self::$decimal_count_cell !== null ? self::$decimal_count_cell : self::$decimal_count,
                    self::$decimal_symbol_cell !== null ? self::$decimal_symbol_cell : self::$decimal_symbol,
                    self::$thousand_symbol_cell !== null ? self::$thousand_symbol_cell : self::$thousand_symbol
                );
            }

            if ($checkCurrencySymbolExist || (isset(self::$currency_symbol_cell) && self::$currency_symbol_cell !== null)) {
                if (self::$currency_symbol_cell !== null) {
                    $returnCellValue = ((int) self::$symbol_position_cell === 0) ? self::$currency_symbol_cell . ' ' . $returnCellValue : $returnCellValue . ' ' . self::$currency_symbol_cell;
                } else {
                    $returnCellValue = ((int) self::$symbol_position === 0) ? self::$currency_symbol . ' ' . $returnCellValue : $returnCellValue . ' ' . self::$currency_symbol;
                }
            }
        } elseif (in_array($formula, self::$and_formula)) {
            /*boolean formula*/
            $returnCellValue = $activeSheet->getCell($position[0] . $position[1])->getCalculatedValue() ? 'true' : 'false';
        } elseif ($formulaGroup === 2) {
            /*dateTime formula*/
            $activeSheet->getStyle($position[0] . $position[1])->getNumberFormat()->setFormatCode(\PhpOffice\PhpSpreadsheet\Style\NumberFormat::FORMAT_DATE_YYYYMMDD);

            preg_match_all('@([a-zA-Z]+)([0-9]+)@', $valueCell[2], $x1);
            if ($formula === 'day' || $formula === 'DAY') {//phpspreadsheet get getCalculatedValue other dateCalculation, dm phpspreadsheet
                if (!($x1[0] !== array())) {
                    $activeSheet->getCell('Z9999')->setValue($valueCell[2]);
                    $activeSheet->getCell($position[0] . $position[1])->setValue('=DAY(Z9999)');
                }
            }
            if ($formula === 'days' || $formula === 'DAYS' || $formula === 'days360' || $formula === 'DAYS360') {
                if (!($x1[0] !== array())) {
                    $vals = explode(';', $valueCell[2]);
                    $activeSheet->getCell('Z9999')->setValue($vals[0]);
                    $activeSheet->getCell('Z9998')->setValue($vals[1]);
                    $activeSheet->getCell($position[0] . $position[1])->setValue('=DAYS(Z9999,Z9998)');
                }
                $returnCellValue = $activeSheet->getCell($position[0] . $position[1])->getCalculatedValue();
            }

            if ($formula === 'DATE') {
                if (count($values) > 1) {
                    $returnCellValue = join('/', $values);
                } else {
                    $returnCellValue = $values[0];
                }
                return array('date', $returnCellValue);
            } else {
                $returnCellValue = $activeSheet->getCell($position[0] . $position[1])->getCalculatedValue();
                if ($formula === 'days' || $formula === 'DAYS'
                    || $formula === 'day' || $formula === 'DAY'
                    || $formula === 'days360' || $formula === 'DAYS360') {
                    return array($formula, $returnCellValue);
                }
            }
        } elseif ($formulaGroup === 3) {
            /*CONCAT formula*/
            $returnCellValue = $activeSheet->getCell($position[0] . $position[1])->getCalculatedValue();
        }

        if ($getRawData) {
            return array($returnCellValue, $returnCellValueRaw);
        }
        return $returnCellValue;
    }

    /**
     * Create table format variable
     *
     * @param object     $tblStyles    Table style data
     * @param null|array $configParams Config params
     *
     * @return void
     */
    public static function setup($tblStyles, $configParams = null)
    {
        if ($configParams === null) {
            $configParams = JComponentHelper::getParams('com_droptables');
        }

        if (!isset(self::$date_formats) || self::$date_formats === '') {
            self::$date_formats    = (!empty($configParams['date_formats'])) ? $configParams['date_formats'] : 'Y-m-d';
            self::$date_formats    = (!empty($tblStyles->date_formats)) ? $tblStyles->date_formats : self::$date_formats;
        }
        if (!isset(self::$symbol_position) || self::$symbol_position === '') {
            self::$symbol_position = (!empty($configParams['symbol_position'])) ? $configParams['symbol_position'] : 0;
            self::$symbol_position = (!empty($tblStyles->symbol_position)) ? $tblStyles->symbol_position : self::$symbol_position;
        }
        if (!isset(self::$currency_symbol) || self::$currency_symbol === '') {
            self::$currency_symbol = (!empty($configParams['currency_sym'])) ? $configParams['currency_sym'] : '$';
            self::$currency_symbol = (!empty($tblStyles->currency_symbol)) ? $tblStyles->currency_symbol : self::$currency_symbol;
        }
        if (!isset(self::$decimal_symbol) || self::$decimal_symbol === '') {
            self::$decimal_symbol  = (!empty($configParams['decimal_sym'])) ? $configParams['decimal_sym'] : '.';
            self::$decimal_symbol  = (!empty($tblStyles->decimal_symbol)) ? $tblStyles->decimal_symbol : self::$decimal_symbol;
        }
        if (!isset(self::$decimal_count) || self::$decimal_count === '') {
            self::$decimal_count   = (!empty($configParams['decimal_count'])) ? $configParams['decimal_count'] : 0;
            self::$decimal_count   = (isset($tblStyles->decimal_count)) ? $tblStyles->decimal_count : self::$decimal_count;
        }
        if (!isset(self::$thousand_symbol) || self::$thousand_symbol === '') {
            self::$thousand_symbol = (!empty($configParams['thousand_sym'])) ? $configParams['thousand_sym'] : ',';
            self::$thousand_symbol = isset($tblStyles->thousand_symbol) ? $tblStyles->thousand_symbol : self::$thousand_symbol;
        }
    }

    /**
     * Render html
     *
     * @param object  $table        Table data
     * @param array   $configParams Config param data
     * @param array   $dataStyle    Data style(tooltip, ...)
     * @param string  $table_hash   Hash string
     * @param boolean $getData      Get data or build header only. Default true
     *
     * @return boolean
     */
    public static function htmlRender($table, $configParams, $dataStyle, $table_hash, $getData = true)
    {
        jimport('joomla.filesystem.folder');
        jimport('joomla.filesystem.file');

        $folder = JPATH_SITE . DIRECTORY_SEPARATOR . 'media' . DIRECTORY_SEPARATOR . 'droptables' . DIRECTORY_SEPARATOR;
        if (!file_exists($folder)) {
            JFolder::create($folder);
            $data = '<html><body bgcolor="#FFFFFF"></body></html>';
            JFile::write($folder . 'index.html', $data);
        }

        $file = $folder . JFile::makeSafe($table->id . '_' . $table->hash . '.html');
        $tblStyles = $table->style;
        if (!isset($tblStyles->table)) {
            $tblStyles->table = new stdClass();
        }

        $datas = is_string($table->data) ? json_decode($table->data) : $table->data;
        $style = is_string($table->style) ? json_decode($table->style) : $table->style;

        if (!file_exists($file)) {
            $files = glob($folder . $table->id . '_*.html');
            foreach ($files as $f) {
                unlink($f);
            }
        } else {
            // sync google sheet automatically on front-end
            return true;
        }

        //merge cells
        if (isset($tblStyles->table->mergeSetting)) {
            $mergeSetting = is_string($tblStyles->table->mergeSetting) ? json_decode($tblStyles->table->mergeSetting, true) : $tblStyles->table->mergeSetting;

            if (is_array($mergeSetting)) {
                $count = count($mergeSetting);
                if ($count > 0) {
                    foreach ($mergeSetting as $key => $ms) {
                        if (is_object($ms)) {
                            $merge = (array)$ms;
                            $mergeSettings[$key] = $merge;
                        } else {
                            $merge = $ms;
                        }

                        if (isset($tblStyles->table->headerOption) && $tblStyles->table->headerOption > $merge[0] && $tblStyles->table->headerOption <= ($merge[0] + $merge[1])) {
                            $tblStyles->table->headerOption = $merge[0] + $merge[1];
                        }
                    }
                }
            }
        } else {
            $mergeSetting = array();
        }

        //header
        if (!isset($tblStyles->table->headerOption) || (int)$tblStyles->table->headerOption < 1) {//header not exist
            $heads = 1;
            if (isset($datas[0])) {
                $header_data = array($datas[0]);
            } elseif (!empty($style->header_data)) {
                $header_data = $style->header_data;
            } else {
                $header_data = null;
            }
        } else {
            $heads = (int)$tblStyles->table->headerOption;
            if (isset($style->header_data[(int)$tblStyles->table->headerOption - 1])) {
                $header_data = $style->header_data;
            } else {//header_data not exist
                $header_data = array();
                for ($header_number = 0; $header_number < $tblStyles->table->headerOption; $header_number++) {
                    $header_data[$header_number] = $datas[$header_number];
                }
            }
        }

        if (!$getData && is_null($datas)) {
            $datas = $header_data;
        }

        $count_Col = 0;
        $colWidths = array();
        $hide_column = array();
        $encode_hide_column = '';
        $column_hiden = false;
        $columnSortType = array();
        $columnSortFormat = array();
        if (isset($tblStyles->cols)) {
            foreach ($tblStyles->cols as $col) {
                if (!empty($col)) {
                    $firstCol = is_object($col) ? $col->{1} : $col[1];
                    $zeroCol = is_object($col) ? (string)$col->{0} : (string)$col[0];
                    $hide_column[] = !empty($firstCol->hide_column) ? (int)$firstCol->hide_column : 0;
                    if (isset($firstCol->width) && $hide_column[$count_Col] !== 1) {
                        $colWidths[] = $firstCol->width;
                    } else {
                        $column_hiden = true;
                    }

                    if (isset($firstCol->sort_type)) {
                        $columnSortType[] = $firstCol->sort_type;
                    } else {
                        $columnSortType[] = '';
                    }
                    if (isset($firstCol->sort_format)) {
                        $columnSortFormat[] = $firstCol->sort_format !== '' ? self::momentjsFormat($firstCol->sort_format) : '';
                    } else {
                        $columnSortFormat[] = '';
                    }

                    $count_Col++;
                }
            }
        }


        //phpcs:ignore PHPCompatibility.Constants.NewConstants.ent_html401Found -- Not support php version 5.4 or earlier
        $encodeSortFormat = htmlspecialchars(json_encode($columnSortFormat), ENT_COMPAT | ENT_HTML401, 'UTF-8');
        //phpcs:ignore PHPCompatibility.Constants.NewConstants.ent_html401Found -- Not support php version 5.4 or earlier
        $encodeSortType = htmlspecialchars(json_encode($columnSortType), ENT_COMPAT | ENT_HTML401, 'UTF-8');

        //phpcs:ignore PHPCompatibility.Constants.NewConstants.ent_html401Found -- Not support php version 5.4 or earlier
        $encodeColWidths = htmlspecialchars(json_encode($colWidths), ENT_COMPAT | ENT_HTML401, 'UTF-8');
        //phpcs:ignore PHPCompatibility.Constants.NewConstants.ent_html401Found -- Not support php version 5.4 or earlier
        $encode_hide_column = htmlspecialchars(json_encode($hide_column), ENT_COMPAT | ENT_HTML401, 'UTF-8');

        if (isset($tblStyles->table->responsive_type)) {
            if ((string)$tblStyles->table->responsive_type === 'hideCols') {
                $responsive_type = 'hideCols';
            } elseif ((string)$tblStyles->table->responsive_type === 'scroll') {
                $responsive_type = 'scroll';
            } else {
                $responsive_type = 'repeatedHeader';
            }
        } else {
            $responsive_type = 'scroll';
        }
        if (!isset($tblStyles->table->freeze_col)) {
            $tblStyles->table->freeze_col = 0;
        }
        if (!isset($tblStyles->table->freeze_row)) {
            $tblStyles->table->freeze_row = 0;
        }

        //filters and sort
        if (!isset($tblStyles->table->enable_filters)) {
            $tblStyles->table->enable_filters = 0;
        }
        if (isset($tblStyles->table->use_sortable) && (int) $tblStyles->table->use_sortable === 1) {
            $sortable = true;
        } else {
            $sortable = false;
        }
        $default_order_sortable = isset($tblStyles->table->default_order_sortable) ? (int)$tblStyles->table->default_order_sortable : 0;
        $default_sort = isset($tblStyles->table->default_sortable) ? (int)$tblStyles->table->default_sortable : 0;

        if ($column_hiden) {
            $default_sort = 0;
        }

        //pagination
        $limit = isset($tblStyles->table->limit_rows) ? (int)$tblStyles->table->limit_rows : 0;
        $enable_pagination = isset($tblStyles->table->enable_pagination) ? (int)$tblStyles->table->enable_pagination : 0;

        //convert to tableshorter dateformat
        $dateFormat = isset($tblStyles->table->date_formats) ? self::momentjsFormat($tblStyles->table->date_formats) : 'mm/dd/yyyy';

        //hyperlink cells
        if (isset($tblStyles->table->hyperlink) && is_string($tblStyles->table->hyperlink)) {
            $tableHyperlink = json_decode($tblStyles->table->hyperlink);
        } elseif (!isset($tblStyles->table->hyperlink)) {
            $tableHyperlink = new stdClass();
        } else {
            $tableHyperlink = $tblStyles->table->hyperlink;
        }

//        var_dump($tableHyperlink);

        $tblCls = ($sortable ? 'sortable' : '');
        $tableDataAttr = array();

        $res_prioritys = array();
        if ($responsive_type === 'hideCols') {
            foreach ($tblStyles->cols as $col) {
                if (is_object($col)) {
                    $firstCol = $col->{1};
                    $colZero = $col->{0};
                } else {
                    $firstCol = $col[1];
                    $colZero = $col[0];
                }
                if (isset($firstCol->res_priority)) {
                    $res_prioritys[(string) $colZero] = ((string)$firstCol->res_priority === 'persistent') ? 'persistent' : (int) $firstCol->res_priority;
                }
            }
            $tableDataAttr[] = 'data-responsive="true"';
            $tableDataAttr[] = 'data-hideCols="true"';
            $tableDataAttr[] = 'data-hideColslanguage="Columns"';
        } else {
            $tableDataAttr[] = 'data-responsive="false"';
            $tableDataAttr[] = 'data-hideCols="false"';

            if (($tblStyles->table->freeze_row) || ($tblStyles->table->freeze_col)) {
                $tblCls .= ' fxdHdrCol';
            }

            if ($responsive_type === 'repeatedHeader') {
                if (isset($tblStyles->table->style_repeated) && $tblStyles->table->style_repeated > 0) {
                    $tblCls .= ' repeatedHeader style_repeated';
                } else {
                    $tblCls .= ' repeatedHeader';
                }
            }
        }

        //check table no body
        $countData = count($datas);
        if ($countData < 2) {
            $content = '<div class="droptablesOneRow droptablesresponsive dataTables-droptables droptablestable ' . (isset($tblStyles->table->class_name) ? $tblStyles->table->class_name : '') . '" id="droptablestable' . (int) $table->id . '" data-tableid="droptableTbl' . (int) $table->id . '">';
        } else {
            $content = '<div class="droptablesresponsive dataTables-droptables droptablestable ' . (isset($tblStyles->table->class_name) ? $tblStyles->table->class_name : '') . '" id="droptablestable' . (int) $table->id . '" data-tableid="droptableTbl' . (int) $table->id . '">';
        }

        // Table type
        if ($table->type === 'html') {
            $tableDataAttr[] = 'data-type="html"';
        } else {
            $tableDataAttr[] = 'data-type="mysql"';
        }
        //pagination
        if (!$enable_pagination || !$limit) {
            $tblCls .= ' disablePager';
        }
        if ($enable_pagination && $limit) {
            $tableDataAttr[] = 'data-paging="true"';
            $tableDataAttr[] = 'data-page-length="' . $limit . '"';
        } else {
            $tableDataAttr[] = 'data-paging="false"';
        }

        //searching
        if ($tblStyles->table->enable_filters) {
            $tblCls .= ' filterable';
        } elseif ($enable_pagination && $limit) {
            $tblCls .= ' enablePager';
        }

        $tableDataAttr[] = ($tblStyles->table->enable_filters) ? 'data-searching="true"' : 'data-searching="false"';

        $tableDataAttr[] = $sortable ? 'data-ordering="true"' : 'data-ordering="false"';
        if ($sortable) {
            $default_order_sortable_str = ($default_order_sortable === 1) ? 'asc' : 'desc';
            $tableDataAttr[] = 'data-ordertarget="'.$default_sort.'"';
            $tableDataAttr[] = 'data-ordervalue="'.$default_order_sortable_str.'"';
        }

        if (!isset($tblStyles->table->table_height)) {
            $tblStyles->table->table_height = 0;
        }

        if (!isset($tblStyles->table->table_breakpoint)) {
            $tblStyles->table->table_breakpoint = 980;
        }

        $tableDataAttr[] = 'data-table-breakpoint="'.$tblStyles->table->table_breakpoint.'"';

        //scrolling
        if ($responsive_type === 'scroll' || $responsive_type === 'repeatedHeader') {
            if ((int)$tblStyles->table->table_height !== 0 && $tblStyles->table->table_height !== '') {
                $table_height = $tblStyles->table->table_height . 'px';
            } else {
                $table_height = 'auto';
            }
            $tableDataAttr[] = 'data-scroll-x="true"';
            $tableDataAttr[] = 'data-scroll-collapse="true"';
            if ($tblStyles->table->freeze_col && $responsive_type !== 'repeatedHeader') {
                //freeze column
                $tableDataAttr[] = 'data-freezecol="'.$tblStyles->table->freeze_col.'"';
            }

            $fix_height = '';
            if ($responsive_type === 'repeatedHeader') {
                $fix_height = '#droptablestable' . (int) $table->id . ' table.repeatedHeaderTrue colgroup col[data-dtr="0"] {width: 0 !important;min-width: unset;}';
            }

            if ((int)$tblStyles->table->freeze_row > 0) {
                if (isset($table_height)) {
                    $tableDataAttr[] = 'data-scroll-y="' . $table_height . '"';
                }
                //freeze row
                $tableDataAttr[] = 'data-freezerow="true"';
            } elseif ($table_height !== 'auto') {
                //fix height when not freeze row
                $fix_height .= '#droptablestable' . (int) $table->id . ' .dataTables_wrapper > .dataTables_scroll {max-height:' . $table_height .';overflow: auto;}';
                if ($tblStyles->table->freeze_col) {
                    $fix_height .= '#droptablestable' . (int) $table->id . ' .dataTables_wrapper > .DTFC_ScrollWrapper {max-height:' . $table_height .';overflow: auto;}';
                }
            } elseif ($table_height === 'auto') {
                //fix height when not freeze row
                $fix_height .= '#droptablestable' . (int) $table->id . ' .dataTables_wrapper > .dataTables_scroll {max-height: unset;overflow: auto;}';
                if ($tblStyles->table->freeze_col) {
                    $fix_height .= '#droptablestable' . (int) $table->id . ' .dataTables_wrapper > .DTFC_ScrollWrapper {max-height: unset;overflow: auto;}';
                }
            }
        }

        $table_align = !isset($tblStyles->table->table_align) ? 'left' : $tblStyles->table->table_align;
        $tableDataAttr[] = 'data-align="'.$table_align.'"';

        if ($datas !== null && count($datas) === $heads) {
            $tableDataAttr[] = 'data-tbody="0"';
        } else {
            $tableDataAttr[] = 'data-tbody="1"';
        }

        $tableDataAttrStr = implode(' ', $tableDataAttr);
        $content .= '<table data-format="' . $dateFormat . '" data-sort-format="' . $encodeSortFormat  . '" data-sort-type="' . $encodeSortType  . '" id="droptableTbl' . (int)$table->id . '" data-id="' . $table->id . '" '.$tableDataAttrStr.' data-hideColumn="' . $encode_hide_column . '" data-colwidths="' . $encodeColWidths . '" class="' . $tblCls . '">';

        $rowNb = 0;
        $limit_rows = 0;
        self::setup($tblStyles->table, $configParams);

        // Fix Col width issue for Mac
        $firstRow = $datas[0];
        $content .= '<colgroup>';
        $countFirstRow = count($firstRow);

        for ($colNb = 0; $colNb < $countFirstRow; $colNb ++) {
            if (!(isset($hide_column[$colNb]) && $hide_column[$colNb] === 1)) {
                $content .= '<col data-dtr="' . $rowNb . '" data-dtc="' . $colNb . '" class="dtc' . $colNb . '">';
            }
        }
        $content .= '</colgroup>';
        $tfoot = '';
        $maxColNb = 0;

        if ($datas !== null) {
            require_once(JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR . 'com_droptables' . DIRECTORY_SEPARATOR . 'classes' . DIRECTORY_SEPARATOR . 'phpspreadsheet' . DIRECTORY_SEPARATOR . 'vendor' . DIRECTORY_SEPARATOR . 'autoload.php');
            $spreadsheet = new \PhpOffice\PhpSpreadsheet\Spreadsheet();
            $activeSheet = $spreadsheet->createSheet(1);
            $maxRows = count($datas);
            $activeSheet->fromArray(self::renderValueCalculateCell($datas, $maxRows), null, 'A1');

            /*convert Separator for cells*/
            foreach ($datas as $krow => $row) {
                if (!empty($tblStyles->table->limit_rows)) {
                    $limit_rows ++;
                }

                if ($rowNb === 0) {
                    $content .= '<thead>';
                } elseif ($rowNb !== 0 && $rowNb === $heads) {
                    $content .= '<tbody>';
                }

                if ($rowNb === 0) {
                    $content .= '<tr class="droptables-header-cells-index dnone">';
                    $headerColNb = 0;
                    foreach ($row as $index_column_header => $col_value) {
                        if (!(isset($hide_column[$index_column_header]) && $hide_column[$index_column_header] === 1)) {
                            $content .= '<th rowspan="1" colspan="1" data-dtr="' . $rowNb . '" data-dtc="' . $headerColNb . '" class="dtr' . $rowNb . ' dtc' . $headerColNb . '">';
                            $content .= '';
                            $content .= '</th>';
                        }
                        $headerColNb++;
                    }
                    $content .= '</tr>';
                }

                // when have pagination---> freeze_row = limit_rows (joomla)
                if ($limit_rows <= $tblStyles->table->freeze_row) {
                    $content .= '<tr class=" row' . $rowNb . '">';
                } else {
                    if ($rowNb === $maxRows - 1) {
                        $content .= '<tr class="droptable_none footer_class row' . $rowNb . '">';
                    } else {
                        $content .= '<tr class="droptable_none row' . $rowNb . '">';
                    }
                }
                $colNb = 0;
                foreach ($row as $k => $col) {
                    $dataSortAttr = '';
                    $cellHtml = '';
                    if ($heads > $rowNb && $header_data !== null) {//in header
                        $col = $header_data[$rowNb][$k];
                    }

                    if (isset($tblStyles->cells[$rowNb . '!' . $colNb])) {
                        $tblStyle = $tblStyles->cells[$rowNb . '!' . $colNb][2];
                    } else {
                        $tblStyle = array();
                    }

                    if ((isset($tblStyle['decimal_count']) && $tblStyle['decimal_count'] !== false)
                        || (isset($tblStyle['decimal_symbol']) && $tblStyle['decimal_symbol'] !== false)
                        || (isset($tblStyle['currency_symbol']) && $tblStyle['currency_symbol'] !== false)) {
                        self::$thousand_symbol_cell = (isset($tblStyle['thousand_symbol']) && $tblStyle['thousand_symbol'] !== false)
                            ? $tblStyle['thousand_symbol'] : ((isset($tblStyle['thousand_symbol_second']) && $tblStyle['thousand_symbol_second'] !== false)
                                ? $tblStyle['thousand_symbol_second'] : self::$thousand_symbol);
                        self::$decimal_count_cell = (isset($tblStyle['decimal_count']) && $tblStyle['decimal_count'] !== false)
                            ? $tblStyle['decimal_count'] : ((isset($tblStyle['decimal_count_second']) && $tblStyle['decimal_count_second'] !== false)
                                ? $tblStyle['decimal_count_second'] : self::$decimal_count);
                        self::$decimal_symbol_cell = (isset($tblStyle['decimal_symbol']) && $tblStyle['decimal_symbol'] !== false)
                            ? $tblStyle['decimal_symbol'] : ((isset($tblStyle['decimal_symbol_second']) && $tblStyle['decimal_symbol_second'] !== false)
                                ? $tblStyle['decimal_symbol_second'] : self::$decimal_symbol);
                        self::$currency_symbol_cell = (isset($tblStyle['currency_symbol']) && $tblStyle['currency_symbol'] !== false)
                            ? $tblStyle['currency_symbol'] : ((isset($tblStyle['currency_symbol_second']) && $tblStyle['currency_symbol_second'] !== false)
                                ? $tblStyle['currency_symbol_second'] : self::$currency_symbol);
                        self::$symbol_position_cell = (isset($tblStyle['symbol_position']) && $tblStyle['symbol_position'] !== false)
                            ? $tblStyle['symbol_position'] : ((isset($tblStyle['symbol_position_second']) && $tblStyle['symbol_position_second'] !== false)
                                ? $tblStyle['symbol_position_second'] : self::$symbol_position);
                        $has_format_cell = true;
                    } else {
                        self::$thousand_symbol_cell = null;
                        self::$decimal_count_cell = null;
                        self::$decimal_symbol_cell = null;
                        self::$currency_symbol_cell = null;
                        self::$symbol_position_cell = null;
                        $has_format_cell = false;
                    }

                    if (isset($tblStyle['date_formats_momentjs']) && $tblStyle['date_formats_momentjs'] !== false) {
                        $has_format_date_cell = 'data-format="' . $tblStyle['date_formats_momentjs'] . '"';
                    } else {
                        $has_format_date_cell = '0';
                    }

                    $position = array();
                    $position[] = self::getNameFromNumber($k);
                    $position[] = $krow + 1;
                    if (isset($col[0]) && $col[0] === '=') {
                        if (preg_match('@^=(DATE|DAY|DAYS|DAYS360|AND|OR|XOR|SUM|MULTIPLY|DIVIDE|COUNT|MIN|MAX|AVG|CONCAT|date|day|days|days360|and|or|xor|sum|multiply|divide|count|min|max|avg|concat)\\((.*?)\\)$@', $col, $matches)) {
                            $formula = strtoupper($matches[1]);
                            //check formula function to replace input value
                            if (in_array($formula, self::$math_formula)) {
                                self::$decimal_count_cell = (isset($tblStyle['decimal_count_second']) && $tblStyle['decimal_count_second'] !== false) ? $tblStyle['decimal_count_second'] : self::$decimal_count_cell;
                                self::$decimal_symbol_cell = (isset($tblStyle['decimal_symbol_second']) && $tblStyle['decimal_symbol_second'] !== false) ? $tblStyle['decimal_symbol_second'] : self::$decimal_symbol_cell;
                                self::$thousand_symbol_cell = (isset($tblStyle['thousand_symbol_second']) && $tblStyle['thousand_symbol_second'] !== false) ? $tblStyle['thousand_symbol_second'] : self::$thousand_symbol_cell;
                                self::$currency_symbol_cell = (isset($tblStyle['currency_symbol_second']) && $tblStyle['currency_symbol_second'] !== false) ? $tblStyle['currency_symbol_second'] : self::$currency_symbol_cell;
                                self::$symbol_position_cell = (isset($tblStyle['symbol_position_second']) && $tblStyle['symbol_position_second'] !== false) ? $tblStyle['symbol_position_second'] : self::$symbol_position_cell;
                            }
                            $calculaterCell = self::calculaterCell2($datas, $matches, $activeSheet, $position);
                        } else {
                            $calculaterCell = $col;
                        }
                        if (is_array($calculaterCell)) {
                            if (strtoupper($calculaterCell[0]) === 'DATE') {
                                if ($has_format_date_cell === '0') {
                                    $has_format_date_cell = ' data-format="1" ';
                                }
                                $calculaterCell = $calculaterCell[1];
                            } elseif (in_array(strtoupper($calculaterCell[0]), array('DAYS360', 'DAYS', 'DAY')) !== false) {
                                $calculaterCell = $calculaterCell[1];
                                $has_format_date_cell = '0';
                            } else {
                                $calculaterCell = 'NaN';
                            }
                        }
                        $cellHtml .= $calculaterCell;
                    } elseif (isset($tblStyle)//html cell
                        && isset($tblStyle['cell_type'])
                        && (string)$tblStyle['cell_type'] === 'html') {
                        if (isset($tableHyperlink->{$rowNb . '!' . $colNb})) {
                            $checkHtml = strpos($col, $tableHyperlink->{$rowNb . '!' . $colNb}->hyperlink);
                        } else {
                            $checkHtml = false;
                        }
                        if (isset($tableHyperlink->{$rowNb . '!' . $colNb}) && $checkHtml === false) {
                            $cellHtml .= '<a target="_blank" href="' . $tableHyperlink->{$rowNb . '!' . $colNb}->hyperlink . '">' . $col . '</a>';
                        } else {
                            $cellHtml .= $col;
                        }
                    } else {//default cell
                        if ($has_format_date_cell !== '0') {
                            $col = $activeSheet->getCell($position[0] . $position[1])->getValue();
                        } elseif ($has_format_cell) {
                            $col1 = preg_replace('/[-|0-9|,|\.|' . self::$currency_symbol_cell . '| ]/', '', $col);

                            $col = preg_replace('/[' . self::$thousand_symbol_cell . '| ]/', '', $col);
                            $col = preg_replace('/[' . self::$decimal_symbol_cell . '| ]/', '.', $col);
                            if ($col1 === '') {
                                $col = preg_replace('/[' . self::$currency_symbol_cell . '| ]/', '', $col);
                                $col = number_format(floatval($col), self::$decimal_count_cell, self::$decimal_symbol_cell, self::$thousand_symbol_cell);
                            }
                            if (isset($tblStyle['currency_symbol']) && $tblStyle['currency_symbol'] !== '' && $col1 === '') {
                                $col = ((int) self::$symbol_position_cell === 0) ? self::$currency_symbol_cell . ' ' . $col : $col . ' ' . self::$currency_symbol_cell;
                            }
                        }
                        $cellHtml .= nl2br(($col || $col === '0') ? $col : '');
                    }

                    //fix wrap text
                    if (isset($tblStyle)
                        && isset($tblStyle['text_wrapping'])
                        && ((string)$tblStyle['text_wrapping'] === 'normal' || (string)$tblStyle['text_wrapping'] === 'hidden')) {
                        if (isset($tblStyle['cell_type']) && (string)$tblStyle['cell_type'] === 'html') {
                            $cellHtml = '<div>' . $cellHtml . '</div>';
                        } else {
                            $cellHtml = '<p>' . $cellHtml . '</p>';
                        }
                    }

                    $isDateFormat = self::validateDate($col ? $col : '', self::$date_formats);
                    if ($isDateFormat) {
                        $newDate = DateTime::createFromFormat(self::$date_formats, $col);
                        $dataSortAttr = 'data-sort="'.$newDate->getTimestamp().'"';
                    }

                    $mergeInfoAndCheckMerge = $rowNb >= $heads ? self::checkMergeInfo($rowNb, $colNb, $mergeSetting, false) : self::checkMergeInfo($rowNb, $colNb, $mergeSetting, true);
                    $mergeInfo              = $mergeInfoAndCheckMerge[0];
                    $mergeSetting           = $mergeInfoAndCheckMerge[1];
                    if ($mergeInfo !== false && ((isset($hide_column[$colNb]) && $hide_column[$colNb] === 1 && $mergeInfo !== '') || !(isset($hide_column[$colNb]) && $hide_column[$colNb] === 1))) {//fix MergerHeader And HidenCol merger[0]
//                    if ($mergeInfo !== false && !(isset($hide_column[$colNb]) && $hide_column[$colNb] === 1)) {
                        if ($rowNb >= $heads) {
                            $content .= '<td ' . $mergeInfo . ' ' . $has_format_date_cell . ' ' . $dataSortAttr .' class="dtr' . $rowNb . ' dtc' . $colNb . '">';
                        } else {
                            $dataPriority = !empty($res_prioritys) && isset($res_prioritys[$colNb]) ? 'data-priority="'.$res_prioritys[$colNb].'"' : 'data-priority="0"';
                            $content   .= '<th ' . $mergeInfo . ' ' . $has_format_date_cell . ' '.$dataPriority.' class="dtr' . $rowNb . ' dtc' . $colNb . '">';
                        }
                        if (isset($tblStyles->cells[$rowNb . '!' . $colNb]) && isset($tblStyle['tooltip_content']) && (string) $tblStyle['tooltip_content'] !== '') {
                            if (!empty($dataStyle['tooltip'][$rowNb . '_' . $colNb])) {
                                $content .= '<span class="droptables_tooltip ">' . $cellHtml . '<span class="droptables_tooltipcontent" data-width="' . $dataStyle['tooltip'][$rowNb . '_' . $colNb] . '">' . $tblStyle['tooltip_content'] . '</span></span>';
                            } else {
                                $content .= '<span class="droptables_tooltip ">' . $cellHtml . '<span class="droptables_tooltipcontent">' . $tblStyle['tooltip_content'] . '</span></span>';
                            }
                        } else {
                            $content .= $cellHtml;
                        }

                        if ($rowNb < $heads) {
                            $content .= '</th>';
                        } else {
                            $content .= '</td>';
                        }
                    }
                    $colNb ++;
                }

                $maxColNb = $maxColNb > $colNb ? $maxColNb : $colNb;

                if ($rowNb === $heads - 1) {
                    $content .= '</thead>';
                } else {
                    $content .= '</tr>';
                }

                $rowNb ++;
            }

            if (!$enable_pagination) {//fix merger cells in row end
                for ($kTfoot = 0; $kTfoot < $maxColNb; $kTfoot++) {
                    if (!(isset($hide_column[$kTfoot]) && $hide_column[$kTfoot] === 1)) {
                        $tfoot .= '<td class="dtr' . ($rowNb - 1) . ' dtc' . $kTfoot . '" style="height: 0;"></td>';
                    }
                }
            }
        }

        $content .= '</tbody>';

        $content .= '<tfoot>';//fix merger cells in row end
        $content .= $tfoot;
        $content .= '</tfoot>';
        $content .= '</table></div>';

        if (isset($fix_height)) {
            $content .= '<style>' . $fix_height . '</style>';
        }

        // Process the content plugins.
        JPluginHelper::importPlugin('content');
//        $dispatcher = JEventDispatcher::getInstance();
        $item = new stdClass();
        $item->text = $content;
        $item->id = 1;
        $tableParams = new JObject;
        Factory::getApplication()->triggerEvent('onContentPrepare', array('com_content.article', &$item, &$tableParams, 0));
//        $dispatcher->trigger('onContentPrepare', array('com_content.article', &$item, &$tableParams, 0));

        $content = $item->text;

        try {
            if ($table->type === 'html') {
                if (!file_put_contents($file, htmlspecialchars($content, ENT_NOQUOTES))) {//phpcs:ignore PHPCompatibility.ParameterValues.NewHTMLEntitiesEncodingDefault.NotSet -- Not support php 5.4
                    echo 'error saving file!';
                    return false;
                }
            } else {
                return htmlspecialchars($content, ENT_NOQUOTES);//phpcs:ignore PHPCompatibility.ParameterValues.NewHTMLEntitiesEncodingDefault.NotSet -- Not support php 5.4
            }
        } catch (Exception $exc) {
            echo 'error saving file!';
            return false;
        }
        return true;
    }

    /**
     * Build pager
     *
     * @param string  $colNb    Col number
     * @param integer $pageSize Table page number
     * @param integer $total    Total
     *
     * @return string
     */
    public static function buildPager($colNb, $pageSize, $total)
    {
        $content = '<tr>';
        $content .= '<td colspan="' . $colNb . '" class="pager form-horizontal" >';
        $content .= '<button type="button" class="btn first">
                            <i class="icon-step-backward glyphicon glyphicon-step-backward"></i>
                         </button>
                         <button type="button" class="btn prev">
                            <i class="icon-arrow-left glyphicon glyphicon-backward"></i>
                         </button>	
                         <span class="pagedisplay">1 to ' . $pageSize . ' (' . $total . ')</span></span> 
                         <!-- this can be any element, including an input -->
                         <button type="button" class="btn next">
                            <i class="icon-arrow-right glyphicon glyphicon-forward"></i>
                         </button>
                         <button type="button" class="btn last">
                            <i class="icon-step-forward glyphicon glyphicon-step-forward"></i>
                         </button>
                         <select class="pagesize input-mini" title="Select page size">';
        $options = array(10, 20, 40, 9999);
        foreach ($options as $v) {
            $content .= '<option value="' . $v . '" ';
            if ($v < 1000) {
                if ((int)$pageSize === $v) {
                    $content .= 'selected="selected" ';
                }
                $content .= '>' . $v . '</option>';
            } else {
                if ($pageSize === 0) {
                    $content .= 'selected="selected" ';
                }
                $content .= '>All</option>';
            }
        }
        $content .= '</select>';
        $content .= '<select class="pagenum input-mini" title="Select page number">';
        if ($pageSize > 0) {
            $max = floor($total / $pageSize);
            for ($i = 1; $i <= $max; $i++) {
                $content .= '<option value="' . $i . '">' . $i . '</option>';
            }
        }
        $content .= '</select>';
        $content .= '</td></tr>';
        return $content;
    }

    //function from http://www.bradino.com/php/excel-column-convert-letters-to-numbers/

    /**
     * Get val cell
     *
     * @param string $col Position of cell
     *
     * @return integer
     */
    private static function convertAlpha($col)
    {
        $col = str_pad($col, 2, '0', STR_PAD_LEFT);
        $i = ((string)$col[0] === '0') ? 0 : (ord($col[0]) - 64) * 26;
        $i += ord($col[1]) - 64;

        return $i;
    }
    /**
     * Check merge cell
     *
     * @param integer $rowNb         Row
     * @param integer $colNb         Col NB
     * @param array   $mergeSettings Setting
     * @param boolean $removeSpan    Remove tag span
     *
     * @return array|boolean
     */
    private static function checkMergeInfo($rowNb, $colNb, $mergeSettings, $removeSpan = false)
    {
        $result = array('', $mergeSettings);
        $count = count($mergeSettings);
        if (!is_array($mergeSettings) || $count === 0) {
            return $result;
        }

        foreach ($mergeSettings as $key => $ms) {
            if (is_object($ms)) {
                $merge = (array)$ms;
                $mergeSettings[$key] = $merge;
            } else {
                $merge = $ms;
            }

            if ((int) $merge[0] === (int) $rowNb && (int) $merge[2] === (int) $colNb) {
                $rowSpan = intval($merge[1]) > 1 ? ' rowspan="' . $merge[1] . '"' : '';
                $colSpan = intval($merge[3]) > 1 ? ' colspan="' . $merge[3] . '"' : '';
                $result[0] = $colSpan . $rowSpan;
            } elseif ($merge[0] <= $rowNb && $rowNb < $merge[0] + $merge[1] && $merge[2] <= $colNb && $colNb < $merge[2] + $merge[3]) {
                if ($removeSpan === true) {
                    $result[0] = false;
                    return $result;
                } else {
//                    $result[0] = ' style="display: none; color:red" class="hidden_droptables"'; error merger cells + hidden columns
                    $result[0] = ' style="display: none"';
                }
            }
        }

        return $result;
    }

    /**
     * Gets a list of the actions that can be performed.
     *
     * @return JObject
     *
     * @since 1.6
     * @todo  Refactor to work with notes
     */
    public static function getActions()
    {
        if (empty(self::$actions)) {
            self::$actions = ContentHelper::getActions('com_droptables');
//            $user = JFactory::getUser();
//            self::$actions = new JObject;
//
//            $actions = JAccess::getActions('com_droptables');
//
//            foreach ($actions as $action) {
//                self::$actions->set($action->name, $user->authorise($action->name, 'com_droptables'));
//            }
        }

        return self::$actions;
    }

    /**
     * Render button
     *
     * @param string $e_name         Element name
     * @param array  $activedButtons Button action
     *
     * @return string
     */
    public static function displayButtons($e_name, $activedButtons)
    {
        $buttons = array();
        $editor = JEditor::getInstance();
        // Get plugins
        $plugins = JPluginHelper::getPlugin('editors-xtd');
        foreach ($plugins as $plugin) {
            if (!in_array($plugin->name, $activedButtons)) {
                continue;
            }

//            $className = 'PlgEditorsXtd' . $plugin->name;
//            if (!class_exists($className))
//            {
//                $className = 'plgButton' . $plugin->name;
//            }
//
//            if ($className === 'plgButtondroptablesbtn') {
////                require_once(JPATH_PLUGINS . DIRECTORY_SEPARATOR . 'editors-xtd' . DIRECTORY_SEPARATOR . 'droptablesbtn' . DIRECTORY_SEPARATOR . 'droptablesbtn.php');
//            }
//
//            if (class_exists($className)) {
//                var_dump("||||");
//                var_dump($className);
//                var_dump(class_exists($className));
//                $dispatcher = $editor->getDispatcher();
//                $plugin = new $className($dispatcher, (array)$plugin);
//            }
//
//            // Try to authenticate
//            if (!method_exists($plugin, 'onDisplay'))
//            {
//                continue;
//            }
//            $button = $plugin->onDisplay($e_name, '', '');
//            if (empty($button))
//            {
//                continue;
//            }
//            if (\is_array($button))
//            {
//                $buttons = array_merge($buttons, $button);
//                continue;
//            }
//            $buttons[] = $button;
//            var_dump($plugin);
//            var_dump($buttons);

            JPluginHelper::importPlugin('editors-xtd', $plugin->name, false);
            $className = 'PlgButton' . $plugin->name;
            if (droptablesBase::isJoomla40()) {
                if (class_exists($className)) {
                    $dispatcher = $editor->getDispatcher();
                    $plugin = new $className($dispatcher, (array)$plugin);
                }
            } else {
                if (class_exists($className)) {
//                    $dispatcher = $editor->getDispatcher();
                    $plugin = new $className($editor, (array)$plugin);
                }
            }

            // Try to authenticate
            $temp = $plugin->onDisplay($e_name, '', '');
            if ($temp && method_exists($plugin, 'onDisplay')) {
                $buttons[] = $temp;
            }
        }

        $return = '';
        if (is_array($buttons) || (is_bool($buttons) && $buttons)) {
            /*
             * This will allow plugins to attach buttons or change the behavior on the fly using AJAX
             */
            if ($e_name !== 'editor1') {
                $return .= "\n<div id=\"" . $e_name . '-xtd-buttons"';
            } else {
                $return .= "\n<div id=\"droptable-xtd-buttons\"";
            }
            $return .= " class=\"btn-toolbar \">\n\n<div class=\"btn-toolbar\"";
            $return .= ">\n";
            $exclue_buttons = array('droptables');
            $exclue_buttons2 = array(JText::_('PLG_READMORE_BUTTON_READMORE'), JText::_('PLG_EDITORSXTD_PAGEBREAK_BUTTON_PAGEBREAK'));
            $i = 0;
            foreach ($buttons as $button) {
                /*
                 * Results should be an object
                 */
                if (droptablesBase::isJoomla40()) {
                    if (!in_array($button->get('name'), $exclue_buttons) && !in_array($button->get('text'), $exclue_buttons2)) {
                        $title = ($button->get('title')) ? $button->get('title') : $button->get('text');
                        $link = ($button->get('link')) ? JURI::base() . $button->get('link') : '';

                        $return .= '<a data-bs-toggle="modal" data-bs-target="#userModal_' . $title . $i . '" class="droptables-button-custom-edittor btn btn-primary button-select"
                            title="' . $title . '">';
                        $return .= $button->get('text') . '</a>';
                        $return .= JHtml::_(
                            'bootstrap.renderModal',
                            'userModal_' . $title . $i,
                            array(
                                'url' => $link,
                                'title' => $title,
                                'closeButton' => true,
                                'height'      => 380,
                                'width'       => 500,
                                'modalWidth'  => 80,
                                'bodyHeight'  => 60
                            )
                        );
                    }
                } else {
                    if (!in_array($button->get('name'), $exclue_buttons) && !in_array($button->get('text'), $exclue_buttons2)) {
                        $modal = ($button->get('modal')) ? ' class="modal-button btn droptables-button-custom-edittor"' : '';
                        $title = ($button->get('title')) ? $button->get('title') : $button->get('text');

                        $link = ($button->get('link')) ? JURI::base() . $button->get('link') : '';
                        $href = ($button->get('link')) ? ' class="btn" href="#' . $title . $i . '"' : '';
//                    $onclick = ($button->get('onclick')) ? ' onclick="' . $button->get('onclick') . '"' : ' return false;"';

                        $return .= '<a' . $modal . ' title="' . $title . '"' . $href . ' data-toggle="modal" data-url=' . $link . '>';
                        $return .= '<i class="icon-' . $button->get('name') . '"></i> ';
                        $return .= $button->get('text') . '</a>';

                        $modalParams = array();
                        $modalParams['url'] = $link;
                        $modalParams['height'] = 380;
                        $modalParams['width'] = 500;
                        $modalParams['closeButton'] = true;
                        $modalBody = '<iframe src="' . $link . '" frameborder="0" width="800" height="600"></iframe>';

                        $return .= JHtml::_(
                            'bootstrap.renderModal',
                            $title . $i,
                            $modalParams,
                            $modalBody
                        );
                    }
                }
                $i++;
            }

            $return .= "</div>\n";
            $return .= "</div>\n";
        }

        return $return;
    }
}
