<?php
/**
 * Droptables
 *
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 *
 * @package   Droptables
 * @copyright Copyright (C) 2014 JoomUnited (http://www.joomunited.com). All rights reserved.
 * @copyright Copyright (C) 2014 Damien Barrère (http://www.crac-design.com). All rights reserved.
 * @license   GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 */


defined('_JEXEC') || die;

/**
 * Class DroptablesUpdate
 */
class DroptablesUpdate
{
    /**
     * Check table update
     *
     * @return void
     */
    public function checkTableUpdate()
    {
        JLoader::register('DroptablesModelTables', JPATH_ADMINISTRATOR . '/components/com_droptables/models/tables.php');
        $model = DroptablesModelTables::getInstance('DroptablesModelTables');
//        $model->getListTable
    }

    /**
     * Update table
     *
     * @param object $item Data
     *
     * @return array
     */
    public function updateTable($item)
    {
        $item->datas  = isset($item->data) ? $item->data : $item->datas;
        $item->params = isset($item->params) ? $item->params : new stdClass();
        $item->type   = isset($item->type) ? $item->type : (isset($item->params->table_type) ? $item->params->table_type : 'html');

        if ($item->style === '') {
            $item->style = new stdClass();
        } else {
            $item->style = is_string($item->style) ? json_decode($item->style) : $item->style;
        }

        $newData = self::parseItem($item);

        return $newData;
    }

    /**
     * Parse data table
     *
     * @param object $item Data Table
     *
     * @return array
     */
    public static function parseItem($item)
    {
        $newData = array();
        $newData['type'] = isset($item->type) ? $item->type : $item->params->table_type;
        $newData['id_category'] = isset($item->id_category) ? $item->id_category : 0;

        /*convert param*/
        $newData['params'] = new stdClass();
        if (isset($item->params)) {
            $newData['params'] = $item->params;
        }
        if (isset($item->style->table)) {
            foreach ($item->style->table as $key => $table) {
                if (!isset($newData['params']->{$key})) {
                    $newData['params']->{$key} = $table;
                    if ($key === 'freeze_row' && (int)$table > 0) {
                        $newData['params']->{$key} = 1;
                        $newData['params']->headerOption = (int)$table;
                    }
                }
            }
        }

        if ($newData['type'] === 'html') {
            //convert data cells
            $newData['datas'] = json_decode($item->datas);
        } else {
            $newData['datas'] = $item->datas;
            $newData['params']->query = $item->datas;
        }

        /*convert style*/
        $newData['style'] = new stdClass();
        $newData['style']->rows = isset($item->style->rows) ? $item->style->rows : new stdClass();
        $newData['style']->cols = isset($item->style->cols) ? $item->style->cols : new stdClass();
//        $newData['style'] = json_encode($newData['style']);

        if (empty($newData['datas']) || is_string($newData['datas'])) {
            $newData['numberRow'] = isset($item->style->rows) ? count((array)$item->style->rows) : 0;
            $newData['numberCol'] = isset($item->style->cols) ? count((array)$item->style->cols) : 0;
        } else {
            $newData['numberRow'] = count($newData['datas']);
            $newData['numberCol'] = count($newData['datas'][0]);
        }

        $newData['css'] = $item->css;

        if (isset($item->style->cells)) {
            $styleCells = self::mergeStyleCell($item->style->cells, $newData['numberRow'], $newData['numberCol']);
            $newData['styleCells'] = $styleCells['styleCells'];
            $newData['params']->cell_types = $styleCells['typeCells'];
        }
        return $newData;
    }

    /**
     * Create range style cells
     *
     * @param object  $cells Style cells
     * @param integer $row   Style row
     * @param integer $col   Style col
     *
     * @return array
     */
    public static function mergeStyleCell($cells, $row, $col)
    {
        $data = array();
        $content = array();
        $typeCells = array();

        for ($i = 0; $i < $row; $i++) {
            for ($j = 0; $j < $col; $j++) {
                if (isset($cells->{$i . '!' . $j})) {
                    $cell = $cells->{$i . '!' . $j};

                    $content[$cell[0] . '|' . $cell[1]] = array($cell[0],$cell[0],$cell[1],$cell[1]);
                    if (isset($cell[2]->width)) {
                        unset($cell[2]->width);
                    }
                    if (isset($cell[2]->height)) {
                        unset($cell[2]->height);
                    }
                    if (isset($cell[2]->cell_type)) {
                        if ($cell[2]->cell_type === 'html') {
                            $typeCells[] = array($cell[0], $cell[1], 1, 1, 'html');
                        } elseif ($cell[2]->cell_type === null) {
                            $typeCells[] = array($cell[0], $cell[1], 1, 1, '');
                        }
                        unset($cell[2]->cell_type);
                    }
                    array_push($content[$cell[0] . '|' . $cell[1]], json_encode($cell[2]));
                    if (isset($content[$cell[0] . '|' . ($cell[1] - 1)])) {
                        $cell_before = $content[$cell[0] . '|' . ($cell[1] - 1)];
                        if (!isset($cell_before[4])) {
                            $cell_before = $content[$cell_before[0] . '|' . $cell_before[2]];
                        }
                        if ($cell_before[4] === $content[$cell[0] . '|' . $cell[1]][4]) {
                            $content[$cell_before[0] . '|' . $cell_before[2]][3] = $content[$cell[0] . '|' . $cell[1]][3];
                            $data[$cell_before[0] . '|' . $cell_before[2]][3] = $content[$cell[0] . '|' . $cell[1]][3];
                            $content[$cell[0] . '|' . $cell[1]][2] = $cell_before[2];
                            unset($content[$cell[0] . '|' . $cell[1]][4]);
                        }
                    }
                    if (isset($content[$cell[0] . '|' . $cell[1]][4])) {
                        $data[$cell[0] . '|' . $cell[1]] = $content[$cell[0] . '|' . $cell[1]];
                    }
                }
            }
        }

        $content = array();
        foreach ($data as $cell) {
            if (isset($data[($cell[0] - 1) . '|' . $cell[2]])) {
                $cell_before = $data[($cell[0] - 1) . '|' . $cell[2]];
                if (!isset($cell_before[4])) {
                    $cell_before = $data[$cell_before[0] . '|' . $cell_before[2]];
                }
                if ($cell_before[4] === $cell[4] && $cell_before[3] === $cell[3]) {
                    $data[$cell_before[0] . '|' . $cell_before[2]][1] = $cell[1];
                    $content[$cell_before[0] . '|' . $cell_before[2]][1] = $cell[1];
                    $data[$cell[0] . '|' . $cell[2]][0] = $cell_before[0];
                    unset($data[$cell[0] . '|' . $cell[2]][4]);
                }
            }
            if (isset($data[$cell[0] . '|' . $cell[2]][4])) {
                $content[$cell[0] . '|' . $cell[2]] = $data[$cell[0] . '|' . $cell[2]];
            }
        }

        $style_cells = array();
        foreach ($content as $cell) {
            $cell[0]++;
            $cell[1]++;
            $cell[2]++;
            $cell[3]++;
            $style_cells[($cell[0] + 1). '|' . ($cell[2] + 1)] = $cell;
        }

        return array('styleCells'=>$style_cells, 'typeCells'=> $typeCells);
    }
}
