<?php
/**
 * Droptables
 *
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 *
 * @package   Droptables
 * @copyright Copyright (C) 2014 JoomUnited (http://www.joomunited.com). All rights reserved.
 * @copyright Copyright (C) 2014 Damien Barrère (http://www.crac-design.com). All rights reserved.
 * @license   GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 */
defined('_JEXEC') || die;

/**
 * Model for component configuration
 */
class DroptablesModelConfiguration extends JModelAdmin
{
    /**
     * Method to get form file config
     *
     * @param array   $data     File data
     * @param boolean $loadData Load data
     *
     * @return boolean
     */
    public function getForm($data = array(), $loadData = true)
    {
        // Add the search path for the admin component config.xml file.
        JForm::addFormPath(JPATH_ADMINISTRATOR . '/components/com_droptables');


        $form = $this->loadForm('com_droptables.config', 'config', array('control' => 'jform', 'load_data' => $loadData), false, '/config');

        if (empty($form)) {
            return false;
        }

        return $form;
    }


    /**
     * Method to save config
     *
     * @param array $jformData Form data
     *
     * @return boolean
     */
    public function saveGlobalConfig($jformData)
    {
        //Load the data form
        $form = $this->getForm();

        $default_parameter = array();
        foreach ($form->getFieldsets() as $name => $fieldsets) {
            foreach ($form->getFieldset($name) as $fieldset) {
                if ($fieldset->type === 'Radio') {
                    $fieldset->value = 0;
                }
                $default_parameter[$fieldset->fieldname] = $fieldset->value;
            }
        }

        $new_parameter = array_replace($default_parameter, $jformData);

        //Set configuration
        $DroptablesComponentHelper = new DroptablesComponentHelper();
        if (!$DroptablesComponentHelper->setParams($new_parameter)) {
            return false;
        }

        return true;
    }

    /**
     * Method to save permissions
     *
     * @param array $jformData Form data
     *
     * @return boolean
     */
    public function savePermissions($jformData)
    {
        // Save the rules.
        if (isset($jformData) && isset($jformData['rules'])) {
            $form = $this->getForm($jformData);

            // Validate the posted data.
            $postedRules = $this->validate($form, $jformData);
            $rules = new JAccessRules($postedRules['rules']);
            $asset = JTable::getInstance('asset');

            if (!$asset->loadByName('com_droptables')) {
                $root = JTable::getInstance('asset');
                $root->loadByName('root.1');
                $asset->name = 'com_droptables';
                $asset->title = 'com_droptables';
                $asset->setLocation($root->id, 'last-child');
            }

            $asset->rules = (string)$rules;

            if (!$asset->check() || !$asset->store()) {
                $this->setError($asset->getError());
                return false;
            }
        }

        return true;
    }
}
