<?php
/* Based on some work of wp Data Tables plugin */
/**
 * Droptables
 *
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 *
 * @package   Droptables
 * @copyright Copyright (C) 2014 JoomUnited (http://www.joomunited.com). All rights reserved.
 * @copyright Copyright (C) 2014 Damien Barrère (http://www.crac-design.com). All rights reserved.
 * @license   GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 */

defined('_JEXEC') || die;

jimport('joomla.application.component.modeladmin');

/**
 * Class DroptablesModelDbtable
 */
class DroptablesModelDbtable extends JModelAdmin
{
    /*     * * For the WP DB type query ** */
    /**
     * Select array
     *
     * @var array
     */
    private $selectArr = array();
    /**
     * Position array
     *
     * @var array
     */
    private $whereArr = array();
    /**
     * Group array
     *
     * @var array
     */
    private $groupArr = array();
    /**
     * From array
     *
     * @var array
     */
    private $fromArr = array();
    /**
     * Inner array
     *
     * @var array
     */
    private $innerJoinArr = array();
    /**
     * Left jont array in sql
     *
     * @var array
     */
    private $leftJoinArr = array();
    /**
     * Jont on array in sql
     *
     * @var array
     */
    private $joinOnArr = array();
    /**
     * Group check
     *
     * @var boolean
     */
    private $hasGroups = false;

    /**
     * Query data
     *
     * @var string
     */
    private $query = '';

    /**
     * Query text
     *
     * @var string
     */
    private $dbQuery = '';

    /**
     * Get form
     *
     * @param array   $data     Data
     * @param boolean $loadData Data loader
     *
     * @return boolean
     * @since  version
     */
    public function getForm($data = array(), $loadData = true)
    {
        $form = $this->loadForm('com_droptables.dbtable', 'dbtable', array('control'   => 'jform',
                                                                           'load_data' => $loadData
        ));
        if (empty($form)) {
            return false;
        }

        return $form;
    }

    /**
     * Get list sql table
     *
     * @return array
     * @since  version
     */
    public function listMySQLTables()
    {
        $dbo    = $this->getDbo();
        $tables = array();

        $dbo->setQuery('SHOW TABLES');
        $result = $dbo->loadRowList();

        // Formatting the result to plain array
        foreach ($result as $row) {
            $tables[] = $row[0];
        }

        return $tables;
    }

    /**
     * Return a list of columns for the selected tables
     *
     * @param object $tables List sql table seleter
     *
     * @return array
     * @since  version
     */
    public static function listMySQLColumns($tables)
    {
        $columns = array('all_columns' => array(), 'sorted_columns' => array());
        if (!empty($tables)) {
            $dbo = JFactory::getDbo();
            foreach ($tables as $table) {
                $columns['sorted_columns'][$table] = array();
                $dbo->setQuery('SHOW COLUMNS FROM ' . $table . ';');
                $table_columns = $dbo->loadRowList();
                foreach ($table_columns as $table_column) {
                    $columns['sorted_columns'][$table][] = $table . '.' . $table_column[0];
                    $columns['all_columns'][]            = $table . '.' . $table_column[0];
                }
            }
        }

        return $columns;
    }

    /**
     * Return a build query and preview table
     *
     * @param array $table_data Table data
     *
     * @return array
     * @since  version
     */
    public function generateQueryAndPreviewdata($table_data)
    {
        $dbo               = $this->getDbo();
        $this->_table_data = $table_data;
        if (!isset($this->_table_data['where_conditions'])) {
            $this->_table_data['where_conditions'] = array();
        }

        if (isset($this->_table_data['grouping_rules'])) {
            $this->hasGroups = true;
        }

        if (!isset($table_data['mysql_columns'])) {
            $table_data['mysql_columns'] = array();
        }

        // Initializing structure for the SELECT part of query
        $this->prepareMySQLSelectBlock($table_data['mysql_columns']);

        // Initializing structure for the WHERE part of query
        $this->prepareMySQLWhereBlock();

        // Prepare the GROUP BY block
        $this->prepareMySQLGroupByBlock();

        // Prepare the join rules
        $this->prepareMySQLJoinedQueryStructure();

        // Prepare the query itself
        $this->dbQuery = $this->buildMySQLQuery();

        if (isset($this->_table_data['default_ordering']) && $this->_table_data['default_ordering']) {
            $this->dbQuery .= ' Order by ' . $this->_table_data['default_ordering'] . ' ' . $this->_table_data['default_ordering_dir'];
        }
        if (preg_match('/union/i', $this->dbQuery)) {
            $result = false;
        } else {
            $result = array(
                'query'   => $this->dbQuery,
                'hasRow' => true,
                'preview' => $this->getQueryPreview()
            );
        }

        if ($result !== false && !empty($this->_table_data['id_table'])) {
            $name = '#__droptables_table_options';
            $query = 'DELETE FROM ' . $name . ' WHERE id_table = ' . (int)$this->_table_data['id_table'] . ' AND option_name = \'header_data\'';
            $dbo->setQuery($query);
            if (!$dbo->execute()) {
                $result = false;
            }
        }

        return $result;
    }

    /**
     * Generate the sample table with 5 rows from MySQL query
     *
     * @return string|false
     * @since  version
     */
    public function getQueryPreview()
    {
        $dbo = $this->getDbo();
        $dbo->setQuery($this->dbQuery . ' LIMIT 10');
        $result = $dbo->loadRowList();

        if (!empty($result)) {
            $headers = $this->_table_data['custom_titles'];
            ob_start();
            include(JPATH_COMPONENT_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'views' . DIRECTORY_SEPARATOR . 'dbtable' . DIRECTORY_SEPARATOR . 'tmpl' . DIRECTORY_SEPARATOR . 'table_preview.inc.php');
            $ret_val = ob_get_contents();
            ob_end_clean();
        } else {
            $ret_val = JText::_('COM_DROPTABLES_LAYOUT_DBTABLES_NO_RESULTS');
        }

        return $ret_val;
    }

    /**
     * Get columns options
     *
     * @param array $custom_mysql Custom query table
     *
     * @return array
     */
    public function applyCustomQuery($custom_mysql)
    {
        $dbo = $this->getDbo();
        $this->query = $custom_mysql;
        $getMessage = null;
        try {
            $dbo->setQuery($this->query);
            $result = $dbo->loadAssocList();
        } catch (Exception $e) {
            $getMessage = $e->getMessage();
        }

        if (!(!empty($result) && is_array($result) && count($result) > 0) && $getMessage !== null) {
            $result = array(
                'hasRow' => false,
                'query' => $this->query,
                'error' => JText::_('COM_DROPTABLES_LAYOUT_DBTABLES_QUERY_ERROR'),
                'preview' => JText::_('COM_DROPTABLES_LAYOUT_DBTABLES_NO_RESULTS')
            );
            return $result;
        }

        $headers = array();
        if (!empty($result[0])) {
            foreach ($result[0] as $column => $row0) {
                $headers[] = $column;
            }
        } else {
            require_once(JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR . 'com_droptables' . DIRECTORY_SEPARATOR . 'classes' . DIRECTORY_SEPARATOR . 'php-sql-parser' . DIRECTORY_SEPARATOR . 'vendor' . DIRECTORY_SEPARATOR . 'autoload.php');
            $parser = new \PHPSQLParser\PHPSQLParser(false, true);
            $custom_mysql = str_replace(array("\n\r", "\r\n", "\n", "\r", '&#10;'), ' ', $custom_mysql);
            $custom_mysql = str_replace('\\', '', $custom_mysql);
            if (strpos($custom_mysql, '{user_id}') !== false) {
                $custom_mysql = str_replace('{user_id}', ' {user_id}', $custom_mysql);
            }
            if (strpos($custom_mysql, '{user_email}') !== false) {
                $custom_mysql = str_replace('{user_email}', ' {user_email}', $custom_mysql);
            }
            $parserMysql = $parser->parse($custom_mysql, true);
            //list columns
            $headers = array();
            $count = count($parserMysql['SELECT']);
            for ($i = 0; $i < $count; $i++) {
                if (!isset($parserMysql['SELECT'][$i]['alias']['name'])) {
                    $nameColumn = explode('.', $parserMysql['SELECT'][$i]['base_expr']);
                    if (!empty($nameColumn[1])) {
                        $headers[] = $nameColumn[1];
                    } else {
                        $headers[] = $parserMysql['SELECT'][$i]['base_expr'];
                    }
                } else {
                    $headers[] = $parserMysql['SELECT'][$i]['alias']['name'];
                }
            }
        }

        ob_start();
        include(JPATH_COMPONENT_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'views' . DIRECTORY_SEPARATOR . 'dbtable' . DIRECTORY_SEPARATOR . 'tmpl' . DIRECTORY_SEPARATOR . 'table_preview.inc.php');
        $ret_val = ob_get_contents();
        ob_end_clean();
        $result = array(
            'hasRow' => true,
            'query' => $this->query,
            'preview' => $ret_val
        );

        return $result;
    }

    /**
     * Helper function to generate the fields structure from MySQL tables
     *
     * @return void
     * @since  version
     */
    private function prepareMySQLSelectBlock()
    {
        foreach ($this->_table_data['mysql_columns'] as $mysql_column) {
            $mysql_column_arr = explode('.', $mysql_column);
            if (!isset($this->selectArr[$mysql_column_arr[0]])) {
                $this->selectArr[$mysql_column_arr[0]] = array();
            }
            $this->selectArr[$mysql_column_arr[0]][] = $mysql_column;

            if (!in_array($mysql_column_arr[0], $this->fromArr)) {
                $this->fromArr[] = $mysql_column_arr[0];
            }
        }
    }

    /**
     * Prepare a Where block for MySQL based
     *
     * @return void
     * @since  version
     */
    private function prepareMySQLWhereBlock()
    {
        if (empty($this->_table_data['where_conditions'])) {
            return;
        }

        foreach ($this->_table_data['where_conditions'] as $where_condition) {
            $where_column_arr = explode('.', $where_condition['column']);

            if (!in_array($where_column_arr[0], $this->fromArr)) {
                $this->fromArr[] = $where_column_arr[0];
            }
            if (!empty($where_condition['column'])) {
                $this->whereArr[$where_column_arr[0]][] = self::buildWhereCondition(
                    $where_condition['column'],
                    $where_condition['operator'],
                    $where_condition['value']
                );
            }
        }
    }

    /**
     * Prepare a GROUP BY block for MySQL based
     *
     * @return void
     * @since  version
     */
    private function prepareMySQLGroupByBlock()
    {
        if (!$this->hasGroups) {
            return;
        }

        foreach ($this->_table_data['grouping_rules'] as $grouping_rule) {
            if (empty($grouping_rule)) {
                continue;
            }
            $this->groupArr[] = $grouping_rule;
        }
    }

    /**
     * Prepares the structure of the JOIN rules for MySQL based tables
     *
     * @return void
     * @since  version
     */
    private function prepareMySQLJoinedQueryStructure()
    {
        if (!isset($this->_table_data['join_rules'])) {
            return;
        }

        foreach ($this->_table_data['join_rules'] as $join_rule) {
            if (empty($join_rule['initiator_column']) || empty($join_rule['connected_column'])) {
                continue;
            }

            $connected_column_arr = explode('.', $join_rule['connected_column']);

            if (in_array($connected_column_arr[0], $this->fromArr) && count($this->fromArr) > 1) {
                if ($join_rule['type'] === 'left') {
                    $this->leftJoinArr[$connected_column_arr[0]] = $connected_column_arr[0];
                } else {
                    $this->innerJoinArr[$connected_column_arr[0]] = $connected_column_arr[0];
                }
                unset($this->fromArr[array_search($connected_column_arr[0], $this->fromArr)]);
            } else {
                if ($join_rule['type'] === 'left') {
                    $this->leftJoinArr[$connected_column_arr[0]] = $connected_column_arr[0];
                } else {
                    $this->innerJoinArr[$connected_column_arr[0]] = $connected_column_arr[0];
                }
            }

            $this->joinOnArr[$connected_column_arr[0]][] = self::buildWhereCondition(
                $join_rule['initiator_table'] . '.' . $join_rule['initiator_column'],
                'eq',
                $join_rule['connected_column'],
                false
            );
        }
    }

    /**
     * Prepares the query text for MySQL based table
     *
     * @return string
     * @since  version
     */
    private function buildMySQLQuery()
    {
        // Build the final output
        $query = 'SELECT ';
        $i     = 0;
        foreach ($this->selectArr as $table_alias => $select_block) {
            $new_select_block = array();
            foreach ($select_block as $select_element) {
                $new_select_block[] = $select_element . ' AS ' . str_replace('.', '_', $select_element);
            }
            $query .= implode(",\n       ", $new_select_block);
            $i ++;
            if ($i < count($this->selectArr)) {
                $query .= ",\n       ";
            }
        }
        $query .= "\nFROM ";
        $query .= implode(', ', array_values(array_filter($this->fromArr))) . "\n";
        if (!empty($this->innerJoinArr)) {
            $i = 0;
            foreach ($this->innerJoinArr as $table_alias => $inner_join_block) {
                $query .= '  INNER JOIN ' . $inner_join_block . "\n";
                if (!empty($this->joinOnArr[$table_alias])) {
                    $query .= '     ON ' . implode("\n     AND ", $this->joinOnArr[$table_alias]) . "\n";
                    unset($this->joinOnArr[$table_alias]);
                }
            }
        }
        if (!empty($this->leftJoinArr)) {
            foreach ($this->leftJoinArr as $table_alias => $left_join_block) {
                $query .= '  LEFT JOIN ' . $left_join_block . "\n";
                if (!empty($this->joinOnArr[$table_alias])) {
                    $query .= '     ON ' . implode("\n     AND ", $this->joinOnArr[$table_alias]) . "\n";
                    unset($this->joinOnArr[$table_alias]);
                }
            }
        }
        if (!empty($this->whereArr)) {
            $query .= "WHERE 1=1 \n   AND ";
            $i     = 0;
            foreach ($this->whereArr as $table_alias => $where_block) {
                $query .= implode("\n   AND ", $where_block);
                $i ++;
                if ($i < count($this->whereArr)) {
                    $query .= "\n   AND ";
                }
            }
        }
        if (!empty($this->groupArr)) {
            $query .= "\nGROUP BY " . implode(', ', $this->groupArr);
        }

        return $query;
    }


    /**
     * Prepares the structure of the WHERE rules for MySQL based tables
     *
     * @param string  $leftOperand  Left operand
     * @param string  $operator     Operator
     * @param string  $rightOperand Right operand
     * @param boolean $isValue      Check value
     *
     * @return string
     * @since  version
     */
    public static function buildWhereCondition($leftOperand, $operator, $rightOperand, $isValue = true)
    {
        $rightOperand = stripslashes($rightOperand); //stripslashes_deep
        $wrap         = $isValue ? "'" : '';
        switch ($operator) {
            case 'eq':
                return $leftOperand . ' = ' . $wrap . $rightOperand . $wrap;
            case 'neq':
                return $leftOperand . ' != ' . $wrap . $rightOperand . $wrap;
            case 'gt':
                return $leftOperand . ' > ' . $wrap . $rightOperand . $wrap;
            case 'gtoreq':
                return $leftOperand . ' >= ' . $wrap . $rightOperand . $wrap;
            case 'lt':
                return $leftOperand . ' < ' . $wrap . $rightOperand . $wrap;
            case 'ltoreq':
                return $leftOperand . ' <= ' . $wrap . $rightOperand . $wrap;
            case 'in':
                return $leftOperand . ' IN (' . $rightOperand . ')';
            case 'like':
                return $leftOperand . ' LIKE ' . $wrap . $rightOperand . $wrap;
            case 'plikep':
                return $leftOperand . ' LIKE ' . $wrap . '%' . $rightOperand . '%' . $wrap;
        }
    }

    /**
     * Get result data of build query
     *
     * @param string $query Query
     *
     * @return array|boolean|null|object
     */
    public function getTableData($query)
    {
        $dbo = $this->getDbo();
        $dbo->setQuery($query);
        $reuslts = $dbo->loadAssocList();
        return $reuslts;
    }

    /**
     * Get query option in db table
     *
     * @param integer|null $id Id table
     *
     * @return boolean|mixed
     */
    public function getQueryOption($id = null)
    {
        if ($id !== null) {
            $db = JFactory::getDbo();
            $name = '#__droptables_table_options';
            $query = 'SELECT t.* FROM ' . $name . ' as t WHERE t.id_table = ' . (int)$id . ' AND t.option_name = \'query_option\'';
            $db->setQuery($query);
            $styles = $db->loadObject();
            if ($styles === false) {
                return false;
            }
            return $styles;
        }
        return false;
    }

    /**
     * Get list table query
     *
     * @return array|boolean
     */
    public function getListQueryDbTable()
    {
        $db = JFactory::getDbo();
        //upload style range
        $db->setQuery('Select t.params, t.id From #__droptables_tables As t '
            . ' WHERE t.type = \'mysql\'');
        $old_tables = $db->loadObjectList();

        if (!empty($old_tables)) {
            $tables_query = array();
            foreach ($old_tables as $old_table) {
                if (is_string($old_table->params)) {
                    $params = json_decode($old_table->params);
                    if (is_array($params)) {
                        $tables_query[] = array($old_table->id, isset($params['mysql_query']) ? $params['mysql_query'] : (isset($params['query']) ? $params['query'] : null));
                    } elseif (is_object($params)) {
                        $tables_query[] = array($old_table->id, isset($params->mysql_query) ? $params->mysql_query : (isset($params->query) ? $params->query : null));
                    }
                }
            }
            return $tables_query;
        }
        return false;
    }

    /**
     * Create new table for selected mysql tables
     *
     * @param array   $table_data Table data
     * @param integer $id_cat     Id category
     *
     * @return boolean
     * @since  version
     */
    public function createTable($table_data, $id_cat)
    {
        $query_option = array();
        $query_option['tables_list'] = $table_data['tables_list'];
        $query_option['column_options'] = $table_data['column_options'];
        unset($table_data['tables_list']);
        unset($table_data['column_options']);
        $params = $table_data;

        if ($id_cat !== null) {
            $id_category = $id_cat;
        } else {
            $modelCategory = $this->getModel('category');
            $id_category = $modelCategory->addCategory('Category');
        }

        $dbo         = $this->getDbo();
        $query       = 'SELECT MAX(c.position) AS lastPos FROM #__droptables_tables as c WHERE c.id_category=' . (int) $id_cat;
        $dbo->setQuery($query);
        $lastPos = $dbo->loadObject();
        $lastPos = $lastPos->lastPos++;
        $style = json_decode('{   "table":{      "alternate_row_even_color":"#fafafa",      "use_sortable":"1"   },   "rows":{      "0":[         0,         {            "height":30,            "cell_padding_top":"3",            "cell_padding_right":"3",            "cell_padding_bottom":"3",            "cell_padding_left":"3",            "cell_font_family":"Arial",            "cell_font_size":"13",            "cell_font_color":"#333333",            "cell_background_color":"#ffffff",            "cell_font_bold":true,            "cell_vertical_align":"middle"         }      ],      "1":[         1,         {            "height":30}      ],      "2":[         2,         {            "height":30}      ],      "3":[         3,         {            "height":30}      ],      "4":[         4,         {            "height":30}      ],      "5":[         5,         {            "height":30}      ],      "6":[         6,         {            "height":30}      ],      "7":[         7,         {            "height":30}      ],      "8":[         8,         {            "height":30}      ]   },   "cols":{      "0":[         0,         {            "cell_border_bottom":"1px solid #d6d6d6",            "width":50,            "cell_text_align":"center",            "cell_font_bold":true,            "cell_padding_top":"3",            "cell_padding_right":"3",            "cell_padding_bottom":"3",            "cell_padding_left":"3",            "cell_font_color":"#333333",            "cell_vertical_align":"middle"         }      ],      "1":[         1,         {            "cell_border_bottom":"1px solid #d6d6d6",            "width":122,"cell_text_align":"center",            "cell_padding_top":"3",            "cell_padding_right":"3",            "cell_padding_bottom":"3",            "cell_padding_left":"3",            "cell_font_color":"#333333",            "cell_vertical_align":"middle"         }      ],      "2":[         2,         {            "cell_border_bottom":"1px solid #d6d6d6",            "width":137,"cell_text_align":"center",            "cell_padding_top":"3",            "cell_padding_right":"3",            "cell_padding_bottom":"3",            "cell_padding_left":"3",            "cell_font_color":"#333333",            "cell_vertical_align":"middle"         }      ],      "3":[         3,         {            "cell_border_bottom":"1px solid #d6d6d6",            "width":133,"cell_text_align":"center",            "cell_padding_top":"3",            "cell_padding_right":"3",            "cell_padding_bottom":"3",            "cell_padding_left":"3",            "cell_font_color":"#333333",            "cell_vertical_align":"middle"         }      ],      "4":[         4,         {            "cell_border_bottom":"1px solid #d6d6d6",            "width":150,"cell_text_align":"center",            "cell_padding_top":"3",            "cell_padding_right":"3",            "cell_padding_bottom":"3",            "cell_padding_left":"3",            "cell_font_color":"#333333",            "cell_vertical_align":"middle"         }      ],      "5":[         5,         {            "cell_border_bottom":"1px solid #d6d6d6",            "width":50,"cell_text_align":"center",            "cell_padding_top":"3",            "cell_padding_right":"3",            "cell_padding_bottom":"3",            "cell_padding_left":"3",            "cell_font_color":"#333333",            "cell_vertical_align":"middle"         }      ]   },   "cells":{         }}');
        $style->table->enable_pagination = $table_data['enable_pagination'];
        $style->table->limit_rows = $table_data['limit_rows'];

        require_once(JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR . 'com_droptables' . DIRECTORY_SEPARATOR . 'classes' . DIRECTORY_SEPARATOR . 'php-sql-parser' . DIRECTORY_SEPARATOR . 'vendor' . DIRECTORY_SEPARATOR . 'autoload.php');
        $parser = new \PHPSQLParser\PHPSQLParser(false, true);
        $creator = new \PHPSQLParser\PHPSQLCreator();
        $params['query'] = str_replace(array("\n\r", "\r\n", "\n", "\r", '&#10;'), ' ', $params['query']);
        $params['query'] = str_replace('\\', '', $params['query']);

        if (strpos($params['query'], '{user_id}') !== false) {
            $params['query'] = str_replace('{user_id}', ' {user_id}', $params['query']);
        }
        if (strpos($params['query'], '{user_email}') !== false) {
            $params['query'] = str_replace('{user_email}', ' {user_email}', $params['query']);
        }
        $parserMysql = $parser->parse($params['query'], true);

        unset($params['query']);
        $params['table_type'] = 'mysql';

        if (empty($params['customquery'])
            && !empty($params['priKey'])
            && !empty($params['table_editing'])
            && !in_array($params['priKey'], $params['mysql_columns'])) {
            //no custom, priKey column not exist in columns be selected then create new query

            array_unshift(
                $parserMysql['SELECT'],
                array('expr_type' => 'colref',
                    'alias' => '',
                    'base_expr' => $params['priKey'],
                    'sub_tree' => false,
                    'delim' => ','
                )
            );

            $params['getPriKeyTableQuery'] = $creator->create($parserMysql);
        } else {
            $params['getPriKeyTableQuery'] = $table_data['query'];
        }

        //phpcs:ignore PHPCompatibility.Constants.NewConstants.json_unescaped_unicodeFound -- the use of JSON_UNESCAPED_UNICODE has check PHP version
        $data_params = json_encode($params, JSON_UNESCAPED_UNICODE);
        $user                 = JFactory::getUser();
        $query                = sprintf(
            'INSERT INTO #__droptables_tables (id_category, title, style, params, created_time, modified_time, author, position, type, css) VALUES ( %d,%s,%s,%s,%s,%s,%d,%d,%s,%s)',
            $id_cat,
            $dbo->quote(Jtext::_('COM_DROPTABLES_MODEL_TABLE_DEFAULT_TITLE')),
            $dbo->quote(json_encode($style)),
            $dbo->quote($data_params),
            $dbo->quote(date('Y-m-d H:i:s')),
            $dbo->quote(date('Y-m-d H:i:s')),
            $user->id,
            $lastPos,
            $dbo->quote('mysql'),
            $dbo->quote('')
        );
        $dbo->setQuery($query);

        if (!$dbo->execute()) {
            return false;
        }

        $id_table = $dbo->insertid();

        $updateTableQuery = $this->updateTableQuery($table_data['query'], $id_table, true);
        if (!$updateTableQuery) {
            return false;
        }
        //create data in wptm_table_options table
        //get list all columns
        if (!empty($query_option['tables_list'])) {
            $query_option['columns_list'] = $this->getTableColumnsOptions($query_option['tables_list'], null);
            $table_alias = $this->getTableAliasOptions($parserMysql);
            if (count($table_alias)) {
                $query_option['table_alias'] = $table_alias;
            }
            $this->updateQueryOption($query_option, $id_table);
        }

        return $id_table;
    }

    /**
     * Update table with new change
     *
     * @param string|integer $id_table   Id table
     * @param array          $table_data Data table
     *
     * @return mixed
     * @since  version
     */
    public function updateTable($id_table, $table_data)
    {
        $query_option = array();
        $query_option['tables_list'] = $table_data['tables_list'];
        $query_option['column_options'] = $table_data['column_options'];
        unset($table_data['tables_list']);
        unset($table_data['column_options']);
        $params = $table_data;
        require_once(JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR . 'com_droptables' . DIRECTORY_SEPARATOR . 'classes' . DIRECTORY_SEPARATOR . 'php-sql-parser' . DIRECTORY_SEPARATOR . 'vendor' . DIRECTORY_SEPARATOR . 'autoload.php');
        $parser = new \PHPSQLParser\PHPSQLParser(false, true);
        $creator = new \PHPSQLParser\PHPSQLCreator();
        $params['query'] = str_replace(array("\n\r", "\r\n", "\n", "\r", '&#10;'), ' ', $params['query']);
        $params['query'] = str_replace('\\', '', $params['query']);

        if (strpos($params['query'], '{user_id}') !== false) {
            $params['query'] = str_replace('{user_id}', ' {user_id}', $params['query']);
        }
        if (strpos($params['query'], '{user_email}') !== false) {
            $params['query'] = str_replace('{user_email}', ' {user_email}', $params['query']);
        }
        $parserMysql = $parser->parse($params['query'], true);

        unset($params['query']);
        $params['table_type'] = 'mysql';

        if (empty($params['customquery'])
            && !empty($params['priKey'])
            && !empty($params['table_editing'])
            && !in_array($params['priKey'], $params['mysql_columns'])) {
            //no custom, priKey column not exist in columns be selected then create new query
            array_unshift(
                $parserMysql['SELECT'],
                array('expr_type' => 'colref',
                    'alias' => '',
                    'base_expr' => $params['priKey'],
                    'sub_tree' => false,
                    'delim' => ','
                )
            );

            $params['getPriKeyTableQuery'] = $creator->create($parserMysql);
        } else {
            $params['getPriKeyTableQuery'] = $table_data['query'];
        }

        // Fields to update.
        $db    = $this->getDbo();
        $query = $db->getQuery(true);

        $fields = array(
            $db->quoteName('params') . ' = ' . $db->quote(json_encode($params)),
            $db->quoteName('modified_time') . ' = ' . $db->quote(date('Y-m-d H:i:s'))
        );

        // Conditions for which records should be updated.
        $conditions = array(
            $db->quoteName('id') . ' = ' . $db->quote($id_table)
        );

        $query->update($db->quoteName('#__droptables_tables'))->set($fields)->where($conditions);

        $db->setQuery($query);
        $ret = $db->execute();

        $updateTableQuery = $this->updateTableQuery($table_data['query'], $id_table, true);

        if (!$updateTableQuery) {
            return false;
        }

        //update data in wptm_table_options table
        //get list all columns
        if (!empty($query_option['tables_list'])) {
            $query_option['columns_list'] = $this->getTableColumnsOptions($query_option['tables_list'], null);
            $table_alias = $this->getTableAliasOptions($parserMysql);
            if (count($table_alias)) {
                $query_option['table_alias'] = $table_alias;
            }
            $this->updateQueryOption($query_option, $id_table);
        }

        return $ret;
    }

    /**
     * Update query option by table id
     *
     * @param array   $query_option Query option
     * @param integer $id           Table id
     *
     * @return array|false|integer|object|void|null
     */
    public function updateQueryOption($query_option, $id)
    {
        $dbo = JFactory::getDbo();
        $name = '#__droptables_table_options';
        $query = 'SELECT t.* FROM ' . $name . ' as t WHERE t.id_table = ' . (int)$id . ' AND t.option_name = \'query_option\'';
        $dbo->setQuery($query);
        $styles = $dbo->loadObject();

        if ($styles === false || empty($styles)) {
            $query = $dbo->getQuery(true);
            $query
                ->insert($dbo->quoteName('#__droptables_table_options'))
                ->values(implode(',', array($dbo->quote($id), $dbo->quote('query_option'), $dbo->quote(json_encode($query_option)))))
                ->columns(implode(',', array('id_table', 'option_name', 'option_value')));
            $dbo->setQuery($query);
        } else {
            $query = 'UPDATE ' . $name . ' SET option_value = ' . $dbo->quote(json_encode($query_option)) . ' WHERE option_name = \'query_option\' AND id_table = ' . (int)$id;
            $dbo->setQuery($query);
        }

        if (!$dbo->execute()) {
            return false;
        }

        return true;
    }

    /**
     * Get table query
     *
     * @param integer $id_table Table id
     *
     * @return object|null
     */
    public function getTableQuery($id_table)
    {
        $db = JFactory::getDbo();
        $name = '#__droptables_table_options';
        $query = 'SELECT t.option_value FROM ' . $name . ' as t WHERE t.id_table = ' . (int)$id_table . ' AND t.option_name = \'mysql_query\'';
        $db->setQuery($query);
        $mysql_query = $db->loadObject();
        if ($mysql_query !== false) {
            if ($mysql_query !== null && is_object($mysql_query)) {
                $mysql_query = $mysql_query->option_value;//object
            } else {
                $mysql_query = null;
            }
        } else {
            $mysql_query = null;
        }

        return $mysql_query;
    }

    /**
     * Update/insert table query
     *
     * @param integer $mysql_query       Query created table
     * @param integer $id                Id table
     * @param boolean $update_when_exist Update when option exist
     *
     * @return boolean|mixed
     */
    public function updateTableQuery($mysql_query, $id, $update_when_exist = true)
    {
        if ($id !== null && !empty($mysql_query)) {
            if (!is_string($mysql_query)) {
                return false;
            }

            $dbo = $this->getDbo();
            $query = $dbo->getQuery(true);
            $header_data_old = $this->getTableQuery($id);

            if ($header_data_old) {
                if ($update_when_exist) {
                    $query->update($dbo->quoteName('#__droptables_table_options'))
                        ->set(array($dbo->quoteName('option_value') . ' = ' . $dbo->quote($mysql_query)))
                        ->where(array($dbo->quoteName('id_table') . ' = ' . (int)$id, $dbo->quoteName('option_name') . ' = ' .  $dbo->quote('mysql_query')));
                    $dbo->setQuery($query);
                }
                if (!$dbo->execute()) {
                    return false;
                }
            } else {
                $query
                    ->insert($dbo->quoteName('#__droptables_table_options'))
                    ->values(implode(',', array($dbo->quote($id), $dbo->quote('mysql_query'), $dbo->quote($mysql_query))))
                    ->columns(implode(',', array('id_table', 'option_name', 'option_value')));
                $dbo->setQuery($query);
                if (!$dbo->execute()) {
                    return false;
                }
            }
        }
        return true;
    }

    /**
     * Get table params
     *
     * @param integer $id_table   Table id
     * @param string  $param_name Name
     *
     * @return object
     */
    public function getTableParams($id_table, $param_name = '')
    {
        $dbo = $this->getDbo();

        $dbo->setQuery('Select params From #__droptables_tables '
            . ' WHERE id = ' . (int)$id_table);

        $result = $dbo->loadObject();
        if (is_string($result)) {
            $result = json_decode($result);
        }

        if (is_object($result) && !empty($result->params) && is_string($result->params)) {
            $result = json_decode($result->params);
        }

        if ($param_name === '') {
            return $result;
        } else {
            if ($result !== null) {
                if (is_array($result)) {
                    if (isset($result[$param_name])) {
                        return $result[$param_name];
                    } else {
                        return null;
                    }
                } elseif (is_object($result)) {
                    if (isset($result->{$param_name})) {
                        return $result->{$param_name};
                    } else {
                        return null;
                    }
                } else {
                    return null;
                }
            }
        }

        return $result;
    }

    /**
     * Get list table alias
     *
     * @param array $parserMysql Data query
     *
     * @return array
     */
    public function getTableAliasOptions($parserMysql)
    {
        $countTable = count($parserMysql['FROM']);
        $tables_list_alias = array();
        for ($i = 0; $i < $countTable; $i++) {
            $table_in_query[] = $parserMysql['FROM'][$i]['table'];
            if (!empty($parserMysql['FROM'][$i]['alias'])) {
                $tables_list_alias[$parserMysql['FROM'][$i]['table']] = $parserMysql['FROM'][$i]['alias']['no_quotes']['parts'][0];
            }
        }

        return $tables_list_alias;
    }

    /**
     * Get columns options
     *
     * @param array      $table_data  Data table
     * @param array|null $listColumns Data table
     *
     * @return array
     */
    public function getTableColumnsOptions($table_data, $listColumns = null)
    {
        $dbo = $this->getDbo();
        $tableColumns = array();
        $i = 0;
        foreach ($table_data as $table) {
            $columns1 = $dbo->getTableColumns($table);
            $table_columns = $dbo->loadAssocList();

            foreach ($table_columns as $column) {
                $column = $this->getColumnInTable($column);
                if ($listColumns !== null) {
                    if (in_array($column['Field'], $listColumns) || in_array($table . '.' . $column['Field'], $listColumns)) {
                        $tableColumns[$i] = $column;
                        $tableColumns[$i]['table'] = $table;
                        $i++;
                    }
                } else {
                    $tableColumns[$i] = $column;
                    $tableColumns[$i]['table'] = $table;
                    $i++;
                }
            }
        }

        return $tableColumns;
    }

    /**
     * Get column option
     *
     * @param array $column Column option raw
     *
     * @return array
     */
    public function getColumnInTable($column)
    {
        $column['priKey'] = false;
        $column['canEdit'] = 0;
        $column['notNull'] = false;
        //check key column
        switch ($column['Key']) {
            case 'MUL':
                $column['priKey'] = false;
                $column['canEdit'] = 1;
                break;
            case 'PRI':
                $column['priKey'] = true;
                $column['canEdit'] = 0;
                break;
            case 'UNI':
                $column['priKey'] = true;
                $column['canEdit'] = 1;
                break;
            default:
                $column['priKey'] = false;
                $column['canEdit'] = 1;
                break;
        }

        switch ($column['Null']) {
            case 'NO':
                $column['notNull'] = true;
                break;
            default:
                $column['notNull'] = false;
                break;
        }
        return $column;
    }

    /**
     * Get model
     *
     * @param string $name   Model name
     * @param string $prefix Prefix
     * @param array  $config Config
     *
     * @return DroptablesModelDbtable|JModelLegacy
     */
    public function getModel($name = '', $prefix = '', $config = array('ignore_request' => true))
    {
        if (!DroptablesHelper::getActions()->get('com_droptables.dbtable')) {
            $this->exitStatus(false, JText::_('COM_DROPTABLES_VIEW_DROPTABLES_ERROR_PERMISSIONS'));
        }
        require_once(JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR . 'com_droptables' . DIRECTORY_SEPARATOR . 'models' . DIRECTORY_SEPARATOR . 'dbtable.php');
        return new DroptablesModelDbtable();
    }
}
