<?php
/**
 * Droptables
 *
 * @package    Joomla.Administrator
 * @subpackage com_droptables
 *
 * @copyright Copyright (C) 2005 - 2016 Open Source Matters, Inc. All rights reserved.
 * @license   GNU General Public License version 2 or later; see LICENSE.txt
 */
defined('JPATH_BASE') || die;

/**
 * Class JFormFieldModaltable Supports a modal table picker.
 */
class JFormFieldModaltable extends JFormField
{
    /**
     * The form field type.
     *
     * @var string
     */
    protected $type = 'Modaltable';

    /**
     * Method to get the field input markup.
     *
     * @return string The field input markup.
     */
    protected function getInput()
    {
        $allowEdit  = ((string) $this->element['edit'] === 'true') ? true : false;
        $allowClear = ((string) $this->element['clear'] !== 'false') ? true : false;

        // Load language
        JFactory::getLanguage()->load('com_droptables', JPATH_ADMINISTRATOR);

        // The active table id field.
        $value = (int) $this->value > 0 ? (int) $this->value : '';

        // Build the script.
        $script = array();

        // Select button script
        $script[] = '	function jSelectTable_' . $this->id . '(id, title, catid, object) {';
        $script[] = '		document.getElementById("' . $this->id . '_id").value = id;';
        $script[] = '		document.getElementById("' . $this->id . '_name").value = title;';

        if ($allowEdit) {
            $script[] = '		if (id == "' . $value . '") {';
            $script[] = '			jQuery("#' . $this->id . '_edit").removeClass("hidden");';
            $script[] = '		} else {';
            $script[] = '			jQuery("#' . $this->id . '_edit").addClass("hidden");';
            $script[] = '		}';
        }

        if ($allowClear) {
            $script[] = '		jQuery("#' . $this->id . '_clear").removeClass("hidden");';
        }

        $script[] = '		jQuery("#tableSelect' . $this->id . 'Modal").modal("hide");';

        if ($this->required) {
            $script[] = '		document.formvalidator.validate(document.getElementById("' . $this->id . '_id"));';
            $script[] = '		document.formvalidator.validate(document.getElementById("' . $this->id . '_name"));';
        }

        $script[] = '	}';

        // Edit button script
        $script[] = '	function jEditTable_' . $value . '(title) {';
        $script[] = '		document.getElementById("' . $this->id . '_name").value = title;';
        $script[] = '	}';

        // Clear button script
        static $scriptClear;

        if ($allowClear && !$scriptClear) {
            $scriptClear = true;

            $script[] = '	function jClearTable(id) {';
            $script[] = '		document.getElementById(id + "_id").value = "";';
            $script[] = '		document.getElementById(id + "_name").value = "' .
                        htmlspecialchars(JText::_('COM_DROPTABLES_SELECT_AN_TABLE', true), ENT_COMPAT, 'UTF-8') . '";';
            $script[] = '		jQuery("#"+id + "_clear").addClass("hidden");';
            $script[] = '		if (document.getElementById(id + "_edit")) {';
            $script[] = '			jQuery("#"+id + "_edit").addClass("hidden");';
            $script[] = '		}';
            $script[] = '		return false;';
            $script[] = '	}';
        }
        // Add the script to the document head.
        JHtml::_('jquery.framework');
        JFactory::getDocument()->addScriptDeclaration(implode("\n", $script));

        // Setup variables for display.
        $html = array();

        $linkTables = 'index.php?option=com_droptables&amp;view=tables&amp;layout=modal&amp;tmpl=component'
                      . '&amp;function=jSelectTable_' . $this->id;

        $linkTable = 'index.php?option=com_droptables&amp;view=table&amp;layout=modal&amp;tmpl=component'
                     . '&amp;task=table.edit'
                     . '&amp;function=jEditTable_' . $value;

        if (isset($this->element['language'])) {
            $linkTables .= '&amp;forcedLanguage=' . $this->element['language'];
            $linkTable  .= '&amp;forcedLanguage=' . $this->element['language'];
        }

        $urlSelect = $linkTables . '&amp;' . JSession::getFormToken() . '=1';
        $urlEdit   = $linkTable . '&amp;id=' . $value . '&amp;' . JSession::getFormToken() . '=1';

        if ($value) {
            $db    = JFactory::getDbo();
            $query = $db->getQuery(true)
                        ->select($db->quoteName('title'))
                        ->from($db->quoteName('#__droptables_tables'))
                        ->where($db->quoteName('id') . ' = ' . (int) $value);
            $db->setQuery($query);

            try {
                $title = $db->loadResult();
            } catch (RuntimeException $e) {
                JError::raiseWarning(500, $e->getMessage());
            }
        }

        if (empty($title)) {
            $title = JText::_('COM_DROPTABLES_SELECT_AN_TABLE');
        }

        $title = htmlspecialchars($title, ENT_QUOTES, 'UTF-8');

        // The current table display field.
        $html[] = '<span class="input-append">';
        $html[] = '<input class="input-medium" id="' . $this->id . '_name" type="text" value="' . $title . '" disabled="disabled" size="35" />';

        // Select table button
        if (version_compare($this->getJoomlaVersion(), '4.0') >= 0) {
            $html[] = '<a  data-toggle="modal" data-target="#ju-btn-connect"  data-bs-toggle="modal" data-bs-target="#tableSelect'.$this->id .'Modal"
             class="btn btn-primary button-select" title="'. JText::_('COM_DROPTABLES_CHANGE_TABLE').'"><span class="icon-file"></span>'.JText::_('JSELECT').'</a>';
        } else {
            $html[] = '<a'
                . ' class="btn hasTooltip"'
                . ' data-toggle="modal"'
                . ' role="button"'
                . ' href="#tableSelect' . $this->id . 'Modal"'
                . ' title="' . JHtml::tooltipText('COM_DROPTABLES_CHANGE_TABLE') . '">'
                . '<span class="icon-file"></span> ' . JText::_('JSELECT')
                . '</a>';
        }

        // Edit table button
        if ($allowEdit) {
            $html[] = '<a'
                      . ' class="btn hasTooltip' . ($value ? '' : ' hidden') . '"'
                      . ' id="' . $this->id . '_edit"'
                      . ' data-toggle="modal"'
                      . ' role="button"'
                      . ' href="#tableEdit' . $value . 'Modal"'
                      . ' title="' . JHtml::tooltipText('COM_DROPTABLES_EDIT_TABLE') . '">'
                      . '<span class="icon-edit"></span> ' . JText::_('JACTION_EDIT')
                      . '</a>';
        }

        // Clear table button
        if ($allowClear) {
            $html[] = '<button'
                      . ' class="btn' . ($value ? '' : ' hidden') . '"'
                      . ' id="' . $this->id . '_clear"'
                      . ' onclick="return jClearTable(\'' . $this->id . '\')">'
                      . '<span class="icon-remove"></span>' . JText::_('JCLEAR')
                      . '</button>';
        }

        $html[] = '</span>';

        // Select table modal
        $html[] = JHtml::_(
            'bootstrap.renderModal',
            'tableSelect' . $this->id . 'Modal',
            array(
                'title'      => JText::_('COM_DROPTABLES_CHANGE_TABLE'),
                'url'        => $urlSelect,
                'height'     => '400px',
                'width'      => '800px',
                'bodyHeight' => '70',
                'modalWidth' => '80',
                'footer'     => '<button type="button" class="btn" data-dismiss="modal" aria-hidden="true">'
                                . JText::_('JLIB_HTML_BEHAVIOR_CLOSE') . '</button>',
            )
        );

        // Edit table modal
        $html[] = JHtml::_(
            'bootstrap.renderModal',
            'tableEdit' . $value . 'Modal',
            array(
                'title'       => JText::_('COM_DROPTABLES_EDIT_TABLE'),
                'backdrop'    => 'static',
                'keyboard'    => false,
                'closeButton' => false,
                'url'         => $urlEdit,
                'height'      => '400px',
                'width'       => '800px',
                'bodyHeight'  => '70',
                'modalWidth'  => '80',
                'footer'      => '<button type="button" class="btn" data-dismiss="modal" aria-hidden="true"'
                                 . ' onclick="jQuery(\'#tableEdit' . $value . 'Modal iframe\').contents().find(\'#closeBtn\').click();">'
                                 . JText::_('JLIB_HTML_BEHAVIOR_CLOSE') . '</button>'
                                 . '<button type="button" class="btn btn-primary" aria-hidden="true"'
                                 . ' onclick="jQuery(\'#tableEdit' . $value . 'Modal iframe\').contents().find(\'#saveBtn\').click();">'
                                 . JText::_('JSAVE') . '</button>'
                                 . '<button type="button" class="btn btn-success" aria-hidden="true"'
                                 . ' onclick="jQuery(\'#tableEdit' . $value . 'Modal iframe\').contents().find(\'#applyBtn\').click();">'
                                 . JText::_('JAPPLY') . '</button>',
            )
        );

        // Note: class='required' for client side validation.
        $class = $this->required ? ' class="required modal-value"' : '';

        $html[] = '<input type="hidden" id="' . $this->id . '_id"' . $class . ' name="' . $this->name . '" value="' . $value . '" />';


        $doc = JFactory::getDocument();
        $doc->addStyleDeclaration('.icon-file:before {margin-right: 10px;}');

        return implode("\n", $html);
    }

    /**
     * Method to return the current joomla version
     *
     * @param string $format Format value to version
     *
     * @return string version
     */
    protected function getJoomlaVersion($format = 'short')
    {
        $method = 'get' . ucfirst($format) . 'Version';

        // Get the joomla version
        $instance = new JVersion();
        $version  = call_user_func(array($instance, $method));

        return $version;
    }

    /**
     * Method to get the field label markup.
     *
     * @return string The field label markup.
     *
     * @since 3.4
     */
    protected function getLabel()
    {
        return str_replace($this->id, $this->id . '_id', parent::getLabel());
    }
}
