<?php
/**
 * Droptables
 *
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 *
 * @package   Droptables
 * @copyright Copyright (C) 2014 JoomUnited (http://www.joomunited.com). All rights reserved.
 * @copyright Copyright (C) 2014 Damien Barrère (http://www.crac-design.com). All rights reserved.
 * @license   GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 */

defined('_JEXEC') || die;

jimport('joomla.application.component.modeladmin');

/**
 * Class DroptablesModelTable
 */
class DroptablesModelTable extends JModelAdmin
{
    /**
     * Old params
     *
     * @var string $paramsOld Table params
     */
    public $paramsOld = '';

    /**
     * Returns a Table object, always creating it.
     *
     * @param string $type   The table type to instantiate
     * @param string $prefix A prefix for the table class name. Optional.
     * @param array  $config Configuration array for model. Optional.
     *
     * @return JTable A database object
     */
    public function getTable($type = 'Table', $prefix = 'DroptablesTable', $config = array())
    {
        return JTable::getInstance($type, $prefix, $config);
    }

    /**
     * Get form
     *
     * @param array   $data     Data
     * @param boolean $loadData Data loader
     *
     * @return boolean
     */
    public function getForm($data = array(), $loadData = true)
    {
        $form = $this->loadForm('com_droptables.table', 'table', array('control' => 'jform', 'load_data' => $loadData));
        if (empty($form)) {
            return false;
        }
        return $form;
    }

    /**
     * Get cells data
     *
     * @param integer $id      Table id
     * @param array   $filters Filter in query
     *
     * @return array|boolean|mixed
     */
    public function getDataCell($id, $filters = array())
    {
        $db = JFactory::getDbo();
        $name = '#__droptable_tbl_' . $id;
        $list_table = $this->getTableData("SHOW TABLES LIKE '%_droptable_tbl_" . $id . "'");
        if (count($list_table) < 1) {
            return false;
        }
//        $db->setQuery('Select t.* From #__droptable_tbl_' . $id . ' As t ');

        $columns = array();
        if (!isset($filters['cols']) || (isset($filters['cols']) && $filters['cols'] === null)) {
            $columns_query = 'SHOW COLUMNS FROM ' . $name;

            $db->setQuery($columns_query);
            $columns_obj = $db->loadObjectList();

            foreach ($columns_obj as $column_obj) {
                if ($column_obj->Field === 'ID' || $column_obj->Field === 'line') {
                    continue;
                }
                $col_index = (int)substr($column_obj->Field, 3) - 1;
                array_push($columns, $column_obj->Field . ' as `a' . $col_index . '`');
            }
        } else {
            array_push($columns, 'line');
            if (isset($filters['cols']) && is_array($filters['cols']) && count($filters['cols'])) {
                foreach ($filters['cols'] as $column) {
                    array_push($columns, 'col' . ($column + 1) . ' as `a' . $column . '`');
                }
            }
        }

        if (count($columns) < 1) {
            return false;
        }

        $limitQuery = '';

        $limit = !empty($filters['limit']) ? $filters['limit'] : -1;
        $page = isset($filters['page']) ? $filters['page'] : 1;
        $headerOffset = isset($filters['headerOffset']) ? (int)$filters['headerOffset'] : 1;

        $where = (isset($filters['where']) && $filters['where'] !== '') ? ' WHERE ' . $filters['where'] : '';
        $order = (isset($filters['order']) && $filters['order'] !== '') ? ' ORDER BY ' . $filters['order'] : ' ORDER BY line ASC';

        if ($limit > -1) {
            if ($page === 1) {
                $offset = 0;
                if ($headerOffset > 0 && $where === '') {
                    $limit -= $headerOffset;
                }
            } else {
                $offset = ($page - 1) * $limit;
                if ($headerOffset > 0) {
                    $offset -= $headerOffset;
                }
            }
            $limitQuery = ' LIMIT ' . intval($offset) . ', ' . intval($limit);
            if ($headerOffset > 0) {
                if ($where !== '') {
                    $where .= ' AND line > ' . ($headerOffset - 1);
                } else {
                    $where = ' WHERE line > ' . ($headerOffset - 1);
                }
            }
        }
        $select = implode(',', $columns);

        if (isset($filters['getLine']) && $filters['getLine'] === true) {
            $select = ' line as DT_RowId, ' . $select;
        }
        $query = 'SELECT ' . $select . ' FROM ' . $name . $where . $order . $limitQuery;
        $db->setQuery($query);
        $data = $db->loadObjectList();

        if (!is_array($data) || empty($data) || is_null($data)) {
            return false;
        }

        if (isset($filters['getLine']) && $filters['getLine'] === true) {
            // Include line as DT_RowId, use for ajax call. You will need to reset array key in use
            $data = array_map(function ($cells) {
                $cells = (array)$cells;
                $DT_RowId = isset($cells['DT_RowId']) ? $cells['DT_RowId'] : false;

                if (!$DT_RowId) {
                    return $cells;
                }
                unset($cells['DT_RowId']);
                $newCells = array_values($cells);
                $newCells['DT_RowId'] = $DT_RowId;

                return $newCells;
            }, $data);
        } else {
            foreach ($data as $k => $v) {
                $v = (array)$v;
                $data[$k] = array_values($v);
            }
        }
        return $data;
    }

    /**
     * Get result style cells of table
     *
     * @param integer $id           Id table
     * @param integer $numberRows   Number row
     * @param integer $numberColumn Number column
     *
     * @return array|boolean
     */
    public function getCellsStyle($id, $numberRows, $numberColumn)
    {
        $db = JFactory::getDbo();
        $name = '#__droptables_range_styles';
        $query = 'SELECT row_start, row_end, col_start, col_end, style FROM ' . $name . ' as t WHERE t.id_table = ' . (int)$id . ' Order by t.id ASC';
        $db->setQuery($query);
        $styles = $db->loadObjectList();
        if ($styles === false) {
            return false;
        }

        $cell_styles = $this->convertRangeStyleToCellStyle($styles, array('row' => $numberRows, 'column' => $numberColumn));
        return $cell_styles;
    }

    /**
     * Merge style cell
     *
     * @param object $item Table data
     *
     * @return mixed
     */
    public function mergeCellTypesIntoCells($item)
    {
        $cell_types = !isset($item->params->cell_types) || $item->params->cell_types === null ? array() : (array)$item->params->cell_types;
        $count = count($cell_types);

        for ($ii = 0; $ii < $count; $ii++) {
            $cell_type = $cell_types[$ii];
            if (isset($cell_type[2])) {
                for ($i = 0; $i < $cell_type[2]; $i++) {
                    for ($j = 0; $j < $cell_type[3]; $j++) {
                        $k = ($cell_type[0] + $i) . '!' . ($cell_type[1] + $j);
                        if (!isset($item->style->cells[$k])) {
                            $item->style->cells[$k] = array();
                            $item->style->cells[$k][0] = $cell_type[0] + $i;
                            $item->style->cells[$k][1] = $cell_type[1] + $j;
                            $item->style->cells[$k][2] = array();
                        }

                        if ($cell_type[4] === 'html') {
                            $item->style->cells[$k][2]['cell_type'] = 'html';
                        } else {
                            $item->style->cells[$k][2]['cell_type'] = '';
                        }
                    }
                }
            }
        }
        return $item->style->cells;
    }

    /**
     * Convert current object to js format
     *
     * @param object $data_styles Data cells style
     * @param array  $count       Number row and column
     *
     * @return array|boolean
     */
    public function convertRangeStyleToCellStyle($data_styles, $count)
    {
        if (empty($data_styles)) {
            return array();
        }
        $styles = array();
        if (!isset($count['row'])) {
            $count['row'] = 10;
        }
        if (!isset($count['column'])) {
            $count['column'] = 10;
        }

        foreach ($data_styles as $style) {
            $style = (array) $style;
            $style['style'] = str_replace(array("\r\n", "\n"), '', $style['style']);
            $row_end = 0 === (int)$style['row_end'] ? $count['row'] : $style['row_end'];
            $col_end = 0 === (int)$style['col_end'] ? $count['column'] : $style['col_end'];
            $style_ranger = json_decode($style['style'], true);
            $new_data = json_decode($style['style']);
            for ($i = $style['row_start'] - 1; $i <= $row_end - 1; $i++) {
                for ($j = $style['col_start'] - 1; $j <= $col_end - 1; $j++) {
                    $index = $i . '!' . $j;
                    $old_data = isset($styles[$index]) ? $styles[$index] : null;
                    if ($old_data === null) {
                        $styles[$index] = array((int)$i, (int)$j, $style_ranger);
                    } else {
                        $old_data_style = $old_data[2];
                        $merged_data = array_merge((array) $old_data_style, (array) $new_data);
                        $styles[$index] = array((int)$i, (int)$j, $merged_data);
                    }
                }
            }
        }

        return $styles;
    }

    /**
     * Get header data
     *
     * @param integer|null $id Id table
     *
     * @return boolean|mixed
     */
    public function getHeaderData($id = null)
    {
        if ($id !== null) {
            $db = JFactory::getDbo();
            $name = '#__droptables_table_options';
            $query = 'SELECT t.option_value FROM ' . $name . ' as t WHERE t.id_table = ' . (int)$id . ' AND t.option_name = \'header_data\'';
            $db->setQuery($query);
            $styles = $db->loadObject();
            if ($styles === false) {
                return false;
            }
            return $styles;
        }
        return false;
    }

    /**
     * Update/insert header content
     *
     * @param integer $header_data Header content
     * @param integer $id          Id table
     *
     * @return boolean|mixed
     */
    public function updateHeaderData($header_data, $id)
    {
        if ($id !== null && !empty($header_data)) {
            if (!is_string($header_data)) {
                $header_data = json_encode($header_data);
            }

            $dbo = $this->getDbo();
            $query = $dbo->getQuery(true);
            $header_data_old = $this->getHeaderData($id);

            if ($header_data_old) {
                $query->update($dbo->quoteName('#__droptables_table_options'))
                    ->set(array($dbo->quoteName('option_value') . ' = ' . $dbo->quote($header_data)))
                    ->where(array($dbo->quoteName('id_table') . ' = ' . (int)$id, $dbo->quoteName('option_name') . ' = ' .  $dbo->quote('header_data')));
                $dbo->setQuery($query);
            } else {
                $query
                    ->insert($dbo->quoteName('#__droptables_table_options'))
                    ->values(implode(',', array($dbo->quote($id), $dbo->quote('header_data'), $dbo->quote($header_data))))
                    ->columns(implode(',', array('id_table', 'option_name', 'option_value')));
                $dbo->setQuery($query);
            }

            if (!$dbo->execute()) {
                return false;
            }
        }
        return true;
    }

    /**
     * Get item
     *
     * @param null|string $id             Item name
     * @param boolean     $get_data       Get data
     * @param boolean     $get_style_cell Get style cells
     * @param boolean     $get_hash       Get hash
     * @param boolean     $limit          Get limit rows
     * @param boolean     $getPriKeyTable Get base column for row identification
     *
     * @return mixed
     */
    public function getItem($id = null, $get_data = true, $get_style_cell = true, $get_hash = false, $limit = false, $getPriKeyTable = false)
    {
        $item = new stdClass();
        $dbo = $this->getDbo();

        if ($get_hash) {
            $dbo->setQuery('Select t.id_category, t.title, t.style, t.css, t.params, t.author, t.type, t.hash From #__droptables_tables As t '
                . ' WHERE t.id = ' . (int)$id);
        } else {
            $dbo->setQuery('Select t.id_category, t.title, t.style, t.css, t.params, t.author, t.type From #__droptables_tables As t '
                . ' WHERE t.id = ' . (int)$id);
        }

        $result = $dbo->loadObject();
        if (!$result || is_null($result)) {
            return false;
        }

        $item->title = $result->title;
        $item->author = $result->author;
        if ($get_hash) {
            $item->hash = $result->hash;
        }
        $item->category = $result->id_category;
        $item->type = $result->type;
        $item->css = preg_replace('/\\\n/', '', $result->css);

        if ($result->style === '') {
            $item->style = new stdClass();
        } else {
            $item->style = json_decode($result->style);
        }

        if (isset($result->params)) {
            $item->params = str_replace(array("\n\r", "\r\n", "\n", "\r", '&#10;'), ' ', $result->params);
            $item->params = json_decode($item->params);
        } else {
            $item->params = new stdClass();
        }

        require_once(JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR . 'com_droptables' . DIRECTORY_SEPARATOR . 'models' . DIRECTORY_SEPARATOR . 'dbtable.php');
        $modelDbTable = new DroptablesModelDbtable();
        $mysql_query = $modelDbTable->getTableQuery($id);

        if (isset($item->params->query) && empty($mysql_query)) {
            $item->params->query = str_replace(array("\n\r", "\r\n", "\n", "\r", '&#10;'), ' ', $item->params->query);
        } elseif (!empty($mysql_query)) {
            $item->params->query = str_replace(array("\n\r", "\r\n", "\n", "\r", '&#10;'), ' ', $mysql_query);
        }

        if (isset($item->params->getPriKeyTableQuery)) {
            $item->params->getPriKeyTableQuery = str_replace(array("\n\r", "\r\n", "\n", "\r", '&#10;'), ' ', $item->params->getPriKeyTableQuery);
        }

        if ((isset($item->params->table_type) && $item->params->table_type === 'mysql') || $item->type !== 'html') {
            $item->params->col_types = array();
            require_once(JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR . 'com_droptables' . DIRECTORY_SEPARATOR . 'models' . DIRECTORY_SEPARATOR . 'dbtable.php');
            $modelDbTable = new DroptablesModelDbtable();

            if (empty($item->query_option)) {
                $query_option = $modelDbTable->getQueryOption($id);
                if (!empty($query_option->option_value) && is_string($query_option->option_value)) {
                    $item->query_option = json_decode($query_option->option_value);
                }
            }

            if (empty($item->query_option) && !empty($item->params->mysql_columns)) {
                $item->query_option = new stdClass();
                $item->query_option->tables_list = $item->params->tables;
                $item->query_option->column_options = $modelDbTable->getTableColumnsOptions($item->params->tables, $item->params->mysql_columns);
                $item->query_option->columns_list = $modelDbTable->getTableColumnsOptions($item->params->tables, null);
            }
        }

        if (!empty($item->style->table)) {
            $item->style->table = (object) array_merge((array) $item->params, (array) $item->style->table);
        } else {
            $item->style->table = $item->params;
        }

        if ($get_data) {
            if (isset($item->params->query) && $item->params->query !== '') {
                $query = $getPriKeyTable ? $item->params->getPriKeyTableQuery : $item->params->query;

                if (isset($item->params->placeholder_query) && (int)$item->params->placeholder_query === 1) {
                    if (strpos($query, '{user_id}') !== false) {
                        $id_user = (int)JFactory::getUser()->id;
                        $query = str_replace('{user_id}', ' ' . $id_user, $query);
                    }

                    if (strpos($query, '{user_email}') !== false) {
                        $email_user = JFactory::getUser()->email;
                        $query = str_replace('{user_email}', ' \'@' . $email_user . '@\'', $query);
                    }
                }

                $dataCell = $this->getTableData($query);

                if (!empty($dataCell[0])) {
                    if (gettype($dataCell) === 'array' && !empty($dataCell)) {
                        foreach ($dataCell as $key => $datas) {
                            $dataCell[$key] = array_values($datas);
                            $header_from_query = array_keys($datas);
                        }
                    }
                } else {
                    require_once(JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR . 'com_droptables' . DIRECTORY_SEPARATOR . 'classes' . DIRECTORY_SEPARATOR . 'php-sql-parser' . DIRECTORY_SEPARATOR . 'vendor' . DIRECTORY_SEPARATOR . 'autoload.php');
                    $parser = new \PHPSQLParser\PHPSQLParser(false, true);
                    $query = str_replace(array("\n\r", "\r\n", "\n", "\r", '&#10;'), ' ', $query);
                    $query = str_replace('\\', '', $query);
                    $parserMysql = $parser->parse($query, true);

                    //list columns
                    //$parserMysql['SELECT'];//array
                    $count = count($parserMysql['SELECT']);
                    $dataCell[0] = array();
                    for ($i = 0; $i < $count; $i++) {
                        if (!isset($parserMysql['SELECT'][$i]['alias']['name'])) {
                            $nameColumn = explode('.', $parserMysql['SELECT'][$i]['base_expr']);
                            if (!empty($nameColumn[1])) {
                                $dataCell[0][] = $nameColumn[1];
                            } else {
                                $dataCell[0][] = $parserMysql['SELECT'][$i]['base_expr'];
                            }
                        } else {
                            $dataCell[0][] = $parserMysql['SELECT'][$i]['alias']['name'];
                        }
                    }
                }
            } else {
                $dataCell = $this->getDataCell($id, array('limit' => $limit));
            }
        }

        if (!empty($dataCell) && is_array($dataCell)) {
            $item->data = $dataCell;
            $numberRows = count($dataCell);
            $numberColumn = count($dataCell[0]);
        } else {
            $item->data = false;
            $numberRows = 0;
            $numberColumn = 0;
        }

        if (!isset($item->style->table->headerOption)) {
            $item->style->table->headerOption = 1;
        }

        if ($item->data !== false && isset($item->style->table->headerOption) && (int)$item->style->table->headerOption > 0) {
            $header_data = $this->getHeaderData($id);
            if ($header_data === null && (int)$item->style->table->headerOption > 0) {
                if ($item->type === 'mysql' && isset($item->params->custom_titles)) {
                    $header_data = array($item->params->custom_titles);
                }
            }

            if ($header_data !== null) {
                if (isset($header_data->option_value)) {
                    $header_data = json_decode($header_data->option_value);
                }
                $item->style->header_data = $header_data;
                unset($item->style->table->header_data);
                unset($item->params->header_data);

                if (!isset($parserMysql)) {
                    for ($i = 0; $i < $item->style->table->headerOption; $i++) {
                        if (isset($header_data[$i]) && (count($header_data[$i]) < count($item->data[$i]))) {
                            if ($i === 0 && isset($item->params->custom_titles) && is_array($item->params->custom_titles)) {
                                $header_data[$i] = $header_data[$i] + $item->params->custom_titles;
                            }
                            $header_data[$i] = $header_data[$i] + $item->data[$i];
                        }

                        if (isset($header_data[$i])) {
                            if ($item->type === 'mysql') {
                                array_unshift($item->data, $header_data[$i]);
                            } else {
                                $item->data[$i] = $header_data[$i];
                            }
                        }
                    }
                } else {
                    $item->data[0] = $header_data[0];
                }
            } elseif (isset($item->query_option) && isset($item->query_option->column_options)) {
                $item->style->header_data = array();
                $item->style->header_data[0] = array();

                if (!empty($header_from_query) && count($header_from_query) === count($item->data[0])) {
                    $item->style->header_data[0] = $header_from_query;
                } else {
                    foreach ($item->query_option->column_options as $column_option) {
                        $item->style->header_data[0][] = $column_option->table . '.' . $column_option->Field;
                    }
                }
                //get header from query, no data row
                if (!isset($parserMysql)) {
                    array_unshift($item->data, $item->style->header_data[0]);
                } else {
                    $item->style->header_data[0] = $item->data[0];
                }
            }
        }

        if ($get_style_cell) {
            $item->style->cells = new stdClass();
            $item->style->cells = $this->getCellsStyle($id, $numberRows, $numberColumn);
            $item->style->cells = $this->mergeCellTypesIntoCells($item);
        }

        //remove stripslashes_deep for $item because limit memories
//        $this->stripslashesDeep($item->style->table);
        $item->params = $this->stripslashesDeep($item->params);
        $item->style->table = $this->stripslashesDeep($item->style->table);
        return $item;
    }
    /**
     * Run stripslashes in array
     *
     * @param mixed $variable Data
     *
     * @return mixed
     */
    public function stripslashesDeep($variable)
    {
        if (is_string($variable)) {
            return stripslashes($variable);
        } elseif (is_array($variable)) {
            foreach ($variable as $i => $value) {
                $variable[$i] = $this->stripslashesDeep($value);
            }
        } elseif (is_object($variable)) {
            $keys = array_keys((array)$variable);
            foreach ($keys as $key) {
                $variable->{$key} = $this->stripslashesDeep($variable->{$key});
            }
        }

        return $variable;
    }

    /**
     * Get table data
     *
     * @param string $query Query select
     *
     * @return mixed
     */
    public function getTableData($query)
    {
        $dbo = $this->getDbo();
        $dbo->setQuery($query);
        $reuslts = $dbo->loadAssocList();

        return $reuslts;
    }

    /**
     * Get all rows
     *
     * @param string $query Query get all rows
     *
     * @return mixed
     */
    public function getTotalRows($query)
    {
        $dbo = $this->getDbo();
        $dbo->setQuery($query);

        $dbo->execute();
        $num_rows = $dbo->getNumRows();

        return $num_rows;
    }

    /**
     * Load form table
     *
     * @return mixed
     */
    protected function loadFormData()
    {
        // Check the session for previously entered form data.
        $data = $this->getItem();
        return $data;
    }

    /**
     * Add tables in category
     *
     * @param string|integer $id_category  Id category
     * @param string|null    $name         Name table
     * @param integer|null   $id_table     Id table
     * @param string|null    $created_time Creat table time
     * @param integer|null   $author       Author
     * @param string|null    $type         Table type
     *
     * @return boolean|array
     */
    public function add($id_category, $name = null, $id_table = null, $created_time = null, $author = null, $type = null)
    {
        $defaultColTypes =  array(
            'varchar(255)',
            'varchar(255)',
            'varchar(255)',
            'varchar(255)',
            'varchar(255)',
            'varchar(255)',
            'varchar(255)',
            'varchar(255)',
            'varchar(255)',
            'varchar(255)'
        );

        $dbo = $this->getDbo();
        $name = $name === null ? JText::_('COM_DROPTABLES_MODEL_TABLE_DEFAULT_TITLE') : $name;
        $params = new stdClass();
        $params->mergeSetting = array();
        $params->hyperlink = new stdClass();
        $params->headerOption = 1;
        $params->col_types = $defaultColTypes;
        $params = json_encode($params);
        $style = new stdClass();
        $style->cols = new stdClass();
        $style->rows = new stdClass();
        for ($i = 0; $i < 10; $i++) {
            $height = new stdClass();
            $height->height = 24;
            $height->cell_border_left = '1px solid #cfcfcf';
            $height->cell_border_right = '1px solid #cfcfcf';
            $style->cols->{$i} = array($i, $height);
        }
        for ($i = 0; $i < 11; $i++) {
            $height = new stdClass();
            $height->height = 24;
            $height->cell_border_top = '1px solid #cfcfcf';
            $height->cell_border_bottom = '1px solid #cfcfcf';
            $style->rows->{$i} = array($i, $height);
        }
        $style = json_encode($style);

        if ($id_table !== null) {
            $values = array($id_table, (int)$id_category, $dbo->quote($name), $dbo->quote($created_time), $dbo->quote(date('Y-m-d H:i:s')), (int)$author,  $dbo->quote($style),  $dbo->quote($params),  $dbo->quote($type), $dbo->quote(''));
            $query = $dbo->getQuery(true);
            $query
                ->insert('#__droptables_tables')
                ->columns(array('id', 'id_category', 'title', 'created_time', 'modified_time', 'author', 'style', 'params', 'type', 'css'))
                ->values(implode(',', $values));
            $dbo->setQuery($query);
        } else {
            $values = array((int)$id_category, $dbo->quote($name), $dbo->quote(date('Y-m-d H:i:s')), $dbo->quote(date('Y-m-d H:i:s')), (int)JFactory::getUser()->id,  $dbo->quote($style),  $dbo->quote($params),  $dbo->quote('html'), $dbo->quote(''));
            $query = $dbo->getQuery(true);
            $query
                ->insert('#__droptables_tables')
                ->columns(array('id_category', 'title', 'created_time', 'modified_time', 'author', 'style', 'params', 'type', 'css'))
                ->values(implode(',', $values));
            $dbo->setQuery($query);
        }

        if (!$dbo->execute()) {
            return false;
        }

        $columns = array();
        for ($i = 1; $i <= 10; $i++) {
            $columnDefault = array('name' => 'col' . $i, 'type' => 'string');
            $columns[] = $columnDefault;
        }

        $insert_id = $dbo->insertid();
        $this->createTblInDb('droptable_tbl_' . $insert_id, $columns);

        $dbo->setQuery(
            'insert INTO #__droptable_tbl_' . $insert_id . " (`line`, `col1`, `col2`, `col3`, `col4`, `col5`, `col6`, `col7`, `col8`, `col9`, `col10`) VALUES
             (0, '', '', '', '', '', '', '', '', '', ''),
            (1, '', '', '', '', '', '', '', '', '', ''),
            (2, '', '', '', '', '', '', '', '', '', ''),
            (3, '', '', '', '', '', '', '', '', '', ''),
            (4, '', '', '', '', '', '', '', '', '', ''),
            (5, '', '', '', '', '', '', '', '', '', ''),
            (6, '', '', '', '', '', '', '', '', '', ''),
            (7, '', '', '', '', '', '', '', '', '', ''),
            (8, '', '', '', '', '', '', '', '', '', ''),
            (9, '', '', '', '', '', '', '', '', '', ''),
            (10, '', '', '', '', '', '', '', '', '', '')"
        );
        if (!$dbo->execute()) {
            return false;
        }
//        $query = $dbo->getQuery(true);
//        $table_name = '#__droptables_range_styles';
//        $values = array($insert_id, 1, 0, 1, 0, $dbo->quote('{"height": "24px"}'));
//        $query
//            ->insert($table_name)
//            ->columns(array('id_table', 'row_start', 'row_end', 'col_start', 'col_end', 'style'))
//            ->values(implode(',', $values));
//        $dbo->setQuery($query);
//        if (!$dbo->execute()) {
//            return false;
//        }
        if ($id_table !== null) {
            return true;
        }
        return array($insert_id, date('Y-m-d H:i:s'), (int)JFactory::getUser()->id);
    }

    /**
     * Get lat ranger style of table
     *
     * @param string $id_table Table id
     *
     * @return array|boolean|object|void|null
     */
    public function getTableLastRange($id_table = '')
    {
        if ($id_table === '') {
            return false;
        }

        $dbo = $this->getDbo();
        $dbo->setQuery('Select * From #__droptables_range_styles '
            . ' WHERE id_table = ' . $id_table . ' ORDER BY id DESC LIMIT 0,1');

        $result = $dbo->loadObject();

        if ($result !== null) {
            return $result;
        } else {
            return null;
        }
    }

    /**
     * Create table in database
     *
     * @param integer $tblName Id table
     * @param array   $columns Column data
     *
     * @return boolean|integer
     */
    public function createTblInDb($tblName, $columns)
    {
        $dbo = $this->getDbo();

        $sql = 'CREATE TABLE #__' . $tblName . ' (
                        ID int(11) NOT NULL AUTO_INCREMENT,
                        line int(11) NOT NULL,';

        foreach ($columns as $column) {
            $sql .= $this->sqlCreateColumn($column) . ', ';
        }

        $sql .= 'PRIMARY KEY  (id) 
                ) ENGINE=MyISAM  DEFAULT CHARSET=utf8mb4 ;';
        // echo $sql; die();
        $dbo->setQuery($sql);
        if (!$dbo->execute()) {
            $this->deleteTblInDb('#__' . $tblName);
            throw new Exception(JText::_('COM_DROPTABLES_MODEL_TABLE_COPY'));
        }

        return true;
    }

    /**
     * Delete a table in database
     *
     * @param string $tblName Name table
     *
     * @return boolean
     */
    public function deleteTblInDb($tblName)
    {
        $dbo = $this->getDbo();
        if (is_int($tblName)) {
            $tblName = $dbo->quoteName('#__droptable_tbl_' . $tblName);
        }
        $dbo->setQuery(
            'DROP TABLE IF EXISTS ' . $tblName
        );

        $result = $dbo->execute();
        return $result;
    }

    /**
     * Create columns for new table
     *
     * @param array $column Column data
     *
     * @return string
     */
    public function sqlCreateColumn($column)
    {
        $name = $column['name'];
        switch ($column['type']) {
            case 'int':
                $sql_block = $name . ' INT(11) ';
                break;

            case 'float':
                $sql_block = $name . ' DECIMAL(16,4) ';
                break;

            case 'date':
                $sql_block = $name . ' DATE ';
                break;

            case 'datetime':
                $sql_block = $name . ' DATETIME ';
                break;

            case 'text':
                $sql_block = $name . ' TEXT ';
                break;

            case 'mediumtext':
                $sql_block = $name . ' MEDIUMTEXT ';
                break;

            default:
                $sql_block = $name . ' VARCHAR(255) ';
                break;
        }

        return $sql_block;
    }

    /**
     * Copy table
     *
     * @param integer|string $id Id table
     *
     * @return boolean|null
     */
    public function copy($id)
    {
        $table = $this->getTable();
        if ($table->load($id)) {
            $table->id = null;
            $table->title = $table->title . JText::_('COM_DROPTABLES_MODEL_TABLE_COPY');
            $table->store();
            $new_id = $table->id;

            //copy style range
            $db = JFactory::getDbo();
            $query = 'SELECT * FROM #__droptables_range_styles as t WHERE t.id_table = ' . (int)$id . ' Order by t.id ASC';
            //phpcs:ignore WordPress.WP.PreparedSQL.NotPrepared -- sql already escaped
            $result = $db->setQuery($query);
            $old_style = $db->loadObjectList();
            $new_data_style = array();
            $new_data_style['action'] = 'insert';
            $new_data_style['styleCells'] = array();

            foreach ($old_style as $style) {
                array_push($new_data_style['styleCells'], array($style->row_start, $style->row_end, $style->col_start, $style->col_end, $style->style));
            }

            $updateCellsStyle = $this->updateCellsStyle($new_id, $new_data_style);

            require_once(JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR . 'com_droptables' . DIRECTORY_SEPARATOR . 'models' . DIRECTORY_SEPARATOR . 'dbtable.php');
            $modelDbTable = new DroptablesModelDbtable();
            $table_query = $modelDbTable->getTableQuery($id);
            if (!empty($table_query)) {
                $modelDbTable->updateTableQuery($table_query, $new_id, true);
                $query_option = $modelDbTable->getQueryOption($id);
                if (!empty($query_option)) {
                    $name = '#__droptables_table_options';
                    $query = 'insert into ' . $name . ' (id_table, option_name, option_value) SELECT "' . $new_id . '", option_name, option_value FROM ' . $name . ' WHERE id_table = ' . (int)$id . ' AND option_name = \'query_option\'';
                    $db->setQuery($query);
                    if (!$db->execute()) {
                        return false;
                    }
                    $name = '#__droptables_table_options';
                    $query = 'insert into ' . $name . ' (id_table, option_name, option_value) SELECT "' . $new_id . '", option_name, option_value FROM ' . $name . ' WHERE id_table = ' . (int)$id . ' AND option_name = \'header_data\'';
                    $db->setQuery($query);
                    if (!$db->execute()) {
                        return false;
                    }
                }
                return $new_id;
            } else {
                if ($this->copyDataTable($new_id, $id)) {
                    return $new_id;
                }
            }
        }
        return false;
    }

    /**
     * Copy data table
     *
     * @param string $new_id New table id
     * @param string $old_id Source table
     *
     * @return boolean
     */
    public function copyDataTable($new_id, $old_id)
    {
        $db = JFactory::getDbo();
        $name = '#__droptable_tbl_' . $new_id;
        $old_name = '#__droptable_tbl_' . $old_id;

        $db->setQuery('CREATE TABLE '. $name .' LIKE ' . $old_name);
        if (!$db->execute()) {
            return false;
        }
        $db->setQuery('INSERT INTO '. $name .' SELECT * FROM ' . $old_name);

        if (!$db->execute()) {
            return false;
        }
        return true;
    }

    /**
     * Set the title of a table
     *
     * @param integer $id    Id table
     * @param string  $title Title category
     *
     * @return integer
     */
    public function setTitle($id, $title)
    {
        if ($title === '' || $title === null) {
            return false;
        }
        $filter = JFilterInput::getInstance();
        $title = $filter->clean($title);

        $table = $this->getTable();
        if (!$table->load($id)) {
            return false;
        }
        if (!$table->bind(array('title' => $title))) {
            return false;
        }
        if (!$table->store()) {
            return false;
        }
        return true;
    }

    /**
     * Get table params
     *
     * @param integer $id_table   Table id
     * @param string  $param_name Name
     *
     * @return object
     */
    public function getTableParams($id_table, $param_name = '')
    {
        $dbo = $this->getDbo();

        $dbo->setQuery('Select params From #__droptables_tables '
            . ' WHERE id = ' . (int)$id_table);

        $result = $dbo->loadObject();
        return $result;
    }

    /**
     * Get table
     *
     * @param integer $id_table Table id
     *
     * @return object
     */
    public function firstGetTable($id_table)
    {
        $dbo = $this->getDbo();

        $dbo->setQuery('Select type, author From #__droptables_tables '
            . ' WHERE id = ' . (int)$id_table);

        $result = $dbo->loadObject();
        return $result;
    }

    /**
     * Save table
     *
     * @param integer $id_table Id table
     * @param array   $datas    Data table
     *
     * @return false|integer
     */
    public function saveTableSynfile($id_table, $datas)
    {
        $dbo = $this->getDbo();

        if (!isset($datas['css'])) {
            $datas['css'] = '';
        }
        $object = new stdClass();

        if (isset($datas['style'])) {
            if (is_string($datas['style'])) {
                $styles = json_decode($datas['style']);
            } else {
                $styles = $datas['style'];
            }

            if (isset($styles->table)) {
                foreach ($styles->table as $key => $table) {
                    if (!isset($datas['params']->{$key})) {
                        $datas['params']->{$key} = $table;
                    }
                }
            }
            unset($styles->table);
            $object->modified_time = date('Y-m-d H:i:s');
            $object->css = $datas['css'];
            $object->hash = date('Y-m-d H:i:s');
            $object->params = $datas['params'];
            $object->style = json_encode($styles);
        } else {
            $object->modified_time = date('Y-m-d H:i:s');
            $object->css = $datas['css'];
            $object->hash = date('Y-m-d H:i:s');
            $object->params = $datas['params'];
        }

        $result = $this->updateTableDatas($id_table, $datas, null);

        $this->updateCellsStyle($id_table, $datas);

        if ((!isset($datas['type']) || $datas['type'] === 'html') && !(empty($datas['datas']) || is_string($datas['datas']))) {
            $countCol = count($datas['datas'][0]);
            if (isset($datas['params']->cell_types)) {
                $downgrade = $this->checkColumnHasHtml($countCol, $datas['params']->cell_types);
            } else {
                $downgrade = false;
            }
            $col_types = $this->checkLengthColumn('#__droptable_tbl_' . $id_table, $countCol, $downgrade);
        }

        if (!empty($col_types) && $col_types !== false) {
            $object->params->col_types = $col_types;
//            $this->updateTypeColumnDataBase($id_table, $object->params->col_types);
        }

        $object->params = json_encode($object->params);
        $object->id = $id_table;
        if (!JFactory::getDbo()->updateObject('#__droptables_tables', $object, 'id', true) && $datas['action'] === 'insert') {
            return false;
        }

        if ($result === false) {
            exit();
        }
        if ((int)$result === 0) {
            $result = $id_table;
        }

        return $result;
    }

    /**
     * Update column type when update 3.6.2
     *
     * @param integer $id Table id
     *
     * @return integer
     */
    public function updateTable362($id)
    {
        $dbo = $this->getDbo();
        $name = '#__droptables_table_options';
        $query = 'SELECT t.option_value FROM ' . $name . ' as t WHERE t.option_name = \'droptables_tables_convert_362\'';
        $dbo->setQuery($query);
        $query_option = $dbo->loadObject();
        if ($query_option !== null && is_object($query_option)) {
            if ((int)$id === -1) {
                $id = json_decode($query_option->option_value);
            }
        }

        $item = $this->getListTableById('WHERE c.type = \'html\'');
        $count = count($item);

        $after_id = -2;

        for ($i = 0; $i < $count; $i++) {
            if ($item[$i]->id > $id) {
                $after_id = $item[$i]->id;
                break;
            }
        }

        if ((int)$after_id > 0) {
            $getTableData = $this->getTableData("SHOW TABLES LIKE '%_droptable_tbl_" . $after_id . "'");
            if (count($getTableData) > 0) {
                $getTableData = $this->getTableData('SHOW COLUMNS FROM #__droptable_tbl_' . $after_id);
                if (count($getTableData) > 2) {
                    $dbo->setQuery('Select params From #__droptables_tables '
                        . ' WHERE id = ' . (int)$after_id);
                    $result = $dbo->loadObject();
                    $paramsOld = json_decode($result->params);
                    if (isset($paramsOld->cell_types)) {
                        $downgrade = $this->checkColumnHasHtml(count($getTableData) - 2, $paramsOld->cell_types);
                    } else {
                        $downgrade = false;
                    }
                    $col_types = $this->checkLengthColumn('#__droptable_tbl_' . $after_id, count($getTableData) - 2, $downgrade);
                    if (!empty($col_types) && $col_types !== false) {
                        $this->updateParams($after_id, array('col_types' => $col_types), 'col_types');
                    }
                }
            } else {
                $after_id = $this->updateTable362($after_id);
            }
        }

        if ($query_option === null) {
            $query = $dbo->getQuery(true);
            $query
                ->insert($dbo->quoteName('#__droptables_table_options'))
                ->values(implode(',', array($dbo->quote('0'), $dbo->quote('droptables_tables_convert_362'), $dbo->quote($after_id))))
                ->columns(implode(',', array('id_table', 'option_name', 'option_value')));
            $dbo->setQuery($query);
        } else {
            $query = 'UPDATE ' . $name . ' SET option_value = ' . $after_id . ' WHERE option_name = \'droptables_tables_convert_362\'';
            $dbo->setQuery($query);
        }
        $result = $dbo->execute();

        if ($after_id === -2) {
            $query = 'UPDATE ' . $name . ' SET option_value = ' . $after_id . ' WHERE option_name = \'droptables_tables_convert\'';
            $dbo->setQuery($query);
            $result = $dbo->execute();
        }

        return $after_id;
    }

    /**
     * Check columns has html cell
     *
     * @param integer $countCol   Clumn count
     * @param array   $cell_types Cell type params
     *
     * @return array
     */
    public function checkColumnHasHtml($countCol, $cell_types)
    {
        $data = array_fill(0, $countCol, 0);
        $count = count($cell_types);
        for ($i = 0; $i < $count; $i++) {
            for ($j = $cell_types[$i][1]; $j < $cell_types[$i][1] + $cell_types[$i][3]; $j++) {
                $data[$j] = 1;
            }
        }
        return $data;
    }

    /**
     * Create a string to alter column
     *
     * @param integer       $startColumn Column start
     * @param integer       $columns     Column end
     * @param string        $name_table  Table name
     * @param boolean|array $downgrade   Has downgrade column type
     *
     * @return array
     */
    public function createStringAlterColumn($startColumn, $columns, $name_table, $downgrade = true)
    {
        $dbo = $this->getDbo();
        $cols = array();
        $col_type = array();
        for ($i = $startColumn; $i <= $columns; $i++) {
            $query = 'SELECT MAX(LENGTH(col' . $i . ')) as number FROM ' . $name_table;

            $dbo->setQuery($query);
            if ($dbo->execute()) {
                $query = 'SELECT MAX(LENGTH(col' . $i . ')) as number FROM ' . $name_table;
                $dbo->setQuery($query);
                $result = $dbo->loadObject();
                $max = $result->number;
                if ($max < 250 && !(is_array($downgrade) && isset($downgrade[$i - 1]) && $downgrade[$i - 1] === 1)) {//varchar
                    array_push($cols, 'MODIFY COLUMN `col' . $i . '` VARCHAR(255)');
                    $col_type[] = 'VARCHAR(255)';
                } elseif ($max < 65000) {//text
                    array_push($cols, 'MODIFY COLUMN `col' . $i . '` TEXT');
                    $col_type[] = 'TEXT';
                } else {
                    array_push($cols, 'MODIFY COLUMN `col' . $i . '` MEDIUMTEXT');
                    $col_type[] = 'MEDIUMTEXT';
                }
            }
        }
        return array($cols, $col_type);
    }

    /**
     * Check max length for columns content
     *
     * @param string        $name_table Table name
     * @param integer       $columns    Number column
     * @param boolean|array $downgrade  Has downgrade column type
     *
     * @return array|boolean
     */
    public function checkLengthColumn($name_table, $columns, $downgrade = true)
    {
        $dbo = $this->getDbo();
        $cols = array();
        $col_type = array();
        if ($columns > 20) {
            $first_rows = $columns % 20;//0
            $number_query = ($columns / 20);//20
            $check = true;
            for ($i = 0; $i <= $number_query; $i++) {
                $data = array();
                if ($i !== (int)$number_query) {
                    $data = $this->createStringAlterColumn($i * 20 + 1, $i * 20 + 20, $name_table, $downgrade);
                } elseif ($first_rows > 0) {
                    $data = $this->createStringAlterColumn($i * 20 + 1, $i * 20 + $first_rows, $name_table, $downgrade);
                }
                if (count($data) > 0) {
                    $query = 'ALTER TABLE ' . $name_table . ' ' . implode(', ', $data[0]);

                    $dbo->setQuery($query);
                    $check = $dbo->execute();
                    $col_type = array_merge($col_type, $data[1]);
                }
            }
        } else {
            $data = $this->createStringAlterColumn(1, $columns, $name_table, $downgrade);
            $cols = $data[0];
            $col_type = $data[1];
            $query = 'ALTER TABLE ' . $name_table . ' ' . implode(', ', $cols);
            $dbo->setQuery($query);
            $check = $dbo->execute();
        }

        if ($check) {
            foreach ($col_type as $k => $col) {
                if (preg_match('/varchar/i', $col)) {
                    $col_type[$k] = 'varchar';
                } elseif (preg_match('/int/i', $col)) {
                    $col_type[$k] = 'int';
                } elseif (preg_match('/decimal/i', $col)) {
                    $col_type[$k] = 'float';
                } elseif (preg_match('/text/i', $col)) {
                    $col_type[$k] = 'text';
                } elseif (preg_match('/date/i', $col)) {
                    $col_type[$k] = 'date';
                } elseif (preg_match('/datetime/i', $col)) {
                    $col_type[$k] = 'datetime';
                } else {
                    $col_type[$k] = 'varchar';
                }
            }
            return $col_type;
        } else {
            return false;
        }
    }

    /**
     * Save table
     *
     * @param array $data Data table
     *
     * @return false|integer
     */
    public function save($data)
    {
        $id = (int)$data[0];
        $datas = $data[1];

        if ((int)$id < 1) {
            return false;
        }

        $dbo = $this->getDbo();

        $paramsOld = $this->getTableParams($id);
        $paramsOld = json_decode($paramsOld->params);

        if (empty($paramsOld)) {
            $paramsOld = new stdClass();
            $paramsOld->mergeSetting = array();
            $paramsOld->hyperlink = new stdClass();
            $paramsOld->headerOption = 1;
        }

        $object = new stdClass();
        $object->id = $id;
        $object->modified_time = date('Y-m-d H:i:s');
        $object->hash = date('Y-m-d H:i:s');
        $upadte_table = false;

        if (!empty($datas['params'])) {
            $params = $datas['params'];

            if (isset($params['cell_types']) && is_string($params['cell_types'])) {
                $params['cell_types'] = json_decode($params['cell_types']);
            }
            if (isset($params['col_types']) && is_string($params['col_types'])) {
                $params['col_types'] = json_decode($params['col_types']);
            }
            if (isset($params['mergeSetting']) && is_string($params['mergeSetting'])) {
                $params['mergeSetting'] = json_decode($params['mergeSetting']);
            }
            if (isset($params['hyperlink']) && is_string($params['hyperlink'])) {
                $params['hyperlink'] = json_decode($params['hyperlink']);
            }
            if (isset($params['alternateColorValue']) && is_string($params['alternateColorValue'])) {
                $params['alternateColorValue'] = json_decode($params['alternateColorValue']);
            }

            if (!isset($params['mergeSetting'])) {
                $params['mergeSetting'] = array();
            }

            if (!isset($params['allAlternate'])) {
                $params['allAlternate'] = new stdClass();
            }

            foreach ($params as $key => $value) {
                $paramsOld->{$key} = $value;
            }
            $object->params = json_encode($paramsOld);

            $upadte_table = true;
        }

        $this->paramsOld = $paramsOld;

        if (isset($datas['css'])) {
            $object->css = $datas['css'];
            $upadte_table = true;
        }

        if (!empty($datas['style'])) {
            $object->style = $datas['style'];
            $upadte_table = true;
        }
        //update params, style, css, hash, time
        if ($upadte_table) {
            $updateNulls = true;
            if (!JFactory::getDbo()->updateObject('#__droptables_tables', $object, 'id', $updateNulls)) {
                return false;
            }
        }

        //update header_data
        if (!empty($datas['header_data'])) {
            if (!is_string($datas['header_data'])) {
                $header_data = json_encode($datas['header_data']);
            } else {
                $header_data = $datas['header_data'];
            }

            $this->updateHeaderData($header_data, $id);
        }

        if (!empty($datas['datas'])) {
            $this->updateTableDatas($id, $datas, $paramsOld);
        }

        return true;
    }

    /**
     * Update style ranger
     *
     * @param integer $index       Index
     * @param integer $length      Length
     * @param integer $row_start   Start row or col
     * @param integer $row_end     End row or col
     * @param boolean $addOrRemove Check remove or add row|col
     * @param integer $max_count   Max number column or row
     *
     * @return array
     */
    public function updateStyleRanger($index, $length, $row_start, $row_end, $addOrRemove, $max_count = 0)
    {
        $new_ranger = array();
        $check = false;
        if ($addOrRemove) {//add
            if ($row_start - 1 >= $index + $length) {//++
                $check = true;
                $row_end = $row_end === 0 ? 0 : $row_end + $length;
                $row_start = $row_start + $length;
            } elseif ($row_start - 1 >= $index && $row_start - 1 < $index + $length) {//++ and copy
                $check = true;

                $new_ranger['start'] = $row_start;
                $new_ranger['end'] = $row_end - 1 >= $index + $length || $row_end === 0 ? $index + $length : $row_end;

                $row_end = $row_end === 0 ? 0 : $row_end + $length;
                $row_start = $row_start + $length;
            } elseif ($row_start - 1 < $index) {
                if ($row_end - 1 >= $index + $length) {
                    $check = true;
                    $row_end = $row_end + $length;
                } elseif ($row_end - 1 < $index + $length && $row_end - 1 >= $index) {
                    $new_ranger['start'] = $index + $length + 1;
                    $new_ranger['end'] = $row_end + $length;
                }
            }
        } else {//remove
            $new_ranger['delete'] = false;
            if ($row_start - 1 >= $index + $length) {//--
                $check = true;
                $row_end = $row_end === 0 ? 0 : $row_end - $length;
                $row_start = $row_start - $length;
            } elseif ($row_start - 1 >= $index && $row_start - 1 < $index + $length) {//++ and copy
                if ($row_end  - 1 >= $index + $length) {
                    $check = true;
                    $row_start = $index + 1;
                    $row_end = $row_end - $length;
                } elseif ($row_end === 0 && $max_count > $index + $length) {
                    $check = true;
                    $row_start = $index + 1;
                } else {//remove
                    $new_ranger['delete'] = true;
                }
            } elseif ($row_start - 1 < $index) {
                if ($row_end - 1 >= $index) {
                    $check = true;
                    $row_end = $row_end - 1 >= $index + $length ? $row_end - $length : $index;
                }
            }
        }

        return array($check, $row_start, $row_end, $new_ranger);
    }

    /**
     * Update style when remove cols or rows
     *
     * @param integer $id_table         Id table
     * @param integer $index            Index
     * @param integer $length           Length
     * @param string  $colRow           Check remove column or row
     * @param array   $count_row_column Number col row
     *
     * @return boolean
     */
    public function updateStyleRemoveColRow($id_table, $index, $length, $colRow, $count_row_column)
    {
        $dbo = $this->getDbo();
        $range_style = '#__droptables_range_styles';

        $params = $this->getTableParams($id_table, '');
        if (is_string($params)) {
            $params = json_decode($params);
        }

        if (isset($params->params) && is_string($params->params)) {
            $params = json_decode($params->params);
        }

        //update params cell_types
        if (isset($params->cell_types)) {
            $count = 0;
            $count_cell_typ = count($params->cell_types);
            $new_cell_type = array();
            for ($i = 0; $i < $count_cell_typ; $i++) {
                $cell_type = $params->cell_types[$i];
                if ($colRow === 'col') {
                    $data = $this->updateStyleRanger($index, $length, (int)$cell_type[1] + 1, (int)$cell_type[3] + (int)$cell_type[1], false, $count_row_column);

                    if ($data[0]) {
                        $new_cell_type[$count] = array($cell_type[0], $data[1] - 1, $cell_type[2], $data[2] - $data[1] + 1, $cell_type[4]);
                    } elseif (!$data[3]['delete']) {
                        $new_cell_type[$count] = $cell_type;
                    }
                }
                if ($colRow === 'row') {
                    $data = $this->updateStyleRanger($index, $length, (int)$cell_type[0] + 1, (int)$cell_type[2] + (int)$cell_type[0], false, $count_row_column);
                    if ($data[0]) {
                        $new_cell_type[$count] = array($data[1] - 1, $cell_type[1], $data[2] - $data[1] + 1, $cell_type[3], $cell_type[4]);
                    } elseif (!$data[3]['delete']) {
                        $new_cell_type[$count] = $cell_type;
                    }
                }
                $count++;
            }
            unset($params->cell_types);
            $params->cell_types = $new_cell_type;

            $object = new stdClass();
            $object->id = $id_table;
            $object->params = $params;
            if (!JFactory::getDbo()->updateObject('#__droptables_tables', $object, 'id', true)) {
                return false;
            }
        }

        //upload style range
        $query = 'SELECT * FROM ' . $range_style . ' as t WHERE t.id_table = ' . (int)$id_table . ' Order by t.id ASC';
        $dbo->setQuery($query);
        $old_style = $dbo->loadObjectList();

        foreach ($old_style as $style) {
            $check = false;
            if ($colRow === 'col') {
                $data = $this->updateStyleRanger($index, $length, (int)$style->col_start, (int)$style->col_end, false, $count_row_column);
                //($check, $row_start, $row_end, $new_ranger);
                if ($data[0] === true) {
                    $check = true;
                    $style->col_start = $data[1];
                    $style->col_end = $data[2];
                }
                if ($data[3]['delete']) {
                    $query = $dbo->getQuery(true);

                    $query->delete($range_style);
                    $query->where(array($dbo->quoteName('id') . ' = ' . $style->id));
                    $dbo->setQuery($query);
                    $result = $dbo->execute();
                }
            }
            if ($colRow === 'row') {
                $data = $this->updateStyleRanger($index, $length, (int)$style->row_start, (int)$style->row_end, false, $count_row_column);
                //($check, $row_start, $row_end, $new_ranger);
                if ($data[0] === true) {
                    $check = true;
                    $style->row_start = $data[1];
                    $style->row_end = $data[2];
                }
                if ($data[3]['delete']) {
                    $query = $dbo->getQuery(true);

                    $query->delete($range_style);
                    $query->where(array($dbo->quoteName('id') . ' = ' . $style->id));
                    $dbo->setQuery($query);
                    $result = $dbo->execute();
                }
            }

            if ($check) {
                $query = $dbo->getQuery(true);
                $query->update($dbo->quoteName($range_style))
                    ->set(array(
                        $dbo->quoteName('col_start') . ' = ' . $style->col_start,
                        $dbo->quoteName('col_end') . ' = ' . $style->col_end,
                        $dbo->quoteName('row_start') . ' = ' . $style->row_start,
                        $dbo->quoteName('row_end') . ' = ' . $style->row_end
                    ))
                    ->where(array($dbo->quoteName('id') . ' = ' . (int)$style->id));
                $dbo->setQuery($query);

                if (!$dbo->execute()) {
                    return false;
                }
            }
        }
    }

    /**
     * Delete old style table
     *
     * @param integer $id Id table
     *
     * @return boolean
     */
    public function deleteOldStyle($id)
    {
        $dbo = $this->getDbo();
        $query = 'DELETE FROM #__droptables_range_styles WHERE id_table = ' . $id;
        $dbo->setQuery($query);
        $result = $dbo->execute();

        $query = 'DELETE FROM #__droptables_table_options WHERE id_table = ' . $id;
        $dbo->setQuery($query);
        $result = $dbo->execute();

        if ($result) {
            $query = 'UPDATE #__droptables_tables SET params = "", style = "", modified_time = ' . $dbo->quote(date('Y-m-d H:i:s')) . ' WHERE id = ' . $id;
            $dbo->setQuery($query);
            $result = $dbo->execute();
        }
        if (!$result) {
            return false;
        }

        return true;
    }

    /**
     * Add more row into table in database
     *
     * @param object $data       Data
     * @param string $table_name Table name
     *
     * @return boolean
     */
    public function addRows($data, $table_name = '')
    {
        if ($table_name === '' || empty($data)) {
            return false;
        }

        //update the rows have line value bigger than data->index
        $dbo = $this->getDbo();
        $query = 'UPDATE ' . $table_name . ' SET line = line + ' . $data->amount . ' WHERE line >= ' . $data->index;
        $dbo->setQuery($query);
        $result = $dbo->execute();

        //insert rows into table in database
        if ($result) {
            for ($i = 0; $i < $data->amount; $i++) {
                $query = 'insert into ' . $table_name . ' (line) values (' . ($data->index + $i) . ')';
                $dbo->setQuery($query);
                $dbo->execute();
            }
        }

        return true;
    }

    /**
     * Delete column(s)
     *
     * @param object $data       Data
     * @param string $table_name Table name
     *
     * @return boolean
     */
    public function deleteColumns($data, $table_name = '')
    {
        if ($table_name === '' || empty($data)) {
            return false;
        }

        $dbo = $this->getDbo();

        $drop_cols = array();
        for ($i = 0; $i < $data->amount; $i++) {
            array_push($drop_cols, 'DROP COLUMN `col' . ($data->index + 1 + $i) . '`');
        }

        $dbo->setQuery('ALTER TABLE ' . $table_name .' ' . implode(', ', $drop_cols));
        $dbo->execute();

        $new_cols_names = array();
        $columns = $this->getTableDataColumns($table_name);
        $count = count($columns);
        for ($i = 0; $i < $count; $i++) {
            $col_index = (int)substr($columns[$i]->Field, 3) - 1;
            if ($col_index >= $data->index + 1) {
                array_push($new_cols_names, 'CHANGE `' . $columns[$i]->Field . '` `col' . ($i + 1) . '` ' . $columns[$i]->Type);
            }
        }

        $dbo->setQuery('ALTER TABLE ' . $table_name . ' ' . implode(', ', $new_cols_names));
        $dbo->execute();
        return true;
    }

    /**
     * Remove row(s) of table in database and update column line
     *
     * @param object $data       Data
     * @param string $table_name Table name
     *
     * @return boolean
     */
    public function removeRows($data, $table_name = '')
    {
        if ($table_name === '' || empty($data)) {
            return false;
        }
        $dbo = $this->getDbo();

        $delete_rows_lines = array();
        for ($i = 0; $i < $data->amount; $i++) {
            array_push($delete_rows_lines, $data->index + $i);
        }
        $max_row_index = $data->index + $data->amount - 1;
        $dbo->setQuery('DELETE FROM ' . $table_name . ' WHERE line IN (' . implode(', ', $delete_rows_lines) . ')');
        if ($dbo->execute()) {
            //update line for other row
            $query = 'UPDATE ' . $table_name . ' SET line = line - ' . $data->amount . ' WHERE line >= ' . $data->index;
            $dbo->setQuery($query);
            $result = $dbo->execute();
        }

        return true;
    }

    /**
     * Function delete table
     *
     * @param integer $id Id table
     *
     * @return false|integer
     */
    public function delete(&$id)
    {
        $dbo = JFactory::getDbo();
        if (is_int($id)) {
            $listId = $id;
            $count = 1;
        } else {
            $listId = json_decode($id, true);
            $count = count($listId);
        }
        $result = false;
        for ($i = 0; $i < $count; $i++) {
            if (is_int($id)) {
                $id_table = $listId;
            } else {
                $id_table = $listId[$i];
            }
            $range_style = '#__droptables_tables';
            $query = 'SELECT t.type FROM ' . $range_style . ' as t WHERE t.id = ' . (int)$id_table;
            $dbo->setQuery($query);
            $types = $dbo->loadObjectList();

            if (!empty($types)) {
                $type = $types[0]->type;
                $query1 = 'DELETE FROM #__droptables_tables WHERE id = ' . $id_table;
                $query2 = 'DELETE FROM #__droptables_range_styles WHERE id_table = ' . $id_table;
                $query3 = 'DELETE FROM #__droptables_table_options WHERE id_table = ' . $id_table;

                $dbo->setQuery($query1);
                if ($dbo->execute()) {
                    $dbo->setQuery($query2);
                    $dbo->execute();

                    $dbo->setQuery($query3);
                    $dbo->execute();
                    $tblName = $dbo->quoteName('#__droptable_tbl_' . $id_table);
                    $result = $this->deleteTblInDb($tblName);
                }
            }
        }

        return $result;
    }

    /**
     * Check the range style exists
     *
     * @param string $row_start Row start
     * @param string $row_end   Row end
     * @param string $col_start Col start
     * @param string $col_end   Col end
     * @param string $id_table  Table id
     *
     * @return boolean|string|null
     */
    public function checkRangeStyleExists($row_start, $row_end, $col_start, $col_end, $id_table = '')
    {
        if ($id_table === '') {
            return false;
        }

        $dbo = $this->getDbo();

        //upload style range
        $dbo->setQuery('Select * From #__droptables_range_styles '
            . ' WHERE row_start = ' . (int)$row_start . ' AND row_end = ' . $row_end . ' AND col_start = ' . $col_start . ' AND col_end = ' . $col_end . ' AND id_table = ' . $id_table);

        $result = $dbo->loadObjectList();

        return $result;
    }

    /**
     * Update style table
     *
     * @param object $data     Data
     * @param string $id_table Table id
     *
     * @return boolean
     */
    public function updateStyle($data, $id_table = '')
    {
        if ($id_table === '' || empty($data)) {
            return false;
        }

        $dbo = $this->getDbo();
        $table_name = '#__droptables_range_styles';

        $range_exists = $this->checkRangeStyleExists($data->row_start, $data->row_end, $data->col_start, $data->col_end, $id_table);
        if ($range_exists === null) {
            $values = array($id_table, $data->row_start, $data->row_end, $data->col_start, $data->col_end, "'" . json_encode($data->style, JSON_FORCE_OBJECT) . "'");
            $query = $dbo->getQuery(true);
            $query
                ->insert($table_name)
                ->columns(array('id_table', 'row_start', 'row_end', 'col_start', 'col_end', 'style'))
                ->values(implode(',', $values));
            $dbo->setQuery($query);
            if (!$dbo->execute()) {
                return false;
            }
        } else {
            $new_style = (array)$data->style;
            $new_style_meger_old = (array)$data->style;

            foreach ($range_exists as $k => $value) {
                if (is_string($value->style)) {
                    $old_styles = json_decode($value->style, true) === null ? json_decode($value->style) : json_decode($value->style, true);
                    if (empty($old_styles)) {
                        $old_styles = array();
                    }
                } else {
                    $old_styles = $value->style;
                }
                $new_value_old_range = array_diff_key($old_styles, $new_style);//not overlap

                //all overlap//delete old style ranger
                $query = $dbo->getQuery(true);
                $query->delete($table_name);
                $query->where(array($dbo->quoteName('id') . ' = ' . $value->id));
                $dbo->setQuery($query);
                $result = $dbo->execute();

                if (!empty($new_value_old_range)) {//has style not overlap
                    $new_style_meger_old = array_merge($new_value_old_range, $new_style);
                }
            }

            $style = json_encode($new_style_meger_old, JSON_FORCE_OBJECT);
            $style = str_replace('"', '\"', $style);

            $values = array($id_table, $data->row_start, $data->row_end, $data->col_start, $data->col_end,  "'" . $style . "'");

            $query = $dbo->getQuery(true);
            $query
                ->insert($table_name)
                ->columns(array('id_table', 'row_start', 'row_end', 'col_start', 'col_end', 'style'))
                ->values(implode(',', $values));
            $dbo->setQuery($query);
            if (!$dbo->execute()) {
                return false;
            }
        }
        return true;
    }

    /**
     * Replace type column
     *
     * @param string $data Column type
     *
     * @return string
     */
    public function getColTypeToModify($data)
    {
        if ($data === 'int') {
            $type = 'INT(11)';
        } elseif ($data === 'float') {
            $type = 'DECIMAL(16,3)';
        } elseif ($data === 'date') {
            $type = 'DATE';
        } elseif ($data === 'datetime') {
            $type = 'DATETIME';
        } elseif ($data === 'text') {
            $type = 'TEXT';
        } else {
            $type = 'VARCHAR(255)';
        }

        return $type;
    }


    /**
     * Update columns type
     *
     * @param integer $id_table   Table id
     * @param string  $table_name Table name
     * @param array   $data       Data table
     *
     * @return void|boolean
     */
    public function updateColumnsTypes($id_table, $table_name, $data)
    {
        global $wpdb;
        $cols = array();
        $columns = $this->getTableDataColumns($table_name);
        foreach ($columns as $k => $column) {
            if (in_array($k, $data['cols'])) {
                $cols[$k] = $data['type'];
            }
        }
        $paramsOld = $this->paramsOld;

        if (empty($paramsOld->col_types)) {
            $cols = $this->getTableDataColumns($table_name);
            $new_col_types = array();
            foreach ($cols as $k => $col) {
                if (preg_match('/varchar/i', $col->Type)) {
                    $new_col_types[$k] = 'varchar';
                } elseif (preg_match('/int/i', $col->Type)) {
                    $new_col_types[$k] = 'int';
                } elseif (preg_match('/decimal/i', $col->Type)) {
                    $new_col_types[$k] = 'float';
                } elseif (preg_match('/text/i', $col->Type)) {
                    $new_col_types[$k] = 'text';
                } elseif (preg_match('/date/i', $col->Type)) {
                    $new_col_types[$k] = 'date';
                } elseif (preg_match('/datetime/i', $col->Type)) {
                    $new_col_types[$k] = 'datetime';
                } else {
                    $new_col_types[$k] = 'varchar';
                }
            }
            $paramsOld->col_types = $new_col_types;

            $object = new stdClass();
            $object->id = $id_table;
            $object->params = json_encode($paramsOld);

            $updateNulls = true;
            if (!JFactory::getDbo()->updateObject('#__droptables_tables', $object, 'id', $updateNulls)) {
                return false;
            }
            $this->paramsOld = $paramsOld;
        } else {
            $count = count($paramsOld->col_types);
            for ($i = 0; $i < $count; $i++) {
                foreach ($cols as $k => $col) {
                    if ($i === $k) {
                        $paramsOld->col_types[$i] = $col;
                    } else {
                        continue;
                    }
                }
            }
            $object = new stdClass();
            $object->id = $id_table;
            $object->params = json_encode($paramsOld);

            $updateNulls = true;
            if (!JFactory::getDbo()->updateObject('#__droptables_tables', $object, 'id', $updateNulls)) {
                return false;
            }
            $this->paramsOld = $paramsOld;
        }
    }

    /**
     * Get rows of table to update by select query of table
     *
     * @param string $query_db     Get data table query
     * @param object $query_option Query options
     * @param object $data         Ajax data
     *
     * @return array|boolean|object|null
     */
    public function getRowsToUpdateInDbTable($query_db, $query_option, $data)
    {
        $column_key =  explode('.', $data->column_key);
        $dbTables = explode('.', $data->column);
        if (!empty($query_option->table_alias)) {
            $dbTables[0] = !empty($query_option->table_alias->{$dbTables[0]}) ? $query_option->table_alias->{$dbTables[0]} : $dbTables[0];
            if (!empty($query_option->table_alias->{$column_key[0]})) {
                $column_key[0] =  $query_option->table_alias->{$column_key[0]};
                $data->column_key = $column_key[0] . '.' . $column_key[1];
            }
        }
        require_once(JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR . 'com_droptables' . DIRECTORY_SEPARATOR . 'classes' . DIRECTORY_SEPARATOR . 'php-sql-parser' . DIRECTORY_SEPARATOR . 'vendor' . DIRECTORY_SEPARATOR . 'autoload.php');
        $parser = new \PHPSQLParser\PHPSQLParser(false, true);
        $creator = new \PHPSQLParser\PHPSQLCreator();
        $query_db = str_replace(array("\n\r", "\r\n", "\n", "\r", '&#10;'), ' ', $query_db);
        $query_db = str_replace('\\', '', $query_db);
        if (strpos($query_db, '{user_id}') !== false) {
            $query_db = str_replace('{user_id}', ' {user_id}', $query_db);
        }
        if (strpos($query_db, '{user_email}') !== false) {
            $query_db = str_replace('{user_email}', ' {user_email}', $query_db);
        }
//        if (isset($item->params->placeholder_query) && (int)$item->params->placeholder_query === 1) {
//            if (strpos($query_db, '{user_id}') !== false) {
//                $id_user = (int)JFactory::getUser()->id;
//                $query_db = str_replace('{user_id}', $id_user, $query_db);
//            }
//        }
        $parserMysql = $parser->parse($query_db, true);

        if (!empty($parserMysql['SELECT'])) {
            $parserMysql['SELECT'] = array();
            array_unshift(
                $parserMysql['SELECT'],
                array(
                    'expr_type' => 'colref',
                    'alias' => false,
                    'base_expr' => $dbTables[0] . '.*',
                    'sub_tree' => false,
                    'delim' => ''
                )
            );
        }

        $var_id = (is_numeric($data->id) && $data->id > 0) ? $data->id : '"' . $data->id . '"';

        if (!empty($parserMysql['WHERE'])) {
            array_push(
                $parserMysql['WHERE'],
                array(
                    'expr_type' => 'operator',
                    'base_expr' => 'AND',
                    'sub_tree' => false
                ),
                array(
                    'expr_type' => 'colref',
                    'base_expr' => $data->column_key,
                    'no_quotes' => array(
                        'delim' => '.',
                        'parts' => array(
                            $column_key[0],
                            $column_key[1]
                        )
                    ),
                    'sub_tree' => false
                ),
                array(
                    'expr_type' => 'operator',
                    'base_expr' => '=',
                    'sub_tree' => false
                ),
                array(
                    'expr_type' => 'const',
                    'base_expr' => $var_id,
                    'sub_tree' => false
                )
            );
        } else {
            $parserMysql['WHERE'] = array();
            array_push(
                $parserMysql['WHERE'],
                array(
                    'expr_type' => 'colref',
                    'base_expr' => $data->column_key,
                    'no_quotes' => array(
                        'delim' => '.',
                        'parts' => array(
                            $column_key[0],
                            $column_key[1]
                        )
                    ),
                    'sub_tree' => false
                ),
                array(
                    'expr_type' => 'operator',
                    'base_expr' => '=',
                    'sub_tree' => false
                ),
                array(
                    'expr_type' => 'const',
                    'base_expr' => $var_id,
                    'sub_tree' => false
                )
            );
        }
        $parserMysql = $creator->create($parserMysql);

        $dataReturn = $this->getTableData($parserMysql);
        if ($dataReturn !== false) {
            return $dataReturn;
        } else {
            return false;
        }
    }

    /**
     * Update style when add cols or rows
     *
     * @param integer $id_table Id table
     * @param integer $index    Index
     * @param integer $length   Length
     * @param boolean $before   Check add before or after
     * @param string  $colRow   Number col row
     *
     * @return boolean
     */
    public function updateStyleAddColRow($id_table, $index, $length, $before, $colRow)
    {
        $dbo = $this->getDbo();
        $range_style = '#__droptables_range_styles';
        if (!$before) {
            $index = $index - $length;
        }

        $params = $this->getTableParams($id_table, '');
        if (is_string($params)) {
            $params = json_decode($params);
        }

        if (isset($params->params) && is_string($params->params)) {
            $params = json_decode($params->params);
        }

        //update params cell_types
        if (isset($params->cell_types)) {
            $new_cell_types = array();
            $count = 0;
            $count_cell_typ = count($params->cell_types);
            for ($i = 0; $i < $count_cell_typ; $i++) {
                $cell_type = $params->cell_types[$i];
                if ($colRow === 'col') {
                    $data = $this->updateStyleRanger($index, $length, (int)$cell_type[1] + 1, (int)$cell_type[3] + (int)$cell_type[1], true);
                    if ($data[0] === true) {
                        $new_cell_types[$count] = array($cell_type[0], $data[1] - 1, $cell_type[2], $data[2] - $data[1] + 1, $cell_type[4]);
                    } else {
                        $new_cell_types[$count] = $cell_type;
                    }
                    if (isset($data[3]['start'])) {
                        $count++;
                        $new_cell_types[$count] = array($cell_type[0],$data[3]['start'] - 1,$cell_type[2],$data[3]['end'] - $data[3]['start'] + 1, $cell_type[4]);
                    }
                }
                if ($colRow === 'row') {
                    $data = $this->updateStyleRanger($index, $length, (int)$cell_type[0] + 1, (int)$cell_type[2] + (int)$cell_type[0], true);

                    if ($data[0] === true) {
                        $new_cell_types[$count] = array($data[1] - 1, $cell_type[1], $data[2] - $data[1] + 1, $cell_type[3], $cell_type[4]);
                    } else {
                        $new_cell_types[$count] = $cell_type;
                    }
                    if (isset($data[3]['start'])) {
                        $count++;
                        $new_cell_types[$count] = array($data[3]['start'] - 1,$cell_type[1],$data[3]['end'] - $data[3]['start'] + 1,$cell_type[3], $cell_type[4]);
                    }
                }
                $count++;
            }

            $params->cell_types = $new_cell_types;

            $object = new stdClass();
            $object->id = $id_table;
            $object->params = json_encode($params);
            $object->modified_time = date('Y-m-d H:i:s');
            $object->hash = date('Y-m-d H:i:s');
            if (!JFactory::getDbo()->updateObject('#__droptables_tables', $object, 'id', true)) {
                return false;
            }
        }

        //upload style range
        $query = 'SELECT * FROM ' . $range_style . ' as t WHERE t.id_table = ' . (int)$id_table . ' Order by t.id ASC';
        $dbo->setQuery($query);
        $old_style = $dbo->loadObjectList();
        $new_ranger = array();

        foreach ($old_style as $style) {
            $check = false;
            if ($colRow === 'col') {
                $data = $this->updateStyleRanger($index, $length, (int)$style->col_start, (int)$style->col_end, true);
                //($check, $row_start, $row_end, $new_ranger);
                if ($data[0] === true) {
                    $check = true;
                    $style->col_start = $data[1];
                    $style->col_end = $data[2];
                }
                if (isset($data[3]['start'])) {
                    $new_ranger[] = '(' . $id_table . ',' . $style->row_start . ',' . $style->row_end . ',' . $data[3]['start'] . ',' . $data[3]['end'] . ",'" . $style->style . "')";
                }
            }
            if ($colRow === 'row') {
                $data = $this->updateStyleRanger($index, $length, (int)$style->row_start, (int)$style->row_end, true);
                //($check, $row_start, $row_end, $new_ranger);
                if ($data[0] === true) {
                    $check = true;
                    $style->row_start = $data[1];
                    $style->row_end = $data[2];
                }
                if (isset($data[3]['start'])) {
                    $new_ranger[] = '(' . $id_table . ',' . $data[3]['start'] . ',' . $data[3]['end'] . ',' . $style->col_start . ',' . $style->col_end . ",'" . $style->style . "')";
                }
            }

            if ($check) {
                $query = $dbo->getQuery(true);
                $query->update($dbo->quoteName($range_style))
                    ->set(array(
                        $dbo->quoteName('col_start') . ' = ' . $style->col_start,
                        $dbo->quoteName('col_end') . ' = ' . $style->col_end,
                        $dbo->quoteName('row_start') . ' = ' . $style->row_start,
                        $dbo->quoteName('row_end') . ' = ' . $style->row_end
                    ))
                    ->where(array($dbo->quoteName('id') . ' = ' . (int)$style->id));
                $dbo->setQuery($query);

                if (!$dbo->execute()) {
                    return false;
                }
            }
        }

        if (count($new_ranger) > 0) {
            $query = 'insert into ' . $range_style . ' (id_table,row_start,row_end,col_start,col_end,style) values ' . implode(',', $new_ranger);
            $dbo->setQuery($query);
            $dbo->execute();
        }
    }

    /**
     * Get all column in current table name
     *
     * @param string  $table_name Table name
     * @param boolean $all        Get all columns
     *
     * @return array
     */
    public function getTableDataColumns($table_name, $all = false)
    {
        $db = JFactory::getDbo();
        $columns_query = 'SHOW COLUMNS FROM ' . $table_name;
        $db->setQuery($columns_query);
        $columns_obj = $db->loadObjectList();
        $columns = array();
        foreach ($columns_obj as $column_obj) {
            if (($column_obj->Field === 'ID' || $column_obj->Field === 'line') && !$all) {
                continue;
            }
            array_push($columns, $column_obj);
        }

        return $columns;
    }

    /**
     * Get recode in query
     *
     * @param string       $table Table
     * @param string|array $where Where parameter
     *
     * @return mixed
     */
    public function getRecord($table, $where)
    {
        $db = JFactory::getDbo();
        $query = $db->getQuery(true);

        $query->select('*');
        $query->from($table);
        if (!is_string($where)) {
            $count = count($where);
            for ($i = 0; $i < $count; $i ++) {
                $query->where($where[$i]);
            }
        } else {
            $query->where($where);
        }
        $db->setQuery($query);

        $result = $db->loadObject();
        return $result;
    }

    /**
     * Crate string cells data before query
     *
     * @param array   $curr_data          Data cells
     * @param integer $length             Length
     * @param integer $max_allowed_packet Max allowed packet
     *
     * @return array
     */
    public function createStringDataCells($curr_data, $length, $max_allowed_packet)
    {
        global $wpdb;
        $data = array();
        $countRow = count($curr_data);
        $first_rows = $countRow % $length;//0
        $number_query = ($countRow / $length);//20
        $maxRowInsert = 0;

//        $countCol = count($curr_data[0]);
//        $stringPrepare = '';
//        for ($ii = 0; $ii < $countCol; $ii++) {
//            $stringPrepare .= ',%s';
//        }
//        $stringPrepare .= ')';
        $countCol = count($curr_data[0]);

        for ($i = 0; $i <= $number_query; $i++) {
            $string = '';
            for ($j = $maxRowInsert; $j < $i * $length + $first_rows; $j++) {
                if ($string !== '') {
                    $string .= ',';
                }

                $string .= '(\'' . $j;

                $stringPrepare = '';
                for ($ii = 0; $ii < $countCol; $ii++) {
                    if (empty($curr_data[$j][$ii]) && !(is_float($curr_data[$j][$ii]) && (int)$curr_data[$j][$ii] === 0)) {
                        $curr_data[$j][$ii] = '';
                    }
                    $curr_data[$j][$ii] = str_replace("\'", "\\\'", $curr_data[$j][$ii]);
                    $curr_data[$j][$ii] = str_replace("'", "\'", $curr_data[$j][$ii]);
                    $stringPrepare .= '\', \'' . $curr_data[$j][$ii];
                }
                $stringPrepare .= '\')';
                $string .= $stringPrepare;
            }
            $maxRowInsert = $i * $length + $first_rows;

            if ($string !== '') {
                $size = strlen($string);
                if ($size < ((int)$max_allowed_packet - 50)) {//
                    $data[$i] = $string;
                } else {
                    return $this->createStringDataCells($curr_data, 2, $max_allowed_packet);
                }
            }
        }
        return $data;
    }

    /**
     * Update style to range cells table
     *
     * @param integer $id_table Id table
     * @param array   $datas    Table data
     *
     * @return void|boolean
     */
    public function updateCellsStyle($id_table, $datas)
    {
        if (!empty($datas['styleCells']) && $datas['action'] === 'insert' && count($datas['styleCells']) > 0) {
            $string = '';

            foreach ($datas['styleCells'] as $item) {
                if (isset($datas['numberRow']) && (int)$item[1] === (int)$datas['numberRow']) {//row end
                    $item[1] = 0;
                }
                if (isset($datas['numberRow']) && (int)$item[3] === (int)$datas['numberCol']) {//column end
                    $item[3] = 0;
                }

                if ($string !== '') {
                    $string .= ',';
                }
                $string .= '(' . $id_table . ',' . $item[0] . ',' . $item[1] . ',' . $item[2] . ',' . $item[3] . ",'" . $item[4] . "')";
            }
            $dbo = $this->getDbo();
            $query = 'insert into #__droptables_range_styles (id_table,row_start,row_end,col_start,col_end,style) values ' . $string;

            $dbo->setQuery($query);

            if (!$dbo->execute()) {
                return false;
            }
        }
    }

    /**
     * Update table params
     *
     * @param integer      $id   Table id
     * @param array        $data New data
     * @param string|array $type List param to update
     *
     * @return boolean
     */
    public function updateParams($id, $data, $type)
    {
        $paramsOld = $this->getTableParams($id);
        if (isset($paramsOld->params) && is_string($paramsOld->params)) {
            $paramsOld = json_decode($paramsOld->params);
        }

        if (empty($paramsOld)) {
            $paramsOld = new stdClass();
            $paramsOld->mergeSetting = array();
            $paramsOld->hyperlink = new stdClass();
            $paramsOld->headerOption = 1;
        }

        if (is_array($type)) {
            foreach ($type as $key => $value) {
                if (isset($data[$value])) {
                    $paramsOld->{$value} = $data[$value];
                }
            }
        } elseif (is_string($type)) {
            if (isset($data[$type])) {
                $paramsOld->{$type} = $data[$type];
            }
        }

        $object = new stdClass();
        $object->id = $id;
        $object->modified_time = date('Y-m-d H:i:s');
        $object->hash = date('Y-m-d H:i:s');
        $object->params = json_encode($paramsOld);
        $updateNulls = true;
        if (!JFactory::getDbo()->updateObject('#__droptables_tables', $object, 'id', $updateNulls)) {
            return false;
        }
        return true;
    }

    /**
     * Update table
     *
     * @param integer $id        Table id
     * @param array   $datas     Ajax data
     * @param object  $paramsOld Table old params
     *
     * @return boolean
     */
    public function updateTableDatas($id, $datas, $paramsOld)
    {
        $dbo = $this->getDbo();
        $droptables_tables = $dbo->quoteName('#__droptables_tables');
        $droptable_tbl = $dbo->quoteName('#__droptable_tbl_' . $id);
        $prefix = $dbo->getPrefix();

        if (is_array($datas['datas'])) {
            $curr_data = $datas['datas'];
        } else {
            $curr_data = json_decode(stripslashes($datas['datas']));
            $curr_data_raw = json_decode($datas['datas']);
            if ($curr_data === null) {
                $curr_data = json_decode($datas['datas']);
            }
        }

        //update  droptables_list_syn_google option in Google App script url
        if (isset($datas['action']) && $datas['action'] === 'insert') {
            //update header_data
            if (!empty($datas['header_data'])) {
                $this->updateHeaderData($datas['header_data'], $id);
            }

            if (!isset($datas['type']) || $datas['type'] === 'html') {
                $object = new stdClass();
                $object->id = $id;
                $object->modified_time = date('Y-m-d H:i:s');
                $object->hash = date('Y-m-d H:i:s');
                $object->style = $datas['style'];
                $updateNulls = false;
                if (!JFactory::getDbo()->updateObject('#__droptables_tables', $object, 'id', $updateNulls)) {
                    return false;
                }
                //remove old table
                $result = $this->deleteTblInDb($droptable_tbl);

                if ($result) {
                    //create new table
                    $col_types = array();
                    if (!empty($curr_data[0]) && is_array($curr_data[0])) {
                        $countCol = count($curr_data[0]);

                        $columns = array();
                        $column = ' (line';
                        for ($i = 1; $i <= $countCol; $i++) {
                            $columnDefault = array('name' => 'col' . $i, 'type' => 'mediumtext');
                            $columns[] = $columnDefault;
                            $column .= ',col' . $i;
                            $col_types[] = 'mediumtext';
                        }
                        $column .= ')';
                    } else {//if table empty
                        $columns = array();
                        for ($i = 1; $i <= 10; $i++) {
                            $columnDefault = array('name' => 'col' . $i, 'type' => 'string');
                            $columns[] = $columnDefault;
                            $col_types[] = 'varchar';
                        }
                        $not_insert = true;
                    }

                    $result = $this->createTblInDb('droptable_tbl_' . $id, $columns);
                }

                if ($result && empty($not_insert)) {
                    $dbo->setQuery(
                        "SHOW VARIABLES LIKE 'max_allowed_packet'"
                    );
                    $max_allowed_packet = $dbo->loadObject();
                    $data_insert = $this->createStringDataCells($curr_data, 5, $max_allowed_packet->Value);
                    $countRow = count($data_insert);
                    for ($i = 0; $i <= $countRow; $i++) {
                        if (isset($data_insert[$i]) && $data_insert[$i] !== '') {
                            $query =
                                'INSERT INTO ' . $droptable_tbl . $column . ' VALUES ' . $data_insert[$i]
                            ;

                            $dbo->setQuery($query);
                            if (!$dbo->execute()) {
                                return false;
                            }
                        }

                        if (!$result) {
                            return false;
                        }
                    }

                    if (isset($col_types)) {
                        $this->updateParams($id, array('col_types' => $col_types), 'col_types');
                    }
                }
            } else {
                $object = new stdClass();
                $object->id = $id;
                $object->modified_time = date('Y-m-d H:i:s');
                $object->hash = date('Y-m-d H:i:s');
                $object->style = $datas['style'];
//                $object->mysql_query = $datas['datas'];//da chuyen vao params
                $updateNulls = false;
                if (!JFactory::getDbo()->updateObject('#__droptables_tables', $object, 'id', $updateNulls)) {
                    return false;
                }
            }
        } else {
            $max_row = (int)$datas['count']['countRows'];
            $max_col = (int)$datas['count']['countCols'];
            $header_data = $this->getHeaderData($id);
            //check edit_cell_mysql
            $edit_cell_mysql = false;
            foreach ($curr_data as $key_curr_data => $data) {
                switch ($data->action) {
                    case 'edit_cell_mysql':
                        $params = $this->getTableParams($id, '');

                        if (isset($params->params) && is_string($params->params)) {
                            $params = json_decode($params->params, true);
                        }

                        if (isset($params['headerOption']) && $params['headerOption'] > 0 && $data->row < $params['headerOption']) {
                            if (count($params['header_data'][$data->row]) < $max_col) {
                                $params['header_data'][$data->row] = array_fill(0, $max_col, '');
                            }
                            $params['header_data'][$data->row][$data->col] = $data->content;

                            try {
                                $object = new stdClass();
                                $object->id = $id;
                                $object->params = $params;
                                if (!JFactory::getDbo()->updateObject('#__droptables_tables', $object, 'id', true)) {
                                    return false;
                                }
                            } catch (Exception $e) {
                                $this->exitStatus('An error occurred!');
                            }
                        } else {
                            if (empty($params['table_editing'])) {
                                $this->exitStatus('You don\'t have permission to change the database!');
                            }

                            require_once(JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR . 'com_droptables' . DIRECTORY_SEPARATOR . 'models' . DIRECTORY_SEPARATOR . 'dbtable.php');
                            $modelDbTable = new DroptablesModelDbtable();
                            $query_option = $modelDbTable->getQueryOption($id);
                            if (!empty($query_option->option_value) && is_string($query_option->option_value)) {
                                $query_option = json_decode($query_option->option_value);
                            }
                            $mysql_query = $modelDbTable->getTableQuery($id);
                            $rowsUpdate = $this->getRowsToUpdateInDbTable($mysql_query, $query_option, $data);
                            if ($rowsUpdate !== false) {
                                $countRow = count($rowsUpdate);
                                $dbTables = explode('.', $data->column);

                                for ($j = 0; $j < $countRow; $j++) {
                                    $column_key =  explode('.', $data->column_key);
                                    $dbTables[0] = str_replace($prefix, '#__', $dbTables[0]);
                                    $object = new stdClass();
                                    $object->{$column_key[1]} = $rowsUpdate[$j][$column_key[1]];
                                    $object->{$dbTables[1]} = $data->content;
                                    if (!JFactory::getDbo()->updateObject($dbTables[0], $object, $column_key[1], true)) {
                                        return false;
                                    }
                                }
                            }
                        }
                        $edit_cell_mysql = true;
                        break;
                    case 'edit_cell':
                        if (empty($paramsOld->query)) {
                            $col_name = 'col' . ($data->col + 1);
                            $row_exists = $this->getRecord('#__droptable_tbl_' . $id, 'line = ' . $data->row);
                            $query = $dbo->getQuery(true);

                            if ($row_exists === null) {
                                $query
                                    ->insert($droptable_tbl)
                                    ->columns(array('line', $col_name))
                                    ->values($data->row . ',' . $dbo->quote($curr_data_raw[$key_curr_data]->content));
                                $dbo->setQuery($query);
                                if (!$dbo->execute()) {
                                    return false;
                                }
                            } else {
                                $query->update($droptable_tbl)
                                    ->set(array($col_name . ' = ' . $dbo->quote($curr_data_raw[$key_curr_data]->content)))
                                    ->where(array($dbo->quoteName('line') . ' = ' . $data->row));
                                $dbo->setQuery($query);

                                if (!$dbo->execute()) {
                                    return false;
                                }
                            }

                            if (isset($paramsOld->headerOption) && $paramsOld->headerOption > 0 && $data->row < $paramsOld->headerOption) {
                                if ($header_data !== null && is_object($header_data)) {
                                    $header_data = json_decode($header_data->option_value);
                                }
                                if (empty($header_data)) {
                                    $header_data = array();
                                }

                                if (isset($header_data[$data->row]) && count($header_data[$data->row]) < $max_col) {
                                    $header_data[$data->row] = array_fill(0, $max_col, '');
                                }
                                $header_data[$data->row][$data->col] = $curr_data_raw[$key_curr_data]->content;
                                $this->updateHeaderData($header_data, $id);
                            }
                        }
                        break;
                    case 'create_col_after_paste':
                        $add_cols = array();
                        for ($i = 0; $i < $data->amount; $i++) {
                            if ($data->index + $i === 0) {
                                array_push($add_cols, 'ADD COLUMN `col' . ($data->index + 1 + $i) . '` text AFTER `line`');
                            } else {
                                array_push($add_cols, 'ADD COLUMN `col' . ($data->index + 1 + $i) . '` text AFTER `col' . ($data->index + $i) . '`');
                            }
                        }
                        $new_cols_names = array();
                        $columns = $this->getTableDataColumns($droptable_tbl);
                        foreach ($columns as $k => $column) {
                            $db_col_index = (int)substr($column->Field, 3);
                            if ($db_col_index >= $data->index + 1) {
                                array_push($new_cols_names, 'CHANGE `' . $column->Field . '` `col' . ($k + 1 + $data->amount) . '` ' . $column->Type);
                            }
                        }
                        $check = $dbo->setQuery(
                            'ALTER TABLE ' . $droptable_tbl . ' ' . implode(', ', $new_cols_names)
                        );
                        $dbo->execute();
                        if ($check) {
                            $dbo->setQuery(
                                'ALTER TABLE ' . $droptable_tbl . ' ' . implode(', ', $add_cols)
                            );
                            $dbo->execute();
                        }

                        //add new columns to header/stylecells
                        $this->updateStyleAddColRow($id, $data->index, $data->amount, $data->left, 'col');
                        break;
                    case 'create_col':
                        $add_cols = array();
                        for ($i = 0; $i < $data->amount; $i++) {
                            if ($data->index + $i === 0) {
                                array_push($add_cols, 'ADD COLUMN `col' . ($data->index + 1 + $i) . '` varchar(255) AFTER `line`');
                            } else {
                                array_push($add_cols, 'ADD COLUMN `col' . ($data->index + 1 + $i) . '` varchar(255) AFTER `col' . ($data->index + $i) . '`');
                            }
                        }
                        $new_cols_names = array();
                        $columns = $this->getTableDataColumns($droptable_tbl);
                        foreach ($columns as $k => $column) {
                            $db_col_index = (int)substr($column->Field, 3);
                            if ($db_col_index >= $data->index + 1) {
                                array_push($new_cols_names, 'CHANGE `' . $column->Field . '` `col' . ($k + 1 + $data->amount) . '` ' . $column->Type);
                            }
                        }
                        $check = $dbo->setQuery(
                            'ALTER TABLE ' . $droptable_tbl . ' ' . implode(', ', $new_cols_names)
                        );
                        $dbo->execute();
                        if ($check) {
                            $dbo->setQuery(
                                'ALTER TABLE ' . $droptable_tbl . ' ' . implode(', ', $add_cols)
                            );
                            $dbo->execute();
                        }

                        //add new columns to header/stylecells
                        $this->updateStyleAddColRow($id, $data->index, $data->amount, $data->left, 'col');
                        break;
                    case 'del_col':
                        if ($max_col < 1) {//nob, del all col
                            $dbo = $this->getDbo();

                            $dbo->setQuery('Select id_category, title, created_time, author, type From #__droptables_tables '
                                . ' WHERE id = ' . (int)$id);

                            $dataTable = $dbo->loadObject();

                            $result = $this->delete($id);
                            if ($result) {
                                $result = $this->add($dataTable->id_category, $dataTable->title, $id, $dataTable->created_time, $dataTable->author, $dataTable->type);
                                if ($result) {
                                    $this->exitStatus(true, 'reload');
                                } else {
                                    return false;
                                }
                            } else {
                                return false;
                            }
                        } else {
                            $this->deleteColumns($data, $droptable_tbl);
                            $this->updateStyleRemoveColRow($id, $data->index, $data->amount, 'col', $data->old_columns);
                        }
                        break;
                    case 'create_row':
                        $this->addRows($data, $droptable_tbl);

                        //add new columns to header/stylecells
                        $this->updateStyleAddColRow($id, $data->index, $data->amount, $data->above, 'row');
                        break;
                    case 'del_row':
                        if ($max_row < 1) {//nob, del all col
                            $dbo = $this->getDbo();

                            $dbo->setQuery('Select id_category, title, created_time, author, type From #__droptables_tables '
                                . ' WHERE id = ' . (int)$id);

                            $dataTable = $dbo->loadObject();

                            $result = $this->delete($id);
                            if ($result) {
                                $result = $this->add($dataTable->id_category, $dataTable->title, $id, $dataTable->created_time, $dataTable->author, $dataTable->type);
                                if ($result) {
                                    $this->exitStatus(true, 'reload');
                                } else {
                                    return false;
                                }
                            } else {
                                return false;
                            }
                        } else {
                            $this->removeRows($data, $droptable_tbl);
                            $this->updateStyleRemoveColRow($id, $data->index, $data->amount, 'row', $data->old_rows);
                        }
                        break;
                    case 'style':
                        $dataRange = new stdClass();
                        foreach ($data->selection as $range) {
                            if (is_null($range[0]) || is_null($range[1]) || is_null($range[2]) || is_null($range[3])) {
                                continue;
                            }
                            $dataRange->row_start = $range[0] + 1;
                            $dataRange->row_end = (int)$range[2] + 1 >= $max_row ? 0 : $range[2] + 1;
                            $dataRange->col_start = $range[1] + 1;
                            $dataRange->col_end = (int)$range[3] + 1 >= $max_col ? 0 : $range[3] + 1;
                            $dataRange->style = $data->style;
                            $this->updateStyle($dataRange, $id);
                        }
                        break;
                    case 'deleteStyle':
                        $dataRange = new stdClass();
                        foreach ($data->selection as $range) {
                            $dataRange->row_start = $range[0] + 1;
                            $dataRange->row_end = (int)$range[2] + 1 >= $max_row ? 0 : $range[2] + 1;
                            $dataRange->col_start = $range[1] + 1;
                            $dataRange->col_end = (int)$range[3] + 1 >= $max_col ? 0 : $range[3] + 1;
                            $range_exists = $this->checkRangeStyleExists($dataRange->row_start, $dataRange->row_end, $dataRange->col_start, $dataRange->col_end, $id);
                            if ($range_exists !== null) {
                                foreach ($range_exists as $k => $value) {
                                    if (is_string($value->style)) {
                                        $old_styles = json_decode($value->style, true) === null ? json_decode($value->style) : json_decode($value->style, true);
                                    } else {
                                        $old_styles = $value->style;
                                    }
                                    if (isset($old_styles[$data->style])) {
                                        unset($old_styles[$data->style]);
                                        $range_style = '#__droptables_range_styles';

                                        $query = $dbo->getQuery(true);

                                        $query->delete($range_style);
                                        $query->where(array($dbo->quoteName('id') . ' = ' . $value->id));
                                        $dbo->setQuery($query);
                                        $result = $dbo->execute();
                                        if (count($old_styles) > 0) {
                                            $values = array($id, $dataRange->row_start, $dataRange->row_end, $dataRange->col_start, $dataRange->col_end, "'" . json_encode($old_styles, JSON_FORCE_OBJECT) . "'");
                                            $query = $dbo->getQuery(true);
                                            $query
                                                ->insert($range_style)
                                                ->columns(array('id_table', 'row_start', 'row_end', 'col_start', 'col_end', 'style'))
                                                ->values(implode(',', $values));
                                            $dbo->setQuery($query);
                                            if (!$dbo->execute()) {
                                                return false;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        break;
                    case 'set_column_type':
                        $type = $this->getColTypeToModify($data->type);
                        $cols = array();
                        $count = count($data->cols);
                        for ($i = 0; $i < $count; $i++) {
                            array_push($cols, 'MODIFY COLUMN `col' . ($data->cols[$i] + 1) . '` ' . $type);
                        }

                        $query = 'ALTER TABLE ' . $droptable_tbl . ' ' . implode(', ', $cols);
                        $dbo->setQuery($query);

                        if ($dbo->execute()) {
                            $this->updateColumnsTypes($id, $droptable_tbl, array('cols' => $data->cols, 'type' => $data->type));
                        } else {
                            return false;
                        }
                        break;
                    case 'set_columns_types_copy':
                    case 'set_columns_types':
                        $cols = array();
                        $return_cols = array();
                        foreach ($data->value as $index => $type_col) {
                            if ($type_col !== null) {
                                $type_col = $this->getColTypeToModify($type_col);
                                array_push($cols, 'MODIFY COLUMN `col' . ((int)$index + 1) . '` ' . $type_col);
                                array_push($return_cols, 'col' . ($index + 1) . ' as col' . $index);
                            }
                        }
                        try {
                            $query = 'ALTER TABLE ' . $droptable_tbl . ' ' . implode(', ', $cols);
                            $dbo->setQuery($query);
                            $dbo->execute();
                            $query = 'SELECT ' . implode(', ', $return_cols) . ' FROM ' . $droptable_tbl . ' Order by line ASC';
                            JFactory::getDbo()->setQuery($query);
                            $data_columns = JFactory::getDbo()->loadObjectList();
                            if ($data_columns) {
//                                $old_params = $this->getTableParams($id, '');
//
//                                //not stripslashes_deep header_data
//                                $old_params = json_decode($old_params);
//                                if (!empty($paramsOld->header_data)) {
//                                    $header_data = $paramsOld->header_data;
//                                    $paramsOld = stripslashes_deep($paramsOld);
//                                    $paramsOld->header_data = $header_data;
//                                }

//                                $this->exitStatus(true, array('id' => $id, 'type' => array('set_columns_types' => $data_columns, 'update_params' => json_encode($old_params))));
                                if ($data->action !== 'set_columns_types_copy') {
                                    $this->exitStatus(true, array('id' => $id, 'type' => array('set_columns_types' => $data_columns)));
                                }
                            }
                        } catch (Exception $e) {
                            return false;
                        }
                        break;
                    case 'set_cells_type':
                        $dataRangeCellTypes = array();
                        $table_params = $this->getTableParams($id, '');

                        if (isset($table_params->params) && is_string($table_params->params)) {
                            $table_params = json_decode($table_params->params, true);
                        }

                        if (is_object($table_params)) {
                            $cell_types = $table_params->cell_types;
                        } else {
                            $cell_types = $table_params['cell_types'];
                        }
                        $maxLengthCellTypes = isset($cell_types) ? count($cell_types) : 0;

                        if (isset($cell_types)) {//update cell_types
                            foreach ($cell_types as $k => $range) {
                                $dataRangeCellTypes[$range[0] . '|' . $range[1] . '|' . $range[2] . '|' . $range[3]] = $k;
                            }
                        } else {//new cell_types
                            $cell_types = array();
                        }
//
//                        if (isset($paramsOld->cell_types)) {//update cell_types
//                            foreach ($paramsOld->cell_types as $k => $range) {
//                                $dataRangeCellTypes[$range[0] . '|' . $range[1] . '|' . $range[2] . '|' . $range[3]] = $k;
//                            }
//                            var_dump($data, $paramsOld);
//                            $maxLengthCellTypes = count($paramsOld->cell_types);
//                        } else {//new cell_types
//                            $paramsOld->cell_types = array();
//                            $maxLengthCellTypes = 0;
//                        }

                        foreach ($data->selection as $range) {
                            $row_start = $range[0];
                            $row_range = (int)$range[2] - (int)$row_start + 1;
                            $col_start = $range[1];
                            $col_range = (int)$range[3] - (int)$col_start + 1;
                            $key = $row_start . '|' . $col_start . '|' . $row_range . '|' . $col_range;

                            if (isset($dataRangeCellTypes[$key])) {
                                $cell_types[$dataRangeCellTypes[$key]][4] = $data->style->cell_type === 'html' ? 'html' : '';
                            } else {
                                $cell_types[$maxLengthCellTypes] = array($row_start, $col_start, $row_range, $col_range, $data->style->cell_type === 'html' ? 'html' : '');
                                $dataRangeCellTypes[$key] = $maxLengthCellTypes;
                                $maxLengthCellTypes++;
                            }
                        }

                        $object = new stdClass();
                        $object->id = $id;
                        if (is_object($table_params)) {
                            $table_params->cell_types = $cell_types;
                        } else {
                            $table_params['cell_types'] = $cell_types;
                        }
                        $object->params = json_encode($table_params);
                        $this->paramsOld = $table_params;

                        $updateNulls = true;
                        if (!JFactory::getDbo()->updateObject('#__droptables_tables', $object, 'id', $updateNulls)) {
                            return false;
                        }
                        break;
//                    case 'change_cells_value':
//                        $row_start = $data->range[0][0];
//                        $row_end = $data->range[0][2];
//                        $col_start = $data->range[0][1];
//                        $col_end = $data->range[0][3];
//                        for ($i = $row_start; $i <= $row_end; $i++) {
//                            for ($j = $col_start; $j <= $col_end; $j++) {
//                                $col_name = 'col' . ($j + 1);
//                                $row_exists = $this->checkRowExists($table_name, $i);
//                                if ($row_exists === null) {
//                                    $wpdb->insert(
//                                        $table_name,
//                                        array(
//                                            'line' => $i,
//                                            $col_name => $data->content
//                                        )
//                                    );
//                                } else {
//                                    $wpdb->update(
//                                        $table_name,
//                                        array(
//                                            'line' => $i,
//                                            $col_name => $data->content
//                                        ),
//                                        array(
//                                            'line' => $i,
//                                        )
//                                    );
//                                }
//                            }
//                        }
//                        break;
//                    case 'set_header_option':
//                        $headerOption = $data->value;
//
//                        $header_data = $this->getTableData($table_name, array( 'where' => ' line < ' . (int)$headerOption));
//                        $params = json_decode($this->getTableParams($id_table, ''), true);
//
//                        $count = count($header_data);
//                        if ($count > 0) {
//                            $params['header_data'] = $header_data;
//                        }
//
//                        try {
//                            $wpdb->update(
//                                $wpdb->prefix . 'droptables_tables',
//                                array(
//                                    'params' => json_encode($params)
//                                ),
//                                array(
//                                    'id' => $id_table
//                                )
//                            );
//                        } catch (Exception $e) {
//                            $this->exitStatus(__('An error occurred!', 'droptables'));
//                        }
//                        break;
                }
            }
        }

        if (!empty($edit_cell_mysql)) {
            $this->exitStatus(true, array('id' => $id, 'type' => array('reload_table' => true)));
        }

        return true;
    }

    /**
     * Convert header rows by cells merge
     *
     * @param integer    $idTable    Id table
     * @param boolean    $updateData Check update data table
     * @param array|null $dataTables Data table
     *
     * @return boolean|array
     */
    public function updateMergeCells($idTable, $updateData, $dataTables = null)
    {
        global $wpdb;
        if ($dataTables === null) {
            $paramsTable = $this->getTableParams($idTable, '');
            if (isset($paramsTable->params) && is_string($paramsTable->params)) {
                $paramsTable = json_decode($paramsTable->params);
            }
            $dataTable = array();
        } else {
            $paramsTable = $dataTables['params'];
            $dataTable = $dataTables['datas'];
        }

        if (is_string($paramsTable)) {
            $paramsTable = json_decode($paramsTable);
        }

        if (is_object($paramsTable)) {
            if (isset($paramsTable->mergeSetting)) {
                $paramsTable->mergeSetting = is_string($paramsTable->mergeSetting) ? json_decode(stripslashes($paramsTable->mergeSetting)) : $paramsTable->mergeSetting;
                $mergeSetting = $paramsTable->mergeSetting;
            }
            $headerOption = isset($paramsTable->headerOption) ?  (int)$paramsTable->headerOption : 1;
        } else {
            if (isset($paramsTable['mergeSetting'])) {
                $paramsTable['mergeSetting'] = is_string($paramsTable['mergeSetting']) ? json_decode(stripslashes($paramsTable['mergeSetting'])) : $paramsTable['mergeSetting'];
                $mergeSetting =  $paramsTable['mergeSetting'];
            }
            $headerOption = isset($paramsTable['headerOption']) ? (int)$paramsTable['headerOption'] : 1;
        }
        $maxHeader = $headerOption;

        if (isset($mergeSetting)) {
            if (is_string($mergeSetting)) {
                $mergeSetting = stripslashes($mergeSetting);
                $mergeSetting = json_decode($mergeSetting);
            }

            if (null !== $mergeSetting) {
                $count = count($mergeSetting);

                $paramsTableMergeSetting = array();

                if ($count > 0) {//check merge and header option
                    for ($i = 0; $i < $count; $i++) {
                        if (is_array($mergeSetting[$i])) {
                            if (empty($mergeSetting[$i][1])) {
                                $paramsTableMergeSetting[$i][0] = $mergeSetting[$i]['row'];
                                $paramsTableMergeSetting[$i][1] = $mergeSetting[$i]['rowspan'];
                                $paramsTableMergeSetting[$i][2] = $mergeSetting[$i]['col'];
                                $paramsTableMergeSetting[$i][3] = $mergeSetting[$i]['colspan'];
                            } else {
                                $paramsTableMergeSetting[$i] = $mergeSetting[$i];
                            }
                        } else {
                            if (empty($mergeSetting[$i]->{1})) {
                                $paramsTableMergeSetting[$i][0] = $mergeSetting[$i]->row;
                                $paramsTableMergeSetting[$i][1] = $mergeSetting[$i]->rowspan;
                                $paramsTableMergeSetting[$i][2] = $mergeSetting[$i]->col;
                                $paramsTableMergeSetting[$i][3] = $mergeSetting[$i]->colspan;
                            } else {
                                $paramsTableMergeSetting[$i] = $mergeSetting[$i];
                            }
                        }
                        $maxHeader = ($paramsTableMergeSetting[$i][0] < $maxHeader && $paramsTableMergeSetting[$i][0] + $paramsTableMergeSetting[$i][1] > $maxHeader) ? $paramsTableMergeSetting[$i][0] + $paramsTableMergeSetting[$i][1] : $maxHeader;
                    }
                }
                if ($maxHeader > $headerOption) {
                    $header_data = $this->getDataCell($idTable, array( 'where' => ' line < ' . (int)$maxHeader));
                    if ($header_data === false) {
                        unset($header_data);
                    }
                    //{"row":2,"col":1,"rowspan":6,"colspan":1}
                }
            }
        }
        if ($updateData && is_array($dataTable)) {
            $header_data = array();
            for ($i = 0; $i < $maxHeader; $i++) {
                if (isset($dataTable[$i])) {
                    $header_data[] = $dataTable[$i];
                }
            }
        }

        if (is_object($paramsTable)) {
            $paramsTable->headerOption = $maxHeader;
            $paramsTable->mergeSetting = isset($paramsTableMergeSetting) ? $paramsTableMergeSetting : array();
        } else {
            $paramsTable['headerOption'] = $maxHeader;
            $paramsTable['mergeSetting'] = isset($paramsTableMergeSetting) ? $paramsTableMergeSetting : array();
        }

        if ($updateData && isset($header_data)) {
            $this->updateHeaderData($header_data, $idTable);
        }
        return $paramsTable;
    }


    /**
     * Set position for table
     *
     * @param string $tables Table data json string
     *
     * @return boolean
     */
    public function setPosition($tables)
    {

        $ids = explode(',', $tables);
        $db = $this->getDbo();
        $i = 1;
        foreach ($ids as $id) {
            $query = $db->getQuery(true);
            $query->update($db->quoteName('#__droptables_tables'))
                ->set(array($db->quoteName('position') . ' = ' . $i))
                ->where(array($db->quoteName('id') . ' = ' . (int)$id));
            $db->setQuery($query);
            if (!$db->execute()) {
                return false;
            }

            $i++;
        }

        return true;
    }


    /**
     * Get count row of table
     *
     * @param integer     $id    Id table
     * @param null|object $table Table data
     *
     * @return integer
     */
    public function countRows($id, $table = null)
    {
        $db = $this->getDbo();
        if (is_null($table)) {
            $db->setQuery('SELECT type, params FROM #__droptables_tables WHERE id = ' . intval($id));
            $table = $db->loadObject();
        }
        $params = $table->params;
        if (is_string($params)) {
            $params = json_decode($table->params);
        }

        if ($table && $table->type === 'mysql') {
            if (!isset($params->tables)) {
                $db = JFactory::getDbo();
                $name = '#__droptables_table_options';
                $query = 'SELECT t.option_value FROM ' . $name . ' as t WHERE t.id_table = ' . (int)$id . ' AND t.option_name = \'query_option\'';
                $db->setQuery($query);
                $query_option = $db->loadObject();
                if ($query_option !== false) {
                    if ($query_option !== null && is_object($query_option)) {
                        $query_option = is_string($query_option->option_value) ? json_decode($query_option->option_value) : $query_option->option_value;//object
                    }
                }
                if (!empty($query_option->tables_list)) {
                    $params->tables = $query_option->tables_list;
                }
                $tables = implode(', ', $params->tables);
                $getTableData = $this->getTableData('SELECT COUNT(*) as total FROM ' . $tables);
                if (gettype($getTableData) === 'array' && !empty($getTableData)) {
                    $total = intval($getTableData[0]['total']);
                } else {
                    $total = intval($getTableData);
                }
            } else {
                $total = 0;
            }
        } else {
            $getTableData = $this->getTableData('SELECT COUNT(*) as total FROM #__droptable_tbl_' . (int)$id);
            if (gettype($getTableData) === 'array' && !empty($getTableData)) {
                $total = intval($getTableData[0]['total']);
            } else {
                $total = intval($getTableData);
            }
        }

        return $total;
    }

    /**
     * Get count filter rows
     *
     * @param integer     $id    Id table
     * @param string      $where Where in query
     * @param null|object $table Data table
     *
     * @return integer
     */
    public function countFilterRows($id, $where = '', $table = null)
    {
        $db = $this->getDbo();
        if (is_null($table)) {
            $db->setQuery('SELECT type, params FROM #__droptables_tables WHERE id = ' . intval($id));
            $table = $db->loadObject();
        }
        $params = $table->params;
        if (is_string($params)) {
            $params = json_decode($table->params);
        }

        if ($where !== '') {
            $where = ' WHERE ' . $where;
        }
        if ($table && $table->type === 'mysql') {
            $tables = implode(', ', $params->tables);
            $getTableData = $this->getTableData('SELECT COUNT(*) as total FROM ' . $tables . $where);
            if (gettype($getTableData) === 'array' && !empty($getTableData)) {
                $total = intval($getTableData[0]['total']);
            } else {
                $total = intval($getTableData);
            }
        } else {
            $getTableData = $this->getTableData('SELECT COUNT(*) as total FROM #__droptable_tbl_' . (int)$id . $where);
            if (gettype($getTableData) === 'array' && !empty($getTableData)) {
                $total = intval($getTableData[0]['total']);
            } else {
                $total = intval($getTableData);
            }
        }

        return $total;
    }

    /**
     * Move table to category
     *
     * @param integer|string $id_table Id table
     * @param integer|string $id_cat   Id category
     *
     * @return boolean
     */
    public function changeCategory($id_table, $id_cat)
    {
        $db = $this->getDbo();
        $query = $db->getQuery(true);
        $query->update($db->quoteName('#__droptables_tables'))
            ->set(array($db->quoteName('id_category') . ' = ' . (int)$id_cat))
            ->where(array($db->quoteName('id') . ' = ' . (int)$id_table));
        $db->setQuery($query);
        if (!$db->execute()) {
            return false;
        }

        return true;
    }

    /**
     * Set table author
     *
     * @param integer $id   Id table
     * @param integer $user Id user
     *
     * @return boolean
     */
    public function setUser($id, $user)
    {
        $db = JFactory::getDbo();
        $name = '#__droptables_tables';
        $query = 'UPDATE ' . $name . ' SET author = ' . $user . ' WHERE id = ' . $id;
        $db->setQuery($query);
        $result = $db->execute();

        //insert rows into table in database
        if (!$result) {
            return false;
        }

        return true;
    }

    /**
     * Update old table
     *
     * @param boolean $id Table id
     *
     * @return boolean|integer|mixed
     */
    public function updateOldTable($id = false)
    {
        $db = JFactory::getDbo();
        $name = '#__droptables_table_options';
        $query = 'SELECT t.option_value FROM ' . $name . ' as t WHERE t.option_name = \'droptables_tables_convert\'';
        $db->setQuery($query);
        $query_option = $db->loadObject();

        if ($id === false) {
            if ($query_option !== null && is_object($query_option)) {
                $query_option = json_decode($query_option->option_value);
            } elseif ($query_option === null) {
                $item = $this->getListTableById();
                if (isset($item[0]) && isset($item[0]->id)) {
                    $query_option = $item[0]->id;
                } else {
                    return false;
                }
            } else {
                return false;//converted
            }
            return $query_option;//start a update cycle
        }

        if ((int)$id > 0) {
            $item  = $this->getItemFromBackup($id);
            require_once(JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR . 'com_droptables' . DIRECTORY_SEPARATOR . 'helpers' . DIRECTORY_SEPARATOR . 'droptablesupdate.php');
            $DroptablesUpdate = new DroptablesUpdate();
            $newData = $DroptablesUpdate->updateTable($item);
            if (!empty($newData['id_category']) && (int)$newData['id_category'] > 0) {
                $newData['params']->headerOption = 1;
                $newData['action'] = 'insert';
                $updateMergeCells = $this->updateMergeCells($id, true, $newData);
                if (is_array($updateMergeCells)) {
                    $newData['params'] = $updateMergeCells;
                }

                if ($this->saveTableSynfile($id, $newData)) {
                    $item = true;
                } else {
                    $this->exitStatus('Error while convert table is id = ' . $id . '!');
                }
            } else {//not update table has id_category = 0
                $item = true;
            }
        }

        if (!empty($item)) {
            $after_id = -2;
            $item = $this->getListTableById();
            $count = count($item);

            for ($i = 0; $i < $count; $i++) {
                if ($item[$i]->id > $id) {
                    $after_id = $item[$i]->id;
                    break;
                }
            }

            //update droptables_tables_convert in table option
            if ($query_option === null) {
                $query = $db->getQuery(true);
                $query
                    ->insert($db->quoteName('#__droptables_table_options'))
                    ->values(implode(',', array($db->quote('0'), $db->quote('droptables_tables_convert'), $db->quote($after_id))))
                    ->columns(implode(',', array('id_table', 'option_name', 'option_value')));
                $db->setQuery($query);
            } else {
                $query = 'UPDATE ' . $name . ' SET option_value = ' . $after_id . ' WHERE option_name = \'droptables_tables_convert\'';
                $db->setQuery($query);
            }
            $result = $db->execute();
            if (!$result) {
                $this->exitStatus('Error while convert table!');
            }

//            /*remove data column*/ xem lai, update function da remove datas column, co the remove lai cho chac
            if ($after_id === -2) {
                //check datas column exist
                $db = JFactory::getDbo();
                $query = 'SHOW COLUMNS from `#__droptables_tables` LIKE \'datas\'';
                $db->setQuery($query);
                $columns_obj = $db->loadObject();
                if ($columns_obj !== null) {
                    $query = 'ALTER TABLE `#__droptables_tables` DROP COLUMN datas';
                    $db->setQuery($query);
                    $result = $db->execute();
                    if (!$result) {
                        $this->exitStatus('Error while convert table!');
                    }
                }
            }

            return $after_id;
        } else {
            $this->exitStatus('Error while convert table is id = ' . $id . '!');
        }
    }

    /**
     * Get list table order by id table
     *
     * @param string|null $where Where query
     *
     * @return boolean|array
     */
    public function getListTableById($where = null)
    {
        $dbo = JFactory::getDbo();
        $query = 'SELECT c.id FROM #__droptables_tables as c ';
        if ($where !== null) {
            $query .= $where;
        }
        $query .= ' ORDER BY c.id ASC';

        $dbo->setQuery($query);
        $listTables = $dbo->loadObjectList();

        return $listTables;
    }

    /**
     * Function get table data from droptables_backup table database
     *
     * @param integer $id Id of table
     *
     * @return array|false|mixed
     */
    public function getItemFromBackup($id)
    {
        $db = JFactory::getDbo();
        $table_name = '#__droptables_backup';
        $query = 'SELECT t.* FROM ' . $table_name . ' as t WHERE t.id = ' . (int)$id;
        $db->setQuery($query);
        $item = $db->loadObject();

        if (!$item || is_null($item)) {
            return false;
        }
        if (is_string($item->params)) {
            $item->params = str_replace(array("\n\r", "\r\n", "\n", "\r", '&#10;'), ' ', $item->params);
            $item->params = json_decode($item->params);
        }

        if ($item->style === '') {
            $item->style = new stdClass();
        } else {
            $item->style = json_decode($item->style);
        }

        $item->css = preg_replace('/\\\n/', '', $item->css);
        return $item;
    }

    /**
     * Craete new row table
     *
     * @param string $table Table id
     * @param string $field Name dbtable
     * @param string $id    Data from ajax
     *
     * @return boolean
     */
    public function deleteRowDbTable($table, $field, $id)
    {
        $result = false;
        if (!empty($field)) {
            $dbo = JFactory::getDbo();
            $prefix = $dbo->getPrefix();
            $dbTables = str_replace($prefix, '#__', $table);

            $query = 'DELETE FROM ' . $dbTables . ' WHERE ' . $dbo->quoteName($field) . ' = ' . $id;
            $dbo->setQuery($query);
            $result = $dbo->execute();
        }

        return $result;
    }

    /**
     * Craete new row table
     *
     * @param integer $id_table Table id
     * @param string  $dbtable  Name dbtable
     * @param array   $datas    Data from ajax
     *
     * @return boolean
     */
    public function createRowDbTable($id_table, $dbtable, $datas)
    {
        $result = false;
        if (!empty($datas)) {
            $db = JFactory::getDbo();
            $prefix = $db->getPrefix();
            $dbtable = str_replace($prefix, '#__', $dbtable);

            $db_data = new stdClass();
            foreach ($datas as $key => $data) {
                $db_data->{$key} = $data;
            }
            try {
                $result = $db->insertObject($dbtable, $db_data);
            } catch (Exception $e) {
                if (isset($e->xdebug_message)) {
                    $this->exitStatus('An error occurred!' . $e->xdebug_message);
                } else {
                    $this->exitStatus('An error occurred!');
                }
            }
        }
        return $result;
    }

    /**
     * Add data of new theme for table
     *
     * @param integer $id_table Id of table
     * @param object  $item     Data of theme
     *
     * @return boolean
     */
    public function addThemeToTable($id_table, $item)
    {
        $db = JFactory::getDbo();
        $curr_data = json_decode($item->data, true);
        $this->createNewTableData($id_table, $curr_data, true);

        //update data in wptm_tables
        $tableContent['style'] = new stdClass();
        $tableContent['style']->rows = json_decode($item->rows_data, true);
        $tableContent['style']->cols = json_decode($item->cols_data, true);
        $object = new stdClass();
        $object->id = $id_table;
        $object->css = '';
        $object->modified_time = date('Y-m-d H:i:s');
        $object->hash = date('Y-m-d H:i:s');
        $object->style = json_encode($tableContent['style']);
        $object->params = json_encode($item->params);
        $updateNulls = false;
        if (!$db->updateObject('#__droptables_tables', $object, 'id', $updateNulls)) {
            return false;
        }
        //update style for cell ranger
        $curr_data = json_decode($item->styles, true);
        $string = '';

        foreach ($curr_data as $item) {
            if ($string !== '') {
                $string .= ',';
            }
            $string .= '(' . $id_table . ',' . $item[0] . ',' . $item[1] . ',' . $item[2] . ',' . $item[3] . ",'" . $item[4] . "')";
        }
        $query = 'insert into #__droptables_range_styles (id_table,row_start,row_end,col_start,col_end,style) values ' . $string;

        $db->setQuery($query);

        if (!$db->execute()) {
            return false;
        }
        return true;
    }


    /**
     * Update cell data to table
     *
     * @param integer $id_table     Table id
     * @param array   $curr_data    Data cell
     * @param boolean $create_table Create new table in db
     *
     * @return boolean
     */
    public function createNewTableData($id_table, $curr_data, $create_table = false)
    {
        $db = JFactory::getDbo();
        $table_name = 'droptable_tbl_' . $id_table;
        $droptable_tbl = $db->quoteName('#__droptable_tbl_' . $id_table);
        $countCol = count($curr_data[0]);

        $columns = array();
        $column = ' (line';
        for ($i = 1; $i <= $countCol; $i++) {
            $columnDefault = array('name' => 'col' . $i, 'type' => 'mediumtext');
            $columns[] = $columnDefault;
            $column .= ',col' . $i;
        }
        $column .= ')';

        if ($create_table) {
            $result = $this->createTblInDb($table_name, $columns);
        }

        if ($result) {
            $db->setQuery(
                "SHOW VARIABLES LIKE 'max_allowed_packet'"
            );
            $max_allowed_packet = $db->loadObject();
            $data_insert = $this->createStringDataCells($curr_data, 5, $max_allowed_packet->Value);
            $countRow = count($data_insert);
            for ($i = 0; $i <= $countRow; $i++) {
                if (isset($data_insert[$i]) && $data_insert[$i] !== '') {
                    $query =
                        'INSERT INTO ' . $droptable_tbl . $column . ' VALUES ' . $data_insert[$i]
                    ;
                    $db->setQuery($query);
                    $result = $db->execute();
                }

                if (!$result) {
                    return false;
                }
            }
        }
        return true;
    }

    /**
     * Validate data
     *
     * @param string      $form  Form
     * @param array       $data  Data
     * @param null|string $group Group
     *
     * @return mixed
     */
    public function validate($form, $data, $group = null)
    {
        $validData = $data;
        if (isset($data['datas']) && $data['datas'] === '') {
            unset($validData['datas']);
            unset($validData['params']);
        }
        return $validData;
    }

    /**
     * Return a json response
     *
     * @param boolean $status Status responsive
     * @param array   $datas  Data array to return with the json string
     *
     * @since 3.5.0
     *
     * @return void
     */
    private function exitStatus($status, $datas = array())
    {
        $response = array('response' => $status, 'datas' => $datas);
        echo json_encode($response);
        JFactory::getApplication()->close();
    }
}
