<?php
/**
 * Droptables
 *
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 *
 * @package   Droptables
 * @copyright Copyright (C) 2014 JoomUnited (http://www.joomunited.com). All rights reserved.
 * @copyright Copyright (C) 2014 Damien Barrère (http://www.crac-design.com). All rights reserved.
 * @license   GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 */

// no direct access
defined('_JEXEC') || die;

jimport('joomla.application.component.modellist');
jimport('joomla.access.access');

/**
 * Class DroptablesModelTables
 */
class DroptablesModelTables extends JModelList
{
    /**
     * Get list query table
     *
     * @return string
     * @since  1.6
     */
    public function getListQuery()
    {
        // Create a new query object.
        $db = $this->getDbo();
        $query = $db->getQuery(true);

        // Select the required fields from the table.
        $query->select(
            $this->getState(
                'list.select',
                't.id, t.id_category, t.title, t.created_time, t.modified_time, t.author, t.type'
            )
        );
        $query->from('#__droptables_tables AS t');
        $query->order('position');
        // Filter by category
        $category = $this->getState('filter.category');
        if ($category) {
            $query->where('t.id_category = ' . $db->quote($category));
        }
        // ACL
        $canDo = DroptablesHelper::getActions();
        if (($canDo->get('core.edit.own') && !$canDo->get('core.edit')) ||
            (!$canDo->get('core.edit.own') && !$canDo->get('core.edit'))) {
            $query->where('t.author=' . (int)JFactory::getUser()->id);
        }

        return $query;
    }

    /**
     * Get form model
     *
     * @return string
     * @since  1.6
     */
    public function getItemsForModal()
    {
        // Create a new query object.
        $db = $this->getDbo();
        $query = $db->getQuery(true);

        // Select the required fields from the table.
        $query->select(
            $this->getState(
                'list.select',
                't.id, t.id_category, t.title, t.created_time, t.modified_time, t.author'
            )
        );
        $query->from('#__droptables_tables AS t');
        // Join over the language
        $query->select('c.title AS category_title')
            ->join('LEFT', $db->quoteName('#__categories') . ' AS c ON c.id = t.id_category');

        // Join over the users for the checked out user.
        $query->select('uc.name AS author')
            ->join('LEFT', '#__users AS uc ON uc.id=t.author');

        $query->order('position');
        // Filter by category
        $category = $this->getState('filter.category');
        if ($category) {
            $query->where('t.id_category = ' . $db->quote($category));
        }

        $limitstart = $this->getStart();
        $limit = $this->getState('list.limit');
        $db->setQuery($query, $limitstart, $limit);
        $items = $db->loadObjectList();

        return $items;
    }

    /**
     * Get list User
     *
     * @return boolean
     */
    public function getUsers()
    {
        $db = JFactory::getDbo();
        $name = '#__users';
        $query = 'SELECT id, name, username, email FROM ' . $name . ' Order by id ASC';
        $db->setQuery($query);
        $styles = $db->loadObjectList();
        if ($styles === false) {
            return false;
        }

        return $styles;
    }

    /**
     * Get list users roles
     *
     * @return boolean
     */
    public function getUserRoles()
    {
        $db = JFactory::getDbo();
        $name = '#__usergroups';
        $query = 'SELECT id, parent_id, title FROM ' . $name . ' Order by id ASC';
        $db->setQuery($query);
        $styles = $db->loadObjectList();
        if ($styles === false) {
            return false;
        }

        return $styles;
    }

    /**
     * Auto-populate the model state.
     * Note. Calling getState in this method will result in recursion.
     *
     * @param null $ordering  Ordering
     * @param null $direction Direction
     *
     * @return void
     * @since  1.6
     */
    protected function populateState($ordering = null, $direction = null)
    {
        $category = JFactory::getApplication()->input->getInt('id_category', 0);
        $this->setState('filter.category', $category);

        parent::populateState($ordering, $direction);

        $this->setState('list.limit', 100000);
    }


    /**
     * Method to add a file into database
     *
     * @param array $data File to add
     *
     * @return inserted row id, false if an error occurs
     */
    public function addTable($data)
    {
        $dbo = $this->getDbo();
        $date = JFactory::getDate();
        $ordering = $this->getNextPosition($data['id_category']);
        $query = 'INSERT INTO #__droptables_files (file,catid,ordering,title,ext,size,created_time,modified_time) 
                  VALUES (' . $dbo->quote($data['file']) . ',' . intval($data['id_category']) . ',' . intval($ordering) . ',' . $dbo->quote($data['title']) . ',' . $dbo->quote($data['ext']) . ',' . (int)$data['size'] . ',' . $dbo->quote($date->toSql()) . ',' . $dbo->quote($date->toSql()) . ')';
        $dbo->setQuery($query);
        if (!$dbo->execute()) {
            return false;
        }
        return $dbo->insertid();
    }


    /**
     * Methode to retrieve file information
     *
     * @param integer $id_file Id table file
     *
     * @return object file, false if an error occurs
     */
    public function getFile($id_file)
    {
        $dbo = $this->getDbo();
        $query = 'SELECT * FROM #__droptables_files WHERE id=' . $dbo->quote($id_file);
        $dbo->setQuery($query);
        if (!$dbo->execute()) {
            return false;
        }
        return $dbo->loadObject();
    }

    /**
     * Get list db_tables
     *
     * @return boolean|mixed
     */
    public function getDbItems()
    {
        $db = $this->getDbo();

        $query = 'SELECT c.* FROM #__droptables_tables as c WHERE c.type = "mysql" ORDER BY c.id_category ASC, c.position ASC ';
        $db->setQuery($query);
        $query_option = $db->loadObjectList();

        return $query_option;
    }
}
