<?php
/**
 * Droptables
 *
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 *
 * @package   Droptables
 * @copyright Copyright (C) 2014 JoomUnited (http://www.joomunited.com). All rights reserved.
 * @copyright Copyright (C) 2014 Damien Barrère (http://www.crac-design.com). All rights reserved.
 * @license   GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 */

// No direct access.
defined('_JEXEC') || die;
use Joomla\CMS\Session\Session;
use Joomla\CMS\HTML\HTMLHelper;

$icon = array(
    'ok'    => '<div class="controls"><i class="material-icons system-checkbox material-icons-success">check_circle</i></div>',
    'alert' => '<div class="controls"><i class="material-icons system-checkbox material-icons-alert">info</i></div>',
    'info'  => '<div class="controls"><img class="system-checkbox material-icons-info bell" src="' . JUri::root(true) . '/administrator/components/com_droptables/assets/images/icon-notification.png" /></div>'
);

$extension_array = array(
    array(
        'id'      => 'curl',
        'title'   => 'Php_Curl',
        'tooltip' =>  'PHP extension Curl is NOT detected. The cache preloading feature will not work (preload a page cache automatically after a cache purge'
    ),
    array(
        'id'      => 'zlib',
        'title'   => 'ext-zlib',
        'tooltip' =>  'PHP extension ext-zlib is NOT detected. The function to sync with google sheet, or import, export excel file will not work'
    ),
    array(
        'id'      => 'xml',
        'title'   => 'ext-xml',
        'tooltip' =>  'PHP extension ext-xml is NOT detected. The function to sync with google sheet, or import, export excel file will not work'
    ),
    array(
        'id'      => 'zip',
        'title'   => 'ext-zip',
        'tooltip' =>  'PHP extension ext-zip is NOT detected. The function to sync with google sheet, or import, export excel file will not work'
    ),
    array(
        'id'      => 'mbstring',
        'title'   => 'ext-mbstring',
        'tooltip' =>  'PHP extension ext-mbstring is NOT detected. The function to sync with google sheet, or import, export excel file will not work'
    )
);

JFactory::getDocument()->addScriptDeclaration("
    var DroptableConfigUrl1 = '" . JRoute::_('index.php?option=com_droptables&view=configuration') . "';
    var DroptableConfigUrl2 = DroptableConfigUrl1 + '&task=configuration.closeConfiguration';
    var DroptableConfigUrl;
    var task_action;
    Joomla.submitbutton = function(task)
    {
        if (task == 'wizard.cancel' || document.getElementById('configuration-form'))
        {
            DroptableConfigUrl = DroptableConfigUrl1 + '&task=' + task;
            task_action = task;
            document.getElementById('drop_config_cancel_close').click();
            document.getElementById('configuration-form-submit').click();
            document.getElementById('configuration-roles-form-submit').click();
        }
    };
");

/**
 * Parse module info.
 * Based on https://gist.github.com/sbmzhcn/6255314
 *
 * @return array
 */
function parsePhpinfo()
{
    ob_start();
    //phpcs:ignore WordPress.PHP.DevelopmentFunctions.prevent_path_disclosure_phpinfo -- Get info modules of phpinfo
    phpinfo(INFO_MODULES);
    $s = ob_get_contents();
    ob_end_clean();
    $s     = strip_tags($s, '<h2><th><td>');
    $s     = preg_replace('/<th[^>]*>([^<]+)<\/th>/', '<info>\1</info>', $s);
    $s     = preg_replace('/<td[^>]*>([^<]+)<\/td>/', '<info>\1</info>', $s);
    $t     = preg_split('/(<h2[^>]*>[^<]+<\/h2>)/', $s, - 1, PREG_SPLIT_DELIM_CAPTURE);
    $r     = array();
    $count = count($t);
    $p1    = '<info>([^<]+)<\/info>';
    $p2    = '/' . $p1 . '\s*' . $p1 . '\s*' . $p1 . '/';
    $p3    = '/' . $p1 . '\s*' . $p1 . '/';
    for ($i = 1; $i < $count; $i ++) {
        if (preg_match('/<h2[^>]*>([^<]+)<\/h2>/', $t[$i], $matchs)) {
            $name = trim($matchs[1]);
            $vals = explode("\n", $t[$i + 1]);
            foreach ($vals as $val) {
                if (preg_match($p2, $val, $matchs)) { // 3cols
                    $r[$name][trim($matchs[1])] = array(trim($matchs[2]), trim($matchs[3]));
                } elseif (preg_match($p3, $val, $matchs)) { // 2cols
                    $r[$name][trim($matchs[1])] = trim($matchs[2]);
                }
            }
        }
    }

    return $r;
}

$phpInfo = parsePhpinfo();

$parameters = $this->form->getFieldset('jutranslation');
$main_settings = $this->form->getFieldset('main');
$main_format = $this->form->getFieldset('format');
//$field->input
/**
 * Create default alternater color option
 *
 * @param object $field   Field
 * @param array  $data    Alternate color data
 * @param array  $plugins List plugin
 *
 * @return string
 */
function changeDefaultOption($field, $data, $plugins)
{
    $html = '';
    if ($field->id === 'jform_alternate_color') {
        $default = '#bdbdbd|#ffffff|#f3f3f3|#ffffff';
        $default .= '|#4dd0e1|#ffffff|#e0f7fa|#a2e8f1';
        $default .= '|#63d297|#ffffff|#e7f9ef|#afe9ca';
        $default .= '|#f7cb4d|#ffffff|#fef8e3|#fce8b2';
        $default .= '|#f46524|#ffffff|#ffe6dd|#ffccbc';
        $default .= '|#5b95f9|#ffffff|#e8f0fe|#acc9fe';
        $default .= '|#26a69a|#ffffff|#ddf2f0|#8cd3cd';
        $default .= '|#78909c|#ffffff|#ebeff1|#bbc8ce';

        $values  = isset($data['alternate_color']) && (string) $data['alternate_color'] !== '' ? $data['alternate_color'] : $default;
        $html = renderContentAlternate($field, $values);
    } elseif ($field->type === 'Radio') {
        $checked = ($data->get($field->fieldname, $field->value)) ? 'checked="checked"' : '';
        $html = '<div class="ju-switch-button">
                            <label class="switch">
                                <input type="checkbox" '.$checked.' name="'.$field->name.'" id="'.$field->id.'" value="1" />
                                <span class="slider"></span>
                            </label></div>';
    } elseif ($field->id === 'jform_include_editor_buttons') {
        $value = ($field->value !== null && $field->value !== '') ? $field->value : $field->default;
        $html = '<input type="text" name="jform[include_editor_buttons]" id="jform_include_editor_buttons2" value="'
            . $value . '" class="form-control input-mini" aria-describedby="jform[include_editor_buttons]-desc" style="display: none">';
        $html .= '<span id="jform_include_editor_buttons" data-value="' . $value . '" class="popup_select droptable_select_box_before" style="max-width: 250px;overflow: hidden;text-overflow: ellipsis;padding-right: 40px;"></span><ul class="droptable_select_box">';
        $html .= '<li class="root data-none" data-value="" style="padding-left: 20px;">' . JText::sprintf('COM_DROPTABLES_UNSELECT_ALL') . '</li>';

        foreach ($plugins as $plugin) {
            $html .= '<li class="root" data-value="' . $plugin . '"><input type="checkbox"/>' . $plugin . '</li>';
        }
        $html .= '</ul>';
//        $html .= '<select name="jform[include_editor_buttons]" id="jform_include_editor_buttons" multiple>';
//        foreach ($plugins as $plugin) {
//            $html .= '<option value="' . $plugin . '">' . $plugin . '</option>';
//        }
//        $html .= '</select>';
    } else {
        $html = $field->input;
    }
    return $html;
}

/**
 * Render content
 *
 * @param object $field  Data format color
 * @param string $values Value
 *
 * @return string
 */
function renderContentAlternate($field, $values)
{
    $html    = '';
    $html    .= '<div id="control_format_style">';
    $html    .= '<div class="label_text">' .  JText::sprintf('COM_DROPTABLES_AUTOMATIC_STYLING')  . ':</div>';
    $html    .= '<div class="control_value" style="display: none">';
    $html    .= '<input type="text" name="jform[alternate_color]" id="' . $field->id . '" value="'. $values .'" class="' . $field->class . '" aria-describedby="jform[alternate_color]-desc" value="' . $values . '">';
    $html    .= '</div>';
    $html    .= '<div id="list_format_style">';
    $arrayValue  = explode('|', $values);
    $count   = count($arrayValue);
    for ($i = 0; $i < $count / 4; $i ++) {
        $i16   = $i * 4;
        $value = array($arrayValue[$i16], $arrayValue[$i16 + 1], $arrayValue[$i16 + 2], $arrayValue[$i16 + 3]);
        $html .= renderListStyle($value, $i);
    }
    $html .= '</div>';
    $html .= '<div id="new_format_style">';
    $html .= '<input type="button" class="active create_format_style" value="New">';
    $html .= '<input type="button" class="droptables_no_active remove_format_style" value="Remove">';
    $html .= '<span class="hide_set_format_style show">' . JText::sprintf('COM_DROPTABLES_SHOW_DETAIL_COLOR') . '
            <i class="material-icons">keyboard_arrow_down</i>
            <i class="material-icons">keyboard_arrow_up</i></span>';
    $html .= '</div>';
    $value = array('#ffffff', '#ffffff', '#ffffff', '#ffffff');
    $html .= renderListStyle($value, 'create');
    $html .= '<div id="save_format_style">';
    $html .= '<input class="ju-button orange-button" type="button" value="' . JText::sprintf('COM_DROPTABLES_DONE') . '">';
    $html .= '<input class="ju-button droptables_no_active" type="button" value="' . JText::sprintf('COM_DROPTABLES_CANCEL') . '">';
    $html .= '</div>';
    $html .= '</div>';
    return $html;
}

/**
 * Render list style color
 *
 * @param array   $value Style value
 * @param integer $order Order number
 *
 * @return string
 */
function renderListStyle($value, $order)
{
    $html = '';
    if ($order !== 'create') {
        $html .= '<div class="pane-color-tile td_' . $order . '">';
        $html .= '<div class="pane-color-tile-header pane-color-tile-band" data-value="' . $value[0] . '" style="background-color:' . $value[0] . ';"></div>';
        $html .= '<div class="pane-color-tile-1 pane-color-tile-band" data-value="' . $value[1] . '" style="background-color:' . $value[1] . ';"></div>';
        $html .= '<div class="pane-color-tile-2 pane-color-tile-band" data-value="' . $value[2] . '" style="background-color:' . $value[2] . ';"></div>';
        $html .= '<div class="pane-color-tile-footer pane-color-tile-band" data-value="' . $value[3] . '" style="background-color:' . $value[3] . ';"></div>';
        $html .= '</div>';
    } else {
        $html .= '<div id="set_color" class="input-pane-set-color">';
        $html .= '<div class="control_value">';
        $html .= '<span>' . JText::sprintf('COM_DROPTABLES_ALTERNATE_COLOR_HEADER') . '</span>';
        $html .= '<input class="pane-set-color-header minicolors minicolors-input observeChanges" data-position="left"
                                   data-control="hue" type="text" value="#ffffff" size="7"/>';
        $html .= '</div>';
        $html .= '<div class="control_value">';
        $html .= '<span>' . JText::sprintf('COM_DROPTABLES_ALTERNATE_COLOR_ONE') . '</span>';
        $html .= '<input class="pane-set-color-1 minicolors minicolors-input observeChanges" data-position="left"
                                   data-control="hue" type="text" value="#ffffff" size="7"/>';
        $html .= '</div>';
        $html .= '<div class="control_value">';
        $html .= '<span>' . JText::sprintf('COM_DROPTABLES_ALTERNATE_COLOR_TWO') . '</span>';
        $html .= '<input class="pane-set-color-2 minicolors minicolors-input observeChanges" data-position="left"
                                   data-control="hue" type="text" value="#ffffff" size="7"/>';
        $html .= '</div>';
        $html .= '<div class="control_value">';
        $html .= '<span>' . JText::sprintf('COM_DROPTABLES_ALTERNATE_COLOR_FOOTER') . '</span>';
        $html .= '<input class="pane-set-color-footer minicolors minicolors-input observeChanges" data-position="left"
                                   data-control="hue" type="text" value="#ffffff" size="7"/>';
        $html .= '</div>';
        $html .= '</div>';
    }
    return $html;
}
$class_name = '';
if (droptablesBase::isJoomla40()) {
    $class_name .= ' joomla4';
} elseif (droptablesBase::isJoomla30()) {
    $class_name .= ' joomla30';
}

?>

<div class="wrap droptables-config<?php echo $class_name; ?>">
        <div class="ju-main-wrapper">
            <div class="ju-left-panel">
                <div class="ju-logo">
                    <a href="https://www.joomunited.com/products/droptables" target="_blank" >
                        <img src="<?php echo JUri::root(true) . '/administrator/components/com_droptables/assets/joomla-css-framework/images/logo-joomUnited-white.png'; ?>"
                             srcset="<?php echo JUri::root(true) . '/administrator/components/com_droptables/assets/joomla-css-framework/images/logo-joomUnited-white.png'; ?>"
                             alt="Joom Logo">
                    </a>
                </div>
                <div class="ju-menu-search">
                    <i class="mi mi-search ju-menu-search-icon"></i>
                    <input type="text" class="ju-menu-search-input" size="16" placeholder="Search settings">
                </div>
                <ul class="tabs ju-menu-tabs">
                    <li class="tab">
                        <a href="#droptables_settings" class="link-tab waves-effect waves-light">
                            <i class="material-icons">home</i>
                            <?php echo JText::sprintf('COM_DROPTABLES_CONFIG_MAIN_SETTING') ?>
                        </a></li>
                    <li class="tab">
                        <a href="#droptables-user-roles" class="link-tab waves-effect waves-light user-roles">
                            <img src="<?php echo JUri::root(true) . '/administrator/components/com_droptables/assets/images/icon-user-roles.svg' ?>" class="mCS_img_loaded">
                            <?php echo JText::sprintf('COM_DROPTABLES_CONFIG_USER_ROLES') ?>
                        </a></li>
                    <li class="tab">
                        <a href="#jutranslation" class="link-tab waves-effect waves-light">
                            <i class="material-icons">text_format</i>
                            <?php echo JText::sprintf('COM_DROPTABLES_CONFIG_TRANSLATION') ?>
                        </a></li>
                    <li class="tab">
                        <a href="#droptables_check" class="link-tab waves-effect waves-light">
                            <i class="material-icons">check_circle_outline</i>
                            <?php echo JText::sprintf('COM_DROPTABLES_CONFIG_SYSTEM_CHECK') ?>
                        </a></li>
                    <li class="tab">
                        <a href="#liveUpdate" class="link-tab waves-effect waves-light">
                            <i class="material-icons">check_circle_outline</i>
                            <?php echo JText::sprintf('COM_DROPTABLES_CONFIGURATION_NAV_NAME_LIVE_UPDATES') ?>
                        </a></li>
                </ul>
            </div>

            <div class="ju-right-panel">
                <form id="configuration-form" method="POST" action="#">
                    <?php echo HTMLHelper::_('form.token'); ?>
                    <div class="ju-content-wrapper" id="droptables_settings">
                        <div class="ju-top-tabs-wrapper">
                            <ul class="tabs ju-top-tabs">
                                <li class="tab active">
                                    <a href="#droptables_main_settings"
                                       class="link-tab"><?php echo JText::sprintf('COM_DROPTABLES_CONFIG_MAIN_SETTING') ?></a>
                                </li>
                                <li class="tab">
                                    <a href="#droptables_format" class="link-tab"><?php echo JText::sprintf('COM_DROPTABLES_CONFIG_FORMAT') ?></a>
                                </li>
                            </ul>
                        </div>
                        <?php
                        $message = JFactory::getSession()->get('droptable_configuration_message');
                        if ($message) :
                            ?>
                            <div id="ju-message-container">
                                <div class="alert alert-<?php echo $message['type'] ? 'success' : 'error';?>">
                                    <button type="button" class="close" data-dismiss="alert">×</button>
                                    <h4 class="alert-heading">Message</h4>
                                    <div class="alert-message"><?php echo JText::sprintf($message['text']); ?></div>
                                </div>
                            </div>
                        <?php endif; ?>
                        <div class="ju-notice-msg ju-notice-success">
                            <p></p>
                            <i class="dashicons dashicons-dismiss ju-notice-close"></i>
                        </div>
                        <div class="ju-notice-msg ju-notice-error">
                            <p></p>
                            <i class="dashicons dashicons-dismiss ju-notice-close"></i>
                        </div>
                        <div id="droptables_main_settings" class="tab-pane droptables_show_hiden_option"
                             data-option="enable_import_excel|export_excel_format|alternate_color|hightlight|enable_autosave|open_table|droptables_sync_method|sync_periodicity|enable_frontend|uninstall_delete_files">
                            <h2 style="margin-top: 15px">
                                <?php echo JText::sprintf('COM_DROPTABLES_CONFIG_MAIN_SETTING') ?></h2>

                            <?php foreach ($main_settings as $k => $field) : ?>
                                <li class="ju-settings-option ju-padding-20" style="<?php echo $field->type === 'Hidden' ? 'display: none' : ''?>">
                                    <div>
                                        <div class="ju-custom-block ju-width-100 control-group <?php echo $field->id; ?>">
                                            <label data-toggle="tooltip" data-placement="top" class="ju-setting-label"
                                                   for="<?php echo $field->id; ?>"
                                                   title="<?php echo JText::sprintf($field->description) ?>"><?php echo strip_tags($field->label) ?></label>
                                            <div class="controls"
                                                <?php if ($field->id === 'jform_include_editor_buttons') : ?>
                                                style="position: relative;"
                                                <?php endif; ?>>
                                                <?php echo changeDefaultOption($field, $this->paramsData, $this->plugins); ?>
                                            </div>
                                        </div>
                                    </div>
                                </li>
                            <?php endforeach; ?>
                        </div>
                        <div id="droptables_format" class="tab-pane droptables_show_hiden_option"
                             data-option="date_formats|symbol_position|decimal_sym|decimal_count|thousand_sym|currency_sym">
                            <h2 style="margin-top: 15px">
                                <?php echo JText::sprintf('COM_DROPTABLES_CONFIG_FORMAT_SETTING') ?></h2>

                            <?php foreach ($main_format as $k => $field) : ?>
                                <li class="ju-settings-option ju-padding-20">
                                    <div>
                                        <div class="ju-custom-block ju-width-100 control-group">
                                            <label data-toggle="tooltip" data-placement="top" class="ju-setting-label" for="<?php echo $field->id;?>"
                                                   title="<?php echo JText::sprintf($field->description) ?>"><?php echo strip_tags($field->label) ?></label>
                                            <div class="controls">
                                                <?php echo $field->input; ?>
                                            </div>

                                        </div>
                                    </div>
                                </li>
                            <?php endforeach; ?>
                        </div>
                    </div>
                    <input type="submit" name="submit" style="display: none" id="configuration-form-submit" class=" ju-button orange-button submit_form" value="Save">
                </form>

                <div class="ju-content-wrapper" id="droptables-user-roles">
                    <?php
                    $message = JFactory::getSession()->get('droptable_configuration_message');
                    if ($message) :
                        JFactory::getSession()->clear('droptable_configuration_message');
                        ?>
                        <div id="ju-message-container">
                            <div class="alert alert-<?php echo $message['type'] ? 'success' : 'error';?>">
                                <button type="button" class="close" data-dismiss="alert">×</button>
                                <h4 class="alert-heading">Message</h4>
                                <div class="alert-message"><?php echo JText::sprintf($message['text']); ?></div>
                            </div>
                        </div>
                    <?php endif; ?>
                    <h2>
                        <?php echo JText::sprintf('COM_DROPTABLES_CONFIG_USER_ROLES') ?></h2>

                    <div class="ju-content-wrapper full-width">
                        <?php
                        echo $this->loadTemplate('role');
                        ?>
                    </div>
                </div>

                <div class="ju-content-wrapper" id="jutranslation" >
                    <?php echo $this->loadTemplate('jutranslation'); ?>
                </div>

                <div class="ju-content-wrapper" id="droptables_check">
                    <h2>
                        <?php echo JText::sprintf('COM_DROPTABLES_CONFIG_SYSTEM_CHECK_SETTING') ?></h2>
                    <div class="text-intro">
                        <blockquote>
                            <?php echo JText::sprintf('COM_DROPTABLES_CONFIG_SYSTEM_CHECK_SETTING_TITLE'); ?>
                        </blockquote>
                    </div>
                    <div class="php_version">
                        <h3>
                            <?php echo JText::sprintf('COM_DROPTABLES_CONFIG_PHP_VER'); ?></h3>
                        <div class="ju-settings-option full-width">
                            <label class="ju-setting-label system-check-label">
                                <?php echo 'PHP '; ?>
                                <?php echo PHP_VERSION ?>
                                <?php echo 'version'; ?>
                            </label>
                            <?php
                            if (version_compare(PHP_VERSION, '7.1.0', '>=')) {
                                //phpcs:ignore WordPress.Security.EscapeOutput -- Echo icon html
                                echo $icon['ok'];
                            } else {
                                //phpcs:ignore WordPress.Security.EscapeOutput -- Echo icon html
                                echo $icon['alert'];
                            }
                            ?>
                        </div>
                        <?php if (version_compare(PHP_VERSION, '7.1.0', '<')) : ?>
                            <p>
                                <?php echo JText::sprintf('COM_DROPTABLES_CONFIG_SYSTEM_CHECK_SETTING_YOUR_PHP_VER'); ?>
                                <?php echo PHP_VERSION; ?>
                                <?php echo '.' . JText::sprintf('COM_DROPTABLES_CONFIG_SYSTEM_CHECK_SETTING_PLEASE_UPDATE_YOUR_PHP_VER'); ?>
                            </p>
                        <?php else : ?>
                            <p style="height: auto">
                                <?php echo JText::sprintf('COM_DROPTABLES_CONFIG_SYSTEM_CHECK_SETTING_GREAT_YOUR_PHP_VER'); ?>
                                <?php echo PHP_VERSION; ?>
                                <?php if (version_compare(PHP_VERSION, '7.3.0', '<')) : ?>
                                    <?php echo ',' . JText::sprintf('COM_DROPTABLES_CONFIG_SYSTEM_CHECK_SETTING_UPDATE_YOUR_PHP_VER'); ?>
                                <?php endif; ?>
                            </p>
                        <?php endif; ?>
                    </div>
                    <?php
                    $i = 0;
                    foreach ($extension_array as $v) :
                        ?>
                        <div class="<?php echo ucfirst($v['id']); ?>">
                            <?php if ($i === 0) : ?>
                                <h3><?php ucfirst('Other Check');
                                    $i = 1; ?></h3>
                            <?php endif; ?>
                            <div class="ju-settings-option full-width">
                                <label for="<?php echo $v['id']; ?>" class="ju-setting-label system-check-label">
                                    <?php echo ucfirst($v['title']); ?>
                                </label>
                                <?php
                                $checkother = false;
                                if (function_exists('get_loaded_extensions')) {
                                    $phpModules = get_loaded_extensions();
                                    if (in_array($v['id'], $phpModules)) {
                                        $checkother = true;
                                    }
                                } else {
                                    if (isset($phpInfo[$v['id']])) {
                                        $checkother = true;
                                    }
                                }
                                if ($v['id'] === 'curl') {
                                    //phpcs:ignore WordPress.Security.EscapeOutput -- Echo icon html
                                    echo ($checkother) ? $icon['ok'] : $icon['alert'];
                                } else {
                                    //phpcs:ignore WordPress.Security.EscapeOutput -- Echo icon html
                                    echo ($checkother) ? $icon['ok'] : $icon['info'];
                                }
                                ?>
                            </div>
                            <?php if (!$checkother) : ?>
                                <p><?php echo $v['tooltip']; ?></p>
                            <?php endif; ?>
                        </div>
                    <?php endforeach; ?>
                </div>
                <div class="ju-content-wrapper" id="liveUpdate" >
                    <?php echo $this->loadTemplate('liveupdate'); ?>
                </div>
            </div>
        </div>
    <input type="button" style="display: none" id="drop_config_cancel_close">
</div>

<style type="text/css">
    .editors-xtd-label {
        display:inline-block;
        width: 180px;
        line-height: 40px;
        overflow: hidden;
        text-overflow: ellipsis;
        white-space: nowrap;
    }
</style>