import tableFunction from "./_functions";
import customRenderer from "./_customRenderer";
import selectOption from "./_toolbarOptions";
import DropChart from "./_charts";

/**
 * Set position for html cell editer popup
 *
 * @param checkScroll Check whether to perform
 * @returns {boolean}
 */
function afterScrollEditors (checkScroll) {
    var handsontableInputHolder = Droptables_element.tableContainer.find('.handsontableInputHolder');
    if (checkScroll === true && (handsontableInputHolder.hasClass('droptables_set_top') || handsontableInputHolder.hasClass('droptables_set_left'))) {
        var position = handsontableInputHolder.position();

        var $table = Droptables_element.tableContainer.find('.wtHider');

        if (handsontableInputHolder.hasClass('droptables_set_top')) {
            var tdOffsetTop = handsontableInputHolder.data('tdOffsetTop');
            var outerHeight = handsontableInputHolder.outerHeight();
            var heightTable = Droptables_element.tableContainer.outerHeight() > $table.outerHeight() ? Droptables_element.tableContainer.outerHeight() : $table.outerHeight();
            var new_top = tdOffsetTop + outerHeight - heightTable + 10;

            if (new_top > 0) {
                handsontableInputHolder.css({top: position.top - new_top});
            }
        }
        if (handsontableInputHolder.hasClass('droptables_set_left')) {
            var tdOffsetLeft = handsontableInputHolder.data('tdOffsetLeft');
            var outerWidth = handsontableInputHolder.outerWidth();
            var widthTable = Droptables_element.tableContainer.outerWidth() > $table.outerWidth() ? Droptables_element.tableContainer.outerWidth() : $table.outerWidth();
            var new_left = tdOffsetLeft + outerWidth - widthTable + 10;

            if (new_left > 0) {
                handsontableInputHolder.css({left: position.left - new_left});
            }
        }
    }
    return false;
}

function initHandsontable(datas) {
    var checkScroll = false;
    var autoScroll;
    Droptables = window.Droptables;
    var modifyRow = null;
    var totalRows = datas.length;
    var copyHeightRow = false;

    window.Droptables_element.tableContainer.handsontable({
        data: datas,
        startRows: 5,
        startCols: 5,
        editor: window.CustomEditor,
        search: true,
        renderAllRows: false,
        fillHandle: (Droptables.userOwn.edit && Droptables.type === 'html') ? {
            autoInsertRow:false,
        } : false,
        modifyAutofillRange: function (startArea, entireArea) {//entireArea is start ranger
            var directionFill = -1;//-1 is not fill, 0 is top, 1 is bottom, 2 is left, 3 is right
            var new_range = [startArea[0], startArea[1], startArea[2], startArea[3]];

            if (startArea[0] !== entireArea[0]) {
                directionFill = 0;
                new_range[2] = entireArea[0] - 1;
            } else if (startArea[2] !== entireArea[2]) {
                directionFill = 1;
                new_range[0] = entireArea[2] + 1;
            } else if (startArea[1] !== entireArea[1]) {
                directionFill = 2;
                new_range[3] = entireArea[1] - 1;
            } else if (startArea[3] !== entireArea[3]) {
                directionFill = 3;
                new_range[1] = entireArea[3] + 1;
            }

            if (directionFill > -1) {
                //update cell type, hyperlink
                tableFunction.updateStyleAutofill(entireArea, new_range, directionFill);
            }
        },
        copyPaste: window.Droptables.type !== 'mysql',
        beforePaste: (data, coords) => {
            console.log(data, coords)
            tableFunction.saveChanges(true)
            //vcl, asynchronous
        },
        beforeCopy: (data, coords) => {
            var selection = table_function_data.selection[0];
            table_function_data.copyData = [];
            table_function_data.copyWidthCols = [];
            table_function_data.copyTypeCols = [];
            table_function_data.copyHeightRows = [];
            table_function_data.copyMergeCells = [];
            table_function_data.copyDataSelection = [selection[2] - selection[0], selection[3] - selection[1]];
            table_function_data.copyHeightRow = [];
            table_function_data.createCellWhenCopy = [];
            jquery.map(window.Droptables.style.rows, function (v) {
                table_function_data.copyHeightRow.push(v[1].height);
            });

            var i2 = 0, j2;
            for (var i = selection[0]; i <= selection[2]; i++) {//row
                j2 = 0;

                for (var j = selection[1]; j <= selection[3]; j++) {
                    if (typeof Droptables.style.cells[i + '!' + j] !== 'undefined') {
                        table_function_data.copyData[i2 + '!' + j2] = window.jquery.extend({}, Droptables.style.cells[i + '!' + j][2]);

                        if (typeof Droptables.newSelect !== 'undefined') {
                            if (Droptables.newSelect === 'row' && typeof Droptables.style.rows[i][1] !== 'undefined') {
                                table_function_data.copyHeightRows[i2 + '!'] = Droptables.style.rows[i][1].height;
                            }

                            if (Droptables.newSelect === 'col' && typeof Droptables.style.cols[j][1] !== 'undefined') {
                                table_function_data.copyWidthCols[j2 + '!'] = Droptables.style.cols[j][1].width;
                                table_function_data.copyTypeCols[j2 + '!'] = Droptables.style.table.col_types[j];
                            }
                        }
                        delete table_function_data.copyData[i2 + '!' + j2].lock_cell;
                        delete table_function_data.copyData[i2 + '!' + j2].height;
                        delete table_function_data.copyData[i2 + '!' + j2].width;
                        delete table_function_data.copyData[i2 + '!' + j2].AlternateColor;
                    }
                    j2++;
                }
                i2++
            }

            console.log(table_function_data.copyTypeCols)
            //check merge cells
            jquery.map(Droptables.style.table.mergeSetting, function (v) {
                if (v.col >= selection[1] && v.row >= selection[0]) {
                    if ((v.col + v.colspan - 1) <= selection[3] && (v.row + v.rowspan - 1) <= selection[2]) {
                        table_function_data.copyMergeCells[(v.row - selection[0]) + '!' + (v.col - selection[1])] = [v.rowspan - 1, v.colspan - 1];
                    }
                }
            });
        },
        afterCopy: function (data, coords) {
        },
        afterPaste: function (data, coords) {
        },
        colHeaders: true,
        rowHeaders: true,
        autoRowSize: false,
        autoColSize: false,
        renderer: customRenderer.render,
        height: tableFunction.calculateTableHeight(window.jquery('#droptable_toolbars')),
        afterInit: function () {
            DropChart.functions.loadCharts();
            Droptables_element.tableContainer.handsontable("selectCell", 0, 0, 0, 0);
            window.table_function_data.allowed_save = true;

            tableFunction.render_css_lock(Droptables, table_function_data);
        },
        beforeRender: function (isForced) {
            window.table_function_data.styleToRender = '';
        },
        beforeChange: function (changes, source) {
            if (source === 'auto_convert_data') {
                return;
            }
            var paste_start = [changes[0][0], changes[0][1]];
            var paste_x = 0, paste_y = 0, before_next_row;
            copyHeightRow = false;
            var updateSettings = {};
            var cols_selected = [];
            var number_cols = changes.length / (changes[changes.length - 1][0] - changes[0][0] + 1);
            var mergeCellsSetting = jquery.extend([], window.Droptables.style.table.mergeSetting);

            table_function_data.createCellWhenCopy = [];

            for (var i = 0; i < changes.length; i++) {
                before_next_row = false;
                if ((i + 1) % number_cols === 0) {
                    before_next_row = true;
                }
                var optionsCell = changes[i];

                //paste style options cells
                if (source === 'CopyPaste.paste' && typeof table_function_data.copyDataSelection !== "undefined" && table_function_data.copyDataSelection[0] >= 0) {
                    if (optionsCell[0] - paste_start[0] > table_function_data.copyDataSelection[0]) {//row
                        paste_y = 0;
                    }
                    if (optionsCell[1] - paste_start[1] > table_function_data.copyDataSelection[1]) {//col
                        paste_x = 0;
                    }
                    if (typeof (window.Droptables.style.cells[optionsCell[0] + '!' + optionsCell[1]]) === 'undefined') {
                        window.Droptables.style.cells[optionsCell[0] + '!' + optionsCell[1]] = [optionsCell[0], optionsCell[1], {}];
                        table_function_data.createCellWhenCopy[optionsCell[0] + '!' + optionsCell[1]] = optionsCell[0] + '!' + optionsCell[1];
                    }
                    if (table_function_data.copyData[paste_y + '!' + paste_x].cell_type === 'html') {
                        setTimeout((optionsCell) => {
                            saveData.push({action: 'set_cells_type', selection: [[optionsCell[0], optionsCell[1], optionsCell[0], optionsCell[1]]], style: {cell_type: 'html'}});
                        }, 300, optionsCell);
                    } else {
                        if (window.Droptables.style.cells[optionsCell[0] + '!' + optionsCell[1]][2].cell_type === 'html') {
                            setTimeout((optionsCell) => {
                                saveData.push({action: 'set_cells_type', selection: [[optionsCell[0], optionsCell[1], optionsCell[0], optionsCell[1]]], style: {cell_type: null}});
                            }, 300, optionsCell);
                        }
                    }
                    let AlternateColor = window.Droptables.style.cells[optionsCell[0] + '!' + optionsCell[1]][2].AlternateColor;
                    window.Droptables.style.cells[optionsCell[0] + '!' + optionsCell[1]][2] = window.jquery.extend({}, {AlternateColor}, table_function_data.copyData[paste_y + '!' + paste_x]);

                    if (typeof table_function_data.copyHeightRows !== 'undefined' && table_function_data.copyHeightRows[paste_y + '!'] > 0) {
                        if (typeof (window.Droptables.style.rows[optionsCell[0]]) === 'undefined') {
                            window.Droptables.style.rows[optionsCell[0]] = [optionsCell[0], {}];
                        }
                        window.Droptables.style.rows[optionsCell[0]][1].height = table_function_data.copyHeightRows[paste_y + '!'];
                        table_function_data.copyHeightRow[optionsCell[0]] = table_function_data.copyHeightRows[paste_y + '!'];
                        copyHeightRow = true;
                    }
                    if (typeof table_function_data.copyWidthCols !== 'undefined' && table_function_data.copyWidthCols[paste_x + '!'] > 0) {
                        if (typeof (window.Droptables.style.cols[optionsCell[1]]) === 'undefined') {
                            window.Droptables.style.cols[optionsCell[1]] = [optionsCell[1], {}];
                            window.Droptables.style.cols[optionsCell[1]][1].res_priority = 0;
                        }
                        window.Droptables.style.cols[optionsCell[1]][1].width = table_function_data.copyWidthCols[paste_x + '!'];
                    }

                    if (typeof table_function_data.copyTypeCols !== 'undefined' && table_function_data.copyTypeCols[paste_x + '!']) {
                        Droptables.style.table.col_types[optionsCell[1]] = 'text';
                        cols_selected[optionsCell[1]] = 'text';
                    }

                    if (typeof table_function_data.copyMergeCells !== 'undefined' && typeof table_function_data.copyMergeCells[paste_y + '!' + paste_x] !== 'undefined') {
                        mergeCellsSetting.push({
                            col: optionsCell[1],
                            colspan: table_function_data.copyMergeCells[paste_y + '!' + paste_x][1] + 1,
                            row: optionsCell[0],
                            rowspan: table_function_data.copyMergeCells[paste_y + '!' + paste_x][0] + 1
                        });

                        updateSettings.mergeCells = mergeCellsSetting;
                    }

                    setTimeout((optionsCell, changes, i, updateSettings, copyHeightRow) => {
                        saveData.push({action: 'style',
                            selection: [[optionsCell[0], optionsCell[1], optionsCell[0], optionsCell[1]]],
                            style: window.Droptables.style.cells[optionsCell[0] + '!' + optionsCell[1]][2]
                        });

                        if (i + 1 === changes.length) {
                            if (copyHeightRow) {
                                updateSettings.manualRowResize = table_function_data.copyHeightRow;
                            }

                            if (tableFunction.droptablesCheckSize(updateSettings) > 0) {
                                Droptables_element.tableContainer.handsontable('updateSettings', updateSettings);
                            }
                        }
                    }, 300, optionsCell, changes, i, updateSettings, copyHeightRow);
                    paste_x++;

                    if (i + 1 === changes.length) {
                        if (tableFunction.droptablesCheckSize(cols_selected) > 0) {
                            if (saveData[0] && saveData[0].action === 'create_col_after_paste') {
                                saveData.splice(1, 0, {
                                    action: 'set_columns_types_copy',
                                    value: cols_selected
                                });
                            } else {
                                saveData.unshift({
                                    action: 'set_columns_types_copy',
                                    value: cols_selected
                                });
                            }
                        }
                    }

                    if (before_next_row) {
                        paste_y++;
                        paste_x = 0;
                    }
                }

                //check validate chart
                if (jquery('#list_chart').find('.chart-menu').length > 0) {//check chart exist
                    if (!DropChart.functions.validateCharts(optionsCell)) {
                        bootbox.alert(Joomla.JText._('COM_DROPTABLES_CHANGE_INVALID_CHART_DATA', 'Invalid chart data'));
                        return false;
                    }
                }
            }
        },
        afterRender: function (isForced) {
            var parser = new (less.Parser);
            var Droptables = window.Droptables;
            var $ = window.jquery;

            window.table_function_data.content = '#mainTabContent .handsontable .ht_master .htCore {' + window.table_function_data.styleToRender + '}';
            if (Droptables.style.table.responsive_type == 'scroll' && Droptables.style.table.freeze_row) {
                window.table_function_data.content += ' #mainTabContent .handsontable .ht_clone_top .htCore {' + window.table_function_data.styleToRender + '}';
            }
            if (Droptables.style.table.responsive_type == 'scroll' && Droptables.style.table.freeze_col) {
                window.table_function_data.content += ' #mainTabContent .handsontable .ht_clone_left .htCore {' + window.table_function_data.styleToRender + '}';
            }
            if (Droptables.style.table.responsive_type == 'scroll' && Droptables.style.table.freeze_row && Droptables.style.table.freeze_col) {
                window.table_function_data.content += ' #mainTabContent .handsontable .ht_clone_corner .htCore {' + window.table_function_data.styleToRender + '}';
            }

            parser.parse(window.table_function_data.content, function (err, tree) {
                if (err) {
                    //Here we can throw the erro to the user
                    return false;
                } else {
                    Droptables.css = $('#jform_css').val();
                    if ($('#headMainCss').length === 0) {
                        $('head').append('<style id="headMainCss"></style>');
                    }
                    $('#headMainCss').text(tree.toCSS());
                    return true;
                }
            });
            if ($('#content_popup_hide').find('select.select_columns option').length < 1) {
                // Build column selection for default sort parameter
                tableFunction.default_sortable(window.Droptables.datas);
            }
            //set Droptables.style.rows , Droptables.style.cols value
            tableFunction.pushDims($, Droptables);

            if (table_function_data.needSaveAfterRender === true) {
                tableFunction.saveChanges();
                table_function_data.needSaveAfterRender = false;
            }

            Droptables.style.table.class_name && $('#tableContainer').addClass(Droptables.style.table.class_name);
        },
        afterChange: function (change, source) {//play when change content cell
            table_function_data.copyWidthCols = [];
            if (source === 'CopyPaste.paste') {
                table_function_data.createCellWhenCopy = [];
                table_function_data.copyHeightRows = [];
                table_function_data.copyHeightRow = [];
            }
            // console.log(change, source)
            if (typeof table_function_data.data_argument !== 'undefined' && source === 'loadData') {
                source = 'edit';
                change = jquery.extend([], table_function_data.data_argument);
            }

            //fix handsontable merge cells remove data of cells
            var i;

            if ((source === 'MergeCells' || source === 'populateFromArray')&& typeof change.length !== 'undefined') {
                for (i = 0; i < change.length; i++) {
                    if (change[i][2] !== null && change[i][3] === null) {
                        Droptables.datas[change[i][0]][change[i][1]] = change[i][2];
                    }
                }
            }
            if (source === 'populateFromArray' && change.length > 0 && change[0][2] === change[0][3]) {
                tableFunction.cleanHandsontable();
            }

            if (source === 'loadData' || source === 'populateFromArray' || !table_function_data.allowed_edit) {
                return; //don't save this change
            }
            var action = ['CopyPaste.paste', 'edit', 'UndoRedo.undo', 'UndoRedo.redo', 'Autofill.fill', 'setDataAtCell'];

            //validate data cells when mergeCells
            if (change) {
                var notSaveData;
                var editHeader = false;
                var mysql_column_edit, mysql_column_key;
                for (i = 0; i < change.length; i++) {
                    if (Droptables.type === 'mysql' && !table_function_data.mysqlEdit && change[i][0] != 0) {
                        break;
                    }

                    if (action.includes(source)) {
                        notSaveData = true;
                        if (typeof change[i][3] === 'undefined' || change[i][3] === 'droptables_change_value_after_set_columns_types') {
                            change[i][3] = change[i][2];//loadData when edit cells > 5ommit
                            notSaveData = false;
                        }
                        if (change[i][3] === change[i][4]) {
                            notSaveData = false;
                        }

                        var optionsCell = change[i];

                        if (typeof Droptables.style.table.headerOption == 'undefined' || optionsCell[0] >= Droptables.style.table.headerOption) {//check cell in table header
                            if (typeof optionsCell[3] !== 'undefined' && optionsCell[3] !== null) {
                                var value = tableFunction.cell_type_to_column(optionsCell, optionsCell[3]);
                                var value_change;
                                if (value === false) {
                                    Droptables_element.mainTabContent.find('td.dtr' + optionsCell[0] + '.dtc' + optionsCell[1]).addClass('invalid_data');
                                    bootbox.alert('The value of the cell does not match the column format. For example, and integer format cannot contain letters.');
                                    return;
                                } else {
                                    Droptables_element.mainTabContent.find('td.dtr' + optionsCell[0] + '.dtc' + optionsCell[1]).removeClass('invalid_data');
                                    if (value !== true) {
                                        if (notSaveData) {
                                            if (Droptables.type !== 'mysql' || optionsCell[0] === 0) {
                                                saveData.push({action: 'edit_cell', row: optionsCell[0], col: optionsCell[1], content: value});
                                            } else if (typeof Droptables.query_option !== 'undefined' && Droptables.query_option.column_options !== null && typeof Droptables.query_option.column_options[optionsCell[1]] !== "undefined") {
                                                if (optionsCell[2] !== value) {
                                                    mysql_column_edit = Droptables.query_option.column_options[optionsCell[1]].table + '.' + Droptables.query_option.column_options[optionsCell[1]].Field;
                                                    mysql_column_key = Droptables.query_option.column_options[table_function_data.keyPosition].table + '.' + Droptables.query_option.column_options[table_function_data.keyPosition].Field;
                                                    value_change = Droptables.datas[optionsCell[0]][table_function_data.keyPosition];
                                                    Droptables.saveDataDbTable.push({
                                                        action: 'edit_cell_mysql',
                                                        row: optionsCell[0],
                                                        col: optionsCell[1],
                                                        content: value,
                                                        id: Droptables.datas[optionsCell[0]][table_function_data.keyPosition],
                                                        column: mysql_column_edit,
                                                        column_key: mysql_column_key
                                                    });
                                                }
                                            }
                                            if (Droptables.style.table.headerOption > 0 && optionsCell[0] < Droptables.style.table.headerOption) {
                                                Droptables.style.header_data[optionsCell[0]][optionsCell[1]] = value;
                                            }
                                        }
                                        if (value !== optionsCell[3]) {//has convert cell value
                                            jquery(Droptables_element.tableContainer).handsontable('setDataAtCell', optionsCell[0], optionsCell[1], value, 'auto_convert_data');
                                        }
                                    } else {
                                        if (notSaveData) {
                                            if (Droptables.type !== 'mysql' || optionsCell[0] === 0) {
                                                saveData.push({action: 'edit_cell', row: optionsCell[0], col: optionsCell[1], content: optionsCell[3]});
                                            } else if (typeof Droptables.query_option !== 'undefined' && Droptables.query_option.column_options !== null && typeof Droptables.query_option.column_options[optionsCell[1]] !== "undefined") {
                                                if (optionsCell[2] !== optionsCell[3]) {
                                                    mysql_column_edit = Droptables.query_option.column_options[optionsCell[1]].table + '.' + Droptables.query_option.column_options[optionsCell[1]].Field;
                                                    mysql_column_key = Droptables.query_option.column_options[table_function_data.keyPosition].table + '.' + Droptables.query_option.column_options[table_function_data.keyPosition].Field;
                                                    value_change = Droptables.datas[optionsCell[0]][table_function_data.keyPosition];
                                                    Droptables.saveDataDbTable.push({
                                                        action: 'edit_cell_mysql',
                                                        row: optionsCell[0],
                                                        col: optionsCell[1],
                                                        content: optionsCell[3],
                                                        id: Droptables.datas[optionsCell[0]][table_function_data.keyPosition],
                                                        column: mysql_column_edit,
                                                        column_key: mysql_column_key
                                                    });
                                                }
                                            }
                                            if (Droptables.style.table.headerOption > 0 && optionsCell[0] < Droptables.style.table.headerOption) {
                                                Droptables.style.header_data[optionsCell[0]][optionsCell[1]] = optionsCell[3];
                                            }
                                        }
                                    }
                                }
                            }
                        } else {
                            editHeader = true;
                            if (notSaveData) {
                                if (Droptables.type !== 'mysql' || optionsCell[0] === 0) {
                                    saveData.push({action: 'edit_cell', row: optionsCell[0], col: optionsCell[1], content: optionsCell[3]});
                                } else if (Droptables.type === 'mysql' && typeof Droptables.query_option !== 'undefined' && Droptables.query_option.column_options !== null && typeof Droptables.query_option.column_options[optionsCell[1]] !== "undefined") {
                                    if (optionsCell[2] !== optionsCell[3]) {
                                        mysql_column_edit = Droptables.query_option.column_options[optionsCell[1]].table + '.' + Droptables.query_option.column_options[optionsCell[1]].Field;
                                        mysql_column_key = Droptables.query_option.column_options[table_function_data.keyPosition].table + '.' + Droptables.query_option.column_options[table_function_data.keyPosition].Field;
                                        value_change = Droptables.datas[optionsCell[0]][table_function_data.keyPosition];
                                        Droptables.saveDataDbTable.push({
                                            action: 'edit_cell_mysql',
                                            row: optionsCell[0],
                                            col: optionsCell[1],
                                            content: optionsCell[3],
                                            id: Droptables.datas[optionsCell[0]][table_function_data.keyPosition],
                                            column: mysql_column_edit,
                                            column_key: mysql_column_key
                                        });
                                    }
                                }
                                if (Droptables.style.table.headerOption > 0 && optionsCell[0] < Droptables.style.table.headerOption) {
                                    Droptables.style.header_data[optionsCell[0]][optionsCell[1]] = optionsCell[3];
                                }
                            }
                        }
                    }
                }
            }

            // clearTimeout(tableFunction.autosaveNotification);

            //change[0][2] & change[0][3] === undefined when edit html cell
            if (action.includes(source) && typeof change[0] !== 'undefined' && !(typeof change[0][2] === 'undefined' && typeof change[0][3] === 'undefined')) {
                table_function_data.needSaveAfterRender = (change[0][2] !== change[0][3] || source ==='setDataAtCell') ? true : false;
            }

            if (change && Droptables.type === 'mysql' && notSaveData && Droptables.saveDataDbTable.length > 0) {//mysql table edited
                if (typeof value !== 'undefined' || editHeader) {
                    saveData = saveData.concat(Droptables.saveDataDbTable);
                    Droptables_element.mainTabContent.addClass('loading_ajax');
                    Droptables.saveDataDbTable = [];
                    tableFunction.saveChanges(true);
                }
                table_function_data.needSaveAfterRender = false;
            }

            //update merge cells when autofill
            if ('Autofill.fill' === source) {
                // tableFunction.cleanHandsontable();
                updateMergeCells(true);
            } else {
                setTimeout(function () {
                    if (jquery(Droptables_element.tableContainer).handsontable('getInstance').undoRedo.isUndoAvailable()) {
                        Droptables_element.primary_toolbars.find('#undo_cell').addClass('active');
                    } else {
                        Droptables_element.primary_toolbars.find('#undo_cell').removeClass('active');
                    }
                    if (jquery(Droptables_element.tableContainer).handsontable('getInstance').undoRedo.isRedoAvailable()) {
                        Droptables_element.primary_toolbars.find('#redo_cell').addClass('active');
                    } else {
                        Droptables_element.primary_toolbars.find('#redo_cell').removeClass('active');
                    }
                }, 100);
            }

            if (table_function_data.needSaveAfterRender === true) {
                tableFunction.saveChanges();
                table_function_data.needSaveAfterRender = false;
            }
        },
        afterCreateRow: function (index, amount) {
            var selector = table_function_data.selection[table_function_data.selectionSize - 1];
            var $ = jquery;
            var Droptables = window.Droptables;
            var above = selector[0] === index ? true : false;//check above/below

            if (typeof (Droptables.style.cells) !== 'undefined') {
                var newCells = {};
                var cell, i, cells;

                for (cell in Droptables.style.cells) {
                    cells = Droptables.style.cells[cell];

                    if (cells[0] < index) {//rows < index
                        newCells[cell] = $.extend([], cells);
                    }
                    if (cells[0] >= index) {//rows >= index
                        if (typeof table_function_data.createCellWhenCopy !== 'undefined' && typeof table_function_data.createCellWhenCopy[cell] !== 'undefined') {
                            newCells[cell] = $.extend([], cells);
                        } else {
                            newCells[parseInt(cells[0] + amount) + '!' + cells[1]]
                                = [cells[0] + amount, cells[1], $.extend({}, cells[2])];
                        }
                    }
                    if (cells[0] <= selector[2] && selector[0] <= cells[0]) {//new rows copy style of selector
                        if (typeof table_function_data.createCellWhenCopy !== 'undefined' && typeof table_function_data.createCellWhenCopy[cell] !== 'undefined') {
                            newCells[cell] = $.extend([], cells);
                        } else {
                            newCells[parseInt(cells[0] - selector[0] + index) + '!' + cells[1]]
                                = [parseInt(cells[0] - selector[0] + index), cells[1], $.extend({}, cells[2])];
                        }
                    }
                }
                Droptables.style.cells = $.extend({}, newCells);

                if (typeof Droptables.style.table.alternateColorValue !== 'undefined') {
                    //update style.table.alternateColorValue by index, amount
                    var alternateColorValue = Droptables.style.table.alternateColorValue;
                    for (i in alternateColorValue) {
                        if (alternateColorValue[i] !== false) {
                            if (above) {//above
                                if (alternateColorValue[i].selection[0] >= index + amount) {
                                    // alternateColorValue > new rows
                                    alternateColorValue[i].selection[0] = alternateColorValue[i].selection[0] + amount;
                                    alternateColorValue[i].selection[2] = alternateColorValue[i].selection[2] + amount;
                                }
                                if (alternateColorValue[i].selection[0] < index + amount && alternateColorValue[i].selection[2] >= index) {
                                    alternateColorValue[i].selection[2] = alternateColorValue[i].selection[2] + amount;
                                }
                            } else {//below
                                if (alternateColorValue[i].selection[0] >= index) {
                                    // alternateColorValue > new rows
                                    alternateColorValue[i].selection[0] = alternateColorValue[i].selection[0] + amount;
                                    alternateColorValue[i].selection[2] = alternateColorValue[i].selection[2] + amount;
                                }
                                if (alternateColorValue[i].selection[0] < index && alternateColorValue[i].selection[2] >= selector[0]) {
                                    alternateColorValue[i].selection[2] = alternateColorValue[i].selection[2] + amount;
                                }
                            }
                        }
                    }
                }
            }

            // if (typeof (Droptables.style.rows) !== 'undefined') {
            //     // index, amount
            //     var new_data = $.extend({}, Droptables.style.rows);
            //     var jj;
            //     if (selector[0] === index) {//left
            //         for (jj = index; jj < tableFunction.droptablesCheckSize(Droptables.style.rows); jj++) {
            //             new_data[jj + amount] = [jj + amount, {}];
            //             new_data[jj + amount][1] = $.extend({}, Droptables.style.rows[jj][1]);
            //         }
            //     } else {
            //         for (jj = index; jj < tableFunction.droptablesCheckSize(Droptables.style.rows) + amount; jj++) {
            //             new_data[jj] = [jj, {}];
            //             new_data[jj][1] = $.extend({}, Droptables.style.rows[jj - amount][1]);
            //         }
            //     }
            //
            //     Droptables.style.rows = $.extend({}, new_data);
            // }

            saveData.push({action: 'create_row', index: index, amount: amount, above: above});

            //remove tooltip in new cells created
            tableFunction.getFillArray([[index, 0, index + amount - 1, Droptables.max_Col - 1]], Droptables, {tooltip_content: null});

            if (typeof (Droptables.style.rows) !== 'undefined') {
                let rows = tableFunction.changeValue(
                    Droptables.style.rows,
                    index,
                    amount,
                    Droptables.max_row,
                    selector[0] === index,
                    copyHeightRow
                );
                if (tableFunction.droptablesCheckSize(rows) > 0)
                    Droptables.style.rows = $.extend({}, {}, rows)
            }

            if (parseInt(Droptables.style.table.headerOption) > 0) {
                delete Droptables.style.header_data;
                Droptables.style.header_data = [];
                for (var j = 0; j < Droptables.style.table.headerOption; j++) {
                    Droptables.style.header_data[j] = Droptables.datas[j];
                }
            }

            // update merged row index and get tableFunction.saveChanges();
            setTimeout(function () {
                updateMergeCells(window.table_function_data.firstRender);
            }, 200)
            tableFunction.create_ranger_cells_lock(Droptables, table_function_data, {type: 'create_row', index: index, amount: amount});

            tableFunction.cleanHandsontable();
        },
        afterRemoveRow: function (index, amount) {
            var selector = table_function_data.selection[table_function_data.selectionSize - 1];
            var $ = jquery;
            var Droptables = window.Droptables;
            if (typeof (Droptables.style.cells) !== 'undefined') {
                var newCells = {};
                var cell, i, cells;

                for (cell in Droptables.style.cells) {
                    cells = Droptables.style.cells[cell];
                    if (cells[0] > index + amount - 1) {
                        newCells[parseInt(cells[0] - amount) + '!' + cells[1]]
                            = [cells[0] - amount, cells[1], $.extend({}, cells[2])];
                    }
                    if (cells[0] < index) {
                        newCells[cell]
                            = [cells[0], cells[1], $.extend({}, cells[2])];
                    }
                }
                Droptables.style.cells = $.extend({}, newCells);

                if (typeof Droptables.style.table.alternateColorValue !== 'undefined') {//update Droptables.style.table.alternateColorValue by index, amount
                    var alternateColorValue = Droptables.style.table.alternateColorValue;
                    for (i in alternateColorValue) {
                        if (alternateColorValue[i].selection[2] >= index && alternateColorValue[i].selection[2] <= (index + amount - 1)) {
                            alternateColorValue[i].selection[2] = index - 1;//selection[2] in selector
                        }
                        if (alternateColorValue[i].selection[2] >= index + amount) {
                            alternateColorValue[i].selection[2] = alternateColorValue[i].selection[2] - amount;//selection[2] > selector
                        }
                        if (alternateColorValue[i].selection[0] >= index && alternateColorValue[i].selection[0] < (index + amount)) {
                            alternateColorValue[i].selection[0] = index;//selection[0] in selector
                        }
                        if (alternateColorValue[i].selection[0] >= index + amount) {
                            alternateColorValue[i].selection[0] = alternateColorValue[i].selection[0] - amount;//selection[0] > selector
                        }
                        if (alternateColorValue[i].selection[0] > alternateColorValue[i].selection[2]) {//alternateColor in selector
                            delete alternateColorValue[i];
                        }
                    }
                }
            }

            if (typeof (Droptables.style.rows) !== 'undefined') {
                // index, amount
                var new_data = $.extend({}, Droptables.style.rows);
                var jj;
                for (jj = index; jj < tableFunction.droptablesCheckSize(Droptables.style.rows); jj++) {
                    if (typeof (Droptables.style.rows[jj + amount]) !== 'undefined') {
                        new_data[jj] = [jj, {}];
                        new_data[jj][1] = $.extend({}, Droptables.style.rows[jj][1]);
                    } else {
                        delete new_data[jj];
                    }
                }
                Droptables.style.rows = $.extend({}, new_data);
            }

            //remove col in header table
            if (parseInt(Droptables.style.table.headerOption) > 0) {
                delete Droptables.style.header_data;
                Droptables.style.header_data = [];
                for (var j = 0; j < Droptables.style.table.headerOption; j++) {
                    Droptables.style.header_data[j] = Droptables.datas[j];
                }
            }

            saveData.push({action: 'del_row', index: index, amount: amount, old_rows: $(Droptables_element.tableContainer).handsontable('countRows') + amount});
            // update merged row index and get tableFunction.saveChanges();
            updateMergeCells(window.table_function_data.firstRender);
            tableFunction.create_ranger_cells_lock(Droptables, table_function_data, {type: 'delete_row', index: index, amount: amount});
            tableFunction.cleanHandsontable();
        },
        afterCreateCol: function (index, amount) {
            var selector = table_function_data.selection[table_function_data.selectionSize - 1];
            var $ = jquery;
            var Droptables = window.Droptables;
            var left = selector[1] === index ? true : false;//check insert left/right
            var createColsWhenPaste = false;

            if (typeof (Droptables.style.cells) !== 'undefined') {
                var newCells = {};
                var cell, i, cells;

                for (cell in Droptables.style.cells) {
                    cells = Droptables.style.cells[cell];
                    if (cells[1] >= index) {//cols >= index
                        if (typeof table_function_data.createCellWhenCopy !== 'undefined' && typeof table_function_data.createCellWhenCopy[cell] !== 'undefined') {
                            newCells[cell] = $.extend([], cells);
                            createColsWhenPaste = true;
                        } else {
                            newCells[cells[0] + '!' + parseInt(cells[1] + amount)]
                                = [cells[0], cells[1] + amount, $.extend({}, cells[2])];
                        }
                    }
                    if (cells[1] < index) {//cols < index
                        newCells[cell]
                            = [cells[0], cells[1], $.extend({}, cells[2])];
                    }
                    if (cells[1] <= selector[3] && selector[1] <= cells[1]) {//new cols copy style of selector
                        if (typeof table_function_data.createCellWhenCopy !== 'undefined' && typeof table_function_data.createCellWhenCopy[cell] !== 'undefined') {
                            newCells[cell] = $.extend([], cells);
                            createColsWhenPaste = true;
                        } else {
                            newCells[cells[0] + '!' + parseInt(cells[1] - selector[1] + index)]
                                = [cells[0], parseInt(cells[1] - selector[1] + index), $.extend({}, cells[2])];
                        }
                    }
                }
                Droptables.style.cells = $.extend({}, newCells);

                if (typeof Droptables.style.table.alternateColorValue !== 'undefined') {//update Droptables.style.table.alternateColorValue by index, amount
                    var alternateColorValue = Droptables.style.table.alternateColorValue;
                    for (i in alternateColorValue) {
                        if (alternateColorValue[i] !== false) {
                            if (left) {//insert left
                                if (alternateColorValue[i].selection[1] >= index) { // alternateColorValue > new rows
                                    alternateColorValue[i].selection[1] = alternateColorValue[i].selection[1] + amount;
                                    alternateColorValue[i].selection[3] = alternateColorValue[i].selection[3] + amount;
                                }
                                if (alternateColorValue[i].selection[1] < index && alternateColorValue[i].selection[3] >= index) {
                                    alternateColorValue[i].selection[3] = alternateColorValue[i].selection[3] + amount;
                                }
                            } else {//insert right
                                if (alternateColorValue[i].selection[1] >= index) { // alternateColorValue > new rows
                                    alternateColorValue[i].selection[1] = alternateColorValue[i].selection[1] + amount;
                                    alternateColorValue[i].selection[3] = alternateColorValue[i].selection[3] + amount;
                                }
                                if (alternateColorValue[i].selection[1] < index && alternateColorValue[i].selection[3] >= index) {
                                    alternateColorValue[i].selection[3] = alternateColorValue[i].selection[3] + amount;
                                }
                            }
                        }
                    }
                }
            }

            if (createColsWhenPaste) {
                if (typeof table_function_data.create_col_after_paste === 'undefined') {
                    table_function_data.create_col_after_paste = []
                }
                table_function_data.create_col_after_paste.push({action: 'create_col_after_paste', index: index, amount: amount, left: left});
            } else {
                saveData.push({action: 'create_col', index: index, amount: amount, left: left});
            }
            //remove tooltip in new cells created
            tableFunction.getFillArray([[0, index, Droptables.max_row - 1, index + amount - 1]], Droptables, {tooltip_content: null});


            if (typeof (Droptables.style.cols) !== 'undefined') {
                // index, amount
                var new_data = $.extend({}, Droptables.style.cols);
                var jj;
                if (selector[1] === index) {//left
                    for (jj = index; jj < tableFunction.droptablesCheckSize(Droptables.style.cols); jj++) {
                        new_data[jj + amount] = [jj + amount, {}];
                        new_data[jj + amount][1] = $.extend({}, Droptables.style.cols[jj][1]);
                    }
                } else {
                    for (jj = index; jj < tableFunction.droptablesCheckSize(Droptables.style.cols) + amount; jj++) {
                        new_data[jj] = [jj, {}];
                        new_data[jj][1] = $.extend({}, Droptables.style.cols[jj - amount][1]);
                    }
                }

                Droptables.style.cols = $.extend({}, new_data);
            }

            for (var i = index; i <= (index + amount - 1) ; i++) {
                if (createColsWhenPaste) {
                    Droptables.style.table.col_types.splice(i, 0, 'text');
                } else {
                    Droptables.style.table.col_types.splice(i, 0, 'varchar');
                }
            }

            if (parseInt(Droptables.style.table.headerOption) > 0) {
                delete Droptables.style.header_data;
                Droptables.style.header_data = [];
                for (var j = 0; j < Droptables.style.table.headerOption; j++) {
                    Droptables.style.header_data[j] = Droptables.datas[j];
                }
            }

            // Build column selection for default sort parameter
            tableFunction.default_sortable(window.Droptables.datas);

            window.jquery(Droptables_element.tableContainer).handsontable('render');

            // update merged row index and get tableFunction.saveChanges();
            setTimeout(function () {
                updateMergeCells(window.table_function_data.firstRender, true);
            }, 100);
            tableFunction.create_ranger_cells_lock(Droptables, table_function_data, {type: 'add_col', index: index, amount: amount});
            tableFunction.cleanHandsontable();
        },
        afterRemoveCol: function (index, amount) {
            var selector = table_function_data.selection[table_function_data.selectionSize - 1];
            var $ = jquery;
            var Droptables = window.Droptables;
            if (typeof (Droptables.style.cells) !== 'undefined') {
                var newCells = {};
                var cell, i, cells;

                for (cell in Droptables.style.cells) {
                    cells = Droptables.style.cells[cell];
                    if (cells[1] > index + amount - 1) {
                        newCells[cells[0] + '!' + parseInt(cells[1] - amount)]
                            = [cells[0], cells[1] - amount, $.extend({}, cells[2])];
                    }
                    if (cells[1] < index) {
                        newCells[cell]
                            = [cells[0], cells[1], $.extend({}, cells[2])];
                    }
                }
                Droptables.style.cells = $.extend({}, newCells);

                if (typeof Droptables.style.table.alternateColorValue !== 'undefined') {//update Droptables.style.table.alternateColorValue by index, amount
                    var alternateColorValue = Droptables.style.table.alternateColorValue;
                    for (i in alternateColorValue) {
                        if (alternateColorValue[i].selection[3] >= index && alternateColorValue[i].selection[3] <= (index + amount - 1)) {
                            alternateColorValue[i].selection[3] = index - 1;//selection[3] in selector
                        }
                        if (alternateColorValue[i].selection[3] >= index + amount) {
                            alternateColorValue[i].selection[3] = alternateColorValue[i].selection[3] - amount;//selection[3] > selector
                        }
                        if (alternateColorValue[i].selection[1] >= index && alternateColorValue[i].selection[1] < (index + amount)) {
                            alternateColorValue[i].selection[1] = index;//selection[0] in selector
                        }
                        if (alternateColorValue[i].selection[1] >= index + amount) {
                            alternateColorValue[i].selection[1] = alternateColorValue[i].selection[1] - amount;//selection[0] > selector
                        }
                        if (alternateColorValue[i].selection[1] > alternateColorValue[i].selection[3]) {//alternateColor in selector
                            delete alternateColorValue[i];
                        }
                    }
                }
            }

            if (typeof (Droptables.style.cols) !== 'undefined') {
                // index, amount
                var new_data = $.extend({}, Droptables.style.cols);
                var jj;
                for (jj = index; jj < tableFunction.droptablesCheckSize(Droptables.style.cols); jj++) {
                    if (typeof (Droptables.style.cols[jj + amount]) !== 'undefined' && Droptables.style.cols[jj + amount] !== null) {
                        new_data[jj] = [jj, {}];
                        new_data[jj][1] = $.extend({}, Droptables.style.cols[jj][1]);
                    } else {
                        delete new_data[jj];
                    }
                }
                Droptables.style.cols = $.extend({}, new_data);
            }

            for (var i = (index + amount - 1); i >= index ; i--) {
                Droptables.style.table.col_types.splice(i, 1);
            }

            //remove col in header table
            if (parseInt(Droptables.style.table.headerOption) > 0) {
                delete Droptables.style.header_data;
                Droptables.style.header_data = [];
                for (var j = 0; j < Droptables.style.table.headerOption; j++) {
                    Droptables.style.header_data[j] = Droptables.datas[j];
                }
            }

            // Build column selection for default sort parameter
            tableFunction.default_sortable(window.Droptables.datas);
            jquery(Droptables_element.tableContainer).data('handsontable').render();
            saveData.push({action: 'del_col', index: index, amount: amount, old_columns: $(Droptables_element.tableContainer).handsontable('countCols') + amount});

            updateMergeCells(window.table_function_data.firstRender);
            tableFunction.create_ranger_cells_lock(Droptables, table_function_data, {type: 'delete_col', index: index, amount: amount});
            tableFunction.cleanHandsontable();
        },
        afterColumnResize: function (col, width) {
            tableFunction.saveChanges();
        },
        beforeRowResize: function (currentRow, newSize, isDoubleClick) {
            if (modifyRow !== null) {//currentRow is current size row
                Droptables.style.rows[modifyRow][1].height = newSize;
            }
        },
        modifyRow: function (row) {
            modifyRow = row;
        },
        afterRowResize: function (row1, height) {
            Droptables.style.rows[row1][1].height = height;

            tableFunction.saveChanges();
        },
        afterSelection: function (r, c, r2, c2, preventScrolling, selectionLayerLevel) {
            if (table_function_data.checkCellValueChange !== false && typeof table_function_data.cellValueChange !== 'undefined') {
                tableFunction.change_value_cells(table_function_data.checkCellValueChange, Droptables_element.cellValue.val());
            }
            selectOption.loadSelection(window.jquery, window.Droptables, [r, c, r2, c2]);
            Droptables.max_row = Droptables.datas.length;
            Droptables.max_Col = Droptables.datas[0].length;

            if (typeof Droptables.datas[Droptables.max_row - 1] !== 'undefined') {
                var max_col_2 = Droptables.datas[Droptables.max_row - 1].length;
                Droptables.max_Col = Droptables.max_Col < max_col_2 ? max_col_2 : Droptables.max_Col;
            }
            if (r * r2 == 0 && (r2 + r) == (Droptables.max_row - 1)) {
                Droptables.newSelect = 'col';
            } else if (c * c2 == 0 && (c2 + c) == Droptables.max_Col - 1) {
                Droptables.newSelect = 'row';
            } else {
                delete Droptables.newSelect;
            }
        },
        afterScrollHorizontally: function () {
            //change position of Editors when ScrollHorizontally
            checkScroll = true;
            clearTimeout(autoScroll);
            autoScroll = setTimeout(function () {
                checkScroll = afterScrollEditors(checkScroll);
            }, 200);
        },
        afterScrollVertically: function () {
            //change position of Editors when ScrollHorizontally
            checkScroll = true;
            clearTimeout(autoScroll);
            autoScroll = setTimeout(function () {
                checkScroll = afterScrollEditors(checkScroll);
            }, 200);
        },
        afterMergeCells: function (cellRange, mergeParent, auto) {
            updateMergeCells(window.table_function_data.firstRender);
        },
        afterUnmergeCells: function (cellRange, auto) {
            updateMergeCells(window.table_function_data.firstRender);
        },
        colWidths: function (index) {
            if (tableFunction.checkObjPropertyNested(window.Droptables.style, 'cols', index, 1, 'width')) {
                return window.Droptables.style.cols[index][1].width;
            } else if (typeof window.Droptables.style.cols === 'object' && (typeof window.Droptables.style.cols[index] === 'undefined' || typeof window.Droptables.style.cols[index][1].width === 'undefined')) {
                return 100;
            }
        },
        rowHeights: function (index) {
            if (tableFunction.checkObjPropertyNestedNotEmpty(window.Droptables.style, 'rows', index, 1, 'height')) {
                return window.Droptables.style.rows[index][1].height;
            } else {
                // Table rows is large than 1000, set default row height to 30
                if (totalRows >= 1000) {
                    return 30;
                }
                var h;
                if (typeof Droptables.style.table.allRowHeight !== 'undefined' && Droptables.style.table.allRowHeight !== '') {
                    h = Droptables.style.table.allRowHeight;
                } else {
                    h = window.Droptables_element.tableContainer.find('.ht_master .htCore tbody tr').eq(index).height();
                }
                return h;
            }
        },
        fixedRowsTop: (Droptables.style.table.responsive_type == 'scroll' && parseInt(Droptables.style.table.freeze_row) > 0) ? Droptables.style.table.headerOption : 0,
        fixedColumnsLeft: (Droptables.style.table.responsive_type == 'scroll') ? parseInt(Droptables.style.table.freeze_col) : 0,
        manualColumnResize: table_function_data.allowed_edit,
        manualRowResize: table_function_data.allowed_edit,
        outsideClickDeselects: false,
        columnSorting: false,
        undo: true,
        mergeCells: Droptables.style.table.mergeSetting,
        readOnly: !(table_function_data.allowed_edit && table_function_data.mysqlEdit),
        // readOnly: false,
        selectionMode: typeof table_function_data.mysqlEdit !== 'undefined' && table_function_data.mysqlEdit ? 'single' : 'range',
        columns: typeof table_function_data.columns !== 'undefined' ? table_function_data.columns : undefined,
        beforeKeyDown: function (e) {
            var evtobj = window.event? event : e;

            if (evtobj.keyCode == 90 && (evtobj.ctrlKey || evtobj.metaKey)) {//undo
                if (!evtobj.shiftKey) {
                    if (!jquery(Droptables_element.tableContainer).handsontable('getInstance').undoRedo.isUndoAvailable()) {
                        tableFunction.status_noti(1, Joomla.JText._('COM_DROPTABLES_SOME_ACTION_CANT_BE_DONE', "Some actions can't be done due to the Technical Problem") + '!');
                    }
                }
                if (evtobj.shiftKey) {
                    if (!jquery(Droptables_element.tableContainer).handsontable('getInstance').undoRedo.isRedoAvailable()) {
                        tableFunction.status_noti(1, Joomla.JText._('COM_DROPTABLES_SOME_ACTION_CANT_BE_DONE', "Some actions can't be done due to the Technical Problem") + '!');
                    }
                }
            }
        },
        cells: function(row, col, prop) {
            return tableFunction.check_cell_readOnly(row, col, prop);
            // var cellProperties = {};

            // if (window.Droptables.type === 'mysql') {
            //     if (row < 1) {
            //         cellProperties.readOnly = false;
            //     }

                // if (Droptables.table_editing != '1' && row >= 1) {
                //     cellProperties.readOnly = true;
                // }
            // }

            // return cellProperties;
        },
        contextMenu: (table_function_data.allowed_edit && window.Droptables.type === 'html')
            ? {
                items: {
                    "cut": {
                        name: Joomla.JText._('COM_DROPTABLES_TABLES_CUT', 'Cut'),
                        hidden: function () {
                            if (Droptables.max_Col * Droptables.max_row > 0) {
                                return false;
                            }
                            return true;
                        }
                    },
                    "copy": {
                        name: Joomla.JText._('COM_DROPTABLES_TABLES_COPY', 'Copy'),
                        hidden: function () {
                            if (Droptables.max_Col * Droptables.max_row > 0) {
                                return false;
                            }
                            return true;
                        },
                    },
                    // "paste": {
                    //     key: 'paste',
                    //     name: 'Paste',
                    //     disabled: function() {
                    //         return clipboardCache.length === 0;
                    //     },
                    //     callback: function() {
                    //         var plugin = this.getPlugin('copyPaste');
                    //
                    //         this.listen();
                    //         plugin.paste(clipboardCache);
                    //     }
                    // },
                    "remove": {
                        name: Joomla.JText._('COM_DROPTABLES_TABLES_REMOVE', 'Remove'),
                        key: 'remove',
                        submenu: {
                            items: [
                                {
                                    key: "remove:remove_row",
                                    name: Joomla.JText._('COM_DROPTABLES_TABLES_REMOVE_ROW', 'Remove row'),
                                    callback: function (key, selection, clickEvent) {
                                        var selection = table_function_data.selection;
                                        var i;
                                        for (i = 0; i < table_function_data.selectionSize; i++) {
                                            if (selection[i][2] != null || selection[i][0] != selection[i][2]) {
                                                window.Droptables_element.tableContainer.handsontable('alter', 'remove_row', selection[i][0], selection[i][2] - selection[i][0] + 1);
                                            } else {
                                                window.Droptables_element.tableContainer.handsontable('alter', 'remove_row', selection[i][0]);
                                            }
                                        }
                                    },
                                },
                                {
                                    key: "remove:remove_col",
                                    name: Joomla.JText._('COM_DROPTABLES_TABLES_REMOVE_COL', 'Remove col'),
                                    callback: function (key, options) {
                                        var selection = table_function_data.selection;
                                        var i;
                                        for (i = 0; i < table_function_data.selectionSize; i++) {
                                            if (selection[i][3] != null || selection[i][1] != selection[i][3]) {
                                                window.Droptables_element.tableContainer.handsontable('alter', 'remove_col', selection[i][1], selection[i][3] - selection[i][1] + 1);
                                            } else {
                                                window.Droptables_element.tableContainer.handsontable('alter', 'remove_col', selection[i][1]);
                                            }
                                        }
                                    },
                                },
                                {
                                    key: "remove:remove_cell_format",
                                    name: Joomla.JText._('COM_DROPTABLES_TABLES_REMOVE_CELL_FORMAT', 'Remove cell format'),
                                    callback: function (key, selection, clickEvent) {
                                        tableFunction.getFillArray(table_function_data.selection, Droptables, {
                                            date_formats: false,
                                            date_formats_momentjs: false,
                                            currency_symbol: false,
                                            symbol_position: false,
                                            decimal_symbol: false,
                                            decimal_count: false,
                                            thousand_symbol: false
                                        }, 'style', true, function () {
                                            window.jquery(Droptables_element.tableContainer).handsontable('render');
                                        });
                                    },
                                    hidden: function () {//hiden when has html cell
                                        if (table_function_data.selectionSize > 0) {
                                            if (!tableFunction.checkCellsOptionsValidate(table_function_data.selection, 'cell_type', 'html')) {
                                                return false;
                                            }
                                        }
                                        return true;
                                    }
                                },
                                {
                                    key: "remove:remove_alternating_color",
                                    name: Joomla.JText._('COM_DROPTABLES_TABLES_REMOVE_ALTERNATING_COLOR', 'Remove automatic styling'),
                                    callback: function (key, selection, clickEvent) {
                                        var i, ij, ik, alternateColorCell, selection_cell = null;
                                        for (i = 0; i < table_function_data.selectionSize; i++) {
                                            if (selection_cell !== null) {
                                                break;
                                            }
                                            for (ij = table_function_data.selection[i][0]; ij <= table_function_data.selection[i][2]; ij++) {
                                                if (selection_cell !== null) {
                                                    break;
                                                }
                                                for (ik = table_function_data.selection[i][1]; ik <= table_function_data.selection[i][3]; ik++) {
                                                    if (typeof Droptables.style.cells[ij + "!" + ik] !== 'undefined' && typeof Droptables.style.cells[ij + "!" + ik][2].AlternateColor !== 'undefined') {
                                                        alternateColorCell = parseInt(Droptables.style.cells[ij + "!" + ik][2].AlternateColor);
                                                        if (typeof Droptables.style.table.alternateColorValue[alternateColorCell] !== 'undefined' && typeof Droptables.style.table.alternateColorValue[alternateColorCell].selection !== 'undefined') {
                                                            selection_cell = Droptables.style.table.alternateColorValue[alternateColorCell].selection;
                                                            Droptables_element.tableContainer.handsontable("selectCell", selection_cell[0], selection_cell[1], selection_cell[2], selection_cell[3]);
                                                            break;
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                        if (selection_cell !== null) {
                                            bootbox.confirm(Joomla.JText._('COM_DROPTABLES_TABLES_WARNING_REMOVE_ALTERNATING_COLOR',
                                                'Are you sure you want to remove all the table automatic styling?'), function (result) {
                                                if (result) {
                                                    table_function_data.oldAlternate[alternateColorCell] = false;
                                                    Droptables.style.table.alternateColorValue[alternateColorCell] = false;
                                                    setTimeout(function () {
                                                        saveData.push({action: 'deleteStyle', selection: table_function_data.selection, style: 'AlternateColor'});
                                                        Droptables_element.droptable_popup.find('.colose_popup').trigger('click');
                                                        tableFunction.saveChanges(true, 'render');
                                                    }, 100);
                                                } else {
                                                    Droptables_element.droptable_popup.find('.colose_popup').trigger('click');
                                                }
                                                ;
                                            });
                                        }
                                    },
                                    hidden: function () {//hiden when has html cell
                                        if (table_function_data.selectionSize > 0) {
                                            return false;
                                        }
                                        return true;
                                    }
                                }],
                        },
                        hidden: function () {
                            if (Droptables.max_Col * Droptables.max_row > 0) {
                                return false;
                            }
                            return true;
                        }
                    },
                    "---------": {},
                    "rows_size": {
                        name: function () {
                            if (typeof Droptables.newSelect !== 'undefined') {
                                var selectSize = table_function_data.selection[table_function_data.selectionSize - 1];
                                if (Droptables.newSelect === 'row') {
                                    if (selectSize[0] !== selectSize[2]) {
                                        return '<span>' + Joomla.JText._('COM_DROPTABLES_TABLES_RESIZE_ROW', 'Resize row') + 's ' + (selectSize[0] + 1) + '-' + (selectSize[2] + 1) + '</span>';
                                    }
                                    return '<span>' + Joomla.JText._('COM_DROPTABLES_TABLES_RESIZE_ROW', 'Resize row') + '</span>';
                                } else {
                                    if (selectSize[1] !== selectSize[3]) {
                                        return '<span>' + Joomla.JText._('COM_DROPTABLES_TABLES_RESIZE_COL', 'Resize column') + 's ' + String.fromCharCode(65 + selectSize[1]) + '-' + String.fromCharCode(65 + selectSize[3]) + '</span>';
                                    }
                                    return '<span>' + Joomla.JText._('COM_DROPTABLES_TABLES_RESIZE_COL', 'Resize column') + '</span>';
                                }
                            }
                        },
                        callback: function (key, selection, clickEvent) {
                            if (Droptables.newSelect === 'row') {
                                Droptables_element.primary_toolbars.find('.table_option[name="resize_row"]').trigger('click');
                            } else {
                                Droptables_element.primary_toolbars.find('.table_option[name="resize_column"]').trigger('click');
                            }
                            return true;
                        },
                        hidden: function () {
                            if (typeof Droptables.newSelect !== 'undefined' && Droptables.max_Col * Droptables.max_row > 0) {
                                return false;
                            }
                            return true;
                        }
                    },
                    "insert": {
                        name: Joomla.JText._('COM_DROPTABLES_TABLES_INSERT', 'Insert'),
                        key: 'insert',
                        submenu: {
                            items: [
                                {
                                    key: "insert:row_above",
                                    name: Joomla.JText._('COM_DROPTABLES_TABLES_INSERT_ABOVE', 'Insert rows above'),
                                    callback: function (key, options) {
                                        var selection = table_function_data.selection[table_function_data.selectionSize - 1];
                                        if (selection[2] != null || selection[0] != selection[2]) {
                                            window.Droptables_element.tableContainer.handsontable('alter', 'insert_row', selection[0], selection[2] - selection[0] + 1);
                                        } else {
                                            window.Droptables_element.tableContainer.handsontable('alter', 'insert_row', selection[0]);
                                        }
                                    },
                                    hidden: function () {
                                        if (Droptables.max_Col > 0) {
                                            return false;
                                        }
                                        return true;
                                    }
                                },
                                {
                                    key: "insert:row_below",
                                    name: Joomla.JText._('COM_DROPTABLES_TABLES_INSERT_BELOW', 'Insert rows below'),
                                    callback: function (key, options) {
                                        var selection = table_function_data.selection[table_function_data.selectionSize - 1];
                                        if (selection[2] != null || selection[0] != selection[2]) {
                                            window.Droptables_element.tableContainer.handsontable('alter', 'insert_row', selection[2] + 1, selection[2] - selection[0] + 1);
                                        } else {
                                            window.Droptables_element.tableContainer.handsontable('alter', 'insert_row', selection[2] + 1);
                                        }
                                    },
                                    hidden: function () {
                                        if (Droptables.max_Col > 0) {
                                            return false;
                                        }
                                        return true;
                                    }
                                },
                                {
                                    key: "insert:col_left",
                                    name: Joomla.JText._('COM_DROPTABLES_TABLES_INSERT_LEFT', 'Insert column left'),
                                    callback: function (key, options) {
                                        var selection = table_function_data.selection[table_function_data.selectionSize - 1];
                                        if (selection[3] != null || selection[1] != selection[3]) {
                                            window.Droptables_element.tableContainer.handsontable('alter', 'insert_col', selection[1], selection[3] - selection[1] + 1);
                                        } else {
                                            window.Droptables_element.tableContainer.handsontable('alter', 'insert_col', selection[1]);
                                        }
                                    },
                                    hidden: function () {
                                        if (Droptables.max_row > 0) {
                                            return false;
                                        }
                                        return true;
                                    }
                                },
                                {
                                    key: "insert:col_right",
                                    name: Joomla.JText._('COM_DROPTABLES_TABLES_INSERT_RIGHT', 'Insert column right'),
                                    callback: function (key, options) {
                                        var selection = table_function_data.selection[table_function_data.selectionSize - 1];
                                        if (selection[3] != null || selection[1] != selection[3]) {
                                            window.Droptables_element.tableContainer.handsontable('alter', 'insert_col', selection[3] + 1, selection[3] - selection[1] + 1);
                                        } else {
                                            window.Droptables_element.tableContainer.handsontable('alter', 'insert_col', selection[3] + 1);
                                        }
                                    },
                                    hidden: function () {
                                        if (Droptables.max_row > 0) {
                                            return false;
                                        }
                                        return true;
                                    }
                                }],
                        }
                    },
                    "undo": {
                        name: Joomla.JText._('COM_DROPTABLES_TABLES_UNDO', 'Undo'),
                        hidden: function () {
                            if (Droptables.max_Col * Droptables.max_row > 0) {
                                return false;
                            }
                            return true;
                        }
                    },
                    "redo": {
                        name: Joomla.JText._('COM_DROPTABLES_TABLES_REDO', 'Redo'),
                        hidden: function () {
                            if (Droptables.max_Col * Droptables.max_row > 0) {
                                return false;
                            }
                            return true;
                        }
                    },
                    "mergeCells": {
                        // name: DroptablesContext.merge,
                    },
                    "protect_range": {
                        name: function () {
                            return '<span>' + Joomla.JText._('COM_DROPTABLES_TABLES_PROTECT_RANGE', 'Protect range') + '</span>';
                        },
                        callback: function (key, selections, clickEvent) {
                            Droptables_element.primary_toolbars.find('.table_option[name="lock_ranger_cells"]').trigger('click');
                            return true;
                        },
                        hidden: function () {
                            if (Droptables.check_super_user) {
                                return false;
                            }
                            return true;
                        }
                    },
                    "Add tooltip": {
                        name: Joomla.JText._('COM_DROPTABLES_TABLES_TOOLTIP', 'Tooltip'),
                        callback: function (key, selection, clickEvent) {
                            Droptables_element.editToolTip.trigger('click');
                        },
                        hidden: function () {
                            if (Droptables.max_Col * Droptables.max_row < 1 || !(typeof default_value.enable_tooltip !== 'undefined' && parseInt(default_value.enable_tooltip) === 1)) {
                                return true;
                            }
                            if (table_function_data.selectionSize > 1
                                || (table_function_data.selection[0][2] - table_function_data.selection[0][0] > 0
                                    || table_function_data.selection[0][3] - table_function_data.selection[0][1] > 0)) {
                                return true;
                            }
                            return false;
                        }
                    },
                    "Column type": {
                        name: Joomla.JText._('COM_DROPTABLES_TABLES_COL_TYPE', 'Column type'),
                        key: 'column_type',
                        submenu: {
                            items: [
                                {
                                    key: "column_type:varchar",
                                    name: function () {
                                        if (typeof table_function_data.type_column_selected !== 'undefined' && (table_function_data.type_column_selected === 'varchar' || table_function_data.type_column_selected.toLowerCase() === 'varchar(255)')) {
                                            return '<span class="selected">VARCHAR</span>';
                                        } else {
                                            return '<span>VARCHAR</span>';
                                        }
                                    },
                                    callback: function (key, options) {
                                        bootbox.confirm(Joomla.JText._('COM_DROPTABLES_ALERT_CHANGE_COLUMN_TYPE'), function (result) {
                                            if (result === true) {
                                                var cols_selected = [];
                                                var i, jj;
                                                for (jj = 0; jj < table_function_data.selectionSize; jj++) {
                                                    for (i = table_function_data.selection[jj][1]; i <= table_function_data.selection[jj][3]; i++) {
                                                        cols_selected[i] = 'varchar';
                                                        Droptables.style.table.col_types[i] = 'varchar';
                                                    }
                                                }

                                                // saveData.push({action: 'set_column_type', cols: unique_cols_selected, type: 'varchar'});
                                                saveData.push({
                                                    action: 'set_columns_types',
                                                    value: cols_selected
                                                });
                                                tableFunction.cleanHandsontable();
                                                tableFunction.saveChanges(true);
                                            }
                                        });
                                    },
                                },
                                {
                                    key: "column_type:int",
                                    name: function () {
                                        if (typeof table_function_data.type_column_selected !== 'undefined' && table_function_data.type_column_selected === 'int') {
                                            return '<span class="selected">INTEGER</span>';
                                        } else {
                                            return '<span>INTEGER</span>';
                                        }
                                    },
                                    callback: function (key, options) {
                                        bootbox.confirm(Joomla.JText._('COM_DROPTABLES_ALERT_CHANGE_COLUMN_TYPE'), function (result) {
                                            if (result === true) {
                                                var cols_selected = [];
                                                var i, jj;
                                                for (jj = 0; jj < table_function_data.selectionSize; jj++) {
                                                    for (i = table_function_data.selection[jj][1]; i <= table_function_data.selection[jj][3]; i++) {
                                                        cols_selected[i] = 'int';
                                                        Droptables.style.table.col_types[i] = 'int';
                                                    }
                                                }

                                                // saveData.push({action: 'set_column_type', cols: unique_cols_selected, type: 'varchar'});
                                                saveData.push({
                                                    action: 'set_columns_types',
                                                    value: cols_selected
                                                });
                                                tableFunction.cleanHandsontable();
                                                tableFunction.saveChanges(true);
                                            }
                                        });
                                    },
                                },
                                {
                                    key: "column_type:float",
                                    name: function () {
                                        if (typeof table_function_data.type_column_selected !== 'undefined' && table_function_data.type_column_selected === 'float') {
                                            return '<span class="selected">FLOAT</span>';
                                        } else {
                                            return '<span>FLOAT</span>';
                                        }
                                    },
                                    callback: function (key, options) {
                                        bootbox.confirm(Joomla.JText._('COM_DROPTABLES_ALERT_CHANGE_COLUMN_TYPE'), function (result) {
                                            if (result === true) {
                                                var cols_selected = [];
                                                var i, jj;
                                                for (jj = 0; jj < table_function_data.selectionSize; jj++) {
                                                    for (i = table_function_data.selection[jj][1]; i <= table_function_data.selection[jj][3]; i++) {
                                                        cols_selected[i] = 'float';
                                                        Droptables.style.table.col_types[i] = 'float';
                                                    }
                                                }

                                                // saveData.push({action: 'set_column_type', cols: unique_cols_selected, type: 'varchar'});
                                                saveData.push({
                                                    action: 'set_columns_types',
                                                    value: cols_selected
                                                });
                                                tableFunction.cleanHandsontable();
                                                tableFunction.saveChanges(true);
                                            }
                                        });

                                    },
                                },
                                {
                                    key: "column_type:date",
                                    name: function () {
                                        if (typeof table_function_data.type_column_selected !== 'undefined' && table_function_data.type_column_selected === 'date') {
                                            return '<span class="selected">DATE</span>';
                                        } else {
                                            return '<span>DATE</span>';
                                        }
                                    },
                                    callback: function (key, options) {
                                        bootbox.confirm(Joomla.JText._('COM_DROPTABLES_ALERT_CHANGE_COLUMN_TYPE'), function (result) {
                                            if (result === true) {
                                                var cols_selected = [];
                                                var i, jj;
                                                for (jj = 0; jj < table_function_data.selectionSize; jj++) {
                                                    for (i = table_function_data.selection[jj][1]; i <= table_function_data.selection[jj][3]; i++) {
                                                        cols_selected[i] = 'date';
                                                        Droptables.style.table.col_types[i] = 'date';
                                                    }
                                                }

                                                // saveData.push({action: 'set_column_type', cols: unique_cols_selected, type: 'varchar'});
                                                saveData.push({
                                                    action: 'set_columns_types',
                                                    value: cols_selected
                                                });
                                                tableFunction.cleanHandsontable();
                                                tableFunction.saveChanges(true);
                                            }
                                        });

                                    },
                                },
                                {
                                    key: "column_type:datetime",
                                    name: function () {
                                        if (typeof table_function_data.type_column_selected !== 'undefined' && table_function_data.type_column_selected === 'datetime') {
                                            return '<span class="selected">DATETIME</span>';
                                        } else {
                                            return '<span>DATETIME</span>';
                                        }
                                    },
                                    callback: function (key, options) {
                                        bootbox.confirm(Joomla.JText._('COM_DROPTABLES_ALERT_CHANGE_COLUMN_TYPE'), function (result) {
                                            if (result === true) {
                                                var cols_selected = [];
                                                var i, jj;
                                                for (jj = 0; jj < table_function_data.selectionSize; jj++) {
                                                    for (i = table_function_data.selection[jj][1]; i <= table_function_data.selection[jj][3]; i++) {
                                                        cols_selected[i] = 'datetime';
                                                        Droptables.style.table.col_types[i] = 'datetime';
                                                    }
                                                }

                                                // saveData.push({action: 'set_column_type', cols: unique_cols_selected, type: 'varchar'});
                                                saveData.push({
                                                    action: 'set_columns_types',
                                                    value: cols_selected
                                                });
                                                tableFunction.cleanHandsontable();
                                                tableFunction.saveChanges(true);
                                            }
                                        });

                                    },
                                },
                                {
                                    key: "column_type:text",
                                    name: function () {
                                        if (typeof table_function_data.type_column_selected !== 'undefined' && table_function_data.type_column_selected === 'text') {
                                            return '<span class="selected">TEXT</span>';
                                        } else {
                                            return '<span>TEXT</span>';
                                        }
                                    },
                                    callback: function (key, options) {
                                        bootbox.confirm(Joomla.JText._('COM_DROPTABLES_ALERT_CHANGE_COLUMN_TYPE'), function (result) {
                                            if (result === true) {
                                                var cols_selected = [];
                                                var i, jj;
                                                for (jj = 0; jj < table_function_data.selectionSize; jj++) {
                                                    for (i = table_function_data.selection[jj][1]; i <= table_function_data.selection[jj][3]; i++) {
                                                        cols_selected[i] = 'text';
                                                        Droptables.style.table.col_types[i] = 'text';
                                                    }
                                                }

                                                // saveData.push({action: 'set_column_type', cols: unique_cols_selected, type: 'varchar'});
                                                saveData.push({
                                                    action: 'set_columns_types',
                                                    value: cols_selected
                                                });
                                                tableFunction.cleanHandsontable();
                                                tableFunction.saveChanges(true);
                                            }
                                        });

                                    },
                                },
                                {
                                    key: "column_type:mediumtext",
                                    name: function () {
                                        if (typeof table_function_data.type_column_selected !== 'undefined' && table_function_data.type_column_selected === 'mediumtext') {
                                            return '<span class="selected">MEDIUMTEXT</span>';
                                        } else {
                                            return '<span>MEDIUMTEXT</span>';
                                        }
                                    },
                                    callback: function (key, options) {
                                        bootbox.confirm(Joomla.JText._('COM_DROPTABLES_ALERT_CHANGE_COLUMN_TYPE'), function (result) {
                                            if (result === true) {
                                                var cols_selected = [];
                                                var i, jj;
                                                for (jj = 0; jj < table_function_data.selectionSize; jj++) {
                                                    for (i = table_function_data.selection[jj][1]; i <= table_function_data.selection[jj][3]; i++) {
                                                        cols_selected[i] = 'mediumtext';
                                                        Droptables.style.table.col_types[i] = 'mediumtext';
                                                    }
                                                }

                                                // saveData.push({action: 'set_column_type', cols: unique_cols_selected, type: 'varchar'});
                                                saveData.push({
                                                    action: 'set_columns_types',
                                                    value: cols_selected
                                                });
                                                tableFunction.cleanHandsontable();
                                                tableFunction.saveChanges(true);
                                            }
                                        });

                                    },
                                },
                            ]
                        },
                        hidden: function () {
                            if (Droptables.max_Col * Droptables.max_row < 1) {
                                return true;
                            }
                            if (table_function_data.selectionSize > 1 || table_function_data.selection[0][3] - table_function_data.selection[0][1] > 0) {
                                return true;
                            }
                            return false;
                        }
                    },
                    "hide_column": {
                        name: function () {
                            if (typeof Droptables.newSelect !== 'undefined' && Droptables.newSelect === 'col') {
                                var selection = table_function_data.selection;
                                if (typeof Droptables.style.cols[selection[0][1]] !== 'undefined'
                                    && Droptables.style.cols[selection[0][1]] !== null
                                    && typeof Droptables.style.cols[selection[0][1]][1].hide_column !== 'undefined'
                                    && Droptables.style.cols[selection[0][1]][1].hide_column == 1) {
                                    return '<span class="selected">' + Joomla.JText._('COM_DROPTABLES_TABLES_HIDE_COL', 'Hide column on front-end') + '</span>';
                                } else {
                                    return '<span>' + Joomla.JText._('COM_DROPTABLES_TABLES_HIDE_COL', 'Hide column on front-end') + '</span>';
                                }
                            }
                        },
                        callback: function (key, selection, clickEvent) {
                            if (Droptables.newSelect === 'col') {
                                var selection = table_function_data.selection;
                                if (typeof Droptables.style.cols[selection[0][1]] !== 'undefined' && Droptables.style.cols[selection[0][1]] !== null) {
                                    if (typeof Droptables.style.cols[selection[0][1]][1].hide_column !== 'undefined' && Droptables.style.cols[selection[0][1]][1].hide_column == 1) {
                                        Droptables.style.cols = tableFunction.fillArray(Droptables.style.cols, {hide_column: 0}, selection[0][1]);
                                    } else {
                                        Droptables.style.cols = tableFunction.fillArray(Droptables.style.cols, {hide_column: 1}, selection[0][1]);
                                    }
                                }
                                tableFunction.saveChanges(true);
                                return true;
                            }
                            return true;
                        },
                        hidden: function () {
                            var selection = table_function_data.selection;
                            if (typeof Droptables.newSelect === 'undefined' || Droptables.newSelect !== "col" || selection[0][3] - selection[0][1] >= 1) {
                                return true;
                            }
                            if (Droptables.max_Col * Droptables.max_row > 0) {
                                return false;
                            }
                            return true;
                        }
                    }
                }
            }
            : {
                items: {
                    "hide_column": {
                        name: function () {
                            if (typeof Droptables.newSelect !== 'undefined' && Droptables.newSelect === 'col') {
                                var selection = table_function_data.selection;
                                if (typeof Droptables.style.cols[selection[0][1]] !== 'undefined'
                                    && Droptables.style.cols[selection[0][1]] !== null
                                    && typeof Droptables.style.cols[selection[0][1]][1].hide_column !== 'undefined'
                                    && Droptables.style.cols[selection[0][1]][1].hide_column == 1) {
                                    return '<span class="selected">' + Joomla.JText._('COM_DROPTABLES_TABLES_HIDE_COL', 'Hide column on front-end') + '</span>';
                                } else {
                                    return '<span>' + Joomla.JText._('COM_DROPTABLES_TABLES_HIDE_COL', 'Hide column on front-end') + '</span>';
                                }
                            }
                        },
                        callback: function (key, selection, clickEvent) {
                            if (Droptables.newSelect === 'col') {
                                var selection = table_function_data.selection;
                                if (typeof Droptables.style.cols[selection[0][1]] !== 'undefined' && Droptables.style.cols[selection[0][1]] !== null) {
                                    if (typeof Droptables.style.cols[selection[0][1]][1].hide_column !== 'undefined' && Droptables.style.cols[selection[0][1]][1].hide_column == 1) {
                                        Droptables.style.cols = tableFunction.fillArray(Droptables.style.cols, {hide_column: 0}, selection[0][1]);
                                    } else {
                                        Droptables.style.cols = tableFunction.fillArray(Droptables.style.cols, {hide_column: 1}, selection[0][1]);
                                    }
                                }
                                return true;
                            }
                            return true;
                        },
                        hidden: function () {
                            if (typeof Droptables.query_option === 'undefined') {
                                return true;
                            }
                            var selection = table_function_data.selection;
                            if (typeof Droptables.newSelect === 'undefined' || Droptables.newSelect !== "col" || selection[0][3] - selection[0][1] >= 1) {
                                return true;
                            }
                            if (Droptables.max_Col * Droptables.max_row > 0) {
                                return false;
                            }
                            return true;
                        }
                    },
                    "remove_alternating_color": {
                        name: Joomla.JText._('COM_DROPTABLES_TABLES_REMOVE_ALTERNATING_COLOR', 'Remove automatic styling'),
                        callback: function (key, selection, clickEvent) {
                            tableFunction.fillArray(table_function_data, {allAlternate: {}});
                            tableFunction.fillArray(Droptables.style.table, {allAlternate: {}});
                            window.Droptables_element.tableContainer.handsontable('render');
                            tableFunction.saveChanges(true, 'render');
                        },
                        hidden: function () {
                            if (typeof table_function_data.allAlternate !== 'undefined' && typeof table_function_data.allAlternate.even !== 'undefined') {
                                return false;
                            }
                            return true;
                        }
                    },
                    "delete_row_dbtable": {
                        name: function () {
                            if (typeof Droptables.newSelect !== 'undefined') {
                                if (Droptables.newSelect === 'row') {
                                    return '<span>' + Joomla.JText._('COM_DROPTABLES_TABLES_DELETE_ROW', 'Delete Row') + '</span>';
                                }
                            }
                        },
                        callback: function (key, selection, clickEvent) {
                            if (Droptables.newSelect === 'row') {
                                if (typeof Droptables.query_option.columns_list !== "undefined") {
                                    var not_create = false;
                                    var warning_craete = Joomla.JText._('COM_DROPTABLES_JS_WARNING_DELETE_DB_TABLE');

                                    if (Droptables.query_option.tables_list.length > 1) {
                                        not_create = true;
                                        warning_craete = Joomla.JText._('COM_DROPTABLES_JS_WARNING_DELETE_MULTIPLE_DB_TABLE');
                                    }

                                    var selection = table_function_data.selection;
                                    if (not_create) {
                                        bootbox.alert(warning_craete);
                                        return true;
                                    } else if (typeof selection[0] !== 'undefined') {
                                        bootbox.confirm(warning_craete, function (result) {
                                            if (result === true) {
                                                var id_row = Droptables.datas[selection[0][0]][table_function_data.keyPosition];
                                                var field = Droptables.query_option.column_options[table_function_data.keyPosition].Field;
                                                var table_name = Droptables.query_option.column_options[table_function_data.keyPosition].table;
                                                tableFunction.deleteRowDbTable(table_name, field, id_row);
                                            } else {
                                                return true;
                                            }
                                        });
                                    }
                                }
                                return true;
                            }
                            return true;
                        },
                        hidden: function () {
                            if (typeof Droptables.query_option === 'undefined') {
                                return true;
                            }
                            var selection = table_function_data.selection;
                            if (typeof Droptables.newSelect === 'undefined' || Droptables.newSelect !== "row" || selection[0][2] - selection[0][0] >= 1 || !table_function_data.mysqlEdit) {
                                return true;
                            }
                            if (Droptables.max_Col * Droptables.max_row > 0) {
                                return false;
                            }
                            return true;
                        }
                    },
                    "create_row_dbtable": {
                        name: function () {
                            if (typeof Droptables.newSelect !== 'undefined') {
                                if (Droptables.newSelect === 'row') {
                                    return '<span>' + Joomla.JText._('COM_DROPTABLES_TABLES_CREATE_ROW', 'Create Row') + '</span>';
                                }
                            }
                        },
                        callback: function (key, selection, clickEvent) {
                            if (Droptables.newSelect === 'row') {
                                var popup = {
                                    'html': Droptables_element.content_popup_hide.find('#create_row_dbtable'),
                                    'showAction': function () {
                                        return true;
                                    },
                                    'submitAction': function () {
                                        if (typeof Droptables.query_option.columns_list !== "undefined") {
                                            var list_value = {};
                                            var dbtable = '';
                                            var not_create = false;
                                            var warning_create = '';
                                            window.jquery.each(Droptables.query_option.columns_list, function (i, v) {
                                                if (v.canEdit == 1) {
                                                    var value = Droptables_element.droptable_popup.find('input.table_column[data-column-name="' + v.table + '.' + v.Field + '"]').val();
                                                    var validate = tableFunction.validate_type_cell(
                                                        v.Type,
                                                        value
                                                    );
                                                    // if (!(typeof list_value[v.table] !== 'undefined')) {
                                                    //     list_value[v.table] = {};
                                                    // }
                                                    // list_value[v.table][v.Field] = value;
                                                    list_value[v.Field] = value;
                                                    dbtable = v.table;
                                                    if (Droptables.query_option.tables_list.length > 1) {
                                                        not_create = true;
                                                        warning_create = Joomla.JText._('COM_DROPTABLES_JS_WARNING_CREATE_MULTIPLE_DB_TABLE');
                                                    } else {
                                                        if (!validate && v.Null !== 'YES') {
                                                            not_create = true;
                                                            warning_create = Joomla.JText._('COM_DROPTABLES_JS_WARNING_CREATE_DB_TABLE');
                                                        }
                                                    }
                                                }

                                                if (Droptables.query_option.columns_list.length == i + 1) {
                                                    if (not_create) {
                                                        bootbox.alert(warning_create);
                                                        return true;
                                                    } else {
                                                        tableFunction.createRowDbTable(dbtable, list_value);
                                                        Droptables_element.droptable_popup.find('.colose_popup').trigger('click');
                                                        return true;
                                                    }
                                                }
                                            });
                                        }
                                        return true;
                                    },
                                    'cancelAction': function () {
                                        return true;
                                    }
                                };
                                selectOption.droptables_popup(false, Droptables_element.droptable_popup, popup, true, false, true);
                            }
                            return true;
                        },
                        hidden: function () {
                            if (typeof Droptables.query_option === 'undefined') {
                                return true;
                            }
                            if (typeof Droptables.newSelect === 'undefined' || Droptables.newSelect !== "row" || !table_function_data.mysqlEdit) {
                                return true;
                            }
                            if (Droptables.max_Col * Droptables.max_row > 0) {
                                return false;
                            }
                            return true;
                        }
                    },
                    "rows_size": {
                        name: function () {
                            if (typeof Droptables.newSelect !== 'undefined') {
                                var selectSize = table_function_data.selection[table_function_data.selectionSize - 1];
                                if (Droptables.newSelect === 'row') {
                                    if (selectSize[0] !== selectSize[2]) {
                                        return '<span>' + Joomla.JText._('COM_DROPTABLES_TABLES_RESIZE_ROW', 'Resize row') + 's ' + (selectSize[0] + 1) + '-' + (selectSize[2] + 1) + '</span>';
                                    }
                                    return '<span>' + Joomla.JText._('COM_DROPTABLES_TABLES_RESIZE_ROW', 'Resize row') + '</span>';
                                } else {
                                    if (selectSize[1] !== selectSize[3]) {
                                        return '<span>' + Joomla.JText._('COM_DROPTABLES_TABLES_RESIZE_COL', 'Resize column') + 's ' + String.fromCharCode(65 + selectSize[1]) + '-' + String.fromCharCode(65 + selectSize[3]) + '</span>';
                                    }
                                    return '<span>' + Joomla.JText._('COM_DROPTABLES_TABLES_RESIZE_COL', 'Resize column') + '</span>';
                                }
                            }
                        },
                        callback: function (key, selection, clickEvent) {
                            if (Droptables.newSelect === 'row') {
                                Droptables_element.primary_toolbars.find('.table_option[name="resize_row"]').trigger('click');
                            } else {
                                Droptables_element.primary_toolbars.find('.table_option[name="resize_column"]').trigger('click');
                            }
                            return true;
                        },
                        hidden: function () {
                            if (typeof Droptables.newSelect === 'undefined' || (Droptables.newSelect !== "row" && Droptables.newSelect !== "col")) {
                                return true;
                            }
                            if (Droptables.max_Col * Droptables.max_row > 0) {
                                return false;
                            }
                            return true;
                        }
                    },
                    "protect_range": {
                        name: function () {
                            return '<span>' + Joomla.JText._('COM_DROPTABLES_TABLES_PROTECT_RANGE', 'Protect range') + '</span>';
                        },
                        callback: function (key, selections, clickEvent) {
                            Droptables_element.primary_toolbars.find('.table_option[name="lock_ranger_cells"]').trigger('click');
                            return true;
                        },
                        hidden: function () {
                            if (Droptables.check_super_user) {
                                return false;
                            }
                            return true;
                        }
                    },
                    "Add tooltip": {
                        name: Joomla.JText._('COM_DROPTABLES_TABLES_TOOLTIP', 'Tooltip'),
                        callback: function (key, selection, clickEvent) {
                            Droptables_element.editToolTip.trigger('click');
                        },
                        hidden: function () {
                            if (Droptables.max_Col * Droptables.max_row < 1 || !(typeof default_value.enable_tooltip !== 'undefined' && parseInt(default_value.enable_tooltip) === 1)) {
                                return true;
                            }
                            if (table_function_data.selectionSize > 1
                                || (table_function_data.selection[0][2] - table_function_data.selection[0][0] > 0
                                    || table_function_data.selection[0][3] - table_function_data.selection[0][1] > 0)) {
                                return true;
                            }
                            return false;
                        }
                    }
                }
            }
    })

    window.Droptables_element.tableContainer.find('.ht_clone_top_left_corner').unbind('click').on('click', function () {
        var max_col = parseInt(Droptables.max_Col) > 0 ? Droptables.max_Col : Droptables.datas[0].length;
        var max_row = parseInt(Droptables.max_row) > 0 ? Droptables.max_row : Droptables.datas.length;
        jquery(Droptables_element.tableContainer).handsontable("selectCells", [[0, 0, max_row - 1, max_col - 1]]);
    });

    // search key
    Droptables_element.primary_toolbars.find('.search-menu').find('#dp-form-search').unbind('keydown.mm').bind('keydown.mm', function (event) {
        event.stopImmediatePropagation();
        if (event.keyCode === 13) {
            Droptables_element.primary_toolbars.find('.search-menu').find('.search_table').trigger('click');
        } else if (jquery(this).val() === '') {
            Droptables_element.primary_toolbars.find('.search-menu').find('.search_table').trigger('click');
        }
    });

    Droptables_element.primary_toolbars.find('.search-menu').find('#dp-form-search').bind('click.search', (e) => {
        event.stopPropagation();
        Droptables_element.primary_toolbars.find('.search-menu').find('#dp-form-search').focus();
        return false;
    });

    Droptables_element.primary_toolbars.find('.search-menu').find('.search_table').click(function (e) {
        jquery(this).parent().addClass('open');
        Droptables_element.primary_toolbars.find('.search-menu').find('#dp-form-search').trigger('click.search');
        var textSearch = Droptables_element.primary_toolbars.find('.search-menu').find('#dp-form-search');
        var queryResult = jquery(Droptables_element.tableContainer).data('handsontable').getPlugin('search').query(textSearch.val());
       jquery(Droptables_element.tableContainer).data('handsontable').render();
    });

    Droptables_element.primary_toolbars.find('.search-menu').find('.reload_search').click(function () {
        Droptables_element.primary_toolbars.find('.search-menu').find('#dp-form-search').val('');
        var queryResult = jquery(Droptables_element.tableContainer).data('handsontable').getPlugin('search').query('');
        jquery(Droptables_element.tableContainer).data('handsontable').render();
        jquery(this).parents('.search-menu').removeClass("open");
    });

    /*select menu option function*/
    selectOption.selectOption();

    /*more function Eg: rename table*/
    window.Droptables_element.primary_toolbars.find('.droptable_name_edit').text(window.Droptables.title);
}


/*
* Function update mergecells when change rows and cols
* */
function updateMergeCells(firstRender) {
    window.Droptables.style.table.mergeSetting = [];
    var ht = jquery(Droptables_element.tableContainer).handsontable('getInstance');
    var mergeSetting = ht.getPlugin('mergeCells').mergedCellsCollection;
    var i = 0;

    if (mergeSetting.mergedCells.length < 1) {//save
        // tableFunction.saveChanges();
    }
    table_function_data.start_merge_cell_col = {};

    for (i = 0; i < mergeSetting.mergedCells.length; i++) {
        window.Droptables.style.table.mergeSetting.push({
            col: mergeSetting.mergedCells[i].col,
            colspan: mergeSetting.mergedCells[i].colspan,
            row: mergeSetting.mergedCells[i].row,
            rowspan: mergeSetting.mergedCells[i].rowspan
        });
        if (typeof table_function_data.start_merge_cell_col[mergeSetting.mergedCells[i].col] == 'undefined') {
            table_function_data.start_merge_cell_col[mergeSetting.mergedCells[i].col] = [];
        }
        table_function_data.start_merge_cell_col[mergeSetting.mergedCells[i].col][mergeSetting.mergedCells[i].row] = i;
    }

    if (table_function_data.allowed_save) {
        // tableFunction.saveChanges(table_function_data.allowed_save);
    }
    var args = Array.prototype.slice.call(arguments, 1);
    if (typeof args[1] === 'undefined' || args[1] === true) {
        tableFunction.saveChanges(true);
    }
}

export {initHandsontable}