(function ($) {
    var getScrollbarWidth = function (selector) {
        var scrollWidth;
        if (typeof selector.get(0) === 'undefined') {
            scrollWidth = 0;
        } else {
            scrollWidth = selector.get(0).offsetWidth - selector.get(0).clientWidth + 5;//fix when width .5px
            if (scrollWidth === 0) {
                scrollWidth = 20;
            }
        }

        return scrollWidth;
    };
    var addMarginTopToFixedColumn = function (currentTable, numberRows) {
        var scrollWrapper = currentTable.closest(".DTFC_ScrollWrapper");
        if (scrollWrapper.length < 1) {
            scrollWrapper = currentTable.closest(".dataTables_wrapper");
        }

        var totalRowHeight = scrollWrapper
            .find(".DTFC_TopBodyWrapper")
            .outerHeight();

        var leftScroll = scrollWrapper.find(".DTFC_LeftBodyLiner table");
        leftScroll.css({
            marginTop: totalRowHeight + "px",
        });
    };
    var filterDelay = null;
    var filterDelayInterval = 250;


    var cloneFixedTopWrapper = function (currentTable, numberRows, numCols) {
        var scrollWrapper = currentTable.closest(".DTFC_ScrollWrapper");
        if (scrollWrapper.length < 1) {
            scrollWrapper = currentTable.closest(".dataTables_wrapper");
        }

        var scrollBody = scrollWrapper
            .find(".dataTables_scroll")
            .find(".dataTables_scrollBody");
        var topWrapper = scrollWrapper.find(".dataTables_scroll").clone();
        var tableScrollHeaderHeight = scrollWrapper
            .find(".dataTables_scrollHead")
            .outerHeight();

        topWrapper.removeClass("dataTables_scroll").addClass("DTFC_TopWrapper");
        topWrapper.css({
            position: "absolute",
            top: tableScrollHeaderHeight + "px",
            left: "0px",
            width: "100%",
            paddingRight: getScrollbarWidth(scrollBody) + "px",
            height: "1px",
        });
        //change scroll head class
        topWrapper
            .find(".dataTables_scrollHead")
            .addClass("DTFC_TopHeadWrapper")
            .removeClass("dataTables_scrollHead")
            .attr("style", "")
            .css({
                position: "relative",
                top: "0",
                left: "0",
                height: "0",
                overflow: "hidden",
            });
        topWrapper
            .find(".DTFC_TopHeadWrapper table")
            .addClass("DTFC_Cloned")
            .unwrap()
            .wrap('<div class="DTFC_TopHeadLiner"></div>');
        topWrapper.find(".DTFC_TopHeadWrapper table thead tr").css({
            height: 0,
        });

        var totalRowHeight = 0;

        var datatableScroll = scrollWrapper.find(".dataTables_scroll");
        var datatableScrollBody = datatableScroll.find(".dataTables_scrollBody");
        var datatableScrollBodyTable = datatableScrollBody.find("table");

        for (var i = 0; i < numberRows; i++) {
            var dataScrollRow = scrollWrapper
                .find(".dataTables_scrollBody table tbody tr")
                .eq(i);
            totalRowHeight += dataScrollRow.outerHeight();
            datatableScrollBodyTable.find("tbody tr").eq(i).addClass("hidden_row");
        }

        topWrapper
            .find(".dataTables_scrollBody")
            .addClass("DTFC_TopBodyWrapper")
            .removeClass("dataTables_scrollBody")
            .attr("style", "")
            .css({
                position: "relative",
                top: "0",
                left: "0",
                height: totalRowHeight + "px",
                overflow: "hidden",
            });
        topWrapper
            .find(".DTFC_TopBodyWrapper table")
            .removeAttr("id")
            .addClass("DTFC_Cloned")
            .wrap('<div class="DTFC_TopBodyLiner"></div>');
        topWrapper.find(".DTFC_TopBodyLiner").css({
            overflowX: "scroll",
        });
        topWrapper.find(".DTFC_TopBodyLiner table thead tr").addClass('hidden_row');
        topWrapper.find(".DTFC_TopBodyLiner table tbody tr.droptable_none").remove();

        topWrapper.appendTo(scrollWrapper);

        //set margin top for original table
        datatableScrollBodyTable.css({
            marginTop: totalRowHeight + "px",
        });

        if (numCols > 0) {
            var topLeftWrapper = scrollWrapper.find(".DTFC_TopWrapper").clone();
            topLeftWrapper
                .addClass("DTFC_TopLeftWrapper")
                .removeClass("DTFC_TopWrapper");
            // console.log(scrollWrapper.find(".DTFC_LeftWrapper").width());
            topLeftWrapper.css({
                padding: 0,
                width: scrollWrapper.find(".DTFC_LeftWrapper").width() + "px",
            });
            topLeftWrapper
                .find(".DTFC_TopBodyLiner")
                .addClass("DTFC_TopLeftBodyLiner")
                .removeClass("DTFC_TopBodyLiner");

            topLeftWrapper.appendTo(scrollWrapper);
        }

        var mainScroll = scrollWrapper.find(".dataTables_scrollBody");
        var topBodyScroll = scrollWrapper.find(".DTFC_TopBodyLiner");
        mainScroll.scroll(function () {
            topBodyScroll.scrollLeft($(this).scrollLeft());
        });
        // initFilterRow(tableDom);
    };

    var calculateHeaderColspanResponsive = function (table, tableDom, colWidths) {
        var header = tableDom.find('thead .row0').eq(0);
        var colspans = [];
        table.columns().every(function (index) {
            var currFirstColWidth = 0;
            var thCol = header.find('th[data-dtc="' + index + '"]');
            var nextColIndexes = [];
            if (thCol.attr('colspan') > 1) {
                var currColIndex = thCol.data('dtc');
                var numberColspan = thCol.attr('colspan');
                colspans.push([currColIndex]);
                nextColIndexes.push(currColIndex);
                for (var i = 0; i < numberColspan; i++) {
                    i++;
                    var nextColIndex = currColIndex + i;
                    nextColIndexes.push(nextColIndex);
                }
            }
        });
    };

    var hideFilterOnResponsive = function (table, tableDom) {
        var filterRow = tableDom.find(".droptables-filter-row");
        table.columns().every(function (index) {
            var thCol = filterRow.find('th[data-dtc="' + index + '"]');
            if (thCol.length < 1) {
                thCol = filterRow.find('th.dtc' + index);
            }
            if (this.responsiveHidden()) {
                thCol.css({display: ""});
            } else {
                thCol.css({display: "none"});
            }
        });
    };

    // UMD
    (function( factory ) {
        "use strict";

        if ( typeof define === 'function' && define.amd ) {
            // AMD
            define( ['jquery'], function ( $ ) {
                return factory( $, window, document );
            } );
        }
        else if ( typeof exports === 'object' ) {
            // CommonJS
            module.exports = function (root, $) {
                if ( ! root ) {
                    root = window;
                }

                if ( ! $ ) {
                    $ = typeof window !== 'undefined' ?
                        require('jquery') :
                        require('jquery')( root );
                }

                return factory( $, root, root.document );
            };
        }
        else {
            // Browser
            factory( jQuery, window, document );
        }
    }
    (function( $, window, document ) {
        $.fn.dataTable.render.moment = function ( from, to, locale ) {
            // Argument shifting
            if ( arguments.length === 1 ) {
                locale = 'en';
                to = from;
                from = 'YYYY-MM-DD';
            }
            else if ( arguments.length === 2 ) {
                locale = 'en';
            }

            return function ( d, type, row ) {
                if (! d) {
                    return type === 'sort' || type === 'type' ? 0 : d;
                }

                var m = window.moment( d, from, locale, true );

                // Order and type get a number value from Moment, everything else
                // sees the rendered value
                return m.format( type === 'sort' || type === 'type' ? 'x' : to );
            };
        };
    }));

    window.droptables_render_tables = function (id_table) {
        var $parent_table = this;

        function change_width_table_when_window_resize_or_visible (table_id) {
            var currTable = $("#" + table_id);
            var tableWrapper = currTable.closest(".dataTables_wrapper");
            var droptablestable = tableWrapper.parent();

            var colWidths = currTable.data('colwidths');
            var totalWidth = 0;

            if (typeof colWidths !== "undefined") {
                for (var i = 0; i < colWidths.length; i++) {
                    totalWidth += colWidths[i];
                }
            }

            if (currTable.data("responsive")) {
                if (totalWidth > 0) {
                    if (droptablestable.width() >= totalWidth) {
                        tableWrapper.css('width', currTable.data('tablewidth'));
                    } else {
                        tableWrapper.css("width", "100%");
                        currTable.css("width", "100%");
                    }
                }
            } else {
                var parent_tableWrapper_width = tableWrapper.parent().width();

                var scrollBarWidth = 1;
                if (parent_tableWrapper_width > 0) {
                    console.log(tableWrapper, parent_tableWrapper_width)
                    if (tableWrapper.width() >= parent_tableWrapper_width) {
                        tableWrapper.css("width", "100%");
                        if (tableWrapper.width() >= currTable.width()) {
                            console.log(currTable.width())
                            tableWrapper.css("width", currTable.width());
                        }
                    } else {
                        scrollBarWidth = getScrollbarWidth(
                            // currTable.closest(".dataTables_scroll").find(".dataTables_scrollBody")
                            typeof currTable.data('freezerow') !== 'undefined' ? currTable.closest(".dataTables_scrollBody") : currTable.closest(".dataTables_scroll")
                        );
                        console.log(currTable.outerWidth() + scrollBarWidth)
                        tableWrapper.css("width", currTable.outerWidth() + scrollBarWidth);
                    }
                } else {//when hidden table container when first load(fix for slide in BEAVER BUILDER)
                    scrollBarWidth = getScrollbarWidth(
                        // currTable.closest(".dataTables_scroll").find(".dataTables_scrollBody")
                        typeof currTable.data('freezerow') !== 'undefined' ? currTable.closest(".dataTables_scrollBody") : currTable.closest(".dataTables_scroll")
                    );
                    console.log(currTable.data('tablewidth') + scrollBarWidth)
                    tableWrapper.css('width', currTable.data('tablewidth') + scrollBarWidth);
                }

                if (tableWrapper.find('.repeatedHeader').length > 0) {
                    var table_breakpoint = tableWrapper.find('.repeatedHeader').data('table-breakpoint');

                    table_breakpoint = parseInt(table_breakpoint) > 0 ? table_breakpoint : 980;

                    if (window.innerWidth <= table_breakpoint) {
                        tableWrapper.find('.repeatedHeader').addClass('repeatedHeaderTrue');
                    } else {
                        tableWrapper.find('.repeatedHeader').removeClass('repeatedHeaderTrue');
                    }
                }
            }
        }

        $(window).resize(function () {
            $(".droptablestable").each(function (index, obj) {
                var droptablestable = $(obj);
                var table_id = droptablestable.data("tableid");

                //check in elementor editor
                if (typeof id_table !== 'undefined' && parseInt(id_table) > 0 && table_id !== 'droptableTbl' + id_table) {
                    return false;
                }

                change_width_table_when_window_resize_or_visible (table_id);
            });
        });
        var keyupDelay = function (callback, ms) {
            var timer = 0;
            return function() {
                var context = this, args = arguments;
                clearTimeout(timer);
                timer = setTimeout(function () {
                    callback.apply(context, args);
                }, ms || 0);
            };
        };
        $(document).ready(function () {
            function droptables_render_table(index, obj) {
                var droptablestable = $(obj);
                var table_id = droptablestable.data("tableid");
                droptablestable.addClass('showed');

                //check in elementor editor
                if (typeof id_table !== 'undefined' && parseInt(id_table) > 0 && table_id !== 'droptablesTbl' + id_table) {
                    return false;
                }

                if (droptablestable.find('.dataTable').length > 0) {
                    return false;
                }

                var tableOptions = {};
                var tableDom = droptablestable.find("#" + table_id);

                if (!tableDom.length) {
                    return;
                }

                if (typeof tableDom.DataTable === 'undefined') {
                    return false;
                }

                var table_id_num = tableDom.data('id');
                var colWidths = tableDom.data('colwidths');
                // tableDom.attr("data-tablewidth", tableDom.width());

                tableOptions.totalWidth = 0;
                if (typeof colWidths !== "undefined") {
                    for (var i = 0; i < colWidths.length; i++) {
                        tableOptions.totalWidth += colWidths[i];
                    }
                }

                //tableWrapper, tableDom, tableOptions.totalWidth
                function change_width (tableWrapper, tableDom, totalWidth) {
                    if (!tableDom.data("change_width")) {
                        var parent_tableWrapper_width = tableWrapper.parent().width();

                        if (tableDom.data("responsive")) {
                            if (totalWidth > 0) {
                                if (parent_tableWrapper_width >= totalWidth) {
                                    tableWrapper.css('width', totalWidth + 'px');
                                    tableDom.css('width', '100%');
                                } else {
                                    tableWrapper.css("width", "100%");
                                    tableDom.css("width", "100%");
                                }
                            }
                        } else {
                            var scrollBarWidth = getScrollbarWidth(
                                typeof tableDom.data('freezerow') !== 'undefined' ? tableDom.closest(".dataTables_scrollBody") : tableDom.closest(".dataTables_scroll")
                            );
                            var width_content = 0;
                            if (totalWidth > tableDom.outerWidth()) {
                                width_content = totalWidth;
                            } else {
                                width_content = tableDom.outerWidth();
                            }
                            if (tableWrapper.width() >= parent_tableWrapper_width || (tableWrapper.width() == 0 && isHidden(tableWrapper[0]))) {
                                document.getElementById(table_id + '_wrapper').style.width = '100%';
                                setTimeout(() => {
                                    if (tableWrapper.width() >= width_content) {
                                        width_content = width_content + scrollBarWidth;
                                    } else {
                                        if (tableWrapper.width() > 0) width_content = tableWrapper.width() + 10;
                                    }

                                    tableWrapper.css("width", width_content);
                                }, 100);
                            } else {
                                tableWrapper.css("width", width_content + scrollBarWidth);
                            }

                            //repeated header responsive
                            setTimeout(() => {
                                if (tableWrapper.find('.repeatedHeader').length > 0) {
                                    var table_breakpoint = tableWrapper.find('.repeatedHeader').data('table-breakpoint');
                                    table_breakpoint = parseInt(table_breakpoint) > 0 ? table_breakpoint : 980;
                                    if (window.innerWidth <= table_breakpoint) {
                                        tableWrapper.find('.repeatedHeader').addClass('repeatedHeaderTrue');
                                    }
                                }
                            }, 100);
                        }
                        tableDom.data("change_width", true);
                    }
                }

                tableDom.attr("data-tablewidth", tableDom.width());

                tableOptions.orderCellsTop = false;
                tableOptions.dom = '<"top">rt<"bottom"pl><"clear">';
                var tableLanguage = {};

                //fix the responsive hiding columns on mobile size
                if (tableDom.data("hidecols")) {
                    tableOptions.dom = '<"top">Bfrt<"bottom"pl><"clear">';
                    tableOptions.buttons = ["colvis"];

                    tableLanguage.buttons = {colvis: tableDom.data("hidecolslanguage")};
                }

                tableLanguage.lengthMenu =
                    '<select><option value="10">10</option><option value="20">20</option><option value="40">40</option><option value="-1">All</option></select>';

                var pagination_merge_cells = [];
                tableOptions.juHideColumn = tableDom.data('hidecolumn');
                tableOptions.juHideColumnClass = [];
                tableOptions.juHideColumns = [];
                if (typeof tableOptions.juHideColumn !== 'undefined' && tableOptions.juHideColumn.length > 0) {
                    var i = 0;
                    var j = 0;
                    for (j = 0; j < tableOptions.juHideColumn.length; j++) {
                        if (tableOptions.juHideColumn[j] > 0) {//hide
                            i++;
                        } else {
                            tableOptions.juHideColumnClass.push(i);
                            tableOptions.juHideColumns[i] = i - tableOptions.juHideColumnClass.length + 1;
                            i++;
                        }
                    }
                }

                if (tableDom.data("paging")) {
                    tableOptions.pagingType = "full_numbers";

                    //if (tableDom.data("type") === 'html') {
                    tableOptions.processing = true;
                    tableOptions.serverSide = true;
                    // var pagination_merge_cells = [];

                    if (typeof droptables_ajaxurl === "undefined") {
                        droptables_ajaxurl = droptables_data.droptables_ajaxurl;
                    }

                    tableOptions.ajax = {
                        'url': droptables_ajaxurl + '&task=fronttable.loadPage&id=' + table_id_num,
                        'type': 'POST',
                        'dataFilter': function(json){
                            var json = jQuery.parseJSON(json);
                            var data = {};
                            data.recordsTotal = json.total;
                            data.recordsFiltered = json.filteredTotal;
                            data.data = json.rows;
                            data.page = json.page;
                            data.draw = json.draw;

                            return JSON.stringify(data); // return JSON string
                        }
                    };

                    tableOptions.lengthMenu = [
                        [10, 20, 40, -1],
                        [10, 20, 40, "All"],
                    ];
                    tableLanguage.paginate = {
                        first:
                            "<i class='icon-step-backward glyphicon glyphicon-step-backward'></i>",
                        previous:
                            "<i class='icon-arrow-left glyphicon glyphicon-backward'></i>",
                        next: "<i class='icon-arrow-right glyphicon glyphicon-forward'></i>",
                        last:
                            "<i class='icon-step-forward glyphicon glyphicon-step-forward'></i>",
                    };
                }

                tableOptions.date_format = tableDom.data("format");

                tableLanguage.first_load = true;
                tableLanguage.left_header = -1;

                droptablestable.find('thead tr').each(function (key, value) {
                    $(value).addClass('row_index' + (key - 1)).data('row-index', (key - 1));
                    tableLanguage.left_header++;
                });
                var number_load = 0;

                tableOptions.fnDrawCallback = function( oSettings ) {
                    droptables_tooltip();
                    setTimeout(() => {
                        var this_oApi = this.oApi;
                        $('.dataTables_wrapper .dataTables_scrollBody thead').hide();
                        $('.DTFC_LeftBodyWrapper thead').hide();

                        var x2 = $.extend([], oSettings.aiDisplay);
                        if (!tableDom.data("paging")) {
                            number_load++;
                            var sort_time = [];
                            var complement_sort_time = [];
                            var order = this.api().order();
                            if (typeof order[0] === 'object') {
                                order = order[0];
                            }
                            oSettings.aiDisplay.forEach(function (value, key) {
                                var i, data, $row = droptablestable.find("tbody .row" + (value + tableLanguage.left_header)), rowSpan = 0;
                                if (tableLanguage.first_load) {
                                    $row.addClass('row_index' + (value + tableLanguage.left_header)).data('row-index', (value + tableLanguage.left_header));
                                } else {
                                    data = droptablestable.find("tbody .row" + (value + tableLanguage.left_header)).data('row-index');
                                    $row.removeClass('row_index' + data).addClass('row_index' + (key + tableLanguage.left_header)).data('row-index', (key + tableLanguage.left_header));
                                }
                                if (oSettings.aiDisplay.length === key + 1) {
                                    tableLanguage.first_load = false;
                                }
                                if (tableDom.data('ordering')) {
                                    if ($row.find('td').filter(function () {
                                        if (this.rowSpan > 1) {
                                            rowSpan = (this.rowSpan - 1) >= rowSpan ? (this.rowSpan - 1) : rowSpan;
                                            return true;
                                        } else {
                                            if ($(this).hasClass('dtc' + order[0])) {
                                                if ($(this).data('timestamp') > 0) {
                                                    sort_time.push([value + tableLanguage.left_header - 1, $(this).data('timestamp')]);
                                                } else {
                                                    complement_sort_time.push([value + tableLanguage.left_header - 1, 0])
                                                }
                                                return true;
                                            }
                                        }
                                        return false;
                                    }).length > 0) {
                                        if (sort_time.length < 1) {
                                            for (i = 0; i < rowSpan; i++) {
                                                x2 = x2.filter(data => data !== (i + value + 1));
                                                var x3 = x2.indexOf(value);
                                                x2.splice((x3 + i + 1), 0, (i + value + 1));
                                            }
                                        }
                                    }

                                    if (key + 1 === oSettings.aiDisplay.length) {
                                        if (number_load === 1) {
                                            if (sort_time.length > 0) {
                                                if (oSettings.aiDisplay.length > sort_time.length) {
                                                    sort_time = sort_time.concat(complement_sort_time);
                                                }
                                                if (order[1] === 'asc') {
                                                    sort_time.sort(function (a, b) {
                                                        return a[1] - b[1]
                                                    });
                                                } else {
                                                    sort_time.sort(function (a, b) {
                                                        return b[1] - a[1]
                                                    });
                                                }
                                                var sort_time2 = sort_time.map(x => x[0]);
                                                oSettings.aiDisplay = $.extend([], sort_time2);
                                                this_oApi._fnDraw(oSettings);
                                            } else {
                                                oSettings.aiDisplay = $.extend([], x2);
                                                this_oApi._fnDraw(oSettings);
                                            }
                                        } else {
                                            number_load = 0;
                                        }
                                    }
                                }
                            });
                        }
                    }, 300);
                    if (tableDom.data("paging")) {
                        change_width(tableWrapper, tableDom, tableOptions.totalWidth);
                    }
                };

                tableOptions.createdRow = function( row, data, dataIndex ) {
                    var keys = Object.keys(data);
                    var $cRow;
                    if (typeof data.DT_RowId !== 'undefined') {
                        $(row).addClass('row' + data.DT_RowId + ' row_index' + ( dataIndex + tableLanguage.left_header)).data('row-index', ( dataIndex + tableLanguage.left_header));
                    } else {
                        $(row).addClass(' row_index' + ( dataIndex + tableLanguage.left_header)).data('row-index', ( dataIndex + tableLanguage.left_header));
                    }

                    if (typeof data.merges !== 'undefined') {
                        $.each(data.merges, function (key, value) {
                            if (typeof tableOptions.juHideColumns !== 'undefined'
                                && typeof tableOptions.juHideColumns[value[2]] !== 'undefined'
                                && tableOptions.juHideColumns[value[2]] > 0) {
                                value[2] = parseInt(value[2]) - tableOptions.juHideColumns[value[2]];
                            } else {
                            }
                            $(row).find('td:nth-child(' + (1 + parseInt(value[2])) + ')').attr('colspan', value[3]).attr('rowspan', value[1]);
                            //merger rows
                            if (typeof pagination_merge_cells[value[0]] == 'undefined') {
                                pagination_merge_cells[value[0]] = [];
                            }
                            pagination_merge_cells[value[0]][value[2]] = value;
                            var rowspanI = 0;
                            for (rowspanI = 0; rowspanI < parseInt(value[1]); rowspanI++) {
                                if (typeof pagination_merge_cells[parseInt(value[0]) + rowspanI] == 'undefined') {
                                    pagination_merge_cells[parseInt(value[0]) + rowspanI] = [];
                                }
                                pagination_merge_cells[parseInt(value[0]) + rowspanI][value[2]] = value;
                            }
                        });
                    }

                    let regex2 = new RegExp('(dtc)([0-9]+)');
                    keys.forEach(function (key, index) {
                        if (key !== 'merges' && key !== 'DT_RowId' && key !== 'format_date_cell') {
                            $cRow = $(row).find('td:nth-child(' + (parseInt(key) + 1).toString() + ')');
                            let columnClass = 'dtc' + key;
                            if (typeof tableOptions.juHideColumn !== 'undefined') {
                                columnClass = 'dtc' + tableOptions.juHideColumnClass[key];
                            }
                            if (typeof tableOptions.pagingType !== 'undefined' && typeof tableOptions.juHideColumn !== 'undefined') {
                                if (key !== 'DT_RowId') {
                                    if ($cRow.length) {
                                        if (typeof data.DT_RowId !== 'undefined') {
                                            $cRow.addClass('dtr' + data.DT_RowId).addClass(columnClass);
                                        } else {
                                            $cRow.addClass(columnClass);
                                        }
                                    }
                                }
                            } else {
                                if (key !== 'DT_RowId') {
                                    if ($cRow.length) {
                                        let classColumn = $cRow.attr("class").split(' ').filter(classname => regex2.exec(classname) !== null)
                                        if (classColumn[0]) {
                                            $cRow.removeClass(classColumn[0]);
                                        }
                                        if (typeof data.DT_RowId !== 'undefined') {
                                            $cRow.addClass('dtr' + data.DT_RowId).addClass(columnClass);
                                        } else {
                                            $cRow.addClass(columnClass);
                                        }
                                    }
                                }
                            }

                            if (key !== 'DT_RowId' && typeof data.DT_RowId !== 'undefined' && typeof pagination_merge_cells[data.DT_RowId] !== 'undefined') {//has merger
                                pagination_merge_cells[data.DT_RowId].forEach(function (value, key) {
                                    var colspanI = 0;
                                    for (colspanI = 0; colspanI < parseInt(value[3]); colspanI++) {
                                        if (!(parseInt(value[0]) == data.DT_RowId && colspanI === 0)) {
                                            $(row).find('td:nth-child(' + (1 + parseInt(value[2]) + colspanI) + ')').css('display', 'none');
                                        }
                                    }
                                });
                            }
                            var value_format = '';
                            var value_timestamp = '';

                            if (typeof $cRow.data('format') !== 'undefined' && $cRow.text() !== '' && $cRow.text() !== null) {
                                if ($cRow.data('format') == '1') {
                                    value_format = moment($cRow.text()).format(tableOptions.date_format);
                                    value_timestamp = moment($cRow.text()).format('X');
                                    $cRow.data('timestamp', value_timestamp).text(value_format).change();
                                } else if ($cRow.data('format') !== '0') {
                                    if (moment($cRow.text(), tableOptions.date_format).format($cRow.data('format')) === 'Invalid date') {
                                        value_format = moment($cRow.text(), $cRow.data('format')).format($cRow.data('format'));
                                        value_timestamp = moment($cRow.text(), $cRow.data('format')).format('X');
                                    } else {
                                        value_format = moment($cRow.text(), tableOptions.date_format).format($cRow.data('format'));
                                        value_timestamp = moment($cRow.text(), tableOptions.date_format).format('X');
                                    }
                                    $cRow.data('timestamp', value_timestamp).text(value_format).change();
                                }
                            } else if ($cRow.text() !== '' && $cRow.data('sort') > 0) {
                                $cRow.data('timestamp', $cRow.data('sort')).change();
                            }

                            if (typeof data.format_date_cell !== 'undefined' && typeof data.format_date_cell[key] !== 'undefined' && $cRow.text() !== '' && $cRow.text() !== null) {
                                if (data.format_date_cell[key] === '1') {
                                    value_format = moment($cRow.text()).format(tableOptions.date_format);
                                    value_timestamp = moment($cRow.text()).format('X');
                                    $cRow.data('timestamp', value_timestamp).text(value_format).change();
                                } else if (data.format_date_cell[key] !== '0') {
                                    if (moment($cRow.text(), tableOptions.date_format).format(data.format_date_cell[key]) === 'Invalid date') {
                                        value_format = moment($cRow.text(), data.format_date_cell[key]).format(data.format_date_cell[key]);
                                        value_timestamp = moment($cRow.text(), data.format_date_cell[key]).format('X');
                                    } else {
                                        value_format = moment($cRow.text(), tableOptions.date_format).format(data.format_date_cell[key]);
                                        value_timestamp = moment($cRow.text(), tableOptions.date_format).format('X');
                                    }
                                    $cRow.data('timestamp', value_timestamp).text(value_format).change();
                                }
                            }
                        }
                    });
                };

                tableOptions.language = tableLanguage;

                var initFilterRow = function(tableDom) {
                    // Apply the search
                    if (tableDom.hasClass("filterable")) {
                        addFilterRowToTable(tableDom);
                    }
                };
                var addFilterRowToTable = function(tbl) {
                    // Add an input to latest th in header
                    tbl.find("thead tr:not(.droptables-header-cells-index):last-child th").each(function(i) {
                        var thContent = $(this).html();
                        var inputHtml = '<br><input onClick="var event = arguments[0] || window.event;event.stopPropagation();" type="text" name="wtmp_col_filter" class="droptables-d-block droptables-filter-input stop-propagation" data-index="' + i + '" value="" />';
                        $(this).html(thContent + inputHtml);
                    });
                };

                if (tableDom.data("ordering")) {
                    // tableOptions.serverSide = false;
                    tableOptions.ordering = true;
                    var dataOrder = [];
                    dataOrder.push(tableDom.data("ordertarget"));
                    dataOrder.push(tableDom.data("ordervalue"));
                    tableOptions.order = dataOrder;
                }

                initFilterRow($(tableDom));
                // tableOptions.columnDefs = [
                // {
                //     target: 1, //index of column
                //     type: 'datetime-moment'
                // }
                // ];

                tableOptions.columnDefs = [{
                    "targets": typeof tableOptions.juHideColumnClass !== 'undefined' ? [...tableOptions.juHideColumnClass.keys()] : [...tableDom.data('hidecolumn').keys()],
                    "render": function ( data, type, row, meta ) {
                        if (typeof data === 'undefined' && typeof row[meta.col] !== 'undefined') {
                            return row[meta.col];
                        }
                        return data;
                    }
                }];

                tableOptions.fnInitComplete = function( settings, json ) {
                    setTimeout(function() {
                        $('.dataTables_wrapper .dataTables_scrollBody thead').hide();
                        $('.DTFC_LeftBodyWrapper thead').hide();
                    }, 500);
                };

                //fix width table when enable the responsive hiding columns on mobile size
                // var table = tableDom.DataTable(tableOptions);
                var table = tableDom.on('init.dt', function () {
                    tableDom.find('colgroup col').hide();
                }).DataTable(tableOptions);
                // var table = tableDom.DataTable(tableOptions);

                $(table.table().container()).on('keyup change', 'input.droptables-filter-input', function (e)
                {
                    e.stopPropagation();
                    columnFilter(table, $(this).data('index'), $(this).val());
                });

                if (typeof tableDom.data("freezecol") !== "undefined") {
                    new $.fn.dataTable.FixedColumns(table, {
                        leftColumns: tableDom.data("freezecol"),
                    });
                }

                var columnFilter = function (table, columnIndex, val) {
                    if (table.column(columnIndex).search() !== val) {
                        window.clearTimeout(filterDelay);
                        filterDelay = window.setTimeout(function() {
                            table.column(columnIndex).search(val).draw();
                        }, filterDelayInterval);
                    }
                };

                if (tableDom.data("responsive") === true) {
                    if ($(".droptables-filter-row").length > 0) {
                        hideFilterOnResponsive(table, tableDom);
                    }
                    //calculateHeaderColspanResponsive(table, tableDom, colWidths);
                    table.on("responsive-resize", function () {
                        if ($(".droptables-filter-row").length > 0) {
                            hideFilterOnResponsive(table, tableDom);
                        }
                    });
                }

                // Change div table wrapper width
                var tableWrapper = tableDom.closest(".dataTables_wrapper");
                var tableAlign = tableDom.data("align");
                var margin = "0 0 0 auto";
                if (tableAlign === "center") {
                    margin = "0 auto";
                } else if (tableAlign === "left") {
                    margin = "0 auto 0 0";
                } else if (tableAlign === "right") {
                    margin = "0 0 0 auto";
                }
                tableWrapper.css("margin", margin);
                tableWrapper.closest(".wptm_table").css("margin", margin);

                if (!tableDom.data("paging")) {
                    change_width (tableWrapper, tableDom, tableOptions.totalWidth);
                }

                //fix height for before td in repeatedHeader
                if (tableWrapper.find('.repeatedHeader').length > 0) {

                }

                table.rows(".hidden_row").remove().draw();

                if (!tableDom.data("paging")) {
                    tableDom
                        .closest(".droptablestable")
                        .find(".dataTables_info")
                        .css("display", "none");
                }

                var hightLight = tableDom.closest(".droptables_table").data("hightlight");
                if (typeof hightLight === "undefined") {
                    hightLight = tableDom.closest(".droptables_dbtable").data("highlight");
                }
                var classHightLight = "droptables-highlight-vertical";

                if (hightLight === 1) {
                    table.on("mouseenter", "td", function () {
                        if (typeof table.cell(this).index() !== "undefined") {
                            var colIdx = table.cell(this).index().column;
                            var rowIdx = table.cell(this).index().row;
                            var affectedRow = 0;
                            var affectedCol = table.cell(this).index().column;

                            $(table.cells().nodes()).removeClass(
                                "droptables-highlight-vertical"
                            );

                            //line, col, both
                            if (hightLight !== 1) {
                                $(table.column(colIdx).nodes()).addClass(
                                    "droptables-highlight-vertical"
                                );
                            }

                            if (hightLight !== 2) {
                                table.row(rowIdx).every(function () {
                                    var row = $(this.node());
                                    row.find("td").addClass("droptables-highlight-vertical");
                                    affectedRow = row.find("td").data("dtr");
                                });
                            }

                            var leftWrapperTable = tableDom
                                .closest(".dataTables_wrapper")
                                .find(".DTFC_LeftBodyLiner table");
                            var topWrapperTable = tableDom
                                .closest(".dataTables_wrapper")
                                .find(".DTFC_TopBodyLiner table");
                            if (leftWrapperTable.length > 0) {
                                leftWrapperTable.find("td").removeClass(classHightLight);
                                leftWrapperTable
                                    .find(".dtr" + affectedRow)
                                    .addClass(classHightLight);
                            }
                            if (topWrapperTable.length > 0) {
                                topWrapperTable.find("td").removeClass(classHightLight);
                                topWrapperTable
                                    .find(".dtc" + affectedCol)
                                    .addClass(classHightLight);
                            }
                        }
                    });
                    table.on("mouseleave", "td", function () {
                        if (typeof table.cell(this).index() !== "undefined") {
                            $(table.cells().nodes()).removeClass(
                                "droptables-highlight-vertical"
                            );

                            var leftWrapperTable = tableDom
                                .closest(".dataTables_wrapper")
                                .find(".DTFC_LeftBodyLiner table");
                            var topWrapperTable = tableDom
                                .closest(".dataTables_wrapper")
                                .find(".DTFC_TopBodyLiner table");
                            if (leftWrapperTable.length > 0) {
                                leftWrapperTable.find("td").removeClass(classHightLight);
                            }
                            if (topWrapperTable.length > 0) {
                                topWrapperTable.find("td").removeClass(classHightLight);
                            }
                        }
                    });
                }

                /*if table not  content*/
                if (tableDom.find('td.dataTables_empty').length > 0) {
                    tableDom.css({width: '95%'});
                }

                //change width table when table visible in toggle
                respondToVisibility = function(element, callback) {
                    var options = {
                        root: document.documentElement
                    }

                    var observer = new IntersectionObserver((entries, observer) => {
                        entries.forEach(entry => {
                            callback(entry.intersectionRatio > 0);
                        });
                    }, options);

                    observer.observe(element);
                }

                respondToVisibility(document.getElementById(table_id), visible => {
                    if(visible) {
                        change_width_table_when_window_resize_or_visible (table_id);
                    } else {
                        console.log("Not Visible");
                    }
                });
            }
            if (typeof id_table !== 'undefined') {
                droptables_render_table(1, $parent_table.find("#droptablestable" + id_table + ".droptablestable"));
            } else {
                $(".droptablestable").each(function (index, obj) {
                    if (!$(obj).is(':visible')) {
                        respondToVisibility(document.getElementById($(obj).data('tableid')), visible => {
                            if(visible) {
                                if (!$(obj).hasClass('showed')) {
                                    droptables_render_table(index, obj);
                                }
                            } else {
                                console.log("Not Visible");
                            }
                        });
                    } else {
                        if (!$(obj).hasClass('showed')) {
                            droptables_render_table(index, obj);
                        }
                    }
                });
            }

            setTimeout(function () {
                droptables_tooltip();
            }, 100);

            function droptables_tooltip() {
                $(".droptables_tooltip ").each(function () {
                    var that = $(this);
                    $(that).tipso({
                        useTitle: false,
                        tooltipHover: true,
                        background: "#000000",
                        color: "#ffffff",
                        offsetY: 0,
                        width: $(that).find(".droptables_tooltipcontent").data("width"),
                        content: $(that).find(".droptables_tooltipcontent").html(),
                        onShow: function (ele, tipso, obj) {
                            //calculate top tipso_bubble when set width
                            var size = realHeight(obj.tooltip());
                            $(obj.tipso_bubble[0]).css(
                                "top",
                                obj.tipso_bubble[0].offsetTop +
                                (size.height - obj.tipso_bubble.outerHeight())
                            );
                        },
                    });
                });

                function realHeight(obj) {
                    var clone = obj.clone();
                    clone.css("visibility", "hidden");
                    $("body").append(clone);
                    var height = clone.outerHeight();
                    var width = clone.outerWidth();
                    console.log(width)
                    clone.remove();
                    return {
                        width: width,
                        height: height,
                    };
                }
            }

            $(".droptables_table .download_droptables")
                .unbind("click")
                .click(function () {
                    if (typeof droptables_ajaxurl === "undefined") {
                        droptables_ajaxurl = droptables_data.droptables_ajaxurl;
                    }

                    var $that = $(this);
                    $that.after('<span class="wptm_downloading" style="margin-left: 10px;font-size: 0.8em;">Downloading...</span>');

                    var id_table = $that.parents(".droptables_table").data("id");
                    var url = droptables_ajaxurl;
                    url += "&task=fronttable.export&id=" +
                        id_table +
                        "&format_excel=xlsx&onlydata=1";
                    $.fileDownload(url, {
                        downloadSuccess: function () {
                            $that.siblings('.wptm_downloading').remove();
                        },
                        failCallback: function (html, url) {
                            $that.siblings('.wptm_downloading').remove();
                        },
                    });
                });
        });
        function isHidden(el) {
            var style = window.getComputedStyle(el);
            return (style.display === 'none')
        }
    }
    window.droptables_render_tables.call();
})(jQuery);
